function [ G ] = objectDynamics( Tau, K )
%OBJECTDYNAMICS Descbribes the dynamics of the object detected and
%identified by the UAV. T contains the time constants, and K the scaling
%factors.
%   The dynamics are represented by
%   Xy(k) = G(z)Xr(k), where
%   G = [g(1,1) 0 g(1,3);
%        0   g(2,2)   0;
%        g(3,1) 0 g(3,3)];

% Init variabels
Tii=Tau(1);
Tij=Tau(2);
Kii=K(1);
Kij=K(2);

gii = tf([Kii],[Tii^2 2*Tii 1]);
gij = tf([Kij 0],[Tij^2 2*Tij 1]);

G = [gii 0 gij;
     0 gii 0;
     gij 0 gii];

end

