/// -*- tab-width: 4; Mode: C++; c-basic-offset: 4; indent-tabs-mode: nil -*-

// user defined variables

// example variables used in Wii camera testing - replace with your own
// variables
#ifdef USERHOOK_VARIABLES

#if WII_CAMERA == 1
WiiCamera           ircam;
int                 WiiRange=0;
int                 WiiRotation=0;
int                 WiiDisplacementX=0;
int                 WiiDisplacementY=0;
#endif  // WII_CAMERA

#if DUNEMODE == ENABLED

/*
* Target system is uint8_t
* Split into four fields a two bits
* 0b 7 6 5 4 3 2 1 0
*
* Bits   Description
* 0-1      How to use xy-component
* 2-4    How to use z -component
* 5      How to use roll-pitch component
* 6-7    How to use yaw-component
*
*/

enum APM_target_masks {
	TARGET_MASK_XY   = 0b00000011,
	TARGET_MASK_Z    = 0b00011100,
	TARGET_MASK_RP   = 0b00100000,
	TARGET_MASK_YAW  = 0b11000000
};

enum APM_target_xy {
	// MASK:              0b00000011
	TARGET_XY_NONE      = 0b00000000,
	TARGET_XY_VEL_BODY  = 0b00000001,
	TARGET_XY_VEL_NED   = 0b00000010,
};

enum APM_target_z {
	// MASK:          0b00011100
	TARGET_Z_NONE   = 0b00000000,
	TARGET_Z_VEL    = 0b00000100,
	TARGET_Z_ALT    = 0b00001000,
	TARGET_Z_THRUST = 0b00001100,
	TARGET_Z_RC     = 0b00010000
};

enum APM_target_rp {
	// MASK:          0b00100000
	TARGET_RP_NONE  = 0b00000000,
	TARGET_RP_SET   = 0b00100000,
};

enum APM_target_yaw {
	// MASK:              0b11000000
	TARGET_YAW_NONE     = 0b00000000,
	TARGET_YAW_SET      = 0b01000000,
	TARGET_YAW_RC       = 0b11000000,
	TARGET_YAW_RESERVED = 0b10000000
};


////////////////////////////////////////////////////////////////////////////////
// Velocity control variables
////////////////////////////////////////////////////////////////////////////////

/* The desired velocity in 2D-body in x and y.
 * 2D-body do not take z into account. It can be compared to as NED with a compensation for yaw angle
 * Range: -4500~4500 (a range outside the range will give a higher velocity then loiter speed)
 */
static int32_t vel_body_x = 0;
static int32_t vel_body_y = 0;

/* The desired velocity in NED in x and y.
 * Range: -4500~4500 (a range outside the range will give a higher velocity then loiter speed)
 */
static int32_t vel_ned_x = 0;
static int32_t vel_ned_y = 0;

/* Loiter speed. Used in calculation of velocity
 */
static int32_t vel_loiter_speed_cms = 500;

////////////////////////////////////////////////////////////////////////////////
// Z control variables
////////////////////////////////////////////////////////////////////////////////

/* The desired altitude.
 * Unit: Centimeters
 */
static int32_t z_alt_cm = 30*100;

// Currently not implemented
/* The desired velocity in z.
 * Range: 0~0
 */
static int32_t z_vel = 0;

/* The desired thrust.
 * Range: 0~1000
 */
static int32_t z_thrust = 0;


////////////////////////////////////////////////////////////////////////////////
// RP control variables
////////////////////////////////////////////////////////////////////////////////

/* The desired roll and pitch angle.
 * Unit: Hundredth of a degree
 */
static int32_t rp_roll_cd  = 0;
static int32_t rp_pitch_cd = 0;

////////////////////////////////////////////////////////////////////////////////
// YAW control variables
////////////////////////////////////////////////////////////////////////////////

/* The desired roll and pitch angle.
 * Unit: Hundredth of a degree
 */
static int32_t yaw_cd  = 0;

////////////////////////////////////////////////////////////////////////////////
// Mode control variables
////////////////////////////////////////////////////////////////////////////////

/* State variables for keeping track of mode of velocity, RP, yaw and z
 * Initializing modes.
 */

static APM_target_xy  mode_xy  = TARGET_XY_NONE;
static APM_target_z   mode_z   = TARGET_Z_RC;
static APM_target_rp  mode_rp  = TARGET_RP_NONE;
static APM_target_yaw mode_yaw = TARGET_YAW_RC;
static bool dune_mode_from_rc  = true;

/*
 * Debug variables
 */

static int32_t vx;
static int32_t vy;
static int32_t thrust;

#endif




#endif  // USERHOOK_VARIABLES


