//***************************************************************************
// Copyright 2007-2014 Universidade do Porto - Faculdade de Engenharia      *
// Laboratório de Sistemas e Tecnologia Subaquática (LSTS)                  *
//***************************************************************************
// This file is part of DUNE: Unified Navigation Environment.               *
//                                                                          *
// Commercial Licence Usage                                                 *
// Licencees holding valid commercial DUNE licences may use this file in    *
// accordance with the commercial licence agreement provided with the       *
// Software or, alternatively, in accordance with the terms contained in a  *
// written agreement between you and Universidade do Porto. For licensing   *
// terms, conditions, and further information contact lsts@fe.up.pt.        *
//                                                                          *
// European Union Public Licence - EUPL v.1.1 Usage                         *
// Alternatively, this file may be used under the terms of the EUPL,        *
// Version 1.1 only (the "Licence"), appearing in the file LICENCE.md       *
// included in the packaging of this file. You may not use this work        *
// except in compliance with the Licence. Unless required by applicable     *
// law or agreed to in writing, software distributed under the Licence is   *
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF     *
// ANY KIND, either express or implied. See the Licence for the specific    *
// language governing permissions and limitations at                        *
// https://www.lsts.pt/dune/licence.                                        *
//***************************************************************************
// Author: Kristian Klausen                                                 *
//***************************************************************************

// DUNE headers.
#include <DUNE/DUNE.hpp>

namespace Control
{
  namespace UAV
  {
    namespace SAR
    {
      namespace GenerateDetection
      {
        using DUNE_NAMESPACES;

        struct Task: public DUNE::Tasks::Task
        {

          IMC::EstimatedState m_estate;

          //! Constructor.
          //! @param[in] name task name.
          //! @param[in] ctx context.
          Task(const std::string& name, Tasks::Context& ctx):
            DUNE::Tasks::Task(name, ctx)
          {

            bind<IMC::EstimatedState>(this);
            bind<IMC::DetectedObject>(this);
          }

          //! Update internal state with new parameter values.
          void
          onUpdateParameters(void)
          {
          }

          void
          consume(const IMC::EstimatedState* msg)
          {
            m_estate = *msg;
          }

          void
          consume(const IMC::DetectedObject* msg)
          {

            // Check that source was neptus
            if( msg->state == IMC::DetectedObject::OBJ_MANUAL)
            {
              // Copy the message, set current lat/lon.
              IMC::DetectedObject copy;
              copy.id = msg->id;
              copy.lat = m_estate.lat;
              copy.lon = m_estate.lon;
              copy.time = Clock::get();
              copy.state = IMC::DetectedObject::OBJ_DETECTED;

              dispatch(copy);

              inf("Dispatched a new detected object!");
            }
          }

          //! Reserve entity identifiers.
          void
          onEntityReservation(void)
          {
          }

          //! Resolve entity names.
          void
          onEntityResolution(void)
          {
          }

          //! Acquire resources.
          void
          onResourceAcquisition(void)
          {
          }

          //! Initialize resources.
          void
          onResourceInitialization(void)
          {
          }

          //! Release resources.
          void
          onResourceRelease(void)
          {
          }

          //! Main loop.
          void
          onMain(void)
          {
            while (!stopping())
            {
              waitForMessages(1.0);
            }
          }
        };
      }
    }
  }
}

DUNE_TASK
