//***************************************************************************
// Copyright 2007-2014 Universidade do Porto - Faculdade de Engenharia      *
// Laboratório de Sistemas e Tecnologia Subaquática (LSTS)                  *
//***************************************************************************
// This file is part of DUNE: Unified Navigation Environment.               *
//                                                                          *
// Commercial Licence Usage                                                 *
// Licencees holding valid commercial DUNE licences may use this file in    *
// accordance with the commercial licence agreement provided with the       *
// Software or, alternatively, in accordance with the terms contained in a  *
// written agreement between you and Universidade do Porto. For licensing   *
// terms, conditions, and further information contact lsts@fe.up.pt.        *
//                                                                          *
// European Union Public Licence - EUPL v.1.1 Usage                         *
// Alternatively, this file may be used under the terms of the EUPL,        *
// Version 1.1 only (the "Licence"), appearing in the file LICENCE.md       *
// included in the packaging of this file. You may not use this work        *
// except in compliance with the Licence. Unless required by applicable     *
// law or agreed to in writing, software distributed under the Licence is   *
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF     *
// ANY KIND, either express or implied. See the Licence for the specific    *
// language governing permissions and limitations at                        *
// https://www.lsts.pt/dune/licence.                                        *
//***************************************************************************
// Author: Kristian Klausen                                                 *
//***************************************************************************

// DUNE headers.
#include <DUNE/DUNE.hpp>

// Storage of detections
#include <map>

namespace Control
{
  namespace UAV
  {
    namespace SAR
    {
      namespace ReceiveDetection
      {
        using DUNE_NAMESPACES;


        struct Task: public DUNE::Tasks::Task
        {

          std::map<int, IMC::DetectedObject> m_detected_objects;

          //! Constructor.
          //! @param[in] name task name.
          //! @param[in] ctx context.
          Task(const std::string& name, Tasks::Context& ctx):
            DUNE::Tasks::Task(name, ctx)
          {
            bind<IMC::DetectedObject>(this);
          }

          void
          consume(const IMC::DetectedObject* msg)
          {
            // Check that source was not simulated
            if( msg->state != IMC::DetectedObject::OBJ_MANUAL)
            {
              // Try to find the object in our list
              std::map<int, IMC::DetectedObject>::iterator it;
              it = m_detected_objects.find(msg->id);

              // Not Found
              if(it == m_detected_objects.end() || msg->id == 0)
              {
                // Store the message if this is a new object
                if (msg->state == IMC::DetectedObject::OBJ_DETECTED)
                {
                  // store and assign a new ID.
                  int id = m_detected_objects.size() + 1;



                  m_detected_objects[id] = *msg;
                  m_detected_objects[id].id = id;
                  m_detected_objects[id].state = IMC::DetectedObject::OBJ_REGISTERED;
                  //dispatch(m_detected_objects[id]);

                  // For now, auto-up to NEED INPUT
                  m_detected_objects[id].state = IMC::DetectedObject::OBJ_INPUT_NEEDED;

                  // Dispatch the object with the new state.
                  dispatch(m_detected_objects[id]);

                  // Info
                  inf("Stored a new object, with ID: %d", id);
                }
                else
                {
                  war("Got a unknown ID. Ignoring. ");
                }
              }
              else
              {
                // Found the id.
                // Check if we should update state
                if (msg->state == IMC::DetectedObject::OBJ_USER_CONFIRMS)
                {
                  m_detected_objects.at(msg->id).state = IMC::DetectedObject::OBJ_CONFIRMED;
                  inf("Confirmed object with id %d", msg->id);

                  // Dispatch updated state.
                  dispatch(m_detected_objects.at(msg->id));
                }
                else if (msg->state == IMC::DetectedObject::OBJ_USER_REJECTS)
                {
                  m_detected_objects.at(msg->id).state = IMC::DetectedObject::OBJ_REJECTED;
                  inf("Rejected object with id %d", msg->id);

                  // Dispatch updated state.
                  dispatch(m_detected_objects.at(msg->id));
                }
                else if (msg->state != IMC::DetectedObject::OBJ_REGISTERED)
                {
                  //war("Got a detected notification of an already registered ID. ");
                }
              }




            }
          }

          //! Update internal state with new parameter values.
          void
          onUpdateParameters(void)
          {
          }

          //! Reserve entity identifiers.
          void
          onEntityReservation(void)
          {
          }

          //! Resolve entity names.
          void
          onEntityResolution(void)
          {
          }

          //! Acquire resources.
          void
          onResourceAcquisition(void)
          {
          }

          //! Initialize resources.
          void
          onResourceInitialization(void)
          {
          }

          //! Release resources.
          void
          onResourceRelease(void)
          {
          }

          //! Main loop.
          void
          onMain(void)
          {
            while (!stopping())
            {
              waitForMessages(1.0);
            }
          }
        };
      }
    }
  }
}

DUNE_TASK
