//***************************************************************************
// Copyright 2007-2014 Universidade do Porto - Faculdade de Engenharia      *
// Laboratório de Sistemas e Tecnologia Subaquática (LSTS)                  *
//***************************************************************************
// This file is part of DUNE: Unified Navigation Environment.               *
//                                                                          *
// Commercial Licence Usage                                                 *
// Licencees holding valid commercial DUNE licences may use this file in    *
// accordance with the commercial licence agreement provided with the       *
// Software or, alternatively, in accordance with the terms contained in a  *
// written agreement between you and Universidade do Porto. For licensing   *
// terms, conditions, and further information contact lsts@fe.up.pt.        *
//                                                                          *
// European Union Public Licence - EUPL v.1.1 Usage                         *
// Alternatively, this file may be used under the terms of the EUPL,        *
// Version 1.1 only (the "Licence"), appearing in the file LICENCE.md       *
// included in the packaging of this file. You may not use this work        *
// except in compliance with the Licence. Unless required by applicable     *
// law or agreed to in writing, software distributed under the Licence is   *
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF     *
// ANY KIND, either express or implied. See the Licence for the specific    *
// language governing permissions and limitations at                        *
// https://www.lsts.pt/dune/licence.                                        *
//***************************************************************************
// Author: Ricardo Martins                                                  *
//***************************************************************************

// DUNE headers.
#include <DUNE/Compression/Compressor.hpp>
#include <DUNE/Compression/Exceptions.hpp>

namespace DUNE
{
  namespace Compression
  {
    void
    Compressor::compress(char* dst, unsigned long dst_len, char* src, unsigned long src_len)
    {
      m_compressed = 0;
      m_processed = 0;

      unsigned long rv = compressBlock(dst, dst_len, src, src_len);
      m_compressed = rv;
      m_compressed_total += rv;
      m_processed = src_len;
      m_processed_total += src_len;
    }

    Utils::ByteBuffer
    Compressor::compress(char* src, unsigned long src_len)
    {
      Utils::ByteBuffer dst;
      compress(dst, src, src_len);
      return dst;
    }

    void
    Compressor::compress(Utils::ByteBuffer& dst, char* src, unsigned long src_len)
    {
      unsigned bfr_size = compressBound(src_len);

      bool done = false;
      while (!done)
      {
        dst.setSize(bfr_size);

        try
        {
          compress(dst.getBufferSigned(), dst.getSize(), src, src_len);
          done = true;
        }
        catch (BufferTooShort& e)
        {
          (void)e;
          bfr_size <<= 1;
        }
      }

      dst.setSize(m_compressed);
    }

    Utils::ByteBuffer
    Compressor::compress(Utils::ByteBuffer& src)
    {
      return compress(src.getBufferSigned(), src.getSize());
    }

    void
    Compressor::compress(Utils::ByteBuffer& dst, Utils::ByteBuffer& src)
    {
      return compress(dst, src.getBufferSigned(), src.getSize());
    }
  }
}
