//***************************************************************************
// Copyright 2007-2014 Universidade do Porto - Faculdade de Engenharia      *
// Laboratório de Sistemas e Tecnologia Subaquática (LSTS)                  *
//***************************************************************************
// This file is part of DUNE: Unified Navigation Environment.               *
//                                                                          *
// Commercial Licence Usage                                                 *
// Licencees holding valid commercial DUNE licences may use this file in    *
// accordance with the commercial licence agreement provided with the       *
// Software or, alternatively, in accordance with the terms contained in a  *
// written agreement between you and Universidade do Porto. For licensing   *
// terms, conditions, and further information contact lsts@fe.up.pt.        *
//                                                                          *
// European Union Public Licence - EUPL v.1.1 Usage                         *
// Alternatively, this file may be used under the terms of the EUPL,        *
// Version 1.1 only (the "Licence"), appearing in the file LICENCE.md       *
// included in the packaging of this file. You may not use this work        *
// except in compliance with the Licence. Unless required by applicable     *
// law or agreed to in writing, software distributed under the Licence is   *
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF     *
// ANY KIND, either express or implied. See the Licence for the specific    *
// language governing permissions and limitations at                        *
// https://www.lsts.pt/dune/licence.                                        *
//***************************************************************************
// Author: Eduardo Marques                                                  *
//***************************************************************************

// DUNE headers.
#include <DUNE/DUNE.hpp>

namespace Maneuver
{
  namespace Goto
  {
    using DUNE_NAMESPACES;

    struct Task: public DUNE::Maneuvers::Maneuver
    {
      //! Desired Path message to send to path control
      IMC::DesiredPath m_path;

      Task(const std::string& name, Tasks::Context& ctx):
        DUNE::Maneuvers::Maneuver(name, ctx)
      {
        bindToManeuver<Task, IMC::Goto>();
        bind<IMC::PathControlState>(this);
        bind<IMC::DetectedObject>(this);
        bind<IMC::GuidanceControlState>(this);
      }

      void
      resumeGoto(void)
      {
        setControl(IMC::CL_PATH);
        dispatch(m_path);
      }

      void
      consume(const IMC::DetectedObject* msg)
      {
        if (msg->state == IMC::DetectedObject::OBJ_INPUT_NEEDED)
        {
          setControl(IMC::CL_GUIDANCE);
          err("Enabled guidance");
        }

        if ( (msg->state == IMC::DetectedObject::OBJ_CONFIRMED) || (msg->state == IMC::DetectedObject::OBJ_REJECTED) )
        {
          err("Object confirmed or rejected.");
          IMC::GuidanceControlState gcs_stop;
          gcs_stop.flags = IMC::GuidanceControlState::FL_DONE;
          dispatch(gcs_stop, Tasks::DF_LOOP_BACK);
        }
      }

      void
      consume(const IMC::GuidanceControlState* pcs)
      {
        war("Goto Done");
        if (pcs->flags & IMC::GuidanceControlState::FL_DONE)
        {
          err("Resume goto");
          resumeGoto();
        }
        else
        {
          signalProgress(pcs->eta);
        }
      }

      void
      consume(const IMC::Goto* maneuver)
      {
        setControl(IMC::CL_PATH);

        m_path.end_lat = maneuver->lat;
        m_path.end_lon = maneuver->lon;
        m_path.end_z = maneuver->z;
        m_path.end_z_units = maneuver->z_units;
        m_path.speed = maneuver->speed;
        m_path.speed_units = maneuver->speed_units;

        dispatch(m_path);
      }

      void
      consume(const IMC::PathControlState* pcs)
      {
        if (pcs->flags & IMC::PathControlState::FL_NEAR)
          signalCompletion();
        else
          signalProgress(pcs->eta);
      }
    };
  }
}

DUNE_TASK
