//***************************************************************************
// Copyright 2007-2014 Universidade do Porto - Faculdade de Engenharia      *
// Laboratório de Sistemas e Tecnologia Subaquática (LSTS)                  *
//***************************************************************************
// This file is part of DUNE: Unified Navigation Environment.               *
//                                                                          *
// Commercial Licence Usage                                                 *
// Licencees holding valid commercial DUNE licences may use this file in    *
// accordance with the commercial licence agreement provided with the       *
// Software or, alternatively, in accordance with the terms contained in a  *
// written agreement between you and Universidade do Porto. For licensing   *
// terms, conditions, and further information contact lsts@fe.up.pt.        *
//                                                                          *
// European Union Public Licence - EUPL v.1.1 Usage                         *
// Alternatively, this file may be used under the terms of the EUPL,        *
// Version 1.1 only (the "Licence"), appearing in the file LICENCE.md       *
// included in the packaging of this file. You may not use this work        *
// except in compliance with the Licence. Unless required by applicable     *
// law or agreed to in writing, software distributed under the Licence is   *
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF     *
// ANY KIND, either express or implied. See the Licence for the specific    *
// language governing permissions and limitations at                        *
// https://www.lsts.pt/dune/licence.                                        *
//***************************************************************************
// Author: Ricardo Martins                                                  *
//***************************************************************************
// Automatically generated.                                                 *
//***************************************************************************

// DUNE headers.
#include <DUNE/DUNE.hpp>

using DUNE_NAMESPACES;

#include "Test.hpp"

int
main(void)
{
  Test test("IMC Serialization/Deserialization");

  {
    IMC::EntityState msg;
    msg.setTimeStamp(0.24445260238);
    msg.setSource(5735U);
    msg.setSourceEntity(18U);
    msg.setDestination(50846U);
    msg.setDestinationEntity(170U);
    msg.state = 181U;
    msg.flags = 93U;
    msg.description.assign("TTMLMIEWPUWRNVVVMGFYEEPNUOBDCXYBGXSEBKPXTQLVLSLQWVKCRFXRFYSHRKEEALFHIVKBIWCXJQHMCJSURDFNOBDKEHHFYWSSDOMFLYGLXYSRUEINRZSDJGWOFTRYQQCAHNBTGOGWIUPVEQSMQPSAROXXPATOMKCKUZDBZBTQOARGQXPWMZZVC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityState msg;
    msg.setTimeStamp(0.686346462035);
    msg.setSource(42258U);
    msg.setSourceEntity(190U);
    msg.setDestination(7671U);
    msg.setDestinationEntity(90U);
    msg.state = 253U;
    msg.flags = 229U;
    msg.description.assign("AOVNVMAHPNUAGDPBCNWLRZFTGZZYAWSSIQYVTWNFMTZCXVSWSWCVINXTWGOEEZUPBQVMIOMJCDUOONRJQJJAUUQEQZXVGLOALSEOFBIKSYTHILDXPRSNCGRNECTSWMENLPKHGXBFQBRBDFWWKMLKXXUKHVHDMLFDEMJUQZTIDZEYHFKJYYYQPKUBPDLRGE");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityState msg;
    msg.setTimeStamp(0.302797354435);
    msg.setSource(36453U);
    msg.setSourceEntity(15U);
    msg.setDestination(12767U);
    msg.setDestinationEntity(19U);
    msg.state = 165U;
    msg.flags = 144U;
    msg.description.assign("WTBLIVTRZUSKWVEFNSAKKIFUUXAOQBAUYXRCPCNMXYJTDDREHLOWGPIPQWCJOVXVATMUBCRKYIZQGGAXSJMNFXMNFWRLLXYSHYPAOZSVDRDGWMSYJQZEO");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityState msg;
    msg.setTimeStamp(0.779204529817);
    msg.setSource(51713U);
    msg.setSourceEntity(79U);
    msg.setDestination(34875U);
    msg.setDestinationEntity(177U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityState msg;
    msg.setTimeStamp(0.304649922779);
    msg.setSource(30503U);
    msg.setSourceEntity(183U);
    msg.setDestination(14395U);
    msg.setDestinationEntity(116U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityState msg;
    msg.setTimeStamp(0.131479483418);
    msg.setSource(29295U);
    msg.setSourceEntity(140U);
    msg.setDestination(46829U);
    msg.setDestinationEntity(46U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityInfo msg;
    msg.setTimeStamp(0.566521575019);
    msg.setSource(39240U);
    msg.setSourceEntity(156U);
    msg.setDestination(50917U);
    msg.setDestinationEntity(105U);
    msg.id = 124U;
    msg.label.assign("ZYZNJCQBUBIORMIEUZSMDERGTNWKWJSSAFCFYCTDWKYIFOGPWOSLDXPSQBQXJYWABDVUXAWVYHQRLBAXHYWNHMODVUUEEHRLXGXGMBLUFTGHDFLZLEPHULKUFHPHGVUOAOEKAAZNTDSWCMYMZQQCIREJKCVKCISPANLFIEOINTXQWRAGYOJVMNQSWDPVDSNHRJPRVIBMMZPVPGVDIKJT");
    msg.component.assign("LPWMVSFXMLYXTEOSHHSXZLIBFGQBIBQFGWTGQTHMARSAVGYIEMUNRUYDFUQJMGLRWSQDSBXABOLIYKZCCGOTKYCSGKXZNYXATIXYHHHDWFJRZWCPKXCEKKHQFLJAVVFNSNCVNGEYDBRZJUCLPWWSLFREITQ");
    msg.act_time = 15297U;
    msg.deact_time = 45051U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityInfo #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityInfo msg;
    msg.setTimeStamp(0.649508677866);
    msg.setSource(10906U);
    msg.setSourceEntity(59U);
    msg.setDestination(44445U);
    msg.setDestinationEntity(131U);
    msg.id = 169U;
    msg.label.assign("URDLPPPVWPZCEHBIFQRMHCKDQCOKCTZVVBYNRVQUJMIQIEUWKRGHFZRFIFIITSZBPTYOOZKYJJCLALVHEJFQHLCWCSZMANCLYDKKTYKXPESEICJZNAKZLIFVRULYSJFQOAPPONAGPRRBSVBBLUVNXCUOAMMDXNPQWWMAAJQBWELTJ");
    msg.component.assign("IYFKUZRBIEXBFIVTODWIQYZNUYFLZTEJQOCBNJVMESOYZYTNGSAPQUQHPEPVLNDPHEDLYAPDMQNCNXBSJMTKHVFISMSGVILRCFJMLKLZQACWURWORRRZNQHDIAWMGAZALHDPRXGXQXFWMYICTXTOAUDXF");
    msg.act_time = 12283U;
    msg.deact_time = 1168U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityInfo #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityInfo msg;
    msg.setTimeStamp(0.952065358252);
    msg.setSource(9766U);
    msg.setSourceEntity(172U);
    msg.setDestination(26792U);
    msg.setDestinationEntity(221U);
    msg.id = 82U;
    msg.label.assign("MTRDHGVHISYAELSKYRIWHJMJDJVCTLEEJTNGTZMMOCPTPFDNYOPBRCWXNKKDW");
    msg.component.assign("NFQUCIGWITZJCBOKPLTZHCCTTQYVNFKKSUBBKEHAXZFZS");
    msg.act_time = 56042U;
    msg.deact_time = 3914U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityInfo #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityInfo msg;
    msg.setTimeStamp(0.690733807713);
    msg.setSource(20211U);
    msg.setSourceEntity(163U);
    msg.setDestination(35837U);
    msg.setDestinationEntity(148U);
    msg.id = 88U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityInfo #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityInfo msg;
    msg.setTimeStamp(0.979726604666);
    msg.setSource(14961U);
    msg.setSourceEntity(190U);
    msg.setDestination(41775U);
    msg.setDestinationEntity(55U);
    msg.id = 173U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityInfo #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityInfo msg;
    msg.setTimeStamp(0.768611815938);
    msg.setSource(63483U);
    msg.setSourceEntity(176U);
    msg.setDestination(46630U);
    msg.setDestinationEntity(140U);
    msg.id = 120U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityInfo #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityList msg;
    msg.setTimeStamp(0.908711669395);
    msg.setSource(14735U);
    msg.setSourceEntity(44U);
    msg.setDestination(17932U);
    msg.setDestinationEntity(234U);
    msg.op = 203U;
    msg.list.assign("YOKTNGDEYDZBEWFAEIGIWUBCIACVVUYTMVBAPDGCHQIGERTKFBQNTJKIMLZHWKJHSFQQORWOGRUAMORSQMNJEPXDIEARRYOEFRZHYSGBUMDXCZKYWVDKTUSFVNWJDKCFQZVPQQUZHPIOFUXHKXLELAFRQTGJJJOVWIXEFBEPPYZZUCHUXKLKADOLDXRPXZBBNPIUTTPMALLCJAXWNXCNYSDOPHWGTYNGASBHJSLOJFG");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityList #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityList msg;
    msg.setTimeStamp(0.756248998368);
    msg.setSource(21423U);
    msg.setSourceEntity(162U);
    msg.setDestination(23538U);
    msg.setDestinationEntity(244U);
    msg.op = 246U;
    msg.list.assign("WVLULWTEHRJSYAZMJWTWRPXOKCEUZVAAHMFQIJSEXXGFKIPENZSERJQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityList #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityList msg;
    msg.setTimeStamp(0.41637191493);
    msg.setSource(44866U);
    msg.setSourceEntity(237U);
    msg.setDestination(19671U);
    msg.setDestinationEntity(113U);
    msg.op = 194U;
    msg.list.assign("FHBIRZMNFVUFWQXELZZFDXWFHVQQHGHQAQUARQPNNJPKTLYEMHXIMOTUSDEUIVKCVGCMKSIUWJANPYDRBJAQSLDOZHCBAKYNMUTZWYAQFBWDFQDJGRKRIHVIRVYORMAJUZVMZGTGWIALCYKTEFJHRPCLBJDXKJFSBZBIWNTOHEJKANCCPEVXDD");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityList #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityControl msg;
    msg.setTimeStamp(0.432276044623);
    msg.setSource(13726U);
    msg.setSourceEntity(75U);
    msg.setDestination(34247U);
    msg.setDestinationEntity(38U);
    msg.op = 142U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityControl msg;
    msg.setTimeStamp(0.848578618129);
    msg.setSource(16742U);
    msg.setSourceEntity(243U);
    msg.setDestination(26955U);
    msg.setDestinationEntity(146U);
    msg.op = 28U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityControl msg;
    msg.setTimeStamp(0.0440717406646);
    msg.setSource(58646U);
    msg.setSourceEntity(249U);
    msg.setDestination(25760U);
    msg.setDestinationEntity(138U);
    msg.op = 123U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CpuUsage msg;
    msg.setTimeStamp(0.765792132597);
    msg.setSource(47840U);
    msg.setSourceEntity(16U);
    msg.setDestination(25012U);
    msg.setDestinationEntity(8U);
    msg.value = 130U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CpuUsage #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CpuUsage msg;
    msg.setTimeStamp(0.262026920611);
    msg.setSource(46609U);
    msg.setSourceEntity(215U);
    msg.setDestination(47401U);
    msg.setDestinationEntity(94U);
    msg.value = 118U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CpuUsage #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CpuUsage msg;
    msg.setTimeStamp(0.550769005609);
    msg.setSource(20413U);
    msg.setSourceEntity(198U);
    msg.setDestination(15318U);
    msg.setDestinationEntity(159U);
    msg.value = 193U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CpuUsage #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TransportBindings msg;
    msg.setTimeStamp(0.955825307864);
    msg.setSource(19137U);
    msg.setSourceEntity(21U);
    msg.setDestination(22174U);
    msg.setDestinationEntity(6U);
    msg.consumer.assign("IKLEMWXVQQAESDWHOZJFZLTLIQNNMKRWEHRZC");
    msg.message_id = 4773U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TransportBindings #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TransportBindings msg;
    msg.setTimeStamp(0.42507517434);
    msg.setSource(38174U);
    msg.setSourceEntity(129U);
    msg.setDestination(13742U);
    msg.setDestinationEntity(140U);
    msg.consumer.assign("UVKPSKDTCAOITPEPGFUPMMLVOPJOSVCTCUPRDIXAMQFHJGHW");
    msg.message_id = 55649U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TransportBindings #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TransportBindings msg;
    msg.setTimeStamp(0.185436847588);
    msg.setSource(19456U);
    msg.setSourceEntity(236U);
    msg.setDestination(45245U);
    msg.setDestinationEntity(196U);
    msg.consumer.assign("FJDFHCIFJKKRLEVQRVGRPQUKCLBIQSOMHDNWTKLMVHUNAPLJHHJXSLFXJLQZSEILAOLVBRGTZJGVCAOXARNXSEJCBIHMAXAOEQHOQBIYWJFYKDWNMWNGEMTVWMYRTGUKSLYMTGYPD");
    msg.message_id = 15407U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TransportBindings #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RestartSystem msg;
    msg.setTimeStamp(0.488090383966);
    msg.setSource(11689U);
    msg.setSourceEntity(136U);
    msg.setDestination(39359U);
    msg.setDestinationEntity(140U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RestartSystem #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RestartSystem msg;
    msg.setTimeStamp(0.417126604739);
    msg.setSource(52957U);
    msg.setSourceEntity(67U);
    msg.setDestination(56308U);
    msg.setDestinationEntity(214U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RestartSystem #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RestartSystem msg;
    msg.setTimeStamp(0.224317659558);
    msg.setSource(56248U);
    msg.setSourceEntity(201U);
    msg.setDestination(32952U);
    msg.setDestinationEntity(226U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RestartSystem #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Parameter msg;
    msg.setTimeStamp(0.978829786301);
    msg.setSource(9538U);
    msg.setSourceEntity(124U);
    msg.setDestination(60914U);
    msg.setDestinationEntity(58U);
    msg.section.assign("EFDHMWMRYCQADIIRYBTNOI");
    msg.param.assign("BPJWUKDNYWDXHCZACERBSWIPJSFXFEUPLSSEAQMBEUIXBTKXNMEFESHEODGKNWPKHDOXVFLACHWNAHJRSXZUJCGCMRKJHWBIKDHBPIJLSGIXHRVPYOKYOVUOKGITVNUQJRETMODBFPQBQCLDHVTZKCFQGGJAOQITNYXOOCZGPTBUFCTVQYXVRMUJWLAMMREVZT");
    msg.value.assign("XYNVXEQKCMUQMUWSSOIWNVLAAQQWRNBVXZPYOASERCVTLHTLHQYALBFDDWNEDCAMCMYSFGCZAJIJZNPCGKEJXXYYWLFSKOHGYEBPJLPIDGFLECPKIXKIECIFTGKYUQZRMWJFATOJMCNRRWRAIOUKXDNWQZRORGVXPFUQDDFJPLPJHQZZUKMHKJMHGHGOZJUHOOFTVSTBBRXVSYMDBSEUUALZGHMNTTVTLNBVOWTBZSFGKXDEBAQCVEIUDPI");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Parameter #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Parameter msg;
    msg.setTimeStamp(0.400152437185);
    msg.setSource(45659U);
    msg.setSourceEntity(8U);
    msg.setDestination(19466U);
    msg.setDestinationEntity(214U);
    msg.section.assign("SZUQJJSFPWEAIWKODEMWWDBEGXPYXNGMZJQOTFSRXKHKCHGFMDCFRZVOBCZFOZYTRXPEMZHMUGLYCLAYPJSWGJROIGCTZVMJALEHFDNIIUBESBNXXQWTHXKQTNOETKPUDDVVWBFMXUOICNWUZZINC");
    msg.param.assign("LNTQZYYKJFXTHAYOBVDBJDNMANGWYGUEGMQNNCRKKIPPSSXHVQUMEOOONUVEPZJWBPANIRVTRZAXKTUIHLPTLYLULMWJOZNTFFFZYCEOOASBGPXLVHXUANTEPSEWKAFSXEQ");
    msg.value.assign("VOXIBBDVKDFRLDIKIKMPHTNYRTTTQOECWSAAPADQXBZREMXMNIWZZBDSZFFNTSWKVIOXBWJZCBWCFIAFGMIROUUHILJBGJGCMYKKHXZZEX");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Parameter #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Parameter msg;
    msg.setTimeStamp(0.24761543388);
    msg.setSource(25764U);
    msg.setSourceEntity(189U);
    msg.setDestination(7516U);
    msg.setDestinationEntity(7U);
    msg.section.assign("RXVALGMGDZMITJPORDUEBFPQZLLIILGOTAEXEINGOQTHMYCASTCGNOQPFYUJKPJMQTQTUHNXVLDXPGDFJBG");
    msg.param.assign("JHMYJSEMBENFAZLDRTGNKUYFWNLKHSJSCRVQEIHWKUTHSZVOYFIYERCXSMDERKCBUTAZGVCRWADWXSNQKIPRUZEZLHJCTBPSHMMOBXTGAMLDOAREFPDPYFB");
    msg.value.assign("OKOFLPSTRGXLIDTLFRCUDRQGLWLBCMXPYJZNIOZSKCE");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Parameter #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ParameterControl msg;
    msg.setTimeStamp(0.965576750255);
    msg.setSource(12343U);
    msg.setSourceEntity(74U);
    msg.setDestination(19205U);
    msg.setDestinationEntity(188U);
    msg.op = 220U;
    IMC::Parameter tmp_msg_0;
    tmp_msg_0.section.assign("GFZGIAHJWHYRONBYSEQQGLAQAAZVFKZUPWNVRYONEQGEPOIYTCHRPZDLBNCNIETJOSBSPYOYGZTIMCXUDPWCQXWLYLUCGPUXAGLHZLUMQVAIPZJLHJARWNPTIGADDICFM");
    tmp_msg_0.param.assign("RBMMFCAWIFTIPPHLHNVOMSGLEKORJKYQ");
    tmp_msg_0.value.assign("HCJPLVRVOWEAANUGKFEMEWBKXZEXPIXISNUMISGRUTMZSWHCZVJXMMZYFDSHQBUIZZNPJLFTPCBEYCKZBTAEVRRRBEJNUYXPFKQYNOIVMLRKAFTQQBRHWLYBDGLITGFFKWVCLOQHCGYOWJRDAKWXI");
    msg.params.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ParameterControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ParameterControl msg;
    msg.setTimeStamp(0.368920364064);
    msg.setSource(39219U);
    msg.setSourceEntity(25U);
    msg.setDestination(25754U);
    msg.setDestinationEntity(240U);
    msg.op = 170U;
    IMC::Parameter tmp_msg_0;
    tmp_msg_0.section.assign("CSXNJQQELLYXGHOGRMKNLJMYXXPRIAFZLBRMVKYREEQEJMGLLNXOFIYIOHPOUAJAHUBYZRTAIWWTCRLPAGZMOEGSYSBFSGCLDXTVSMJBHDNUPDAUKTBGQRCVDVESWVNBZRVENVDKUJNSQYMRVLZCCWHZTAWOKOJOCPSNYIPWHMXHLCDDITHDVZVUSZIIEADMPUEPCKTFACBAFBFTQHUBGKXGJDQKFZFWTWQWQM");
    tmp_msg_0.param.assign("AEXLQXXNEZYDCCMXTZVSTUTSUIPUPOVWZFKJAJHHAEZEFQRHUTINEAQAWOTLFILGXWLCSMLQCVBMABYMLKYGGPUOOKWPQRHIDKHJQWWGDBXSUEGNHXSRTRJJBDVXKBZVQKDMAWLYUIJEZICFBNASFNYPGCBJOSYSFPKIMPZTOMMPQROCUSOEZEYHNZPGDVWQHANRUFHYJPKDTUWSBGQNDYYGFCIVBALTRHOZDJVN");
    tmp_msg_0.value.assign("NDEXHGGLUDQPVKLMQPLA");
    msg.params.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ParameterControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ParameterControl msg;
    msg.setTimeStamp(0.262985253924);
    msg.setSource(51833U);
    msg.setSourceEntity(64U);
    msg.setDestination(9357U);
    msg.setDestinationEntity(136U);
    msg.op = 26U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ParameterControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevCalibrationControl msg;
    msg.setTimeStamp(0.843907717569);
    msg.setSource(484U);
    msg.setSourceEntity(64U);
    msg.setDestination(64409U);
    msg.setDestinationEntity(235U);
    msg.op = 84U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevCalibrationControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevCalibrationControl msg;
    msg.setTimeStamp(0.50653193425);
    msg.setSource(1126U);
    msg.setSourceEntity(173U);
    msg.setDestination(42392U);
    msg.setDestinationEntity(238U);
    msg.op = 51U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevCalibrationControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevCalibrationControl msg;
    msg.setTimeStamp(0.114838535921);
    msg.setSource(4642U);
    msg.setSourceEntity(71U);
    msg.setDestination(8850U);
    msg.setDestinationEntity(168U);
    msg.op = 132U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevCalibrationControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevCalibrationState msg;
    msg.setTimeStamp(0.346222379263);
    msg.setSource(43761U);
    msg.setSourceEntity(199U);
    msg.setDestination(15695U);
    msg.setDestinationEntity(152U);
    msg.total_steps = 170U;
    msg.step_number = 217U;
    msg.step.assign("MUGXIYZZILPAZTYTTJNBCOFDQAPSZCXXEHYHUYJTMIELVWLMQLQJDKTXCKBRAJ");
    msg.flags = 171U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevCalibrationState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevCalibrationState msg;
    msg.setTimeStamp(0.956575652495);
    msg.setSource(47210U);
    msg.setSourceEntity(112U);
    msg.setDestination(18821U);
    msg.setDestinationEntity(219U);
    msg.total_steps = 248U;
    msg.step_number = 81U;
    msg.step.assign("ZCHGMURDSDHILLKEHCFHJCWSWZMXTUYAIBFDYGDXSQVTPRCCUPPFUCVKAVHPWJKCNWJKJWYHNYNRGETFSXBIEKQGBWQULRMSAROUXKRUNHBQADZEYTOWOMVADFJOIYNBLQUREXBMYDVXFIMXDTDZZTLZTTKZRQYMFRNPOSUAPTBFEJPJNEYXVWLMQMPPCMB");
    msg.flags = 95U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevCalibrationState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevCalibrationState msg;
    msg.setTimeStamp(0.293426313228);
    msg.setSource(42267U);
    msg.setSourceEntity(213U);
    msg.setDestination(23814U);
    msg.setDestinationEntity(68U);
    msg.total_steps = 254U;
    msg.step_number = 228U;
    msg.step.assign("SKUCJQOVAKAIRKCTWFYLQJGRLZBBDPWUAVTOTWDCJXIMBZFUDSSDYWJAISTILMFGFDXLWAEZASXUVDZRVTJPSDFTXITEGJVNBQVXJMVCUHCLTVXQKHRQCPWSEVHPHUNIZQHMGRJZPHHHOONJWGPRWDLCMXAMXYYYEXJPEEWGFGRYLZONGQMOKZNUQNTPVOZKUBRGLNSMHIRYNAOTPQPHKABBYENENABRKYMOGEIMSCL");
    msg.flags = 233U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevCalibrationState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityActivationState msg;
    msg.setTimeStamp(0.513766277544);
    msg.setSource(52158U);
    msg.setSourceEntity(167U);
    msg.setDestination(58698U);
    msg.setDestinationEntity(209U);
    msg.state = 68U;
    msg.error.assign("WMAGDHOETZXYFJHEKYNZXJJCBCXUMKOONBVTWALXAYJIUQMT");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityActivationState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityActivationState msg;
    msg.setTimeStamp(0.274874666805);
    msg.setSource(3585U);
    msg.setSourceEntity(63U);
    msg.setDestination(50703U);
    msg.setDestinationEntity(93U);
    msg.state = 10U;
    msg.error.assign("MVCHTHJBPRP");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityActivationState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityActivationState msg;
    msg.setTimeStamp(0.0591631882079);
    msg.setSource(1253U);
    msg.setSourceEntity(93U);
    msg.setDestination(55082U);
    msg.setDestinationEntity(150U);
    msg.state = 104U;
    msg.error.assign("PZRAVTCJLBDTGUZCSBYJUEERPBN");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityActivationState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityActivationState msg;
    msg.setTimeStamp(0.458964102955);
    msg.setSource(3200U);
    msg.setSourceEntity(226U);
    msg.setDestination(41428U);
    msg.setDestinationEntity(209U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityActivationState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityActivationState msg;
    msg.setTimeStamp(0.137703446957);
    msg.setSource(9914U);
    msg.setSourceEntity(247U);
    msg.setDestination(63421U);
    msg.setDestinationEntity(4U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityActivationState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityActivationState msg;
    msg.setTimeStamp(0.80906923199);
    msg.setSource(26718U);
    msg.setSourceEntity(142U);
    msg.setDestination(43581U);
    msg.setDestinationEntity(76U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityActivationState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleOperationalLimits msg;
    msg.setTimeStamp(0.933800408279);
    msg.setSource(49292U);
    msg.setSourceEntity(72U);
    msg.setDestination(53351U);
    msg.setDestinationEntity(124U);
    msg.op = 247U;
    msg.speed_min = 0.951407052911;
    msg.speed_max = 0.939164089796;
    msg.long_accel = 0.803661535057;
    msg.alt_max_msl = 0.359197878555;
    msg.dive_fraction_max = 0.285829339467;
    msg.climb_fraction_max = 0.50925035208;
    msg.bank_max = 0.335751459181;
    msg.p_max = 0.577728841582;
    msg.pitch_min = 0.553518610804;
    msg.pitch_max = 0.627214905863;
    msg.q_max = 0.895960004188;
    msg.g_min = 0.542107822375;
    msg.g_max = 0.528128093672;
    msg.g_lat_max = 0.782648319988;
    msg.rpm_min = 0.102012882657;
    msg.rpm_max = 0.059746495659;
    msg.rpm_rate_max = 0.779252918463;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleOperationalLimits #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleOperationalLimits msg;
    msg.setTimeStamp(0.69336528664);
    msg.setSource(44402U);
    msg.setSourceEntity(110U);
    msg.setDestination(54582U);
    msg.setDestinationEntity(220U);
    msg.op = 164U;
    msg.speed_min = 0.534175018526;
    msg.speed_max = 0.814868044802;
    msg.long_accel = 0.431937309019;
    msg.alt_max_msl = 0.860109493199;
    msg.dive_fraction_max = 0.604296045381;
    msg.climb_fraction_max = 0.248630504212;
    msg.bank_max = 0.343154524126;
    msg.p_max = 0.489311909148;
    msg.pitch_min = 0.0738462641489;
    msg.pitch_max = 0.143725777614;
    msg.q_max = 0.730485492186;
    msg.g_min = 0.552746277805;
    msg.g_max = 0.308522911785;
    msg.g_lat_max = 0.744406784766;
    msg.rpm_min = 0.144558924368;
    msg.rpm_max = 0.523557252443;
    msg.rpm_rate_max = 0.319256176703;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleOperationalLimits #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleOperationalLimits msg;
    msg.setTimeStamp(0.246960811797);
    msg.setSource(11619U);
    msg.setSourceEntity(89U);
    msg.setDestination(48997U);
    msg.setDestinationEntity(122U);
    msg.op = 8U;
    msg.speed_min = 0.290630055386;
    msg.speed_max = 0.705329845454;
    msg.long_accel = 0.150345279696;
    msg.alt_max_msl = 0.490283315329;
    msg.dive_fraction_max = 0.790341839967;
    msg.climb_fraction_max = 0.735797106441;
    msg.bank_max = 0.769891108441;
    msg.p_max = 0.697300405367;
    msg.pitch_min = 0.540468873506;
    msg.pitch_max = 0.25752852872;
    msg.q_max = 0.999312801786;
    msg.g_min = 0.674901945527;
    msg.g_max = 0.362329441207;
    msg.g_lat_max = 0.717118739248;
    msg.rpm_min = 0.418324407835;
    msg.rpm_max = 0.472266842538;
    msg.rpm_rate_max = 0.0149525785732;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleOperationalLimits #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SimulatedState msg;
    msg.setTimeStamp(0.203391915571);
    msg.setSource(1599U);
    msg.setSourceEntity(99U);
    msg.setDestination(26001U);
    msg.setDestinationEntity(246U);
    msg.lat = 0.346979041322;
    msg.lon = 0.974983337705;
    msg.height = 0.0624396317579;
    msg.x = 0.653379935431;
    msg.y = 0.226466093655;
    msg.z = 0.138941341759;
    msg.phi = 0.122395772058;
    msg.theta = 0.149633355834;
    msg.psi = 0.700786985508;
    msg.u = 0.766482167642;
    msg.v = 0.905061680505;
    msg.w = 0.872856949987;
    msg.p = 0.26702148743;
    msg.q = 0.536140210806;
    msg.r = 0.212943895044;
    msg.svx = 0.579793016935;
    msg.svy = 0.433594406135;
    msg.svz = 0.413192927468;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SimulatedState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SimulatedState msg;
    msg.setTimeStamp(0.277899818703);
    msg.setSource(49762U);
    msg.setSourceEntity(249U);
    msg.setDestination(6151U);
    msg.setDestinationEntity(123U);
    msg.lat = 0.121942873636;
    msg.lon = 0.918609273225;
    msg.height = 0.505319955935;
    msg.x = 0.184490575919;
    msg.y = 0.945211969662;
    msg.z = 0.550431775197;
    msg.phi = 0.665869137225;
    msg.theta = 0.737674934018;
    msg.psi = 0.221684063155;
    msg.u = 0.955153449771;
    msg.v = 0.401259038316;
    msg.w = 0.788815804528;
    msg.p = 0.296268191632;
    msg.q = 0.969939498027;
    msg.r = 0.911311190153;
    msg.svx = 0.372788144951;
    msg.svy = 0.344721304492;
    msg.svz = 0.660335474226;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SimulatedState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SimulatedState msg;
    msg.setTimeStamp(0.112960261196);
    msg.setSource(29780U);
    msg.setSourceEntity(149U);
    msg.setDestination(27446U);
    msg.setDestinationEntity(250U);
    msg.lat = 0.727689504837;
    msg.lon = 0.246410689926;
    msg.height = 0.533051613646;
    msg.x = 0.671520531895;
    msg.y = 0.00184803161022;
    msg.z = 0.740775941313;
    msg.phi = 0.990391034689;
    msg.theta = 0.805665483346;
    msg.psi = 0.544123354381;
    msg.u = 0.727003146683;
    msg.v = 0.204056594044;
    msg.w = 0.651054472815;
    msg.p = 0.995608392645;
    msg.q = 0.222427456514;
    msg.r = 0.512759798141;
    msg.svx = 0.780548609625;
    msg.svy = 0.986524466437;
    msg.svz = 0.202871955225;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SimulatedState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LeakSimulation msg;
    msg.setTimeStamp(0.205556950111);
    msg.setSource(54264U);
    msg.setSourceEntity(99U);
    msg.setDestination(58579U);
    msg.setDestinationEntity(22U);
    msg.op = 2U;
    msg.entities.assign("MFZNPYDTIXBAZDOXECRJEUEBKJEAYRUIBBXTJDQCGCRSIQROUYMPPEOYHGT");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LeakSimulation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LeakSimulation msg;
    msg.setTimeStamp(0.143638022568);
    msg.setSource(4535U);
    msg.setSourceEntity(123U);
    msg.setDestination(33095U);
    msg.setDestinationEntity(243U);
    msg.op = 158U;
    msg.entities.assign("KKLDNNMTDIVQEHWELWQECZXPBSAARTPYLCFVXHZKHQPGPUQEUEOVPLMQT");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LeakSimulation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LeakSimulation msg;
    msg.setTimeStamp(0.958101290381);
    msg.setSource(26176U);
    msg.setSourceEntity(105U);
    msg.setDestination(18089U);
    msg.setDestinationEntity(68U);
    msg.op = 212U;
    msg.entities.assign("OHMFHVGIDURSGTYQFSTXUSBRYBIUETBZNOMXJCOOMDSEADKTTKNRBIROVIPNJLSHEXYYLZCCWFSJYEKBFCMEAHVLZJSVAJGCPAVEXNWIPMZFQVLAIXYDBZTAWABJCKXOTBYKOEFNMEOXGKDPQIPMURIJBGFDCNIKIBSVFHMURMOUZRUDDCKHHRNDHJQYWGQPUNKLEZCGJJWSHYNNRRAXCQAXDTGQXUPVWQMSTPAWKFLQZEHWZQLLVYPGFPLG");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LeakSimulation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UASimulation msg;
    msg.setTimeStamp(0.294724431111);
    msg.setSource(44595U);
    msg.setSourceEntity(99U);
    msg.setDestination(26722U);
    msg.setDestinationEntity(96U);
    msg.type = 41U;
    msg.speed = 5592U;
    const char tmp_msg_0[] = {80, 121, 9, 75, -87, -22, -34, 120, -120, -31, -2, 98, 37, -16, 97, 92, 79, 93, 90, -99, -51, -107, -115, -21, -53, 55, 65, -68, 62, 78, 105, -118, 98, -123, 64, -80, -115, 78, 78, -94, -77, 20, -12, -94, 6, 123, -69, 93, -3, -125, 124, -68, 62, -118, -121, 25, -76, 13, 58, 79, -37, 54, 48, -98, 25, -18, 99, -108, -53, -13, -18, -27, 18, -46, -117, -9, -46, 31, -101, 109, -108, -15, 33, -113, -97, -74, -39, -91, 13, -65, 15, -75, 0, 14, 123, 69, -29, 34, -29, 41, 86, 41, 78, -121, 14, -38, 18, 6, -114, -127, -20, -78, 47, 51, 126, -19, 123, -48, -45, 91, 2, -118, 98, -61, 100, 116, 47, -89, 11, 119, -11, -83, -107, -103, 9, 92, -119, 46, -18, 6, 53, 81, 28, -92, 20, -121, 20, 55, 85, -107, 63, -52, 97, 43, 97, 3, -108, 34, 96, 93, -28, -91, -121, 22, -64, 89, 18, -61, 110, 40};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UASimulation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UASimulation msg;
    msg.setTimeStamp(0.708043939594);
    msg.setSource(22723U);
    msg.setSourceEntity(254U);
    msg.setDestination(56806U);
    msg.setDestinationEntity(61U);
    msg.type = 64U;
    msg.speed = 21693U;
    const char tmp_msg_0[] = {-28, 96, -35, -52, 107, 82, 2, 120, 48, 126, -30, 65, 24, 44, 10, 68, -123, -116, 16, 31, -80, 0, 121, -41, -88, -87, 6, -97, 67, -1, -125, 57, -20, 98, -87, -93, -34, -79, 30, -48, -121, -9, 68, -98, 77, -91, 24, 62, 25, -71, -25, -84, -113, -38, -88, -43, -51, 106, 47, 62, -125, 97, 95, -60, 53, 47, -111, 76, -13, -50, 100, 7, 57, -62, -72, 72, -4, -84, -32, -12, -22, 119, 71, 97, 108, 9, -92, 95, -6, 37, -42, 114, 11, 122, -55, -125, 61, -33, 3, 68, 84, -22, -41, 101, -114, 126, -113, -72, -26, -77, -69, 44, 31, 18, 53, 97, -63, -87, 57, -56, 66, 88, -41, 89, -44, 28, 87, -107, 114, -34, 58, -102, -32, 28, 74, -101, -59, 15, -24, -33, -88, -105, -63, -89, -82, 98, 39, -29, -116, 123, 92, -95, 17, -111, 77, -98, 106, -86, 69, -16, -28, -68, 45, -14, 40, -46, -38, 44, 41, 1, 89, -24, 115, 17, 82, -94, -74, 48, -30, -111, 16, -41, 18, 105, -101, 116, -80, -106, 90, -30, 14, -36, 36, 15, -41, 48, -85, -78, -68, -19, 82, 9, 19, 47, 59, -16, 28, 50, -27, -71, 19, -72, -50, 46, -121, -93, -61, 73, -105, -63, 1, -1, 4, 101, -12, -91, 67, -64, -18, 13, 104, 21, -68, -115, -89, -96};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UASimulation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UASimulation msg;
    msg.setTimeStamp(0.758854480707);
    msg.setSource(54358U);
    msg.setSourceEntity(100U);
    msg.setDestination(47175U);
    msg.setDestinationEntity(185U);
    msg.type = 106U;
    msg.speed = 44369U;
    const char tmp_msg_0[] = {109, -52, -31, -7, 36, 9, -82, 32, 69, 53, -104, -57, -34, 56, 56, 59, 90, 73, 58, -88, 38, -86, 118, 57, 119, -65, 104, -117, -73, 74, 85, 21, 94, -11, -49, -120, 9, 120, -68, -35, -74, -62, 54, 94, 30, 7, 103, 126, 43, 7, -8, -115, -53, -122, -112, 105, 40, 6, -35, 62, 121, 4, -22, -105, 68, -75, -99, -48, 12, -23, 41, -120, 116, 9, -55, -128, 107, 120, 80, -74, -90, -67, -98, -40, -80, 122, -88, 62, -44, 97, -108, -126, -7, 117, 50, 8, 42, -93, -21, -23, 46, 110, 124, -34, -55, 121, 124, 32, 120, 64, -115, -100, 6, -36, -5, 103, -46, -79, 8, 38, 14, -78, 120, -42, 107, 29, -82, -94, 9, 124, 8, -115, 114, -61, -5, 3, -37, 114, 20, 41, 1, 89, 51, 7, -98, -90, -14, 26, -5, -119, 51, -59, 23, -63, -5, 109, 81, -106, -25, -110, -109, -89, -18, -46, 44, -77, 42, 24, 92, -59, -68, -49, -93, -17, -71, 77, -72, -10, 55, 70, -7, -61, -109, 78, -124, -4, 94, 88, -90, -124, 34, 2, -51, 15, -109, 34, 15, -47, 119, 88, 53, -99, 8, -13, -13, -69, -7, 71, 16, 117, -96, 65, 41, -19, -50, 33, 75, 11, 102, 78, -20, -91, -21, 118, 81, -100, 93, -104, 99, -34, 9, 88, 110, -42, 70, 113, 14, -35, 31};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UASimulation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DynamicsSimParam msg;
    msg.setTimeStamp(0.154033157342);
    msg.setSource(26477U);
    msg.setSourceEntity(73U);
    msg.setDestination(18974U);
    msg.setDestinationEntity(253U);
    msg.op = 43U;
    msg.tas2acc_pgain = 0.367506571985;
    msg.bank2p_pgain = 0.38431602021;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DynamicsSimParam #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DynamicsSimParam msg;
    msg.setTimeStamp(0.160723538018);
    msg.setSource(28867U);
    msg.setSourceEntity(35U);
    msg.setDestination(25819U);
    msg.setDestinationEntity(137U);
    msg.op = 36U;
    msg.tas2acc_pgain = 0.692329372055;
    msg.bank2p_pgain = 0.290616420542;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DynamicsSimParam #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DynamicsSimParam msg;
    msg.setTimeStamp(0.975676201035);
    msg.setSource(36492U);
    msg.setSourceEntity(90U);
    msg.setDestination(17549U);
    msg.setDestinationEntity(40U);
    msg.op = 197U;
    msg.tas2acc_pgain = 0.00251087857302;
    msg.bank2p_pgain = 0.739440717463;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DynamicsSimParam #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::StorageUsage msg;
    msg.setTimeStamp(0.206066623349);
    msg.setSource(48633U);
    msg.setSourceEntity(55U);
    msg.setDestination(6098U);
    msg.setDestinationEntity(130U);
    msg.available = 3952744270U;
    msg.value = 223U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("StorageUsage #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::StorageUsage msg;
    msg.setTimeStamp(0.305056432983);
    msg.setSource(51596U);
    msg.setSourceEntity(139U);
    msg.setDestination(25819U);
    msg.setDestinationEntity(122U);
    msg.available = 740337601U;
    msg.value = 76U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("StorageUsage #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::StorageUsage msg;
    msg.setTimeStamp(0.422110104168);
    msg.setSource(23967U);
    msg.setSourceEntity(85U);
    msg.setDestination(42429U);
    msg.setDestinationEntity(145U);
    msg.available = 3421239026U;
    msg.value = 100U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("StorageUsage #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CacheControl msg;
    msg.setTimeStamp(0.136553370499);
    msg.setSource(17254U);
    msg.setSourceEntity(211U);
    msg.setDestination(34390U);
    msg.setDestinationEntity(61U);
    msg.op = 165U;
    msg.snapshot.assign("PIDMLPJAWLGMFXXNXRYDRCGMIFJUTWUBSITKRMQMSTOHPFPZPYCFHJTHJYBIUFONVH");
    IMC::FollowReference tmp_msg_0;
    tmp_msg_0.control_src = 28697U;
    tmp_msg_0.control_ent = 184U;
    tmp_msg_0.timeout = 0.856450124746;
    tmp_msg_0.loiter_radius = 0.445119612995;
    tmp_msg_0.altitude_interval = 0.722301369881;
    msg.message.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CacheControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CacheControl msg;
    msg.setTimeStamp(0.397718360693);
    msg.setSource(34596U);
    msg.setSourceEntity(46U);
    msg.setDestination(52707U);
    msg.setDestinationEntity(164U);
    msg.op = 163U;
    msg.snapshot.assign("XINQXYHRNSBFJMJSKXVGZZDXBCOMNBBIUYZJFSPFPMXGZVAQIMZHCHTVIHMYNAPKVOSERHRRADWLLTXRWQWMAOLWVXWDDNCUZPYVAKBKWWUTOPHVSFOEGWPQCWXTGTDDAOYQIKNRAIGUDUSEUBTZQHXYMCSHDESJPTYSCROGSNEIJTRANWEYNKTRFQFPLJMVHFRVFBGPQILMKG");
    IMC::Rpm tmp_msg_0;
    tmp_msg_0.value = 7531;
    msg.message.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CacheControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CacheControl msg;
    msg.setTimeStamp(0.273905927768);
    msg.setSource(2139U);
    msg.setSourceEntity(77U);
    msg.setDestination(59716U);
    msg.setDestinationEntity(140U);
    msg.op = 123U;
    msg.snapshot.assign("BWWEIHNYJPITEPQKPTHGBCWFWKLNJAXZRNBLIJMZDLAIZOANOKJUKTQAQX");
    IMC::SetEntityParameters tmp_msg_0;
    tmp_msg_0.name.assign("FAPOYDACSASIVHMRSNCQNMXCPBNMEFFSAQYCZJJMKIVGYXOBRIYAUOKUJQBUZQKVFBFTAQBGLLEXWQNKIOPRZLPHDXUNGVUODGXKEHDDFMURJLRNBOGLWDITXDZVTGGWNVYZGLYTWTHPDKFRUYEWNEXGHUMCZKPTOPYVPSPIOHTGWBVZAJZMSFBJKXXRMJMELUAZJEHOWURACC");
    msg.message.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CacheControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LoggingControl msg;
    msg.setTimeStamp(0.594255111867);
    msg.setSource(13888U);
    msg.setSourceEntity(206U);
    msg.setDestination(46880U);
    msg.setDestinationEntity(201U);
    msg.op = 198U;
    msg.name.assign("JIKBDWKVAHADKKQDRLWVPYTHBIHMFFWIDPJSMXZREZVFECJLOJFBUXYLBKFYKSPPMZERDVRNVQZUEOYLVTDGCNTOCZWTZNDMXUGRCFESXUHNVCQNXQTMIQABTMJFZPSLYGQXCSQMJDOOJOLPAFGNGUGJJNQTCXKEYKTZMBNRCIAIHXASSZUKNWRQYHPVPHCWRHGPTISUHBIWOIMGEOZFWVOR");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LoggingControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LoggingControl msg;
    msg.setTimeStamp(0.366788685246);
    msg.setSource(11913U);
    msg.setSourceEntity(213U);
    msg.setDestination(39675U);
    msg.setDestinationEntity(184U);
    msg.op = 253U;
    msg.name.assign("HFJODXHQFCWJOUEKFMYOOXLIJOALXSOTOWZEQVQGRRMVRNZMJNZUHUFACAAMEAMPVTJRKRNUKOBRYDKVRBQIUBPKIVAURZNQHOTBOYCQZLTGCDPBGSIYLXSHDBHFWLVIVWNATVBKKZMPYCFCYSLBESL");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LoggingControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LoggingControl msg;
    msg.setTimeStamp(0.744532655337);
    msg.setSource(27275U);
    msg.setSourceEntity(251U);
    msg.setDestination(32473U);
    msg.setDestinationEntity(161U);
    msg.op = 24U;
    msg.name.assign("XWRWSCXYWDMBENFWUPCBOZFOVPITIUKNJAPXQMDBZIMSHNIVLGLEIGYCSBPSCAJGHDRTABUNTJVOUZUITDFQYFQDPSGUZYCGRDMHSOHMJYNHAMENNRVTHRZYJSCERYXCHPREJSJLAIZXGOOBOKFGFPBIIZKYWMCTEWTVXLJKSFLDHVMBELIKDKJXHZBTGXNOQGXUMMZRDEQATNLBRAPVNLCUJGWFUKCFEALSQZVHAXWQTAYPKDWYFOWRQUK");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LoggingControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LogBookEntry msg;
    msg.setTimeStamp(0.974241386719);
    msg.setSource(51569U);
    msg.setSourceEntity(94U);
    msg.setDestination(48469U);
    msg.setDestinationEntity(13U);
    msg.type = 216U;
    msg.htime = 0.42921711436;
    msg.context.assign("XYUZGVZOWXQWWDTAQDVBZRKFJWSUTERLYNEGHJUYHNFCYDDDJDHESYYOROKWQPKDVBVJJHILPMSKMXQCALNEPJLWVKDTSFRWYGUOMQXJTAOQISIXBBSAKEMEEIUFVJUHFRVKAZATNRRQNOMZONMVHXGDSTXUFTRPEONDZFBSACBTCLYPGLXCFXUJGYKIHFYILEWXGGUOTPTLCVLPCNVNKPAMQ");
    msg.text.assign("KIHAQUYKMCDZPQQPAKOCODUUGZXPGMMDJYEJGDHZXCWABRQXCUFDHAGTAITNWIKPABXYEUKLJJWHRXOEOTYATREWJPLZHBMYZQNZNJHBEFSROEFMRBNJMSJZGSLZGRFCKAVTUTVFCMYIIEVRSBALDKKULLDTIVYAKCFOLHPXEWIXOTNIHQURGFXIPPVHKVVOBFCZSWDNYBEGQFHGBSWVNCPVLDOLNSWNUYZVWJT");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LogBookEntry #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LogBookEntry msg;
    msg.setTimeStamp(0.397082466681);
    msg.setSource(45226U);
    msg.setSourceEntity(142U);
    msg.setDestination(58997U);
    msg.setDestinationEntity(107U);
    msg.type = 43U;
    msg.htime = 0.0309846113511;
    msg.context.assign("PDLGLNEYOTGSPBRVWCSFKNHHJXRDJQHVABMUQLDIORRQYDCEMZCIIWF");
    msg.text.assign("LHIDHOYFRCATDCQXOSBTEJSVITQUOYQZWVATLUYABJMEUAXKNFVGYURWLSKHKJPXDZIRBYEFNJKKTQWEEKCITXCTSVBIIHQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LogBookEntry #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LogBookEntry msg;
    msg.setTimeStamp(0.0631620393435);
    msg.setSource(6221U);
    msg.setSourceEntity(195U);
    msg.setDestination(50659U);
    msg.setDestinationEntity(168U);
    msg.type = 220U;
    msg.htime = 0.338702839326;
    msg.context.assign("PNPTPFOEZRZVTZLHICMXQBEQNWGQSGFVRJSPWDOJJPCUCHFDQEBOLOGRFWWYFPWHVRSVKDYMCTGVGNVKYSUFBSIJTWBNBRFXQDZYITETZOIJNXNILMMHDUEZKBGTCJXQWWRFZEBRCGICLSTEMQNDCDALGMDYIPDYGOASLUNLUKSQKJGZFAMDJAUOHIAUZOU");
    msg.text.assign("DVWOYPRDNGZQOSUYMRPRGVZUJYANKHAJBZYOSHWKAWIBIOCAWRSDOQMCFAXLHAFNLIOWVDBYHQCZNUPGEXKGVXBFFHYDUFKOGIMRGEPKNREWADVLVMSANZEQUQOHVGGTWSQBTBXTAHQTJLDQESJEJMKVXWXZTZKLCHSZ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LogBookEntry #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LogBookControl msg;
    msg.setTimeStamp(0.96040123812);
    msg.setSource(32857U);
    msg.setSourceEntity(19U);
    msg.setDestination(22850U);
    msg.setDestinationEntity(12U);
    msg.command = 127U;
    msg.htime = 0.540302995652;
    IMC::LogBookEntry tmp_msg_0;
    tmp_msg_0.type = 128U;
    tmp_msg_0.htime = 0.138339159134;
    tmp_msg_0.context.assign("LRCZKYOSZYEMMFDJWLAUKGVQVWHTRFGRYCRZBUHDJSBPRVTQXLTFYJEWXDGPNIOQOIGSFBRVGMJENBJEIUPWROWQVZMVOMAHZMVASBNDZPUGRIHTUKDQXNEPCEACZUFOLSERYYKADQJTFDUPCAMFZNUTNDQFHOZBVPBDYTKSKBRINHMHLUD");
    tmp_msg_0.text.assign("XCPRRLZUACHDBDGGTCLWJJVXKRSNFXMQIZITYSZOGQZQBHQHSRFYYLKLIBYNMZPQZZIWAAPFXDYRHEVJBIILPQCPULDSKCBMPXORNAWDNAJBSSOPDYWQXYFCGITFFDFGEIJQGHMRZ");
    msg.msg.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LogBookControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LogBookControl msg;
    msg.setTimeStamp(0.319536124209);
    msg.setSource(16827U);
    msg.setSourceEntity(253U);
    msg.setDestination(64428U);
    msg.setDestinationEntity(5U);
    msg.command = 76U;
    msg.htime = 0.699582982351;
    IMC::LogBookEntry tmp_msg_0;
    tmp_msg_0.type = 219U;
    tmp_msg_0.htime = 0.727851882381;
    tmp_msg_0.context.assign("PMEJMZXLHBCTDTUUDGUJVYNPINTQZNBVILHOAWDBGZKYOUIWBGLNHRKLEWDSKVYQFLWMKLUPSKJFLFIHOHMAJGOHBUVALWEBEEJXCPKZNMWNSJHFIKGVXKYQPZGOIBTWECQAF");
    tmp_msg_0.text.assign("XAXEBFQGSQWWIYMFRDSHUIXDRNKKNLFAIVROKFDNYFOITVRBBGIEYVLKUATIKGFLFXOMOSDHCTFUZZOOMHKDCGJJIMTAVDGUZEYYLQHWKZWAGMWDJAWLEHLCBJDBLCPNLGQVJJNRNHMEGDOGCFVUAVHNMBMPXETYAVNOXTXMJGIOPQIUZSSBWQPRNJTMZ");
    msg.msg.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LogBookControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LogBookControl msg;
    msg.setTimeStamp(0.877647586714);
    msg.setSource(28355U);
    msg.setSourceEntity(106U);
    msg.setDestination(55159U);
    msg.setDestinationEntity(69U);
    msg.command = 27U;
    msg.htime = 0.48086835944;
    IMC::LogBookEntry tmp_msg_0;
    tmp_msg_0.type = 150U;
    tmp_msg_0.htime = 0.363934268912;
    tmp_msg_0.context.assign("DOSSNLPYIHEIQFZQHNDUBIWNSBJKDGCWEWUIBXBYRPJTRWBZEOBENOUCKZEYPMDMRUEDVDYAQBQALZZIIGKHHARXYNHXSWRKJVMGMLVTQUKTVCPHEIJ");
    tmp_msg_0.text.assign("JCHCQYBZFHCGMKUWUDAQEMBINYLCUQLLCOUSRLPBJLJHYYYIWLRPWPXZMLFTNXWFJQTTNSTFDVNVGHSXBCVEEZFKZOALAUNKQYYDAVVWIWQPAJEGXUJAOIFSIVNILYGOBJDUSDOKMGWOKAARRMAJT");
    msg.msg.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LogBookControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ReplayControl msg;
    msg.setTimeStamp(0.0427559603233);
    msg.setSource(43463U);
    msg.setSourceEntity(150U);
    msg.setDestination(23383U);
    msg.setDestinationEntity(105U);
    msg.op = 63U;
    msg.file.assign("HYFAGWOONWEGVDLNRSPOUMQRLVMTRFGDBIXKCYWDEVLRKKPNZRGVNIFTXNPJYUGCFHCBBLSBPXHKUFQWPKPYXMWGCHXKXSPC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ReplayControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ReplayControl msg;
    msg.setTimeStamp(0.744553495379);
    msg.setSource(6828U);
    msg.setSourceEntity(242U);
    msg.setDestination(36662U);
    msg.setDestinationEntity(117U);
    msg.op = 215U;
    msg.file.assign("JCEJRLCOGAQPFGAFHTEZKXVLVPMRNFJNMUTPTPHXBFYFRXWBHHEHGMJQKIQDSTOZVDSDUYMBKSGDYMRDLZVIYKCEZJAHLKMKCWHIENOFEVMNOMXVETQYBQVGBLNHCPTRXPOLLNTZUSLNQDNWBJOUIHZLDZSBIPAXRKCTHEPASPWKVWGYIYAONSWVJQSGCJUXUXZRGUXQTKDQGAWLAQFSWEUDJMA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ReplayControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ReplayControl msg;
    msg.setTimeStamp(0.864823565086);
    msg.setSource(12078U);
    msg.setSourceEntity(45U);
    msg.setDestination(14548U);
    msg.setDestinationEntity(76U);
    msg.op = 194U;
    msg.file.assign("QGJOUFOGWUIFIMGUQWWGDHIPYBLNXYUDMFSQXLFTZDCSCPGUZARVYYKLPADOWUZSPHJXGMWJKDIOIIDHAAAEIVRSQMTJFRHCVTJDEBQWNCBSMOIQZSPLYTRZMXJGMYALCFYAOXWZPPYJKMKVVJJNOKHYZQKVWMTUOQLZXNANTIXDTELLLVEWEVFVNACTGFBJBPLCI");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ReplayControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ClockControl msg;
    msg.setTimeStamp(0.821920388629);
    msg.setSource(34876U);
    msg.setSourceEntity(15U);
    msg.setDestination(63097U);
    msg.setDestinationEntity(8U);
    msg.op = 63U;
    msg.clock = 0.512685003618;
    msg.tz = 39;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ClockControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ClockControl msg;
    msg.setTimeStamp(0.315475527264);
    msg.setSource(42719U);
    msg.setSourceEntity(125U);
    msg.setDestination(5079U);
    msg.setDestinationEntity(102U);
    msg.op = 126U;
    msg.clock = 0.442599483912;
    msg.tz = 55;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ClockControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ClockControl msg;
    msg.setTimeStamp(0.882637224312);
    msg.setSource(182U);
    msg.setSourceEntity(213U);
    msg.setDestination(58499U);
    msg.setDestinationEntity(129U);
    msg.op = 205U;
    msg.clock = 0.258205246405;
    msg.tz = -71;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ClockControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Heartbeat msg;
    msg.setTimeStamp(0.758129326952);
    msg.setSource(2059U);
    msg.setSourceEntity(76U);
    msg.setDestination(11313U);
    msg.setDestinationEntity(218U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Heartbeat #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Heartbeat msg;
    msg.setTimeStamp(0.895645731122);
    msg.setSource(21412U);
    msg.setSourceEntity(180U);
    msg.setDestination(16378U);
    msg.setDestinationEntity(138U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Heartbeat #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Heartbeat msg;
    msg.setTimeStamp(0.675099457855);
    msg.setSource(54226U);
    msg.setSourceEntity(174U);
    msg.setDestination(41839U);
    msg.setDestinationEntity(172U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Heartbeat #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Announce msg;
    msg.setTimeStamp(0.329350740699);
    msg.setSource(42046U);
    msg.setSourceEntity(78U);
    msg.setDestination(35892U);
    msg.setDestinationEntity(197U);
    msg.sys_name.assign("PTWBJEOIWMBCNEAYRZOUOKVFMIGAGDBUPTUSTNQZDULFZVMEDHOCYGKQSMEILXDMQQZAITXDGBCXYAHYSNZRPBXRUIPPYIKNBRESQRYM");
    msg.sys_type = 4U;
    msg.owner = 15290U;
    msg.lat = 0.242569619361;
    msg.lon = 0.299348417404;
    msg.height = 0.358949760335;
    msg.services.assign("NBXYFXJPZQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Announce #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Announce msg;
    msg.setTimeStamp(0.628428828582);
    msg.setSource(12336U);
    msg.setSourceEntity(180U);
    msg.setDestination(12460U);
    msg.setDestinationEntity(155U);
    msg.sys_name.assign("GASXVHWYNKCHVRGRGPQWAFRSHIZTUEVFZRWQXGCBBXBFWPISAVMJEFLZYFVTZIGDHPEGKHXVWWXBYYLNKNGOBOZUGFTDPJCFHRPCKOUKFJCTHAASCCYRUWWEBZUGJVAENIMYKFTMEIYYKQXMOESXTJJAODDMRXLIDTSKPLLKMINPMSECSBXPIDLABYZOSSGBQENUTHNRILVUCPOMZR");
    msg.sys_type = 75U;
    msg.owner = 32255U;
    msg.lat = 0.479537241162;
    msg.lon = 0.58121225781;
    msg.height = 0.750813479185;
    msg.services.assign("ALTOGBYORBMPSUKJFHVXAZMRZKRENQNSGSHYJWCCHJWEJWIPXKYMAGHHIRYIWPJKLGMOCELWKKDAJTUCIIYFXFUQAJUWPUPGBOSWADAIZXXBFSOXVGPNNEVYPXZDCNNEYLHWXKTABBRZNHUFYZFFMQUIOVTDBVJTTUFDEZRCZKJASERJ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Announce #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Announce msg;
    msg.setTimeStamp(0.10171755214);
    msg.setSource(10096U);
    msg.setSourceEntity(214U);
    msg.setDestination(8670U);
    msg.setDestinationEntity(153U);
    msg.sys_name.assign("OPMTXAEAIXKNGXRTKIUZRVHYRHDKTNLDSHQYEKJM");
    msg.sys_type = 88U;
    msg.owner = 57873U;
    msg.lat = 0.315342245795;
    msg.lon = 0.372782509733;
    msg.height = 0.599535865482;
    msg.services.assign("KBBLRGSQCJNLYAHROIRKWGVUEORAHYJKKBWGFLLZCENXSMGSOZXVQHDAWXNBMYNIJZEXWWDZCUYNFGZJEATOTKEWJR");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Announce #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AnnounceService msg;
    msg.setTimeStamp(0.86012459652);
    msg.setSource(17953U);
    msg.setSourceEntity(138U);
    msg.setDestination(62406U);
    msg.setDestinationEntity(43U);
    msg.service.assign("GNTJFLXCUKZPGRDNSEYCZVOGMCOMJTLCASKOHNM");
    msg.service_type = 49U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AnnounceService #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AnnounceService msg;
    msg.setTimeStamp(0.391920324296);
    msg.setSource(20317U);
    msg.setSourceEntity(37U);
    msg.setDestination(14027U);
    msg.setDestinationEntity(95U);
    msg.service.assign("EQGRBOYLZEGXTLFSBQYFLCMZLEMUXPULLZOWAOUTIJFEMJQKCFWGFWOBUGSUNPFQIDHWPVUTBSITNQAWKNIIAOKKHRVRMRCAZPCPHMIXROKYYWDPHAWNMJVZYNQMJHLCWBTBVFPGOGZXVBRVFIRSKXJZHTAFDXNXUVQ");
    msg.service_type = 254U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AnnounceService #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AnnounceService msg;
    msg.setTimeStamp(0.319935803399);
    msg.setSource(5449U);
    msg.setSourceEntity(209U);
    msg.setDestination(14809U);
    msg.setDestinationEntity(108U);
    msg.service.assign("RJKPFIHXOIZHFWECPOTGDWTWIANMSHAHBCYOKZZXCAQWKTMYVLPCCCXDHMSHFCYOZTJQSOCHJCTLZQNMXGYVGUOKUFFXJYLZEREZWRPSMRUQRP");
    msg.service_type = 45U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AnnounceService #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RSSI msg;
    msg.setTimeStamp(0.3305965293);
    msg.setSource(27551U);
    msg.setSourceEntity(181U);
    msg.setDestination(554U);
    msg.setDestinationEntity(36U);
    msg.value = 0.361135650905;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RSSI #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RSSI msg;
    msg.setTimeStamp(0.896705947005);
    msg.setSource(15888U);
    msg.setSourceEntity(115U);
    msg.setDestination(45484U);
    msg.setDestinationEntity(241U);
    msg.value = 0.0943298897606;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RSSI #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RSSI msg;
    msg.setTimeStamp(0.60407780265);
    msg.setSource(58296U);
    msg.setSourceEntity(169U);
    msg.setDestination(40911U);
    msg.setDestinationEntity(140U);
    msg.value = 0.306983764624;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RSSI #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VSWR msg;
    msg.setTimeStamp(0.367391560041);
    msg.setSource(63819U);
    msg.setSourceEntity(143U);
    msg.setDestination(586U);
    msg.setDestinationEntity(140U);
    msg.value = 0.885485294065;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VSWR #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VSWR msg;
    msg.setTimeStamp(0.787735360717);
    msg.setSource(64250U);
    msg.setSourceEntity(99U);
    msg.setDestination(22275U);
    msg.setDestinationEntity(54U);
    msg.value = 0.296140154826;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VSWR #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VSWR msg;
    msg.setTimeStamp(0.386382561066);
    msg.setSource(45625U);
    msg.setSourceEntity(163U);
    msg.setDestination(54814U);
    msg.setDestinationEntity(105U);
    msg.value = 0.121870788797;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VSWR #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LinkLevel msg;
    msg.setTimeStamp(0.0796438337668);
    msg.setSource(64576U);
    msg.setSourceEntity(30U);
    msg.setDestination(34264U);
    msg.setDestinationEntity(88U);
    msg.value = 0.913815554097;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LinkLevel #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LinkLevel msg;
    msg.setTimeStamp(0.973453184289);
    msg.setSource(3954U);
    msg.setSourceEntity(193U);
    msg.setDestination(61063U);
    msg.setDestinationEntity(230U);
    msg.value = 0.988875710895;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LinkLevel #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LinkLevel msg;
    msg.setTimeStamp(0.74421904637);
    msg.setSource(15994U);
    msg.setSourceEntity(234U);
    msg.setDestination(53452U);
    msg.setDestinationEntity(175U);
    msg.value = 0.49006847637;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LinkLevel #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Sms msg;
    msg.setTimeStamp(0.625044368909);
    msg.setSource(26617U);
    msg.setSourceEntity(2U);
    msg.setDestination(58021U);
    msg.setDestinationEntity(195U);
    msg.number.assign("PMMGWXSOHSYLKNHFTSXEKCKGPTQZEUAPDKETZLJRLMXMVFCDZHNYVBMXFXIOIUNPICLEKNIKFFJJBJNTPJIGGHZCMZURAYEIKYAILHSDQWDQSSHSXWBJUNUJHBBJICEAZBARJQNUXLVQVLCPWSKBQAFOMRPRNVRCRNWRTHYAXMBVOAYYTHUXYDGWKTNGVXDAESJYMRUDLFUIMPPCKVLZDCFGVLUQWOZRZQWOOHGETBIPDEGT");
    msg.timeout = 51983U;
    msg.contents.assign("UPOCWMMAJTVZNNMBEVGQEYPXMWOXIFYKMHWLCAOQDSKKNNXBDBZILBFISDBOSJYTAHTUIIUPSAJIUVVFMFHTEUYTSNFCPZLUYIMOYRHEYQDFRSOZDLMWFERNHDOGCWWXPHBTVLWFUGCOGIMXCLPPRCQUAYSJFHJUVVDKOEPRXJQNQCJCQEMZZJBIHBKGWZQGGKZTYPSQTZBSVXBRXDSVZKNLAJPANWXRVIWATOXREHHF");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Sms #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Sms msg;
    msg.setTimeStamp(0.389347850505);
    msg.setSource(7149U);
    msg.setSourceEntity(195U);
    msg.setDestination(65492U);
    msg.setDestinationEntity(170U);
    msg.number.assign("BZXIFCAIHUWNOJYQZHFQFQMAKOUJUCVPWYGRMXZQYJJLTGBZJRPTDXPXTXVHFGYNPGL");
    msg.timeout = 33119U;
    msg.contents.assign("COOCXSIXINRMUVYX");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Sms #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Sms msg;
    msg.setTimeStamp(0.71244929657);
    msg.setSource(15606U);
    msg.setSourceEntity(87U);
    msg.setDestination(32014U);
    msg.setDestinationEntity(225U);
    msg.number.assign("DYADIHFYLNNBVGWGVOHXPQEXAWPXAULTZFIUHCCKYWRKTVFPRUCBXILEYMWNOMKRWBABTJCKHWTVDQLIRZDHBDKTCAMPTDPVUAJOQRAZMRBUJBSUSQMJQQZOFLNFJKCWEEZWUWKHXFYRACFGLVFJNPGUQRRZEZIEGNQGWMMOJIETRPUOXTXFEVHGKOBVPPSCLMZYIAYEYGSGDJHLSDYIMEXTQLIJCSTYDBXNB");
    msg.timeout = 45505U;
    msg.contents.assign("QIDPXNFCZBXSWZUNWXVKGNJGDGXMBEFQTIKXUEWFTPVCRFNEFPPUEQAGKAPRNEWODLSWTUPMNYAGVCIKHJRXHHZVTZHRFPRELFATBQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Sms #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SmsTx msg;
    msg.setTimeStamp(0.261020428975);
    msg.setSource(442U);
    msg.setSourceEntity(215U);
    msg.setDestination(54628U);
    msg.setDestinationEntity(60U);
    msg.seq = 2613013797U;
    msg.destination.assign("FPLEZOSBLNIKKAPSTKALAHNBIUDFXCTBOJNZGEQBJEWTDAACEKISXMDGSYMDMUQYOPHZFKWDMURVKZRBIGYRCWSPCIJRHSOOTR");
    msg.timeout = 55092U;
    const char tmp_msg_0[] = {6, 69, -126, -80, -99, 120, 106, 109, -44, 100, 85, 46, -73, 110, -39, 99, -78, -64, 71, -10, 54, 70, -25, 20, -125, -75, -10, -46, -63, 26, -25, -24, -112, -69, 51, 49, -127, 99, 54, -99, 6, -5, 108, -128, -17, 109, 126, -7, -31, -86, -82, -127, -123, -26, 89, -88, 122, -48, -61, -87, 93, 25, -36, 65, -42, -17, 112, -42, -34, 112, -58, -46, -90, 21, -123, 100, -75, -30, -108, -106, -122, -58, -42, 98, 118, 82, 84, 20, -127, -19, -122, 121, 48, 66, -28, -79, -11, 19, 24, -113, -48, -107, 41, 118, -44, 11, 55, 44, 12, -81, 20, 48, 79, -6, 88, -80, -106, 75, 31, -104, 59, -39, -24, 68, 56, -15, 25, -4, 46, -52, -62, 104, -107, -55, 39, 52, 36, -106, -21, 2, -19, 27, 61, 66, -73, -114, -49, 101, 60, 16, 0, 22, 78, 53, 15, -22, 83, 121, -28, -59, -21, 106, -40, -113, -127, -4, -109, 6, 48, 31, -61, 104, 9, 61, -58, 125, -41, 52, 72, -127, -33, -87, 14, 115, 109, -121, 66, -39, -39, 34, -80, -85, -82, 41, -42, -102, -76, -22, -117, 78, -47, -43, 66, -24, -14, 126, -105, 13, -49, 1, 33, 47, -13, 118, -56, 95, 117, 98, -8, 100, 32, -89, 54, -41, -17, -21, 106, 64, -57, 114, 112, -60, -29, -5, -123, -40, 42, -84, -112, 93, -114, 39, -24, 102, -81, -21, -92, 98, 25};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SmsTx #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SmsTx msg;
    msg.setTimeStamp(0.713651202567);
    msg.setSource(32155U);
    msg.setSourceEntity(44U);
    msg.setDestination(59837U);
    msg.setDestinationEntity(220U);
    msg.seq = 1672873256U;
    msg.destination.assign("XAHPYASMSCGZREFQZZTBITEJXHECXZACSOMWEHYGAIKOHGSXDFWQLYPNCTDDVFDSIZUJORJJXTRMCIXLKHMJIKKVVDVQCPFRKKNXGBPASFNRFAUUZWGAXMNNYVUMDLVLRZOGWBWFKWCTCVKVNTLSMCQDPHYXYQBYQLMPEH");
    msg.timeout = 40773U;
    const char tmp_msg_0[] = {-85, -47, -51, -81, 31, -121, 94, -32, -79, 115, -101, 89, -73, 53, 121, 43, 33, -105, -115, -36, -54, 17, -114, 121, -72, -79, 39, -119, 99, -77, 45, -116, 33, 123, 66, 0, 120, 73, -111, 49, -7, 68, 78, -58, 42, 77, 89, -127, 112, 46, -39, 52, -125, 19, -96, -87, 41, 78, -71, 108, -6, 28, -2, -54, 19, 29, 124, 56, -15, 88, -1, -97, -66, -38, -68, 112, 28, -47, 110, -89, -7, -37, -126, -13, 17, -64, -1, 93, 120, -50, -114, -105, 84, 122, -120, 58, -97, -103, -43, 35, -98, 78, 61, 66, 115, 87, -23, -31, 31, 78, -19, 38, 20, -52, -105, 65, 90, 113, -26, -73, -16, 42, -57, -14, 17, 40, -126, 10, 69, -82, 57, -15, -81, -53, -89, -17, 12, -12, 35, 121, -49, 66, 102, 18, 3, 25, 104, 110, 89, -87, -63, -85, -90, -18, -30, 77, -41, 81, -99, 49, 86};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SmsTx #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SmsTx msg;
    msg.setTimeStamp(0.10767344044);
    msg.setSource(13295U);
    msg.setSourceEntity(186U);
    msg.setDestination(2847U);
    msg.setDestinationEntity(14U);
    msg.seq = 694103581U;
    msg.destination.assign("RFWYRYEAGNOXTOJCJEQDNCKFYSUBDZXFYEPISKMWVUTUNXELOVNICVPZYVNWESFBLJAQMEPDYFAMHSIMVXMQWYNLAKGGWKYJMBGMPFNCWRQZPRSBDRARFJRFHTIKJKBTZXUITGPVBUJAAHOOSUJG");
    msg.timeout = 27068U;
    const char tmp_msg_0[] = {-100, 7, -90, 33, -36, 25, -93, -65, 9, 50, 48, -4, 76, 64, 71, -63, -18, -125, -128, -102, 67, 18, -54, -43, 101, 29, -66, -50, 40, -87, -17, -4, -81, -100, -9, 68, 94, 16, 77, 77, 28, -48, -55, -52, 80, -2, 76, 6, 119, -87, 93, 59, 84, -77, -91, 96, 11, 2, 55, -54, 119, 119, 105, 85, -4, -58, 39, 3, 69, -15, -62, 124, -58, 95, 85, -67, 119, -101, 26, 86, -118, 34, -92, 120, 56, 44, 113, -46, 15, -121, -128, -93, 111, 27, 10, -66, -121, -75, -62, 123, 60, -122, 82, -12, 77, -45, 52, 61, 85, -124, -19, -87, 40, -95, -92, -121, -59, 54, 9, 10, 18, -60, -45, 2, 24, -80, 87, 93, -123, 108, 104, -89, 10, 57, 75, 99, -41, -35, 101, 69, 63, -73, 33, -82, -49, -82, -18, 126, -48, -115, -63, -101, 38, -64, -80, 82, -55, -58, 75, -61, -18, 105, 29, -56, 13, 2, 28, -38, -113, 124, 70, -77, 90, -59, 69, 20, 105, 111, -55, 50, -118, 88, -106, 17, -91, -3, 29, 44, 12, 71, -106, 118, 22, -106, 11, -29, 70, -60, 59, -57, -77, -71, 66, -104, -70, 115, 41, 61, -6, -50, 126, 122, 29, 126, 99, -77, -37, 108, 96, 62, 17, 15, 33, -59, 103, 40, 103, 50, -110, -10, -11, 115, 34, -2, 78, 31, 88, 102, -123, 100, 86, 75, 45, 91, -90, -58, 70, -119, -64, -117};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SmsTx #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SmsRx msg;
    msg.setTimeStamp(0.901822085509);
    msg.setSource(48271U);
    msg.setSourceEntity(39U);
    msg.setDestination(64226U);
    msg.setDestinationEntity(208U);
    msg.source.assign("TVPTDDLCUZVSCNMCTTTQJMXZAHMYPOLI");
    const char tmp_msg_0[] = {62, -58, 108, -74, 50, -103, 37, 12, -29, -23, 77, -7, 31, -77, 124, 44, -87, 31, -91, 84, 29, 91, 18, -25, 62, 122, -23, 54, -125, -7, 97, 2, -105, -24, 12, -78, 37, 107, 101, 16, -39, -41, -50, -39, -105, -75, -26, -83, 43, 124, 71, 21, -77, -32, 59, 64, 84, 46, 68, 62, 43, 101, -56, -53, -120, -6, 116, 81, 56, 94, 17, -104, -128, -53};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SmsRx #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SmsRx msg;
    msg.setTimeStamp(0.602195077512);
    msg.setSource(40593U);
    msg.setSourceEntity(53U);
    msg.setDestination(41902U);
    msg.setDestinationEntity(86U);
    msg.source.assign("TNLUFQLVDIDFUESFWZHACIJWMSIQTXAZCTENNGORFSGXNWSQEMBLVKNBPINJDSJSUUKKSOHGXDJHOQBNDRKEEFUGSTWSRPJCGTETCBIOPNILUZRCODHBKWBDF");
    const char tmp_msg_0[] = {68, -9, 123, -52, -45, -70, -35, -99, 56, 68, 97, 113, -102, -27, -29, -72, -70, -41, -109, -11, 103};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SmsRx #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SmsRx msg;
    msg.setTimeStamp(0.383490425708);
    msg.setSource(60119U);
    msg.setSourceEntity(219U);
    msg.setDestination(26460U);
    msg.setDestinationEntity(225U);
    msg.source.assign("KYFOZXZHRIMSZSRHLYMIHSUUKTFOBGHTEBIJLCSTARDOZKMOKLWXCHAKYELJGEFQUXSGKIRPDBWGYVBACWGNTIIQZILJPENAONKPXRVLFGKTOWCPBQ");
    const char tmp_msg_0[] = {-105, 100, 55, -100, -58, 9, -28, 60, 0, -21, -73, -46, -35, 12, 111, 25, 59, -30, -54, -117, 115, -116, 117, 3, 44, 111, 42, 84, -9, -108, -55, 67, 9, 45, 126, 114, -67, 22, 88, 105, -50};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SmsRx #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SmsState msg;
    msg.setTimeStamp(0.160809851235);
    msg.setSource(5325U);
    msg.setSourceEntity(84U);
    msg.setDestination(16619U);
    msg.setDestinationEntity(6U);
    msg.seq = 372742449U;
    msg.state = 164U;
    msg.error.assign("HSEBNSBLPIDFCOHBFNJLYUSDEBTQTZDVIDUASJMEJJXMNWFIYHGXUAXRXHVBCRVUWJAKGXMYWQRUPIS");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SmsState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SmsState msg;
    msg.setTimeStamp(0.831036332318);
    msg.setSource(63323U);
    msg.setSourceEntity(68U);
    msg.setDestination(61037U);
    msg.setDestinationEntity(170U);
    msg.seq = 3148072398U;
    msg.state = 163U;
    msg.error.assign("YWWSVJCJOAGOTUZALZEEAFOENDFARKDVZUYZSCYKDQUZYRKAZPDXPFWRWPMUPKNOLXYPOXRRKTJHHQTGUIMFWSULWMFE");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SmsState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SmsState msg;
    msg.setTimeStamp(0.365458434553);
    msg.setSource(16936U);
    msg.setSourceEntity(203U);
    msg.setDestination(1601U);
    msg.setDestinationEntity(88U);
    msg.seq = 4101766917U;
    msg.state = 26U;
    msg.error.assign("VSHFZIFZQVBWDAJTCGZTRSDMLEZHYPQRVEAWLHGYRNQXJXKZTIJRVRLGLXWOLQNMAECHMVXNSHVNTPJUFQXMWUKSDWMNMRSCLMNWNOGIFJJUIHSXYOABUPNFTRJKEVPJEUCXFWRDFTYBKIGUMCPUTQDEUYAXOOLUXUAGZOSZBWLFYBQFPSVLMTPGJNDKBRBDHAKQDOIVHOPTQKHTZBZFSEDZWEPDEBMGEJWYCHYCSIBII");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SmsState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TextMessage msg;
    msg.setTimeStamp(0.000703126990089);
    msg.setSource(15799U);
    msg.setSourceEntity(131U);
    msg.setDestination(56621U);
    msg.setDestinationEntity(66U);
    msg.origin.assign("XUCSNVGDKNKCVPJANBMHQQHOCYCRUTFCTNGFEZWYYZIVPTJWYTDBRWMWMJVFBMGBHPHLWISZDFFELVHBPUXWOVJPVSQCBCCEYNWJOOKSVDRDPRXLLIDNXTUKEEZIAPSQRWEUTJRRGKYYOTXSMIZQKVITOKYIHABQHXTUNCZBEMHQYALGWPZGJDFFOFXAZOAXEISWSEYNLPUPAUFXUSKLMIMOLMHABMRNH");
    msg.text.assign("TJVXSGHSOEMTEZBCCRFUEYKJJKRCHUFPAQPXHBMIWZTYXKNUDYCABLVHCHDLWFVNVXJU");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TextMessage #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TextMessage msg;
    msg.setTimeStamp(0.890457656578);
    msg.setSource(3450U);
    msg.setSourceEntity(20U);
    msg.setDestination(32410U);
    msg.setDestinationEntity(43U);
    msg.origin.assign("EMIMFZOQZOIKPHGWUOMEWACWBYXJGCCHIAXVAKJADUUPNJGKCFDNHVBZDJEVBYNJCUHCPAUXNIRGUZXHBP");
    msg.text.assign("ONRBMFELCKMETIDBRUVBTNQDCTALXSGFJJVGJUTGSEBBOSCQEFLPFRVECVZUGYIMPNJZOUXKSBQCAXDEWZXTYDZHOGUJZHNHLRAJULRNPZKOSEKNVHDGPZWLDUNVXWNSLVHKQQAMWRKHMHWFIMNCELDU");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TextMessage #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TextMessage msg;
    msg.setTimeStamp(0.739966072622);
    msg.setSource(602U);
    msg.setSourceEntity(132U);
    msg.setDestination(23290U);
    msg.setDestinationEntity(21U);
    msg.origin.assign("VZWLABESWSAHESNZHAYGXKBYESLPIRMPKYDRGFIANGQYZBHSHDLJYJIMUDFYEAUKNCHPZGBRSIRMCQKINUMLFNKEMQYAUXWZZGUOSTUWRIQPKYNDAFSBFVXJPGZGALLGPJDJOPFPGVQBVHUKTPVRQZIQMCELMKDLCUKNVCPCJZNEBRWTTCKLHVDSOTLVFFMNQTEDZQDRXYVQBCWNOITFXWX");
    msg.text.assign("ZDCJVUJVKCUNGITYNBMDOWWSVPOAFMPAQKHXEDTCZCDYPERRKMZOQWAYPTUDSSCFLHXEYOYJFTLCRNLIPUEOUUHBKMWTPHZHZYNNKVSHSQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TextMessage #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IridiumMsgRx msg;
    msg.setTimeStamp(0.0519042135997);
    msg.setSource(3604U);
    msg.setSourceEntity(86U);
    msg.setDestination(51146U);
    msg.setDestinationEntity(242U);
    msg.origin.assign("QMCVLIWMZQCIGQGURNEMKI");
    msg.htime = 0.887364414501;
    msg.lat = 0.655184207191;
    msg.lon = 0.129817765969;
    const char tmp_msg_0[] = {63, 41, -34, 41, 87, 88, 82, 32, -99, 78, 32, 46, 105, 62, -113, 119, -35, -90, 87, -35, -35, -65, -2, 45, 45, -15, 65, -34, 96, 48, -121, 14, 100, -16, 118, -112, -41, 29, 97, -33, -32, -116, 12, 40, -70, -109, -10, -103, 3, -96, 57, -97, -28, 39, 96, 85, -89, 27, 40, 104, 41, 22, 108, 123, 15, -35, 4, 102, 1, 59, -59, 47, -34, -46, -102, 50, 51, -122, 104, -45, 83, 24, 77, 114, -101, -69, -33, 100, -29, -52, -89, 13, 81, -96, -8, -54, 90, -62, -8, -105, -79, -118, 89, -54, -93, -99, -27, -84, -41, 19, 38, 25, 36, 126, -23, -78, 3, -28, 44, 79, 76, -127, 22, -15, -105, -49, 0, 98, 14, -93, -88, -81, 16, -121, 1, 116, -110, 9, 118, -118, -45, -96, -101, 53, 62, -14, -80, -119, 42, -33, -67, -111, -30, -28, -36, 22, 56, 72, 110, 38, 78, 26, -41, 12, -109, -118, -85, 10, -53, -23, 88, 78, -117, -62, -82, -105, -80, -15, -8, -12, 71, 80, -14, -54, 112};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IridiumMsgRx #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IridiumMsgRx msg;
    msg.setTimeStamp(0.77683199582);
    msg.setSource(54200U);
    msg.setSourceEntity(125U);
    msg.setDestination(50890U);
    msg.setDestinationEntity(55U);
    msg.origin.assign("KVELFASOMAGUBUITADLZRWKMLHPORIPJRBSEWMBVCGGZZNQELECMHDDPBTYCZTURQDYBXORQZLVCFINYKXWZUOAKIESZYCJCEUQNKJCMHULVGGOMSONUSQJFHEQKGLAEDJDDULBYAPVFLFOWNIQXUVVWXXHFOITZBMKUNIYFRXMHXZVSTAEOASEWATDSLKTWWBMIFWJBQGQPCPDKKFHYHWCSDXGYNSGJYRJBIJXCFZTO");
    msg.htime = 0.199890672987;
    msg.lat = 0.717799150594;
    msg.lon = 0.611128999551;
    const char tmp_msg_0[] = {116, 30, 82, -123, -39, 88, 95, -51, -5, -63, -24, -75, 35, 39, 7, 59, -30, 23, 29, -15, 65, -6, -100, 56, -124, 107, -22, -39, 53, -31, -117, -88, -128, -123, 108, -88, -86, -25, 115, -128, 101, 90, 63, -24, 38, 91, -39, 102, 16, -125, 4, -83, -119, 30, -101, -65, 4, -85, -71, 5, -49, -122, -9, -39, -57, -13, -63, 9, 37, -109, -8, -113, 7, 21, -127, 56, -64, -48, 107, 88, 5, -33, -26, -51, -60, -11, -126, 89, 41, -97, 76, -117, 110, 100, -49, -93, -114, -13, -73, -36, -84, 80, 126, -101, 36, -128, -98, 29, -59, 118, 29, 124, 116, -100, 104, -96, 69, 125, -31, -24, -96, -49, -16, 5, 28, -13, 84, 97, -5, -12, 32, -24, -61, -36, 80, -67, -73, -20, 74, 105, 72, -17, 119, -110, -31, -11, 35, 95, -42, -109, 40, -76, -122, 90, 117, -10, 74, 107, 8, 49, -48, -50, 59, 90, -83, -95, -62, 83, 109, 8, 1, 7, 10, -50, 88, 99, 11, 63, 79, -28, 55, 116, -96, -62, 97, 118, -7, -71, 56, 121, 27, 67, -87, 56, -85, 98, 111, -101, -54, -84, -31, -65, -113, -84, -97, -2, -74, 46, -105, -104, 83, 74, 43, 19, -78, 117, 81, -23, 86, -91, -88, 45, 87, -9, -126, 21, 125, 65, 106, -38, 103, -117, 96, -33, 32, 29, 114, -53, 69, 100, 43, 93, -91, -24};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IridiumMsgRx #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IridiumMsgRx msg;
    msg.setTimeStamp(0.888792163183);
    msg.setSource(42353U);
    msg.setSourceEntity(11U);
    msg.setDestination(57781U);
    msg.setDestinationEntity(59U);
    msg.origin.assign("ZCDSTUNNMCKNFARYNYBPCAMHSKVJYQBEXURHNQLRXKGXXQISGZBRKLBQXTJNITXSHJIYIAOQDMYLLAKGWKTYBZESEIHJFLJFMDJMLEEZSVSRCPLPBXMTELQSBWKIOCGORFUOYCOXFWWLOEOZSIDAPPBFJVFGNZBEAMZPQYDQUMGDQUBDSGTIOZIFTNJXQVYZA");
    msg.htime = 0.226031998282;
    msg.lat = 0.289816843919;
    msg.lon = 0.0310572745537;
    const char tmp_msg_0[] = {45, 39, -72, -57, -90, -50, -110, -54, -66, 77, -71, 16, 116, -52, 52, 101, 65, 28, -124, -90, -123, 102, -109, 85, -76, -43, 31, -102, 49, 27, 47, -11, -90, -35, 80, -58, 110, -8, -48, 20, 62, -118, 30, 7, -75, -25, -4, -84, -117, -42, 66, -19, -3, 57, 94, -123, -127, 124, -98, -51, -102, -107, -24, 92, 92, 97, -80, 51, 84, -111, -17, 59, -121, 40, 101, -22, -100, -81, -96, 78, -8, 27, 110, 5, 66, 11, 18, 39, -126, 70, -116, -77, 117, 98, 23, 117, -36, -44, 39, -127, -20, 92, 123, -117, -34, 46, 21, 79, -126, 115, 102, 76, -108, 80, -80, -4, 88, 42, -99, 22, 125, -82, 119, -31, -107, -50, -18, -115, -51, -79, -98, -12, 49, 83, 8, -6, 112, 1, 11, 41, 96, -106, 79, -59, -31, -127, 28, -104, 29, 70, -115, 92, -56, 45, 93, -30, 91, 46, -109, -109, -69, -75, 53, 51, -67, -23, -67, -44, 37, 33, 33, 116, -12, 101, -16, 66, -36, 6, 74, 48, 106, 82, -51, -85, -20, 118, -57};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IridiumMsgRx #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IridiumMsgTx msg;
    msg.setTimeStamp(0.46392728314);
    msg.setSource(19146U);
    msg.setSourceEntity(244U);
    msg.setDestination(62275U);
    msg.setDestinationEntity(203U);
    msg.req_id = 58434U;
    msg.ttl = 411U;
    msg.destination.assign("AQGPREEUIGCZNQPRHHIBFRMCZVMVHCWHRPMDLWYKWJYWRFKIABXVTMERQFFLPYANBRLXSJXZLIQXMYSPJUXPSNWHA");
    const char tmp_msg_0[] = {120, -36, -88, 15, 48, 112, 105, -6, -44, -15, 112, -116, -26, 96, 124, 37, 56, 23, 69, 51, 102, -23, -52, -113, 17, -32, -70, 114, -1, 27, -16, 82, 83, -60, 36, -44};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IridiumMsgTx #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IridiumMsgTx msg;
    msg.setTimeStamp(0.450112207623);
    msg.setSource(13569U);
    msg.setSourceEntity(88U);
    msg.setDestination(48865U);
    msg.setDestinationEntity(168U);
    msg.req_id = 53505U;
    msg.ttl = 37188U;
    msg.destination.assign("AARBTTBBNENKGSIGPNLLDJCRHUHWZEMOBNGDVDATLAEQRWGIWUEICTDPRVZNIMOVXKKUQIULBXSTGFVKUNJPFKMMJTARMAWZYRQXYWGJHVSZSINHFOXVLFNNPEDQTBJOLDTFKGWCFWBIEPBFCQOMELWNMJSQFBXKYCCJZBUZFMYOKRCJDQIUMKSLCJY");
    const char tmp_msg_0[] = {51, 92, 107, 67, 107, -99, -92, -109, -20, 34, 37, 32, -94, -8, -91, -13, 13, -64, -34, -90, -119, -121, -56, -62, 122, 43, 109, -55, 26, -114, 122, -119, -82, -100, -50, -127, 69, 82, -73, 30, 73, 73, -13, -122, 109, 103, -123, 104, 16, -57, -91, -45, 10, -52, 40, 103, -72, -38, -94, -65, -3, -44, -84, 24, 118, -14, -36, -72, -27, -75, -21, 89, -122, -100, 123, -127, 45, -125, -123, 4, -3, 100, -110, 115, -79, -120, -21, 122, 71, 93, -80, 40, 23, 50, -34, 8, -8, -62, -76, 4, -78, -59, 79, 0, -84, -83, -62, -10, 109, -104, 75, -63, 18, -117, 71, -47, 110, 108, -114, -89, 23, -36, -89, -120, -90, -102, -2, 10, 77, 122, 37, -37, -15, -16, 77, -109, 116, 101, 105, 118, -36, -76, 27, -41, -70, 72, -114, -23, 5, 7, -64, -122, 111, 96, 39, 11};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IridiumMsgTx #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IridiumMsgTx msg;
    msg.setTimeStamp(0.9625351889);
    msg.setSource(63810U);
    msg.setSourceEntity(221U);
    msg.setDestination(7685U);
    msg.setDestinationEntity(253U);
    msg.req_id = 56824U;
    msg.ttl = 39871U;
    msg.destination.assign("LHGATKNPGTHMCUHOHZUESAJSUYBACFDCZSLKZGQNDNTYWNRQUHDDYBHERIRGDRUEMYLJHRGAUJKJJXYAISNNMGDYZWOCI");
    const char tmp_msg_0[] = {33, 104, -21, 7, -4, -89, 26, 86, 86, -82, -82, 69, -83, 12, -110, -4, 5, -11, -89, -27, -29, 117, -17, 49, 26, 3, -28, 45, -118, -96, 93, 2, -15, -73, 51, 14, -7, 86, -82, -68, 8, -20, -28, 95, 30, 94, 12, -63, -31, 11, 62, -78, 33, 59, 71, 73, -51, 103, -37, 54, -85, -25, -73, -38, -44, 47, 112, 97, 71, -14, 10, 86, -68, -25, 72, 24, 64, -112, -84, -18, 14, 60, -25, -2, -60, 0, -78, 126, -70, -104, -71, -68, -119, 66, 64, -38, 11, -96, -33, 113, -84, 37, 23, -118, 79, 38, -125, 96, 93, -93, -41, -81, -101, 20, 64, 110, 37, -15, 51, -25, 1, 11, 76, 19, -18, 89, -93, 111, -12, -27, -113, -126, 52};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IridiumMsgTx #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IridiumTxStatus msg;
    msg.setTimeStamp(0.814080463854);
    msg.setSource(10845U);
    msg.setSourceEntity(200U);
    msg.setDestination(11686U);
    msg.setDestinationEntity(229U);
    msg.req_id = 24550U;
    msg.status = 229U;
    msg.text.assign("OMSSZIZARNCNDMCXBXBUPJYPRPWLZSCFSJCYBKHNJJVFGGVDETWVBJKZIZEHNXQYUOGWANFSCZXRFAYORANMNLRBTODEVDJURHWQLLVMZGZGKTZWHTQTYDJTQNGEAYPCXIHIUOMEXTMFRFQBJZKGKQIKGWEPEPQFWCXALAGOSONKOAVUMFPIEPY");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IridiumTxStatus #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IridiumTxStatus msg;
    msg.setTimeStamp(0.519936166505);
    msg.setSource(622U);
    msg.setSourceEntity(217U);
    msg.setDestination(62933U);
    msg.setDestinationEntity(192U);
    msg.req_id = 33184U;
    msg.status = 125U;
    msg.text.assign("QYFIFIYFOJJPLNQINXWATUCOKPZWTSOPWIZGYWRENEDNJPLXXPTBIVPMNOOYOSVQDJHLWMNKCJOEKJVJO");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IridiumTxStatus #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IridiumTxStatus msg;
    msg.setTimeStamp(0.123459548595);
    msg.setSource(44658U);
    msg.setSourceEntity(46U);
    msg.setDestination(15543U);
    msg.setDestinationEntity(113U);
    msg.req_id = 30659U;
    msg.status = 134U;
    msg.text.assign("UTHIPZHQPRMGIKRDUEXKBQSGEMICYASDSOUOUIQUQNWMVZOPIRHIRFSPYYGXPOGXEEHJSCLIQHKLDPVOYPYTVWFKUTCSIAFABLZRDBGMWNHNNREMXNWAEMDOTPGLREKOJ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IridiumTxStatus #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GroupMembershipState msg;
    msg.setTimeStamp(0.85639037025);
    msg.setSource(61955U);
    msg.setSourceEntity(1U);
    msg.setDestination(50939U);
    msg.setDestinationEntity(81U);
    msg.group_name.assign("MMIRCSPBOMPMHBTAIEEUHDWXVQWGVFZZDUENFLDRUCRAVXMZIGD");
    msg.links = 1980754570U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GroupMembershipState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GroupMembershipState msg;
    msg.setTimeStamp(0.933414783382);
    msg.setSource(47076U);
    msg.setSourceEntity(166U);
    msg.setDestination(39197U);
    msg.setDestinationEntity(234U);
    msg.group_name.assign("USAFBQWIKGOMCBXQZWTCFTJMCKIFGVEYHPRPVUFAOEBTOOKKKGQBOUARWHYNAFPD");
    msg.links = 1719475463U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GroupMembershipState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GroupMembershipState msg;
    msg.setTimeStamp(0.191343428425);
    msg.setSource(4139U);
    msg.setSourceEntity(156U);
    msg.setDestination(54557U);
    msg.setDestinationEntity(214U);
    msg.group_name.assign("BDQJHSPHPWEHPQECCWDXJXAHXNMMSHEYQFNKUCZTUAKCPJZSIFEGOLUFVBZPVOFOBYQNXNRFELDIANWTMOKURLLCJKYCIHYOVDRZWVHYSVMLJWERFBARAFRUACKRFNSUQRPVLIKWDENTXQGTIGZTHLISHOPUWTYIPXBETETGYNRVAGGMUHJPACJWRQXEUQFYZNDDDKDPZXJCMIMBGMFCSQVOMBSY");
    msg.links = 1696702280U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GroupMembershipState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SystemGroup msg;
    msg.setTimeStamp(0.604666984818);
    msg.setSource(59062U);
    msg.setSourceEntity(183U);
    msg.setDestination(20331U);
    msg.setDestinationEntity(40U);
    msg.groupname.assign("WBXIWVFOOJTPZJBDEVYYTNUDCVLAHOMHZVEDTUIOGEHGRGJRZLKIFWTETOKCAYXHRSXSNLZCCWQQWFPNVBAFNAUEAARRJSMIUDBGDKKPFVUFFGPNUJJYPITFXNZSSYXBLAUZPAECYPIJRJOQOWZXAQSZWMUSDXFWN");
    msg.action = 74U;
    msg.grouplist.assign("UGFETJJNXJXLATWNEIUORLFGGVBDLKMXPRFCXYFRLKMMELCKUJXBOYPSTAPPAJWGVOMRKITTUWVQROUJRHSBMUYYEYKWZLEDVVLZGDKMCTUSCMWZAHVHHOMZZCBNEBZBLKSAWVICXNPNQRSBNCIIPYGAWSPNFCSAEUIIAIOHFHLZTNMPXINCSJORDXPVRFKDQQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SystemGroup #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SystemGroup msg;
    msg.setTimeStamp(0.0798632639543);
    msg.setSource(12807U);
    msg.setSourceEntity(225U);
    msg.setDestination(53139U);
    msg.setDestinationEntity(227U);
    msg.groupname.assign("BKLNYDAKYUDDCOADTKHECHADVYUHCATEBERRZQBXQJVXMFNPZYNBNPEWBUKISSMKPVAWLMOTQWOPJFHFDREHBPRGZPCXYLFIVSOFBJMZGSPOQBLRAQPUHG");
    msg.action = 127U;
    msg.grouplist.assign("YODNSUKCEVWMOOBRUXALAUKRGQNAIIYRZJZMHXWCRDPDDDMVPQLNJTSJFOWUWLYAJDNQWMOMGXXYKIEQKKZTSGIXQXVUDTBPEHWWWXMIZVKPGSTVAOH");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SystemGroup #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SystemGroup msg;
    msg.setTimeStamp(0.560307714949);
    msg.setSource(16050U);
    msg.setSourceEntity(229U);
    msg.setDestination(5683U);
    msg.setDestinationEntity(208U);
    msg.groupname.assign("INZGPXYBMMFTGCHGZDOIQQUCNYHUZDPVFYQUAILKYZFQRJMSQRYZYHWMEOQCQACAANVMISPBZTJVIFGEUCTTVHCUPFNETFAGDJPRLJBSUKLWSGZXEDRXIJVBORGTXNPUWHJWTUSOCLFOGWSDCHJOWRIWCJAJSKD");
    msg.action = 165U;
    msg.grouplist.assign("OWPVUSNANNANVUYQTKLJKLJDWEGOQURWIUCYNHFFCWJPHVUPKUEZBGGVSTTUSRZRXEFGVXCATDZCPIQWFASOTOJQTQCWMHRMXLPMRPIRINZBYABORKMXZQBVOUFCQYV");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SystemGroup #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblRange msg;
    msg.setTimeStamp(0.322481156966);
    msg.setSource(15823U);
    msg.setSourceEntity(55U);
    msg.setDestination(34930U);
    msg.setDestinationEntity(76U);
    msg.id = 99U;
    msg.range = 0.675042304793;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblRange #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblRange msg;
    msg.setTimeStamp(0.219972207559);
    msg.setSource(19702U);
    msg.setSourceEntity(127U);
    msg.setDestination(26041U);
    msg.setDestinationEntity(140U);
    msg.id = 227U;
    msg.range = 0.908617883414;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblRange #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblRange msg;
    msg.setTimeStamp(0.187491393983);
    msg.setSource(14701U);
    msg.setSourceEntity(114U);
    msg.setDestination(17628U);
    msg.setDestinationEntity(77U);
    msg.id = 69U;
    msg.range = 0.725499956494;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblRange #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblDetection msg;
    msg.setTimeStamp(0.949243164835);
    msg.setSource(36660U);
    msg.setSourceEntity(150U);
    msg.setDestination(48986U);
    msg.setDestinationEntity(98U);
    msg.tx = 167U;
    msg.channel = 92U;
    msg.timer = 15398U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblDetection #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblDetection msg;
    msg.setTimeStamp(0.0826351933465);
    msg.setSource(49431U);
    msg.setSourceEntity(78U);
    msg.setDestination(24953U);
    msg.setDestinationEntity(213U);
    msg.tx = 231U;
    msg.channel = 48U;
    msg.timer = 7886U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblDetection #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblDetection msg;
    msg.setTimeStamp(0.99912353948);
    msg.setSource(3764U);
    msg.setSourceEntity(55U);
    msg.setDestination(9355U);
    msg.setDestinationEntity(6U);
    msg.tx = 206U;
    msg.channel = 18U;
    msg.timer = 34597U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblDetection #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblBeacon msg;
    msg.setTimeStamp(0.726756511788);
    msg.setSource(17914U);
    msg.setSourceEntity(164U);
    msg.setDestination(33804U);
    msg.setDestinationEntity(7U);
    msg.beacon.assign("IMEDKDQTLNVKLTHMIDYCYXAFAGNKQXRFXJWEAFYRWZCGRBOVMINQOMTNVZPWDHWGDMASCDSYWRMSOEPCUFCESDNZLSKWZDHRYUIQXPRXBVEYZPPGBKJESYGYXRZKZBXQZSNLCUBTUWROKVTHLNXJQRAQWGNITGOKUOTPEGOMUVPBEJFJJOICFASELCHUCZQVHQHCDKAQGYARNFJYNIUFBMBHMHGVOXIKVL");
    msg.lat = 0.928096082103;
    msg.lon = 0.104462297806;
    msg.depth = 0.527164280445;
    msg.query_channel = 57U;
    msg.reply_channel = 18U;
    msg.transponder_delay = 47U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblBeacon #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblBeacon msg;
    msg.setTimeStamp(0.54325171913);
    msg.setSource(10974U);
    msg.setSourceEntity(228U);
    msg.setDestination(10380U);
    msg.setDestinationEntity(177U);
    msg.beacon.assign("JJGGYEJTNAFPIXVWCALOWRFDPWXYFYWB");
    msg.lat = 0.945790366406;
    msg.lon = 0.0012340253323;
    msg.depth = 0.610296173399;
    msg.query_channel = 107U;
    msg.reply_channel = 165U;
    msg.transponder_delay = 58U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblBeacon #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblBeacon msg;
    msg.setTimeStamp(0.787892982612);
    msg.setSource(26732U);
    msg.setSourceEntity(164U);
    msg.setDestination(25625U);
    msg.setDestinationEntity(26U);
    msg.beacon.assign("XRTZEGFIZNJWVYJZMERNDVLZEGNJZJPUJRPGJTSBWBVWNHGOGCHPNBCZMOHPDEVFSMUILMRUSHKTGUWCSIFODDQHDMTMWZECTRIOQISKDBWWDIAXRMODBJONEQUVYCTPYHROFRSATACYJYCXDEIFUPSQAWGFUKLXCAKBZJHPVPEUXXMWIGVJMLYAGOTPVXABQYEN");
    msg.lat = 0.299308612645;
    msg.lon = 0.0161375944745;
    msg.depth = 0.756097774562;
    msg.query_channel = 84U;
    msg.reply_channel = 106U;
    msg.transponder_delay = 254U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblBeacon #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblConfig msg;
    msg.setTimeStamp(0.673369638301);
    msg.setSource(48446U);
    msg.setSourceEntity(58U);
    msg.setDestination(3055U);
    msg.setDestinationEntity(107U);
    msg.op = 99U;
    IMC::LblBeacon tmp_msg_0;
    tmp_msg_0.beacon.assign("IRXDHKTSEBPGUODQWTRLGCXIVAEVPOVYEUIXZVXGTQDCYHKCBJMGIFFAWPBEJGRRLSJSAINRBSPMNDYNEIZAPAIQNYWLFFJVQGPCHSSUQTFKTAEMDBXKIJNRMGLZMUOTZL");
    tmp_msg_0.lat = 0.693748414691;
    tmp_msg_0.lon = 0.677188232434;
    tmp_msg_0.depth = 0.94792176158;
    tmp_msg_0.query_channel = 191U;
    tmp_msg_0.reply_channel = 135U;
    tmp_msg_0.transponder_delay = 223U;
    msg.beacons.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblConfig #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblConfig msg;
    msg.setTimeStamp(0.553042488452);
    msg.setSource(59106U);
    msg.setSourceEntity(123U);
    msg.setDestination(11909U);
    msg.setDestinationEntity(54U);
    msg.op = 4U;
    IMC::LblBeacon tmp_msg_0;
    tmp_msg_0.beacon.assign("IPGIBATJPTFLPVHOZDVMLOCULIYTMQUEKDSRDORPSBOISRBAGEUJPNNVFJLYKYXNHKRFKBXCEZ");
    tmp_msg_0.lat = 0.894783798589;
    tmp_msg_0.lon = 0.59429690562;
    tmp_msg_0.depth = 0.260007661149;
    tmp_msg_0.query_channel = 54U;
    tmp_msg_0.reply_channel = 19U;
    tmp_msg_0.transponder_delay = 248U;
    msg.beacons.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblConfig #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblConfig msg;
    msg.setTimeStamp(0.464368260862);
    msg.setSource(63697U);
    msg.setSourceEntity(217U);
    msg.setDestination(55427U);
    msg.setDestinationEntity(57U);
    msg.op = 187U;
    IMC::LblBeacon tmp_msg_0;
    tmp_msg_0.beacon.assign("OBDAAWYLZGRNGPUTVQXYJVNGTXRFHFTIAUZSIVKOKYBBJOUJFAMOKXLNYPYUNTHWKCWQHFEOWXMPSLRCNSWKHRLGAZTOJEQQIJZDAXVAFTYOTCCISICLCIREXUZQRJRVVOSEPNZUWZCQBHTJBBDEVLQEKTIIVMFDMWRBFPPS");
    tmp_msg_0.lat = 0.528049711362;
    tmp_msg_0.lon = 0.525547022692;
    tmp_msg_0.depth = 0.592003058862;
    tmp_msg_0.query_channel = 13U;
    tmp_msg_0.reply_channel = 230U;
    tmp_msg_0.transponder_delay = 203U;
    msg.beacons.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblConfig #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticRange msg;
    msg.setTimeStamp(0.579849306571);
    msg.setSource(4877U);
    msg.setSourceEntity(197U);
    msg.setDestination(35125U);
    msg.setDestinationEntity(224U);
    msg.address = 139U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticRange #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticRange msg;
    msg.setTimeStamp(0.359069295963);
    msg.setSource(55929U);
    msg.setSourceEntity(161U);
    msg.setDestination(21120U);
    msg.setDestinationEntity(16U);
    msg.address = 112U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticRange #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticRange msg;
    msg.setTimeStamp(0.810409820258);
    msg.setSource(13375U);
    msg.setSourceEntity(111U);
    msg.setDestination(19608U);
    msg.setDestinationEntity(122U);
    msg.address = 226U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticRange #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticRangeReply msg;
    msg.setTimeStamp(0.549205187035);
    msg.setSource(12442U);
    msg.setSourceEntity(23U);
    msg.setDestination(47166U);
    msg.setDestinationEntity(128U);
    msg.address = 117U;
    msg.status = 4U;
    msg.range = 0.114811169433;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticRangeReply #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticRangeReply msg;
    msg.setTimeStamp(0.339405385072);
    msg.setSource(5779U);
    msg.setSourceEntity(204U);
    msg.setDestination(24512U);
    msg.setDestinationEntity(116U);
    msg.address = 78U;
    msg.status = 99U;
    msg.range = 0.92412368119;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticRangeReply #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticRangeReply msg;
    msg.setTimeStamp(0.0446852603734);
    msg.setSource(25445U);
    msg.setSourceEntity(32U);
    msg.setDestination(63470U);
    msg.setDestinationEntity(14U);
    msg.address = 165U;
    msg.status = 156U;
    msg.range = 0.522022309774;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticRangeReply #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticMessage msg;
    msg.setTimeStamp(0.0967299976444);
    msg.setSource(2598U);
    msg.setSourceEntity(25U);
    msg.setDestination(4635U);
    msg.setDestinationEntity(144U);
    IMC::SetPWM tmp_msg_0;
    tmp_msg_0.id = 172U;
    tmp_msg_0.period = 4184367705U;
    tmp_msg_0.duty_cycle = 1574613699U;
    msg.message.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticMessage #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticMessage msg;
    msg.setTimeStamp(0.537781345051);
    msg.setSource(49887U);
    msg.setSourceEntity(188U);
    msg.setDestination(20352U);
    msg.setDestinationEntity(234U);
    IMC::SmsRx tmp_msg_0;
    tmp_msg_0.source.assign("QJJTQIFJUVPCMVINHZFATFMBCWJCEBQFLMOCEMAHTABUFZEUREBDZGXRMEERQOSXHSDXGXWOWJRBNAGPVOVGRLIZZXJRLXYSYNDQNJABDOXWZIIXTIMHKPUPCRYIJTHQGCSHWSVVPHIRMWDEKJMTLLCSBNDPGEDQRMQFFLLWNFSYTUABHTUEGHBGSVISOADCAQUKHPAKKLKNONYOZFYQZYODFCLWBXR");
    const char tmp_tmp_msg_0_0[] = {-33, 27, -30, -121, -8, -32, 64, -17, 96, 97, 63, -39, -50, -70, 89, -15, 67, 118, -118, 59, -75, 0, 96, -107, -43, -58, 55, -21, 125, 33, -3, -43, -104, -65, 1, 28, 37, -20, 113, 2, 98, 42, -119, 56, 92, 66, 49, 20, 40, -41, 78, 65, -105, -30, 121, 94, 58, -9, 39, -111, -22, -44, -105, -56, 48, 92, 122, -118, 22, -45, 120, -97, 118, 21, 65, -44, 123, -21, 117, 94, -89, -109, -12, -6, 12, -11, -36, -23, -42, 57, -52, -123, -52, -66, 11, 7, -31, -24, -109, 8, -7, -71, 91, 52, -125, -53, 87, -74, -120, 112, -84, -105, 50, 105, 111, 76, -35, 8, 123, -5, 27};
    tmp_msg_0.data.assign(tmp_tmp_msg_0_0, tmp_tmp_msg_0_0 + sizeof(tmp_tmp_msg_0_0));
    msg.message.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticMessage #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticMessage msg;
    msg.setTimeStamp(0.424020645815);
    msg.setSource(11267U);
    msg.setSourceEntity(150U);
    msg.setDestination(56347U);
    msg.setDestinationEntity(225U);
    IMC::DesiredHeading tmp_msg_0;
    tmp_msg_0.value = 0.817157795624;
    msg.message.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticMessage #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticDiagnostic msg;
    msg.setTimeStamp(0.185754000888);
    msg.setSource(22532U);
    msg.setSourceEntity(94U);
    msg.setDestination(16994U);
    msg.setDestinationEntity(238U);
    msg.enable = 163U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticDiagnostic #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticDiagnostic msg;
    msg.setTimeStamp(0.793355191089);
    msg.setSource(46051U);
    msg.setSourceEntity(137U);
    msg.setDestination(33945U);
    msg.setDestinationEntity(54U);
    msg.enable = 20U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticDiagnostic #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticDiagnostic msg;
    msg.setTimeStamp(0.435124717602);
    msg.setSource(52720U);
    msg.setSourceEntity(17U);
    msg.setDestination(19427U);
    msg.setDestinationEntity(154U);
    msg.enable = 72U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticDiagnostic #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticNoise msg;
    msg.setTimeStamp(0.401775891875);
    msg.setSource(26592U);
    msg.setSourceEntity(174U);
    msg.setDestination(7521U);
    msg.setDestinationEntity(0U);
    msg.summary = 79U;
    msg.level = 219U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticNoise #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticNoise msg;
    msg.setTimeStamp(0.572326757591);
    msg.setSource(32803U);
    msg.setSourceEntity(99U);
    msg.setDestination(19870U);
    msg.setDestinationEntity(249U);
    msg.summary = 188U;
    msg.level = 226U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticNoise #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticNoise msg;
    msg.setTimeStamp(0.598098474448);
    msg.setSource(30889U);
    msg.setSourceEntity(148U);
    msg.setDestination(33578U);
    msg.setDestinationEntity(251U);
    msg.summary = 123U;
    msg.level = 159U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticNoise #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticPing msg;
    msg.setTimeStamp(0.304517439894);
    msg.setSource(5349U);
    msg.setSourceEntity(172U);
    msg.setDestination(62082U);
    msg.setDestinationEntity(55U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticPing #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticPing msg;
    msg.setTimeStamp(0.782205666475);
    msg.setSource(61426U);
    msg.setSourceEntity(50U);
    msg.setDestination(1261U);
    msg.setDestinationEntity(222U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticPing #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticPing msg;
    msg.setTimeStamp(0.0182023245178);
    msg.setSource(35059U);
    msg.setSourceEntity(78U);
    msg.setDestination(4892U);
    msg.setDestinationEntity(253U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticPing #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticPingReply msg;
    msg.setTimeStamp(0.632964676636);
    msg.setSource(26595U);
    msg.setSourceEntity(12U);
    msg.setDestination(58064U);
    msg.setDestinationEntity(99U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticPingReply #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticPingReply msg;
    msg.setTimeStamp(0.813194423055);
    msg.setSource(51010U);
    msg.setSourceEntity(23U);
    msg.setDestination(34613U);
    msg.setDestinationEntity(176U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticPingReply #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticPingReply msg;
    msg.setTimeStamp(0.934630933606);
    msg.setSource(26489U);
    msg.setSourceEntity(107U);
    msg.setDestination(41432U);
    msg.setDestinationEntity(170U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticPingReply #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticOperation msg;
    msg.setTimeStamp(0.733956411517);
    msg.setSource(42524U);
    msg.setSourceEntity(210U);
    msg.setDestination(15821U);
    msg.setDestinationEntity(238U);
    msg.op = 45U;
    msg.system.assign("LIBZFYQTZLOSSFHGDLBLNRYIHGNEESMYGXPKCFAJOWUDOAMDCAKKRSUQMWUDYZAPTEXJKOYXWJMRFSJAWJWMUNRFPPBSJNHBXVPJXIZPEMRLTRACIYRSKTCGFIVVCBOSQBNGUVYNUKYLTHQHIDBUUPFKEQFZVKXALQVQHLDGJIEXCJXKFOXGDMTRAZVEYZICLTGSEOFWDTWVOUZQHOQPDLINUBMKBZAXWVJINTWCSATDEVQCHYGOHNNMBP");
    msg.range = 0.694601883599;
    IMC::EstimatedStreamVelocity tmp_msg_0;
    tmp_msg_0.x = 0.874111805437;
    tmp_msg_0.y = 0.279853336478;
    tmp_msg_0.z = 0.521959175225;
    msg.msg.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticOperation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticOperation msg;
    msg.setTimeStamp(0.0746030480528);
    msg.setSource(3663U);
    msg.setSourceEntity(238U);
    msg.setDestination(61529U);
    msg.setDestinationEntity(145U);
    msg.op = 33U;
    msg.system.assign("WCABQTXESXLEZMMOCUBXNIKTHUNNJDDLHGQLSGYJIPPFTVMYMHORIRYTDQOVMOZJZSEBCSSIKDEGJBLLGACUVBPVOKRQHLZFXGKFZDTANDQEKVFFJYUCTOAJYUIEMKOWUQMPYSGAWNWDCRGUHTPOXJGZRWKXYAVGBSPVVETAFHPZLFIHRAWOVMWUQ");
    msg.range = 0.453334813599;
    IMC::EntityInfo tmp_msg_0;
    tmp_msg_0.id = 171U;
    tmp_msg_0.label.assign("AQRIGQWQILWTMZJYHUGPWDFMQLQCNMTOEPIFBZIDNFDCRNVMHKWGEJYRILOLPOZHXTVJWPAMXTSJENPSDWFMBAVKYEMXZMIHSIIEHBTKFGSWBXDHQEUXJCOOPKL");
    tmp_msg_0.component.assign("HIGGTOZMPNJAXKHZFUJCCZGBXYFRCSZJMWKQWKCKMVEQQNRAEBCTWUVFSDEYSGJWODWOACEAWNRYHQSFMLGFWRAIDAEMNKUQDZBCIIWIOVVNALXJRKFPGBMOREGPRXIHBZKYPOJNEUTUQLDYJXLBPDDJDMOVEFHHEVJAHBNWFQTKUUPVVGODZQSBEYZCBFAUZHKTFLYBTXRHSTPSHIPLLYSMUNOTZC");
    tmp_msg_0.act_time = 2469U;
    tmp_msg_0.deact_time = 18408U;
    msg.msg.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticOperation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticOperation msg;
    msg.setTimeStamp(0.363491169419);
    msg.setSource(57382U);
    msg.setSourceEntity(58U);
    msg.setDestination(42130U);
    msg.setDestinationEntity(200U);
    msg.op = 97U;
    msg.system.assign("RWSOJWRBZYOHIUKPHWCNSSXJEOSZSQLUOKLUCBERQYLFYCVGXKUIRIZMOMKSJARSUNDLZCGPKPBUFVFWOQYEYVGQKLIPKXEXXBPCZMMLMXTZAMUZYNDGQTMTDIJFFXVVLGNYUNAOCU");
    msg.range = 0.0260855969329;
    IMC::EntityList tmp_msg_0;
    tmp_msg_0.op = 182U;
    tmp_msg_0.list.assign("HWZLUPYANKPREYMSEPRDTEMUJFPBBMVNBFKUADDOIUQXBABZEMXVFKQHLHUEDVBKQTWESCRCYFHLKZXTLXGIVGNINSQRJPGBHSPYTQHJSINJOVWNFPCYTOWOMI");
    msg.msg.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticOperation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticSystemsQuery msg;
    msg.setTimeStamp(0.463908550496);
    msg.setSource(64155U);
    msg.setSourceEntity(53U);
    msg.setDestination(62964U);
    msg.setDestinationEntity(13U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticSystemsQuery #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticSystemsQuery msg;
    msg.setTimeStamp(0.993060336841);
    msg.setSource(40346U);
    msg.setSourceEntity(169U);
    msg.setDestination(33014U);
    msg.setDestinationEntity(147U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticSystemsQuery #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticSystemsQuery msg;
    msg.setTimeStamp(0.831082851161);
    msg.setSource(15870U);
    msg.setSourceEntity(120U);
    msg.setDestination(37831U);
    msg.setDestinationEntity(94U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticSystemsQuery #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticSystems msg;
    msg.setTimeStamp(0.915514040693);
    msg.setSource(7174U);
    msg.setSourceEntity(186U);
    msg.setDestination(15815U);
    msg.setDestinationEntity(142U);
    msg.list.assign("LHPIRQIFDMNPXFHBTZLJGMBHBESRBMOKPKILRCEXXFVPRTKSNFGRLQNGUWFDBCEPDDJBOPAWXFMDIVNETXJISEAKZOSGWXOCAAKJZOMGWNHWXSGQMYNLVZNBUKDRWYTZOMJYHLEQOJEGYRVYUVTEUHTNGZBVRQYXVGJZWVQUFJQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticSystems #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticSystems msg;
    msg.setTimeStamp(0.682488157168);
    msg.setSource(35201U);
    msg.setSourceEntity(54U);
    msg.setDestination(49923U);
    msg.setDestinationEntity(204U);
    msg.list.assign("XGLZHJANJYTWQKRVSXNWHEHDGRZXNFEKNGJLLINRGHTJZPOCCUTUFJOENXADXWEVGIBCTQRGXZEFPKLAEDWAWVZEYCMJHHJCMFQGDLEOIJKMYVBMMYRHBAUGSVWTQU");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticSystems #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AcousticSystems msg;
    msg.setTimeStamp(0.333542879344);
    msg.setSource(37651U);
    msg.setSourceEntity(98U);
    msg.setDestination(47776U);
    msg.setDestinationEntity(100U);
    msg.list.assign("JSFWREKEGRFXWLYLTKPMTWDNSURIRIUTVGCOUONGRMLBMKQZZABNZTSHPKYNGEUMIKABLAVLARHMQOOJHKZBVZAUOHIYWUEKVPTOJSICHVWBAFXDATJDQOYCOYWGLIFHQMBFLMXQRVYRQPFZPSNXPVDJSTXQJHMNIEGGNECHPCDDYBPARUBQNCNTNZDFRYEJQEKWUIZCOVMJLGBDSFWUMTJVUGTAZCEWHKPSAELCVWXFCYDQJSIK");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AcousticSystems #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Rpm msg;
    msg.setTimeStamp(0.531943170927);
    msg.setSource(46176U);
    msg.setSourceEntity(51U);
    msg.setDestination(49619U);
    msg.setDestinationEntity(88U);
    msg.value = 23852;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Rpm #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Rpm msg;
    msg.setTimeStamp(0.489631525682);
    msg.setSource(57680U);
    msg.setSourceEntity(42U);
    msg.setDestination(30347U);
    msg.setDestinationEntity(105U);
    msg.value = -366;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Rpm #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Rpm msg;
    msg.setTimeStamp(0.310028311333);
    msg.setSource(65501U);
    msg.setSourceEntity(202U);
    msg.setDestination(3492U);
    msg.setDestinationEntity(74U);
    msg.value = -12987;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Rpm #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Voltage msg;
    msg.setTimeStamp(0.2752470981);
    msg.setSource(17219U);
    msg.setSourceEntity(192U);
    msg.setDestination(23576U);
    msg.setDestinationEntity(151U);
    msg.value = 0.572980539906;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Voltage #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Voltage msg;
    msg.setTimeStamp(0.800570515548);
    msg.setSource(24800U);
    msg.setSourceEntity(187U);
    msg.setDestination(28455U);
    msg.setDestinationEntity(155U);
    msg.value = 0.658004445695;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Voltage #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Voltage msg;
    msg.setTimeStamp(0.910348936287);
    msg.setSource(36602U);
    msg.setSourceEntity(4U);
    msg.setDestination(57101U);
    msg.setDestinationEntity(140U);
    msg.value = 0.881885626361;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Voltage #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Current msg;
    msg.setTimeStamp(0.706033820335);
    msg.setSource(24213U);
    msg.setSourceEntity(137U);
    msg.setDestination(11632U);
    msg.setDestinationEntity(249U);
    msg.value = 0.593782736944;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Current #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Current msg;
    msg.setTimeStamp(0.0305606638052);
    msg.setSource(30322U);
    msg.setSourceEntity(19U);
    msg.setDestination(7169U);
    msg.setDestinationEntity(23U);
    msg.value = 0.879427422744;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Current #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Current msg;
    msg.setTimeStamp(0.40234078042);
    msg.setSource(5067U);
    msg.setSourceEntity(19U);
    msg.setDestination(16971U);
    msg.setDestinationEntity(11U);
    msg.value = 0.0577130166273;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Current #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GpsFix msg;
    msg.setTimeStamp(0.620645771093);
    msg.setSource(21862U);
    msg.setSourceEntity(121U);
    msg.setDestination(55957U);
    msg.setDestinationEntity(250U);
    msg.validity = 21006U;
    msg.type = 113U;
    msg.utc_year = 336U;
    msg.utc_month = 252U;
    msg.utc_day = 50U;
    msg.utc_time = 0.994926545703;
    msg.lat = 0.43358927382;
    msg.lon = 0.0192585081616;
    msg.height = 0.973353642533;
    msg.satellites = 33U;
    msg.cog = 0.54601690798;
    msg.sog = 0.33568434927;
    msg.hdop = 0.391949576277;
    msg.vdop = 0.0545003221987;
    msg.hacc = 0.571910960922;
    msg.vacc = 0.993728221172;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GpsFix #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GpsFix msg;
    msg.setTimeStamp(0.496744076592);
    msg.setSource(52088U);
    msg.setSourceEntity(216U);
    msg.setDestination(2434U);
    msg.setDestinationEntity(65U);
    msg.validity = 64830U;
    msg.type = 229U;
    msg.utc_year = 58490U;
    msg.utc_month = 134U;
    msg.utc_day = 40U;
    msg.utc_time = 0.381910261298;
    msg.lat = 0.22102430478;
    msg.lon = 0.764860722837;
    msg.height = 0.0781014021134;
    msg.satellites = 208U;
    msg.cog = 0.611962742107;
    msg.sog = 0.224268205305;
    msg.hdop = 0.645356775562;
    msg.vdop = 0.922274473389;
    msg.hacc = 0.793055580469;
    msg.vacc = 0.58808974155;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GpsFix #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GpsFix msg;
    msg.setTimeStamp(0.354289568521);
    msg.setSource(528U);
    msg.setSourceEntity(158U);
    msg.setDestination(21446U);
    msg.setDestinationEntity(2U);
    msg.validity = 21299U;
    msg.type = 5U;
    msg.utc_year = 39858U;
    msg.utc_month = 50U;
    msg.utc_day = 193U;
    msg.utc_time = 0.934451300602;
    msg.lat = 0.35765541374;
    msg.lon = 0.991420777085;
    msg.height = 0.29426894088;
    msg.satellites = 254U;
    msg.cog = 0.910717202464;
    msg.sog = 0.491470255756;
    msg.hdop = 0.300337279912;
    msg.vdop = 0.388303270234;
    msg.hacc = 0.240276391181;
    msg.vacc = 0.548212737321;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GpsFix #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EulerAngles msg;
    msg.setTimeStamp(0.627811890842);
    msg.setSource(50704U);
    msg.setSourceEntity(236U);
    msg.setDestination(20257U);
    msg.setDestinationEntity(118U);
    msg.time = 0.637032061333;
    msg.phi = 0.138892770207;
    msg.theta = 0.211106038509;
    msg.psi = 0.103975040789;
    msg.psi_magnetic = 0.788468637936;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EulerAngles #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EulerAngles msg;
    msg.setTimeStamp(0.867903430143);
    msg.setSource(34796U);
    msg.setSourceEntity(90U);
    msg.setDestination(53546U);
    msg.setDestinationEntity(214U);
    msg.time = 0.361872378307;
    msg.phi = 0.440579129861;
    msg.theta = 0.273491052632;
    msg.psi = 0.201035451602;
    msg.psi_magnetic = 0.970428905703;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EulerAngles #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EulerAngles msg;
    msg.setTimeStamp(0.593939431287);
    msg.setSource(21742U);
    msg.setSourceEntity(83U);
    msg.setDestination(46215U);
    msg.setDestinationEntity(82U);
    msg.time = 0.488549311023;
    msg.phi = 0.0735027731527;
    msg.theta = 0.319200101309;
    msg.psi = 0.159438429212;
    msg.psi_magnetic = 0.604231507117;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EulerAngles #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EulerAnglesDelta msg;
    msg.setTimeStamp(0.600263195602);
    msg.setSource(15645U);
    msg.setSourceEntity(88U);
    msg.setDestination(29159U);
    msg.setDestinationEntity(219U);
    msg.time = 0.74666637987;
    msg.x = 0.746952943016;
    msg.y = 0.689337811866;
    msg.z = 0.533372421353;
    msg.timestep = 0.856720122723;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EulerAnglesDelta #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EulerAnglesDelta msg;
    msg.setTimeStamp(0.876169908694);
    msg.setSource(22132U);
    msg.setSourceEntity(163U);
    msg.setDestination(18518U);
    msg.setDestinationEntity(179U);
    msg.time = 0.811647005663;
    msg.x = 0.691001873874;
    msg.y = 0.567165141419;
    msg.z = 0.0267636330606;
    msg.timestep = 0.270513030283;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EulerAnglesDelta #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EulerAnglesDelta msg;
    msg.setTimeStamp(0.044036141058);
    msg.setSource(49096U);
    msg.setSourceEntity(138U);
    msg.setDestination(24971U);
    msg.setDestinationEntity(38U);
    msg.time = 0.802395609241;
    msg.x = 0.786950024706;
    msg.y = 0.709766401449;
    msg.z = 0.170876279595;
    msg.timestep = 0.160474422847;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EulerAnglesDelta #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AngularVelocity msg;
    msg.setTimeStamp(0.87714185433);
    msg.setSource(24257U);
    msg.setSourceEntity(99U);
    msg.setDestination(62245U);
    msg.setDestinationEntity(116U);
    msg.time = 0.111635955289;
    msg.x = 0.332435238061;
    msg.y = 0.197962015643;
    msg.z = 0.487539253724;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AngularVelocity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AngularVelocity msg;
    msg.setTimeStamp(0.952309810714);
    msg.setSource(35252U);
    msg.setSourceEntity(15U);
    msg.setDestination(22587U);
    msg.setDestinationEntity(82U);
    msg.time = 0.598219944206;
    msg.x = 0.804098587638;
    msg.y = 0.821405296414;
    msg.z = 0.15948927988;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AngularVelocity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AngularVelocity msg;
    msg.setTimeStamp(0.112925725509);
    msg.setSource(5950U);
    msg.setSourceEntity(216U);
    msg.setDestination(16188U);
    msg.setDestinationEntity(248U);
    msg.time = 0.138206472305;
    msg.x = 0.328978830944;
    msg.y = 0.725538662308;
    msg.z = 0.769513393567;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AngularVelocity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Acceleration msg;
    msg.setTimeStamp(0.787696007637);
    msg.setSource(27692U);
    msg.setSourceEntity(104U);
    msg.setDestination(38739U);
    msg.setDestinationEntity(134U);
    msg.time = 0.517699627498;
    msg.x = 0.784461759015;
    msg.y = 0.573597227459;
    msg.z = 0.793255132739;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Acceleration #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Acceleration msg;
    msg.setTimeStamp(0.694780797169);
    msg.setSource(38826U);
    msg.setSourceEntity(222U);
    msg.setDestination(735U);
    msg.setDestinationEntity(168U);
    msg.time = 0.693333046715;
    msg.x = 0.721351947989;
    msg.y = 0.891159129089;
    msg.z = 0.690190548601;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Acceleration #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Acceleration msg;
    msg.setTimeStamp(0.884946713031);
    msg.setSource(30448U);
    msg.setSourceEntity(66U);
    msg.setDestination(34259U);
    msg.setDestinationEntity(249U);
    msg.time = 0.929874157037;
    msg.x = 0.181934078549;
    msg.y = 0.0125049334311;
    msg.z = 0.867006401611;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Acceleration #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MagneticField msg;
    msg.setTimeStamp(0.0464245631794);
    msg.setSource(15098U);
    msg.setSourceEntity(241U);
    msg.setDestination(54715U);
    msg.setDestinationEntity(87U);
    msg.time = 0.703994032692;
    msg.x = 0.987307343578;
    msg.y = 0.224242358644;
    msg.z = 0.1811121673;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MagneticField #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MagneticField msg;
    msg.setTimeStamp(0.763742810792);
    msg.setSource(54746U);
    msg.setSourceEntity(146U);
    msg.setDestination(32792U);
    msg.setDestinationEntity(51U);
    msg.time = 0.897933444348;
    msg.x = 0.0561711497562;
    msg.y = 0.170427986569;
    msg.z = 0.763387156913;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MagneticField #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MagneticField msg;
    msg.setTimeStamp(0.135203152624);
    msg.setSource(58307U);
    msg.setSourceEntity(37U);
    msg.setDestination(55207U);
    msg.setDestinationEntity(186U);
    msg.time = 0.0363842209345;
    msg.x = 0.437820910993;
    msg.y = 0.677258357167;
    msg.z = 0.188946148019;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MagneticField #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GroundVelocity msg;
    msg.setTimeStamp(0.443091137243);
    msg.setSource(4729U);
    msg.setSourceEntity(250U);
    msg.setDestination(32871U);
    msg.setDestinationEntity(203U);
    msg.validity = 58U;
    msg.x = 0.175959351483;
    msg.y = 0.0692535454499;
    msg.z = 0.953970738629;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GroundVelocity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GroundVelocity msg;
    msg.setTimeStamp(0.00369695848523);
    msg.setSource(36828U);
    msg.setSourceEntity(140U);
    msg.setDestination(64065U);
    msg.setDestinationEntity(4U);
    msg.validity = 95U;
    msg.x = 0.656661496713;
    msg.y = 0.0226108297334;
    msg.z = 0.833868328016;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GroundVelocity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GroundVelocity msg;
    msg.setTimeStamp(0.838043782131);
    msg.setSource(16374U);
    msg.setSourceEntity(138U);
    msg.setDestination(17042U);
    msg.setDestinationEntity(81U);
    msg.validity = 41U;
    msg.x = 0.926788093744;
    msg.y = 0.955223519081;
    msg.z = 0.404787887917;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GroundVelocity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::WaterVelocity msg;
    msg.setTimeStamp(0.108714343914);
    msg.setSource(53600U);
    msg.setSourceEntity(74U);
    msg.setDestination(61704U);
    msg.setDestinationEntity(131U);
    msg.validity = 228U;
    msg.x = 0.707001308085;
    msg.y = 0.86713303859;
    msg.z = 0.02525210811;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("WaterVelocity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::WaterVelocity msg;
    msg.setTimeStamp(0.915269691908);
    msg.setSource(32450U);
    msg.setSourceEntity(198U);
    msg.setDestination(58344U);
    msg.setDestinationEntity(188U);
    msg.validity = 15U;
    msg.x = 0.415325807326;
    msg.y = 0.546461654099;
    msg.z = 0.157180991692;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("WaterVelocity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::WaterVelocity msg;
    msg.setTimeStamp(0.198433493196);
    msg.setSource(9635U);
    msg.setSourceEntity(129U);
    msg.setDestination(44768U);
    msg.setDestinationEntity(252U);
    msg.validity = 0U;
    msg.x = 0.580105238134;
    msg.y = 0.882372193619;
    msg.z = 0.857846809613;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("WaterVelocity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VelocityDelta msg;
    msg.setTimeStamp(0.65665760826);
    msg.setSource(44060U);
    msg.setSourceEntity(193U);
    msg.setDestination(31232U);
    msg.setDestinationEntity(247U);
    msg.time = 0.26817299867;
    msg.x = 0.808241800023;
    msg.y = 0.648985254617;
    msg.z = 0.199012425174;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VelocityDelta #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VelocityDelta msg;
    msg.setTimeStamp(0.529650019957);
    msg.setSource(11454U);
    msg.setSourceEntity(79U);
    msg.setDestination(40317U);
    msg.setDestinationEntity(2U);
    msg.time = 0.102935019733;
    msg.x = 0.0552475576853;
    msg.y = 0.358809186551;
    msg.z = 0.745144890691;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VelocityDelta #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VelocityDelta msg;
    msg.setTimeStamp(0.55894356132);
    msg.setSource(47780U);
    msg.setSourceEntity(120U);
    msg.setDestination(63163U);
    msg.setDestinationEntity(128U);
    msg.time = 0.884862588694;
    msg.x = 0.238744684338;
    msg.y = 0.110490689254;
    msg.z = 0.5310258334;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VelocityDelta #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Distance msg;
    msg.setTimeStamp(0.347809925647);
    msg.setSource(27628U);
    msg.setSourceEntity(59U);
    msg.setDestination(17759U);
    msg.setDestinationEntity(144U);
    msg.validity = 106U;
    IMC::BeamConfig tmp_msg_0;
    tmp_msg_0.beam_width = 0.773818318478;
    tmp_msg_0.beam_height = 0.596298415361;
    msg.beam_config.push_back(tmp_msg_0);
    msg.value = 0.670580798297;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Distance #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Distance msg;
    msg.setTimeStamp(0.469907033921);
    msg.setSource(31063U);
    msg.setSourceEntity(138U);
    msg.setDestination(63338U);
    msg.setDestinationEntity(41U);
    msg.validity = 45U;
    msg.value = 0.91357390617;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Distance #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Distance msg;
    msg.setTimeStamp(0.666346497253);
    msg.setSource(63627U);
    msg.setSourceEntity(223U);
    msg.setDestination(28984U);
    msg.setDestinationEntity(212U);
    msg.validity = 4U;
    IMC::BeamConfig tmp_msg_0;
    tmp_msg_0.beam_width = 0.823936928739;
    tmp_msg_0.beam_height = 0.490221821021;
    msg.beam_config.push_back(tmp_msg_0);
    msg.value = 0.861377269291;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Distance #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Temperature msg;
    msg.setTimeStamp(0.505021019676);
    msg.setSource(8001U);
    msg.setSourceEntity(118U);
    msg.setDestination(23972U);
    msg.setDestinationEntity(155U);
    msg.value = 0.727672280057;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Temperature #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Temperature msg;
    msg.setTimeStamp(0.865382748215);
    msg.setSource(5914U);
    msg.setSourceEntity(2U);
    msg.setDestination(25881U);
    msg.setDestinationEntity(241U);
    msg.value = 0.809147926155;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Temperature #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Temperature msg;
    msg.setTimeStamp(0.603031816919);
    msg.setSource(26768U);
    msg.setSourceEntity(11U);
    msg.setDestination(59119U);
    msg.setDestinationEntity(93U);
    msg.value = 0.720579988899;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Temperature #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Pressure msg;
    msg.setTimeStamp(0.703124730621);
    msg.setSource(48182U);
    msg.setSourceEntity(126U);
    msg.setDestination(35082U);
    msg.setDestinationEntity(164U);
    msg.value = 0.944166350428;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Pressure #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Pressure msg;
    msg.setTimeStamp(0.131760491623);
    msg.setSource(15712U);
    msg.setSourceEntity(191U);
    msg.setDestination(25238U);
    msg.setDestinationEntity(142U);
    msg.value = 0.0455005824037;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Pressure #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Pressure msg;
    msg.setTimeStamp(0.997782472971);
    msg.setSource(14545U);
    msg.setSourceEntity(50U);
    msg.setDestination(43351U);
    msg.setDestinationEntity(133U);
    msg.value = 0.171285890447;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Pressure #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Depth msg;
    msg.setTimeStamp(0.721572967662);
    msg.setSource(20195U);
    msg.setSourceEntity(67U);
    msg.setDestination(43500U);
    msg.setDestinationEntity(2U);
    msg.value = 0.426366350352;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Depth #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Depth msg;
    msg.setTimeStamp(0.0803263814045);
    msg.setSource(55437U);
    msg.setSourceEntity(53U);
    msg.setDestination(40274U);
    msg.setDestinationEntity(228U);
    msg.value = 0.308425093822;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Depth #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Depth msg;
    msg.setTimeStamp(0.835471962938);
    msg.setSource(56082U);
    msg.setSourceEntity(2U);
    msg.setDestination(57243U);
    msg.setDestinationEntity(134U);
    msg.value = 0.922694781717;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Depth #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DepthOffset msg;
    msg.setTimeStamp(0.718717110688);
    msg.setSource(40733U);
    msg.setSourceEntity(96U);
    msg.setDestination(24532U);
    msg.setDestinationEntity(140U);
    msg.value = 0.86729168127;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DepthOffset #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DepthOffset msg;
    msg.setTimeStamp(0.17872078809);
    msg.setSource(16043U);
    msg.setSourceEntity(116U);
    msg.setDestination(47728U);
    msg.setDestinationEntity(245U);
    msg.value = 0.68265130372;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DepthOffset #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DepthOffset msg;
    msg.setTimeStamp(0.472113980911);
    msg.setSource(40397U);
    msg.setSourceEntity(10U);
    msg.setDestination(16538U);
    msg.setDestinationEntity(153U);
    msg.value = 0.156097380982;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DepthOffset #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SoundSpeed msg;
    msg.setTimeStamp(0.61823107195);
    msg.setSource(26965U);
    msg.setSourceEntity(127U);
    msg.setDestination(6462U);
    msg.setDestinationEntity(122U);
    msg.value = 0.98626217049;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SoundSpeed #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SoundSpeed msg;
    msg.setTimeStamp(0.334216989651);
    msg.setSource(25224U);
    msg.setSourceEntity(178U);
    msg.setDestination(20817U);
    msg.setDestinationEntity(152U);
    msg.value = 0.882723566106;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SoundSpeed #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SoundSpeed msg;
    msg.setTimeStamp(0.417516912629);
    msg.setSource(28941U);
    msg.setSourceEntity(33U);
    msg.setDestination(52295U);
    msg.setDestinationEntity(52U);
    msg.value = 0.0152500837113;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SoundSpeed #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::WaterDensity msg;
    msg.setTimeStamp(0.607982939545);
    msg.setSource(40815U);
    msg.setSourceEntity(44U);
    msg.setDestination(50526U);
    msg.setDestinationEntity(103U);
    msg.value = 0.598700531092;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("WaterDensity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::WaterDensity msg;
    msg.setTimeStamp(0.0405318846325);
    msg.setSource(5738U);
    msg.setSourceEntity(224U);
    msg.setDestination(8486U);
    msg.setDestinationEntity(140U);
    msg.value = 0.402086266945;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("WaterDensity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::WaterDensity msg;
    msg.setTimeStamp(0.326554519447);
    msg.setSource(62727U);
    msg.setSourceEntity(98U);
    msg.setDestination(11850U);
    msg.setDestinationEntity(56U);
    msg.value = 0.0443573852617;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("WaterDensity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Conductivity msg;
    msg.setTimeStamp(0.0674119947256);
    msg.setSource(16434U);
    msg.setSourceEntity(188U);
    msg.setDestination(35003U);
    msg.setDestinationEntity(186U);
    msg.value = 0.310075686076;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Conductivity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Conductivity msg;
    msg.setTimeStamp(0.545651645476);
    msg.setSource(53472U);
    msg.setSourceEntity(171U);
    msg.setDestination(36886U);
    msg.setDestinationEntity(73U);
    msg.value = 0.193547525304;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Conductivity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Conductivity msg;
    msg.setTimeStamp(0.937687994647);
    msg.setSource(13696U);
    msg.setSourceEntity(134U);
    msg.setDestination(27459U);
    msg.setDestinationEntity(180U);
    msg.value = 0.379500430492;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Conductivity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Salinity msg;
    msg.setTimeStamp(0.304380076522);
    msg.setSource(15759U);
    msg.setSourceEntity(67U);
    msg.setDestination(25577U);
    msg.setDestinationEntity(181U);
    msg.value = 0.788535913219;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Salinity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Salinity msg;
    msg.setTimeStamp(0.154297041912);
    msg.setSource(28678U);
    msg.setSourceEntity(100U);
    msg.setDestination(35474U);
    msg.setDestinationEntity(243U);
    msg.value = 0.137825294158;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Salinity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Salinity msg;
    msg.setTimeStamp(0.0617521451967);
    msg.setSource(47334U);
    msg.setSourceEntity(73U);
    msg.setDestination(2727U);
    msg.setDestinationEntity(211U);
    msg.value = 0.493508612669;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Salinity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::WindSpeed msg;
    msg.setTimeStamp(0.540927766944);
    msg.setSource(14801U);
    msg.setSourceEntity(16U);
    msg.setDestination(24905U);
    msg.setDestinationEntity(180U);
    msg.direction = 0.986618860189;
    msg.speed = 0.818555070525;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("WindSpeed #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::WindSpeed msg;
    msg.setTimeStamp(0.63857514552);
    msg.setSource(32672U);
    msg.setSourceEntity(15U);
    msg.setDestination(51945U);
    msg.setDestinationEntity(113U);
    msg.direction = 0.405940623894;
    msg.speed = 0.84612693184;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("WindSpeed #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::WindSpeed msg;
    msg.setTimeStamp(0.121121429677);
    msg.setSource(37229U);
    msg.setSourceEntity(63U);
    msg.setDestination(40769U);
    msg.setDestinationEntity(171U);
    msg.direction = 0.306243808135;
    msg.speed = 0.0251719998135;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("WindSpeed #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RelativeHumidity msg;
    msg.setTimeStamp(0.261978907565);
    msg.setSource(4048U);
    msg.setSourceEntity(54U);
    msg.setDestination(48596U);
    msg.setDestinationEntity(121U);
    msg.value = 0.508686779677;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RelativeHumidity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RelativeHumidity msg;
    msg.setTimeStamp(0.73031663141);
    msg.setSource(21955U);
    msg.setSourceEntity(246U);
    msg.setDestination(40941U);
    msg.setDestinationEntity(210U);
    msg.value = 0.398088631319;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RelativeHumidity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RelativeHumidity msg;
    msg.setTimeStamp(0.705351482229);
    msg.setSource(25360U);
    msg.setSourceEntity(24U);
    msg.setDestination(49110U);
    msg.setDestinationEntity(100U);
    msg.value = 0.432762181037;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RelativeHumidity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevDataText msg;
    msg.setTimeStamp(0.0201642931168);
    msg.setSource(47851U);
    msg.setSourceEntity(24U);
    msg.setDestination(60468U);
    msg.setDestinationEntity(166U);
    msg.value.assign("SPCKYVTDACTWKYEBAWKNWHRDRJABFJNJTOHSP");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevDataText #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevDataText msg;
    msg.setTimeStamp(0.684730047523);
    msg.setSource(8473U);
    msg.setSourceEntity(212U);
    msg.setDestination(53025U);
    msg.setDestinationEntity(137U);
    msg.value.assign("NVTSNQALGPMPXWHLUQEWSDJSFKJHABACGIMUZ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevDataText #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevDataText msg;
    msg.setTimeStamp(0.360841942349);
    msg.setSource(6411U);
    msg.setSourceEntity(140U);
    msg.setDestination(26339U);
    msg.setDestinationEntity(126U);
    msg.value.assign("EIDKYAQOFABHNYULXLWAURQYWFWZOOHPZWHSJXUDOGKXPEGXTDJANYDFRPCPPCKAVGZPIMLWRHRVLUVTKSJNHGBLZBTYLANZMUMHXME");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevDataText #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevDataBinary msg;
    msg.setTimeStamp(0.701440199185);
    msg.setSource(16537U);
    msg.setSourceEntity(118U);
    msg.setDestination(34804U);
    msg.setDestinationEntity(109U);
    const char tmp_msg_0[] = {-122, -104, -48, 119, -63, -18, 82, 59, -85, -73, 92, 106, 116, -60, -24, -11, -73, -102, 44, 8, 18, -81, 11, 70, 3, 112, -60, 61, 110, -76};
    msg.value.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevDataBinary #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevDataBinary msg;
    msg.setTimeStamp(0.567911780203);
    msg.setSource(20418U);
    msg.setSourceEntity(166U);
    msg.setDestination(49274U);
    msg.setDestinationEntity(163U);
    const char tmp_msg_0[] = {122, 1, 112, 7, -47, 120, -87, 73, -90, -122, -101, 53, 85, -60, 34, -64, -56, -69, 106, 3, -17, 11, -56, 52, 95, -123, -34, -120, 2, 35, 84, 82, 126, 14, 48, -19, 38, -41, -96, 67, 121, -92, -107, -126, 60, 109, -76, -114, -97, 53, -65, -7, -73, 101, 98, 43, -101, 44, 108, 35, 86, -79, -128, -62, 6, 100, 49, 107, -29, 79, -66, 35, -5, 97, 85, 123, -49, -42, -21, -125, -8, -112, -105, -118, 23, 47, 54, 10, -5, 90, -74, 119, -87, -82, 84, -39, -126, -12, 78, 6, 41, 10, -60, -85, 125, 12, -47, -63, 32, 67, 76, -119, 108, 62, -22, 11, -74, 43, 45, -37, 117, 121, -111, -71, 74, -6, 58, 9, 43, -126, -60, -95, -39, -71, -33, 73, 67, 63, -63, -9, -38, 27, 89, -83, -116, -72, 18, -104};
    msg.value.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevDataBinary #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DevDataBinary msg;
    msg.setTimeStamp(0.339720125583);
    msg.setSource(3617U);
    msg.setSourceEntity(200U);
    msg.setDestination(63712U);
    msg.setDestinationEntity(213U);
    const char tmp_msg_0[] = {50, 30, -13, 80, -87, 114, -22, -128, 44, 29, -59, -110, -37, 13, -102, 38, -11, -29, -123, -111, -21, -84, 66, 30, 14, 53, -38, -21, -99, -90, -113, 103, -81, -95, 95, -24, 11, 50, -8, 122, -51, 6, 68, -72, 7, 70, -112, -116, 16, -17, 123, -60, 22, 35, 10, -123, -94, -123, 108, 90, -118, -118, -128, -103, 101, 72, 122, -12, -10, -86, 71, -101, -71, 32};
    msg.value.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DevDataBinary #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SonarConfig msg;
    msg.setTimeStamp(0.830775375784);
    msg.setSource(40036U);
    msg.setSourceEntity(94U);
    msg.setDestination(36857U);
    msg.setDestinationEntity(24U);
    msg.frequency = 1763381209U;
    msg.min_range = 34947U;
    msg.max_range = 43372U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SonarConfig #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SonarConfig msg;
    msg.setTimeStamp(0.701093486493);
    msg.setSource(61491U);
    msg.setSourceEntity(232U);
    msg.setDestination(18943U);
    msg.setDestinationEntity(238U);
    msg.frequency = 3472273902U;
    msg.min_range = 43792U;
    msg.max_range = 57014U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SonarConfig #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SonarConfig msg;
    msg.setTimeStamp(0.184629811386);
    msg.setSource(7346U);
    msg.setSourceEntity(186U);
    msg.setDestination(19399U);
    msg.setDestinationEntity(250U);
    msg.frequency = 2981352670U;
    msg.min_range = 61257U;
    msg.max_range = 48646U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SonarConfig #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SonarData msg;
    msg.setTimeStamp(0.526322543467);
    msg.setSource(44694U);
    msg.setSourceEntity(127U);
    msg.setDestination(16578U);
    msg.setDestinationEntity(217U);
    msg.type = 17U;
    msg.frequency = 3973636103U;
    msg.min_range = 25304U;
    msg.max_range = 59860U;
    msg.bits_per_point = 217U;
    msg.scale_factor = 0.584192203357;
    const char tmp_msg_0[] = {42, -109, 1, 10, -43, 35, 0, -6, -5, -33, 50, -51, -83, -33, 73, 71, 30, 53, 55, -102, -101, -41, 107, -75, -53, -67, 4, -109, 42, 16, 97, -104, -23, 115};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SonarData #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SonarData msg;
    msg.setTimeStamp(0.892760659928);
    msg.setSource(32395U);
    msg.setSourceEntity(138U);
    msg.setDestination(51944U);
    msg.setDestinationEntity(148U);
    msg.type = 144U;
    msg.frequency = 2272980624U;
    msg.min_range = 63192U;
    msg.max_range = 20022U;
    msg.bits_per_point = 65U;
    msg.scale_factor = 0.858647624856;
    const char tmp_msg_0[] = {-112, -76, -4, 98, -16, 26, 26, -54, -80, 47, -49, -2, -19, -49, -102, -91, -17, 105, 109, 53, 118, 28, 4, 39, 119, 111, 31, 49, 20, 83, 61, 56, 40, -71, -81, -81, -27, 63, 83, 54, 99, 95, -65, -32, -7, 85, 51, 107, 76, -80, -119, 37, -54, -21, -56, 87, -86, 6, -127, -19, 61, -124, -72, -57, 52, 8, -33, 11, 2, -40, -32, -8, 116, -49, -118, 32, -13, -120, 68, -37, -24, 17, 96, -46, -104, 15, 98, -84, 68, 89, -29, 120, -78, -105, 34, -18, -49, -18, -64, -111, -73, 44, -38, -48, -75, -70, -75, 54, -104, -111, 57, 38, -74, -104, -84, -117, -22, 7, 120, 65, -1, -64, -121, -36, 119, -71, 55, -16, 109, 0, -37, 74, 34, -102, 2, 4, -73, 96, 33, 27, 122, -43, 60, 49, -11, 23, -92, -115, 122, -78, 111, 41, -46, -43, 55, 113, -25, 31, -125, -22, -8, -39, -52, 112, -38, -10, 28, 63, 105, 52, 72, -11, 55, -66, 84, 66, -13, 15, 19, -113, -26, -77, 88, 91, 2, -48, 90, 116, -59, 125};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SonarData #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SonarData msg;
    msg.setTimeStamp(0.102320328715);
    msg.setSource(33476U);
    msg.setSourceEntity(88U);
    msg.setDestination(23087U);
    msg.setDestinationEntity(171U);
    msg.type = 188U;
    msg.frequency = 661307264U;
    msg.min_range = 12868U;
    msg.max_range = 33009U;
    msg.bits_per_point = 94U;
    msg.scale_factor = 0.41107967254;
    IMC::BeamConfig tmp_msg_0;
    tmp_msg_0.beam_width = 0.310167387976;
    tmp_msg_0.beam_height = 0.616221635192;
    msg.beam_config.push_back(tmp_msg_0);
    const char tmp_msg_1[] = {-110, 14, -68, 29, -108, -76, 87, -93, -99, 117, 20, 41, 92, 24, 126, -32, -67, -114, 83, -111, -91, -25, 47, -82, -116, 37, 4, -46, -48, 122, -27, 95, -47, -70, 100, -91, -123, -17, -55, 59, -47, -10, -42, 17, 47, -36, 62, -120, 88, -110, -88, -28, 62, 90, 49, 120, -29, -115, -10, 88, -110, 90, 108, -34, 16, -13, 22, 57, -50, -58, -85, 98, 49, 20, -20, -41, 1, -3, -16, -1, -127, -37, 28, -98, -27, -126, 43, 37, -101, -26, -19, 11, 97, -55, 58, 75, 12, 32, -123, -17, -67, -8, 45, -27, 118, 81, -39, 82, 25};
    msg.data.assign(tmp_msg_1, tmp_msg_1 + sizeof(tmp_msg_1));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SonarData #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Pulse msg;
    msg.setTimeStamp(0.2009986141);
    msg.setSource(18342U);
    msg.setSourceEntity(208U);
    msg.setDestination(10435U);
    msg.setDestinationEntity(196U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Pulse #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Pulse msg;
    msg.setTimeStamp(0.872926667554);
    msg.setSource(34489U);
    msg.setSourceEntity(29U);
    msg.setDestination(54861U);
    msg.setDestinationEntity(0U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Pulse #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Pulse msg;
    msg.setTimeStamp(0.523043064);
    msg.setSource(16568U);
    msg.setSourceEntity(163U);
    msg.setDestination(54359U);
    msg.setDestinationEntity(93U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Pulse #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PulseDetectionControl msg;
    msg.setTimeStamp(0.614884109088);
    msg.setSource(46145U);
    msg.setSourceEntity(20U);
    msg.setDestination(22254U);
    msg.setDestinationEntity(55U);
    msg.op = 64U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PulseDetectionControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PulseDetectionControl msg;
    msg.setTimeStamp(0.533880015854);
    msg.setSource(16018U);
    msg.setSourceEntity(126U);
    msg.setDestination(51530U);
    msg.setDestinationEntity(134U);
    msg.op = 144U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PulseDetectionControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PulseDetectionControl msg;
    msg.setTimeStamp(0.26236092871);
    msg.setSource(1589U);
    msg.setSourceEntity(53U);
    msg.setDestination(29058U);
    msg.setDestinationEntity(106U);
    msg.op = 201U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PulseDetectionControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FuelLevel msg;
    msg.setTimeStamp(0.251461773299);
    msg.setSource(24376U);
    msg.setSourceEntity(147U);
    msg.setDestination(36448U);
    msg.setDestinationEntity(155U);
    msg.value = 0.7402729365;
    msg.confidence = 0.357813408805;
    msg.opmodes.assign("NQVPXOMZZEUFKKSXBLTTYNHBOVCFLQBGMCIEJZZGVMJYLYAADUHUGPNISSHCAQNFRHACLXOVJMXUGYTHNVYFOMQJFBIXCETSRCEDBKRKGZQXGLEUWTVGOBKZBENKDEJUNDADXVNXKODUMLZEXIVANPTLKIDEIYWSLRJJCTACODRNJIGQUMFPSFZZGCWIWWLMDIPAQWMIOAYZEJPDOBTHQSG");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FuelLevel #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FuelLevel msg;
    msg.setTimeStamp(0.622803224245);
    msg.setSource(5387U);
    msg.setSourceEntity(139U);
    msg.setDestination(57644U);
    msg.setDestinationEntity(113U);
    msg.value = 0.00234249321833;
    msg.confidence = 0.633792469277;
    msg.opmodes.assign("ZBKUTDOMGGFYXEFCYUPKINZQRGSEZDCSFVCLIEOZKORGCTMEEORVUYWFXHVRLSVQNWQUPWOYCLXZJRATHHCGAFGEOKQWKVTEGTSILZQMHACIPTJIISVNAQNFGGBBKPQSIDCWM");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FuelLevel #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FuelLevel msg;
    msg.setTimeStamp(0.413405313998);
    msg.setSource(32176U);
    msg.setSourceEntity(212U);
    msg.setDestination(36131U);
    msg.setDestinationEntity(232U);
    msg.value = 0.639664034633;
    msg.confidence = 0.470765211802;
    msg.opmodes.assign("MQSCWBAIIEMPHELXRTNEYXCHRHNOLRNAGNJASSPMMUVSXPFVGSHBWHRGZKDSVKUJVXOYPCLXGLMLWVYMZONLGKWZHBQOQADHCERRQYSQVIXGFEPFQBN");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FuelLevel #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GpsNavData msg;
    msg.setTimeStamp(0.329151617863);
    msg.setSource(30466U);
    msg.setSourceEntity(16U);
    msg.setDestination(12693U);
    msg.setDestinationEntity(196U);
    msg.itow = 3130207757U;
    msg.lat = 0.669247402592;
    msg.lon = 0.716097264019;
    msg.height_ell = 0.0732195999058;
    msg.height_sea = 0.536979770258;
    msg.hacc = 0.39370873637;
    msg.vacc = 0.223080439939;
    msg.vel_n = 0.116634775811;
    msg.vel_e = 0.665679383261;
    msg.vel_d = 0.541854308427;
    msg.speed = 0.382370339369;
    msg.gspeed = 0.434876484575;
    msg.heading = 0.525935064835;
    msg.sacc = 0.591791328519;
    msg.cacc = 0.757290886673;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GpsNavData #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GpsNavData msg;
    msg.setTimeStamp(0.932251049885);
    msg.setSource(25982U);
    msg.setSourceEntity(66U);
    msg.setDestination(10430U);
    msg.setDestinationEntity(6U);
    msg.itow = 980982611U;
    msg.lat = 0.526868633302;
    msg.lon = 0.992739038767;
    msg.height_ell = 0.0619981161103;
    msg.height_sea = 0.31347637237;
    msg.hacc = 0.3845950443;
    msg.vacc = 0.684072394615;
    msg.vel_n = 0.751510893534;
    msg.vel_e = 0.27664873143;
    msg.vel_d = 0.622293934101;
    msg.speed = 0.0458807447872;
    msg.gspeed = 0.608200599909;
    msg.heading = 0.533655483947;
    msg.sacc = 0.408352575696;
    msg.cacc = 0.713248456832;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GpsNavData #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GpsNavData msg;
    msg.setTimeStamp(0.514632237676);
    msg.setSource(4338U);
    msg.setSourceEntity(13U);
    msg.setDestination(24073U);
    msg.setDestinationEntity(196U);
    msg.itow = 3497502190U;
    msg.lat = 0.237044823677;
    msg.lon = 0.520453687042;
    msg.height_ell = 0.021634008817;
    msg.height_sea = 0.847333837341;
    msg.hacc = 0.163893516561;
    msg.vacc = 0.887135719948;
    msg.vel_n = 0.446273114753;
    msg.vel_e = 0.0539987401723;
    msg.vel_d = 0.705100005201;
    msg.speed = 0.984729644938;
    msg.gspeed = 0.54782527491;
    msg.heading = 0.868497989221;
    msg.sacc = 0.870606181524;
    msg.cacc = 0.373192777894;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GpsNavData #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ServoPosition msg;
    msg.setTimeStamp(0.963849233848);
    msg.setSource(11443U);
    msg.setSourceEntity(185U);
    msg.setDestination(49933U);
    msg.setDestinationEntity(184U);
    msg.id = 132U;
    msg.value = 0.853035629749;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ServoPosition #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ServoPosition msg;
    msg.setTimeStamp(0.307936389471);
    msg.setSource(7292U);
    msg.setSourceEntity(66U);
    msg.setDestination(33796U);
    msg.setDestinationEntity(66U);
    msg.id = 211U;
    msg.value = 0.356474972528;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ServoPosition #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ServoPosition msg;
    msg.setTimeStamp(0.297854927739);
    msg.setSource(27015U);
    msg.setSourceEntity(251U);
    msg.setDestination(11205U);
    msg.setDestinationEntity(79U);
    msg.id = 192U;
    msg.value = 0.181195687416;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ServoPosition #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DeviceState msg;
    msg.setTimeStamp(0.670010297486);
    msg.setSource(54772U);
    msg.setSourceEntity(243U);
    msg.setDestination(38583U);
    msg.setDestinationEntity(221U);
    msg.x = 0.433761883441;
    msg.y = 0.142165634673;
    msg.z = 0.798481897306;
    msg.phi = 0.363087782185;
    msg.theta = 0.148998389441;
    msg.psi = 0.0229365574853;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DeviceState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DeviceState msg;
    msg.setTimeStamp(0.00136403498888);
    msg.setSource(27232U);
    msg.setSourceEntity(99U);
    msg.setDestination(61278U);
    msg.setDestinationEntity(101U);
    msg.x = 0.58124525966;
    msg.y = 0.921795320187;
    msg.z = 0.535887986826;
    msg.phi = 0.720160035753;
    msg.theta = 0.561442584777;
    msg.psi = 0.567468102422;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DeviceState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DeviceState msg;
    msg.setTimeStamp(0.597688658092);
    msg.setSource(65373U);
    msg.setSourceEntity(106U);
    msg.setDestination(6155U);
    msg.setDestinationEntity(12U);
    msg.x = 0.720145895799;
    msg.y = 0.303960332261;
    msg.z = 0.119630905893;
    msg.phi = 0.916475632878;
    msg.theta = 0.386440325883;
    msg.psi = 0.951744838985;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DeviceState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::BeamConfig msg;
    msg.setTimeStamp(0.499829592247);
    msg.setSource(18749U);
    msg.setSourceEntity(183U);
    msg.setDestination(40425U);
    msg.setDestinationEntity(73U);
    msg.beam_width = 0.873315691511;
    msg.beam_height = 0.962610518524;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("BeamConfig #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::BeamConfig msg;
    msg.setTimeStamp(0.264817687625);
    msg.setSource(10045U);
    msg.setSourceEntity(114U);
    msg.setDestination(4534U);
    msg.setDestinationEntity(217U);
    msg.beam_width = 0.295000663689;
    msg.beam_height = 0.574766436126;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("BeamConfig #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::BeamConfig msg;
    msg.setTimeStamp(0.0601983433519);
    msg.setSource(21202U);
    msg.setSourceEntity(73U);
    msg.setDestination(30924U);
    msg.setDestinationEntity(118U);
    msg.beam_width = 0.404674504742;
    msg.beam_height = 0.251420674572;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("BeamConfig #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DataSanity msg;
    msg.setTimeStamp(0.988331295455);
    msg.setSource(24571U);
    msg.setSourceEntity(130U);
    msg.setDestination(12453U);
    msg.setDestinationEntity(185U);
    msg.sane = 64U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DataSanity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DataSanity msg;
    msg.setTimeStamp(0.0825581684196);
    msg.setSource(50061U);
    msg.setSourceEntity(228U);
    msg.setDestination(28337U);
    msg.setDestinationEntity(90U);
    msg.sane = 112U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DataSanity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DataSanity msg;
    msg.setTimeStamp(0.38507246792);
    msg.setSource(46820U);
    msg.setSourceEntity(196U);
    msg.setDestination(52989U);
    msg.setDestinationEntity(105U);
    msg.sane = 109U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DataSanity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CameraZoom msg;
    msg.setTimeStamp(0.219304156911);
    msg.setSource(15103U);
    msg.setSourceEntity(65U);
    msg.setDestination(8999U);
    msg.setDestinationEntity(250U);
    msg.id = 122U;
    msg.zoom = 221U;
    msg.action = 74U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CameraZoom #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CameraZoom msg;
    msg.setTimeStamp(0.197429343022);
    msg.setSource(20995U);
    msg.setSourceEntity(222U);
    msg.setDestination(57172U);
    msg.setDestinationEntity(174U);
    msg.id = 156U;
    msg.zoom = 213U;
    msg.action = 173U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CameraZoom #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CameraZoom msg;
    msg.setTimeStamp(0.841033317969);
    msg.setSource(45864U);
    msg.setSourceEntity(205U);
    msg.setDestination(54855U);
    msg.setDestinationEntity(56U);
    msg.id = 232U;
    msg.zoom = 115U;
    msg.action = 224U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CameraZoom #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetThrusterActuation msg;
    msg.setTimeStamp(0.0334486061472);
    msg.setSource(40012U);
    msg.setSourceEntity(207U);
    msg.setDestination(43698U);
    msg.setDestinationEntity(209U);
    msg.id = 89U;
    msg.value = 0.373227542824;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetThrusterActuation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetThrusterActuation msg;
    msg.setTimeStamp(0.330926703179);
    msg.setSource(30676U);
    msg.setSourceEntity(107U);
    msg.setDestination(56894U);
    msg.setDestinationEntity(192U);
    msg.id = 68U;
    msg.value = 0.00687548621762;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetThrusterActuation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetThrusterActuation msg;
    msg.setTimeStamp(0.478611454773);
    msg.setSource(47860U);
    msg.setSourceEntity(174U);
    msg.setDestination(37817U);
    msg.setDestinationEntity(26U);
    msg.id = 173U;
    msg.value = 0.459448529594;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetThrusterActuation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetServoPosition msg;
    msg.setTimeStamp(0.0546345898692);
    msg.setSource(33336U);
    msg.setSourceEntity(244U);
    msg.setDestination(51394U);
    msg.setDestinationEntity(93U);
    msg.id = 168U;
    msg.value = 0.973263569277;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetServoPosition #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetServoPosition msg;
    msg.setTimeStamp(0.556740231399);
    msg.setSource(22599U);
    msg.setSourceEntity(38U);
    msg.setDestination(55666U);
    msg.setDestinationEntity(180U);
    msg.id = 94U;
    msg.value = 0.171616114043;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetServoPosition #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetServoPosition msg;
    msg.setTimeStamp(0.976513809793);
    msg.setSource(20911U);
    msg.setSourceEntity(199U);
    msg.setDestination(61498U);
    msg.setDestinationEntity(45U);
    msg.id = 14U;
    msg.value = 0.477441006264;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetServoPosition #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetControlSurfaceDeflection msg;
    msg.setTimeStamp(0.64171092172);
    msg.setSource(30090U);
    msg.setSourceEntity(101U);
    msg.setDestination(24167U);
    msg.setDestinationEntity(253U);
    msg.id = 71U;
    msg.angle = 0.771313180914;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetControlSurfaceDeflection #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetControlSurfaceDeflection msg;
    msg.setTimeStamp(0.636396376246);
    msg.setSource(56619U);
    msg.setSourceEntity(186U);
    msg.setDestination(23690U);
    msg.setDestinationEntity(120U);
    msg.id = 26U;
    msg.angle = 0.555813383071;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetControlSurfaceDeflection #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetControlSurfaceDeflection msg;
    msg.setTimeStamp(0.33699480581);
    msg.setSource(64224U);
    msg.setSourceEntity(89U);
    msg.setDestination(48487U);
    msg.setDestinationEntity(233U);
    msg.id = 183U;
    msg.angle = 0.97922512498;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetControlSurfaceDeflection #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteActionsRequest msg;
    msg.setTimeStamp(0.302501875279);
    msg.setSource(34625U);
    msg.setSourceEntity(8U);
    msg.setDestination(5354U);
    msg.setDestinationEntity(109U);
    msg.op = 222U;
    msg.actions.assign("CQFBMJDXHPNTOXRFJEARRSFGSGRATGBSDEOPTVUVIFJTYXKZZPSTUUCKYYWDNTHBFNOMMNTOCASOHZGEIUKLJTJPLIMAIAZEGBJBCBICICEVZBLGAHGOXHUEQWLWWOLSWKPSXXEFFUBZQYCWRNHAXBIIKVMDNPXUDYRGLRDTAYHEQYCIYKD");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteActionsRequest #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteActionsRequest msg;
    msg.setTimeStamp(0.442189380342);
    msg.setSource(41661U);
    msg.setSourceEntity(165U);
    msg.setDestination(19090U);
    msg.setDestinationEntity(196U);
    msg.op = 186U;
    msg.actions.assign("NYNZCBEFULBFGTATFCGWVGCGWPDBEUIRRZOJJRNKXTCBKPSPBMDZJHEEHMIMMFWUPAHVYTOHGTWLRAVMVZQLTHULNISIOHSKTF");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteActionsRequest #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteActionsRequest msg;
    msg.setTimeStamp(0.0494755544729);
    msg.setSource(48652U);
    msg.setSourceEntity(162U);
    msg.setDestination(9729U);
    msg.setDestinationEntity(43U);
    msg.op = 137U;
    msg.actions.assign("NOTEKFQBYRESPIINLIEMS");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteActionsRequest #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteActions msg;
    msg.setTimeStamp(0.294193288762);
    msg.setSource(2621U);
    msg.setSourceEntity(16U);
    msg.setDestination(52965U);
    msg.setDestinationEntity(187U);
    msg.actions.assign("YYMMSJWRDFWQSYDKGFBTKUOAXKTPNLCKXZZCKGEKLSBDXQNSQOYAOZJCDAXVLZFRIMCFKUUIVR");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteActions #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteActions msg;
    msg.setTimeStamp(0.657002007336);
    msg.setSource(53923U);
    msg.setSourceEntity(89U);
    msg.setDestination(40560U);
    msg.setDestinationEntity(66U);
    msg.actions.assign("MREBMGPDEXIFIKWGVVHZWHXHWDSQXNOXXNDMEKVQAFZREJKINOOAPNEUCYCRAMZGRFUPBHQJNBTYMPYZCYTOYSEBTXNKWLVLYGBRLSCOAKZHDAOYCE");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteActions #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteActions msg;
    msg.setTimeStamp(0.0286140278656);
    msg.setSource(58425U);
    msg.setSourceEntity(116U);
    msg.setDestination(61313U);
    msg.setDestinationEntity(50U);
    msg.actions.assign("BKTURSHESDFWSJXEXILHDZYRWOVQNAWQGOEXYBEEGMZZEBXGPFAQWTFGAZDNPSSVLLIMEGTBFGCTCFPXZVULVELNVIMATHUAIRXAUCOYXTADUKJQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteActions #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ButtonEvent msg;
    msg.setTimeStamp(0.179226937928);
    msg.setSource(51523U);
    msg.setSourceEntity(38U);
    msg.setDestination(21201U);
    msg.setDestinationEntity(179U);
    msg.button = 35U;
    msg.value = 251U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ButtonEvent #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ButtonEvent msg;
    msg.setTimeStamp(0.018214201138);
    msg.setSource(60309U);
    msg.setSourceEntity(182U);
    msg.setDestination(37385U);
    msg.setDestinationEntity(142U);
    msg.button = 23U;
    msg.value = 0U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ButtonEvent #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ButtonEvent msg;
    msg.setTimeStamp(0.311175408774);
    msg.setSource(16839U);
    msg.setSourceEntity(100U);
    msg.setDestination(49391U);
    msg.setDestinationEntity(3U);
    msg.button = 18U;
    msg.value = 2U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ButtonEvent #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LcdControl msg;
    msg.setTimeStamp(0.44911608155);
    msg.setSource(52416U);
    msg.setSourceEntity(15U);
    msg.setDestination(47406U);
    msg.setDestinationEntity(78U);
    msg.op = 29U;
    msg.text.assign("PESTYKNBAENTFIHRDXNCHFATGDMAIKUIEVXMKZFDXPNLNDKQQWUJQRUGBZFHCJOQBMTDKAPBKPLOTMHWUUIFYTOVGVBYPOEAMCNNVJCBWOFEMVCUILXFTZWHGYOHSEKBFCSJMCSIWWPXLRLHDJYZQXYHZEIZARQRBLXJSSDPOGILVBUJJAZSPFFVQHQANTLNALRCGDSWGYCZTPWWVCZEMYLAGXJWVMNXROBXZH");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LcdControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LcdControl msg;
    msg.setTimeStamp(0.100291438157);
    msg.setSource(8418U);
    msg.setSourceEntity(230U);
    msg.setDestination(20297U);
    msg.setDestinationEntity(48U);
    msg.op = 25U;
    msg.text.assign("UHZMMYBKTHKDINBJCJZVXEINMJTUXROGYKSMWBVOTKDGIXHUSFNZOAJKCYIAGFQGQBDWSFXDKAVHJEYPZLEARUZMWBPJXWMCBPIAOORHDDLAFOLIFKSQMMRESISEFKVWRCEDEMZILQOPN");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LcdControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LcdControl msg;
    msg.setTimeStamp(0.995576684657);
    msg.setSource(34871U);
    msg.setSourceEntity(111U);
    msg.setDestination(16842U);
    msg.setDestinationEntity(97U);
    msg.op = 171U;
    msg.text.assign("TSXLWRUEWZLGBBZNINUZSDTLESDYUUETCJBJSURFSANNVVDYXTMGGHESUGVVJQCOOEHJGEAAXTLTIXPYWSCQBZKJSRKOBOWEXKLPQBTQCSTPMVKIMKFQHPEFLNIQLYZWVZKAMHGWQNHCVAZEIAGHKLWNHOXCFVYDMIRHTJJPMRAFVHNVKGNWPZOXFRLDADPRUFA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LcdControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PowerOperation msg;
    msg.setTimeStamp(0.730021058483);
    msg.setSource(7521U);
    msg.setSourceEntity(240U);
    msg.setDestination(39089U);
    msg.setDestinationEntity(158U);
    msg.op = 148U;
    msg.time_remain = 0.00137753806376;
    msg.sched_time = 0.557273012977;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PowerOperation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PowerOperation msg;
    msg.setTimeStamp(0.536037743691);
    msg.setSource(9570U);
    msg.setSourceEntity(92U);
    msg.setDestination(9608U);
    msg.setDestinationEntity(48U);
    msg.op = 186U;
    msg.time_remain = 0.474640384092;
    msg.sched_time = 0.872071220092;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PowerOperation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PowerOperation msg;
    msg.setTimeStamp(0.0719530688949);
    msg.setSource(62563U);
    msg.setSourceEntity(108U);
    msg.setDestination(147U);
    msg.setDestinationEntity(190U);
    msg.op = 242U;
    msg.time_remain = 0.718978635417;
    msg.sched_time = 0.4376459811;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PowerOperation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PowerChannelControl msg;
    msg.setTimeStamp(0.0485605984104);
    msg.setSource(14846U);
    msg.setSourceEntity(93U);
    msg.setDestination(49109U);
    msg.setDestinationEntity(101U);
    msg.name.assign("SJVKFPBDUXDLJWYQEUPQTJAQQVDDVKOEHSIUMVGWRQADGQCDPHMVRNITYBZMHZGNYYUIRBGXKUQPWGAWWYTVJNTSSZHHLKLSCOJZCPNAVLIXCNJJZPXXEHETOBWFOLWODIVOHLOQABLNCSHZ");
    msg.op = 139U;
    msg.sched_time = 0.0176539477694;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PowerChannelControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PowerChannelControl msg;
    msg.setTimeStamp(0.222998489108);
    msg.setSource(50628U);
    msg.setSourceEntity(198U);
    msg.setDestination(43607U);
    msg.setDestinationEntity(165U);
    msg.name.assign("CZZTODHOEQWEOUPWSUBNWHCXSEIUROSJRYZLGVTGRSRBMHVTCYMBDBXEMHDWRNALUDNGJZQCTXPTIAHRQYJRLPLMYHCYNDPJAIGOVWUIHNZTJQESIADUAYQVCBFMTTOSXNZLVQVLFOXZBHPXMFIWYKQPARZCDCEGAGFXPGKMNWKKTKKJCUBVKZBKYINDUYAXVWBUPRAJOHDKPCMSJEGZEH");
    msg.op = 84U;
    msg.sched_time = 0.659545081006;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PowerChannelControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PowerChannelControl msg;
    msg.setTimeStamp(0.45371262244);
    msg.setSource(64583U);
    msg.setSourceEntity(191U);
    msg.setDestination(62949U);
    msg.setDestinationEntity(158U);
    msg.name.assign("XGEQWYTMMFAEBJHOKWVLEFPBCCDQARXAWKROMGSQGJCCXHSHCRHTXDDZXCAATEFSAIHVETLNMGKIBRCPDWIJZFQIRRERBJEIGYKHDYUJFLKGQSPSBVZVVNXHPZGOOBLMUPETHNEJYWFNYZJBYMNXDSQAAKJGCDTHIMZUMDWLKURIDGFCEUZYXMOGUSRKQPZQFOKKROJPSYLNYYVALQXSM");
    msg.op = 131U;
    msg.sched_time = 0.209214796164;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PowerChannelControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryPowerChannelState msg;
    msg.setTimeStamp(0.844899683128);
    msg.setSource(10473U);
    msg.setSourceEntity(236U);
    msg.setDestination(48055U);
    msg.setDestinationEntity(22U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryPowerChannelState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryPowerChannelState msg;
    msg.setTimeStamp(0.165802256335);
    msg.setSource(41876U);
    msg.setSourceEntity(228U);
    msg.setDestination(30676U);
    msg.setDestinationEntity(19U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryPowerChannelState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryPowerChannelState msg;
    msg.setTimeStamp(0.643086878185);
    msg.setSource(41718U);
    msg.setSourceEntity(57U);
    msg.setDestination(10358U);
    msg.setDestinationEntity(56U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryPowerChannelState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PowerChannelState msg;
    msg.setTimeStamp(0.97485999985);
    msg.setSource(14719U);
    msg.setSourceEntity(197U);
    msg.setDestination(50715U);
    msg.setDestinationEntity(22U);
    msg.name.assign("XSOXJQRQRYJWABCBEKDYDPPMCWTHRRCWYKVJAUFETKILCKUCGIDGYAVXZLRPNLHCAQOCMYRHYJJNDIYUOGHMUVFZKOPWRQGTJTCNUQLQFBWSBUQVBGWPAIIOSOPJLRHZLTUQFBMDESKUXXIAW");
    msg.state = 104U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PowerChannelState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PowerChannelState msg;
    msg.setTimeStamp(0.874075106558);
    msg.setSource(60962U);
    msg.setSourceEntity(234U);
    msg.setDestination(14457U);
    msg.setDestinationEntity(143U);
    msg.name.assign("DPLEUKCIIJWRRXZQQWKDXTUGOBOPCHPZFWZDWLXMOLBBFFNSEHTCOLYAUNYFEGEDPJJYRMPXQRNXZMWSBJJHTDSYNEYPYOAJHVIUVLGMOINNZWPSICMGTQMWUGJVFTONBPDSGTHAJRHPEEVXH");
    msg.state = 247U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PowerChannelState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PowerChannelState msg;
    msg.setTimeStamp(0.971890949528);
    msg.setSource(62685U);
    msg.setSourceEntity(109U);
    msg.setDestination(35453U);
    msg.setDestinationEntity(252U);
    msg.name.assign("PQGAILNPOXQGJFXU");
    msg.state = 199U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PowerChannelState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LedBrightness msg;
    msg.setTimeStamp(0.0818598361919);
    msg.setSource(12879U);
    msg.setSourceEntity(68U);
    msg.setDestination(55176U);
    msg.setDestinationEntity(221U);
    msg.name.assign("ANTJINRNFUQPMGIGWOGZMOGQYKTEWSFOHVFSUP");
    msg.value = 28U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LedBrightness #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LedBrightness msg;
    msg.setTimeStamp(0.150248039463);
    msg.setSource(6562U);
    msg.setSourceEntity(187U);
    msg.setDestination(5702U);
    msg.setDestinationEntity(205U);
    msg.name.assign("IXNWEOSKITCBXQQJEUBNODADLJUQYWGSZGSZRIKPZCECXHONPZPBGEDKBCADRTPWIAVYSDVNQCRG");
    msg.value = 154U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LedBrightness #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LedBrightness msg;
    msg.setTimeStamp(0.517343790278);
    msg.setSource(4213U);
    msg.setSourceEntity(1U);
    msg.setDestination(57310U);
    msg.setDestinationEntity(204U);
    msg.name.assign("QPFNODHVNCDULABWDFZXZIGDUWFJSJPTOWAXVBPTQVNHMOGAVVGGQXWJNGXWCWIKPMEYKCBMQRPJGNSGJXAWDZEIZUVFVODMQTLEIIHMRY");
    msg.value = 147U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LedBrightness #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryLedBrightness msg;
    msg.setTimeStamp(0.53005578108);
    msg.setSource(47617U);
    msg.setSourceEntity(123U);
    msg.setDestination(6251U);
    msg.setDestinationEntity(172U);
    msg.name.assign("TZWQQHPXBKMDBGMRLBVRB");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryLedBrightness #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryLedBrightness msg;
    msg.setTimeStamp(0.146996594417);
    msg.setSource(47212U);
    msg.setSourceEntity(25U);
    msg.setDestination(6375U);
    msg.setDestinationEntity(114U);
    msg.name.assign("EWTINQLOLIPYMHKZUSIFTHCYQYFBNRZONKSWJWJANURWJIFXMVIPTJCSEPUNPVAEDSESKXLYDEUDEJBBQSDRBODIQQSYGVCLXPCIGGIUBQOFCGKGAQUMFHVOLSWDHGJRWXICNPHRAGJTAZDBFLVCYCWKEYBPDZGUOLAVDZFYNANRHTTZM");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryLedBrightness #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryLedBrightness msg;
    msg.setTimeStamp(0.822475163995);
    msg.setSource(9258U);
    msg.setSourceEntity(22U);
    msg.setDestination(47542U);
    msg.setDestinationEntity(24U);
    msg.name.assign("NYEFZJKLWPTCUIZCBUZEHXKSOGBYMLTVYFWLQGMCCHCECDQLESVKBQPJRHFTTWDZOIAMWXJAEDECBJYVAFCGRDBPUQOLADXBJCMHRRLDAPIHFXWGZQYYBZBANUEHGNJPNBQGLTIFVNVUNGKVCGFPYRZHQVIPSRNHTSMYHTDKYR");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryLedBrightness #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetLedBrightness msg;
    msg.setTimeStamp(0.0933038244617);
    msg.setSource(31285U);
    msg.setSourceEntity(181U);
    msg.setDestination(19954U);
    msg.setDestinationEntity(206U);
    msg.name.assign("MXSLXSRQHGOWKYOHUQTCJWMDTQZCPECN");
    msg.value = 185U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetLedBrightness #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetLedBrightness msg;
    msg.setTimeStamp(0.174734179863);
    msg.setSource(34783U);
    msg.setSourceEntity(10U);
    msg.setDestination(33685U);
    msg.setDestinationEntity(88U);
    msg.name.assign("MOSQOQCKHSLWZTFYKPGMSJAICMLUJSZBGEVSRULEZPVGHBTIYWIIKTYPCFTGHNSPXUADAMEVDTBMJKQGRFPKDMUHJLKRXEAFNFYWPX");
    msg.value = 140U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetLedBrightness #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetLedBrightness msg;
    msg.setTimeStamp(0.853857635486);
    msg.setSource(30931U);
    msg.setSourceEntity(215U);
    msg.setDestination(59107U);
    msg.setDestinationEntity(171U);
    msg.name.assign("JEYWOEVHUK");
    msg.value = 168U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetLedBrightness #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetPWM msg;
    msg.setTimeStamp(0.261228695514);
    msg.setSource(51128U);
    msg.setSourceEntity(44U);
    msg.setDestination(34284U);
    msg.setDestinationEntity(228U);
    msg.id = 244U;
    msg.period = 2294053654U;
    msg.duty_cycle = 202558740U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetPWM #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetPWM msg;
    msg.setTimeStamp(0.398158637319);
    msg.setSource(19998U);
    msg.setSourceEntity(207U);
    msg.setDestination(7422U);
    msg.setDestinationEntity(69U);
    msg.id = 166U;
    msg.period = 457788400U;
    msg.duty_cycle = 1358528103U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetPWM #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetPWM msg;
    msg.setTimeStamp(0.20174903904);
    msg.setSource(33023U);
    msg.setSourceEntity(253U);
    msg.setDestination(16212U);
    msg.setDestinationEntity(152U);
    msg.id = 121U;
    msg.period = 3334826531U;
    msg.duty_cycle = 533098972U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetPWM #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PWM msg;
    msg.setTimeStamp(0.466742002557);
    msg.setSource(17286U);
    msg.setSourceEntity(14U);
    msg.setDestination(36434U);
    msg.setDestinationEntity(97U);
    msg.id = 5U;
    msg.period = 2381786842U;
    msg.duty_cycle = 2895659901U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PWM #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PWM msg;
    msg.setTimeStamp(0.295195844208);
    msg.setSource(42278U);
    msg.setSourceEntity(55U);
    msg.setDestination(9359U);
    msg.setDestinationEntity(245U);
    msg.id = 19U;
    msg.period = 860437288U;
    msg.duty_cycle = 497358669U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PWM #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PWM msg;
    msg.setTimeStamp(0.521864351268);
    msg.setSource(4458U);
    msg.setSourceEntity(200U);
    msg.setDestination(25401U);
    msg.setDestinationEntity(222U);
    msg.id = 240U;
    msg.period = 4244760754U;
    msg.duty_cycle = 3618696082U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PWM #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EstimatedState msg;
    msg.setTimeStamp(0.30880350475);
    msg.setSource(7658U);
    msg.setSourceEntity(192U);
    msg.setDestination(13373U);
    msg.setDestinationEntity(114U);
    msg.lat = 0.999139702521;
    msg.lon = 0.239151426488;
    msg.height = 0.40744083778;
    msg.x = 0.0319889321318;
    msg.y = 0.0785251978381;
    msg.z = 0.06073827143;
    msg.phi = 0.943626670729;
    msg.theta = 0.027527543444;
    msg.psi = 0.490520668332;
    msg.u = 0.685307152873;
    msg.v = 0.525887128467;
    msg.w = 0.298794438508;
    msg.vx = 0.311534403227;
    msg.vy = 0.0375893040207;
    msg.vz = 0.515440232248;
    msg.p = 0.257227660597;
    msg.q = 0.919410244248;
    msg.r = 0.181834493074;
    msg.depth = 0.718159625964;
    msg.alt = 0.843685608785;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EstimatedState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EstimatedState msg;
    msg.setTimeStamp(0.0489078587071);
    msg.setSource(41267U);
    msg.setSourceEntity(208U);
    msg.setDestination(62051U);
    msg.setDestinationEntity(70U);
    msg.lat = 0.852478601891;
    msg.lon = 0.834608098425;
    msg.height = 0.705942270574;
    msg.x = 0.712868158103;
    msg.y = 0.169034913782;
    msg.z = 0.65592379383;
    msg.phi = 0.707996269357;
    msg.theta = 0.0937002825889;
    msg.psi = 0.316032753049;
    msg.u = 0.355416373938;
    msg.v = 0.95698146002;
    msg.w = 0.54514254234;
    msg.vx = 0.569371244785;
    msg.vy = 0.23839543653;
    msg.vz = 0.295910280543;
    msg.p = 0.850431605486;
    msg.q = 0.492180006635;
    msg.r = 0.804730993323;
    msg.depth = 0.957855458387;
    msg.alt = 0.673707879088;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EstimatedState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EstimatedState msg;
    msg.setTimeStamp(0.786966600913);
    msg.setSource(38921U);
    msg.setSourceEntity(254U);
    msg.setDestination(44659U);
    msg.setDestinationEntity(97U);
    msg.lat = 0.0815704722849;
    msg.lon = 0.87450198375;
    msg.height = 0.0618993862937;
    msg.x = 0.797050337685;
    msg.y = 0.748634488475;
    msg.z = 0.574046141677;
    msg.phi = 0.708548415824;
    msg.theta = 0.966034576893;
    msg.psi = 0.70191532193;
    msg.u = 0.159622701876;
    msg.v = 0.0450691549223;
    msg.w = 0.523550878473;
    msg.vx = 0.283424901957;
    msg.vy = 0.0907635120953;
    msg.vz = 0.506769892768;
    msg.p = 0.626791748788;
    msg.q = 0.2879375428;
    msg.r = 0.554300782962;
    msg.depth = 0.0779028758318;
    msg.alt = 0.58439183942;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EstimatedState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EstimatedStreamVelocity msg;
    msg.setTimeStamp(0.0899397031043);
    msg.setSource(16002U);
    msg.setSourceEntity(83U);
    msg.setDestination(17490U);
    msg.setDestinationEntity(56U);
    msg.x = 0.888293894539;
    msg.y = 0.391351827725;
    msg.z = 0.0364299344915;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EstimatedStreamVelocity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EstimatedStreamVelocity msg;
    msg.setTimeStamp(0.506134490604);
    msg.setSource(60029U);
    msg.setSourceEntity(81U);
    msg.setDestination(34671U);
    msg.setDestinationEntity(29U);
    msg.x = 0.700231171089;
    msg.y = 0.306526295068;
    msg.z = 0.920885192243;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EstimatedStreamVelocity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EstimatedStreamVelocity msg;
    msg.setTimeStamp(0.935711303469);
    msg.setSource(915U);
    msg.setSourceEntity(213U);
    msg.setDestination(2565U);
    msg.setDestinationEntity(74U);
    msg.x = 0.176540692877;
    msg.y = 0.259760387232;
    msg.z = 0.447160159131;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EstimatedStreamVelocity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IndicatedSpeed msg;
    msg.setTimeStamp(0.721674188649);
    msg.setSource(24191U);
    msg.setSourceEntity(140U);
    msg.setDestination(35422U);
    msg.setDestinationEntity(246U);
    msg.value = 0.115086188166;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IndicatedSpeed #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IndicatedSpeed msg;
    msg.setTimeStamp(0.759684510074);
    msg.setSource(51956U);
    msg.setSourceEntity(36U);
    msg.setDestination(38043U);
    msg.setDestinationEntity(227U);
    msg.value = 0.268259540296;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IndicatedSpeed #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IndicatedSpeed msg;
    msg.setTimeStamp(0.480070889979);
    msg.setSource(43335U);
    msg.setSourceEntity(178U);
    msg.setDestination(38869U);
    msg.setDestinationEntity(107U);
    msg.value = 0.916536002589;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IndicatedSpeed #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrueSpeed msg;
    msg.setTimeStamp(0.630317371606);
    msg.setSource(35264U);
    msg.setSourceEntity(34U);
    msg.setDestination(49228U);
    msg.setDestinationEntity(138U);
    msg.value = 0.149839340792;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrueSpeed #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrueSpeed msg;
    msg.setTimeStamp(0.682193365038);
    msg.setSource(1904U);
    msg.setSourceEntity(90U);
    msg.setDestination(55479U);
    msg.setDestinationEntity(238U);
    msg.value = 0.778130793368;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrueSpeed #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrueSpeed msg;
    msg.setTimeStamp(0.265012969207);
    msg.setSource(9472U);
    msg.setSourceEntity(172U);
    msg.setDestination(41447U);
    msg.setDestinationEntity(151U);
    msg.value = 0.095596729359;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrueSpeed #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::NavigationUncertainty msg;
    msg.setTimeStamp(0.190962973665);
    msg.setSource(40049U);
    msg.setSourceEntity(53U);
    msg.setDestination(14963U);
    msg.setDestinationEntity(98U);
    msg.x = 0.448969863639;
    msg.y = 0.571770443664;
    msg.z = 0.825162946008;
    msg.phi = 0.731916005808;
    msg.theta = 0.11963170734;
    msg.psi = 0.590780754729;
    msg.p = 0.744101954132;
    msg.q = 0.509519958142;
    msg.r = 0.767891232721;
    msg.u = 0.973838997878;
    msg.v = 0.237872153311;
    msg.w = 0.416556540758;
    msg.bias_psi = 0.968830960915;
    msg.bias_r = 0.744634441718;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("NavigationUncertainty #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::NavigationUncertainty msg;
    msg.setTimeStamp(0.861818143248);
    msg.setSource(11420U);
    msg.setSourceEntity(73U);
    msg.setDestination(11914U);
    msg.setDestinationEntity(55U);
    msg.x = 0.476592968553;
    msg.y = 0.258039208685;
    msg.z = 0.63507467942;
    msg.phi = 0.217509300862;
    msg.theta = 0.513320816982;
    msg.psi = 0.442350536569;
    msg.p = 0.501445834237;
    msg.q = 0.887537944216;
    msg.r = 0.802457829262;
    msg.u = 0.686667053931;
    msg.v = 0.281840318165;
    msg.w = 0.816109679636;
    msg.bias_psi = 0.21005791266;
    msg.bias_r = 0.27490762066;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("NavigationUncertainty #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::NavigationUncertainty msg;
    msg.setTimeStamp(0.890386892303);
    msg.setSource(40381U);
    msg.setSourceEntity(61U);
    msg.setDestination(12319U);
    msg.setDestinationEntity(253U);
    msg.x = 0.0830157115276;
    msg.y = 0.413274847706;
    msg.z = 0.0779773025903;
    msg.phi = 0.810076386762;
    msg.theta = 0.952557852972;
    msg.psi = 0.462711108979;
    msg.p = 0.952166524426;
    msg.q = 0.697248142157;
    msg.r = 0.294630799092;
    msg.u = 0.867371724044;
    msg.v = 0.0993486736687;
    msg.w = 0.202459239164;
    msg.bias_psi = 0.357299752824;
    msg.bias_r = 0.540697559066;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("NavigationUncertainty #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::NavigationData msg;
    msg.setTimeStamp(0.823495544707);
    msg.setSource(64019U);
    msg.setSourceEntity(175U);
    msg.setDestination(25118U);
    msg.setDestinationEntity(242U);
    msg.bias_psi = 0.452549647245;
    msg.bias_r = 0.921544679109;
    msg.cog = 0.402018265441;
    msg.cyaw = 0.153888393506;
    msg.lbl_rej_level = 0.724298059658;
    msg.gps_rej_level = 0.79848929856;
    msg.custom_x = 0.189213138705;
    msg.custom_y = 0.771168227177;
    msg.custom_z = 0.460921092311;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("NavigationData #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::NavigationData msg;
    msg.setTimeStamp(0.107054919129);
    msg.setSource(37949U);
    msg.setSourceEntity(155U);
    msg.setDestination(224U);
    msg.setDestinationEntity(229U);
    msg.bias_psi = 0.0281839900472;
    msg.bias_r = 0.503709182984;
    msg.cog = 0.670087681552;
    msg.cyaw = 0.624951637592;
    msg.lbl_rej_level = 0.562756653857;
    msg.gps_rej_level = 0.324319308482;
    msg.custom_x = 0.0328672454827;
    msg.custom_y = 0.841140299467;
    msg.custom_z = 0.406691427746;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("NavigationData #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::NavigationData msg;
    msg.setTimeStamp(0.553579221436);
    msg.setSource(35695U);
    msg.setSourceEntity(1U);
    msg.setDestination(1548U);
    msg.setDestinationEntity(150U);
    msg.bias_psi = 0.0193279911882;
    msg.bias_r = 0.461452400601;
    msg.cog = 0.171012278548;
    msg.cyaw = 0.542604204127;
    msg.lbl_rej_level = 0.279629158597;
    msg.gps_rej_level = 0.727000486936;
    msg.custom_x = 0.607392104596;
    msg.custom_y = 0.59980367117;
    msg.custom_z = 0.269456155091;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("NavigationData #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GpsFixRejection msg;
    msg.setTimeStamp(0.770986117181);
    msg.setSource(44078U);
    msg.setSourceEntity(244U);
    msg.setDestination(45058U);
    msg.setDestinationEntity(56U);
    msg.utc_time = 0.851185806034;
    msg.reason = 191U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GpsFixRejection #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GpsFixRejection msg;
    msg.setTimeStamp(0.266422893042);
    msg.setSource(38347U);
    msg.setSourceEntity(124U);
    msg.setDestination(41831U);
    msg.setDestinationEntity(129U);
    msg.utc_time = 0.689687393934;
    msg.reason = 20U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GpsFixRejection #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GpsFixRejection msg;
    msg.setTimeStamp(0.888974564783);
    msg.setSource(1468U);
    msg.setSourceEntity(74U);
    msg.setDestination(15701U);
    msg.setDestinationEntity(187U);
    msg.utc_time = 0.171561227116;
    msg.reason = 105U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GpsFixRejection #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblRangeAcceptance msg;
    msg.setTimeStamp(0.729994226289);
    msg.setSource(2821U);
    msg.setSourceEntity(150U);
    msg.setDestination(32502U);
    msg.setDestinationEntity(37U);
    msg.id = 17U;
    msg.range = 0.515653008114;
    msg.acceptance = 140U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblRangeAcceptance #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblRangeAcceptance msg;
    msg.setTimeStamp(0.700827069821);
    msg.setSource(43023U);
    msg.setSourceEntity(96U);
    msg.setDestination(47193U);
    msg.setDestinationEntity(193U);
    msg.id = 42U;
    msg.range = 0.833074856613;
    msg.acceptance = 150U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblRangeAcceptance #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblRangeAcceptance msg;
    msg.setTimeStamp(0.206859181527);
    msg.setSource(31508U);
    msg.setSourceEntity(7U);
    msg.setDestination(47787U);
    msg.setDestinationEntity(37U);
    msg.id = 0U;
    msg.range = 0.0610064314026;
    msg.acceptance = 35U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblRangeAcceptance #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DvlRejection msg;
    msg.setTimeStamp(0.0503804242362);
    msg.setSource(63918U);
    msg.setSourceEntity(83U);
    msg.setDestination(29947U);
    msg.setDestinationEntity(34U);
    msg.type = 20U;
    msg.reason = 137U;
    msg.value = 0.28406890781;
    msg.timestep = 0.999286213759;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DvlRejection #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DvlRejection msg;
    msg.setTimeStamp(0.527078373454);
    msg.setSource(39899U);
    msg.setSourceEntity(56U);
    msg.setDestination(12392U);
    msg.setDestinationEntity(22U);
    msg.type = 34U;
    msg.reason = 252U;
    msg.value = 0.714262296584;
    msg.timestep = 0.623081554479;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DvlRejection #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DvlRejection msg;
    msg.setTimeStamp(0.29811542024);
    msg.setSource(34194U);
    msg.setSourceEntity(70U);
    msg.setDestination(57576U);
    msg.setDestinationEntity(233U);
    msg.type = 244U;
    msg.reason = 132U;
    msg.value = 0.2672197928;
    msg.timestep = 0.131161276382;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DvlRejection #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::NavigationReset msg;
    msg.setTimeStamp(0.772912681541);
    msg.setSource(17279U);
    msg.setSourceEntity(190U);
    msg.setDestination(31042U);
    msg.setDestinationEntity(207U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("NavigationReset #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::NavigationReset msg;
    msg.setTimeStamp(0.250984267528);
    msg.setSource(32795U);
    msg.setSourceEntity(45U);
    msg.setDestination(34103U);
    msg.setDestinationEntity(19U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("NavigationReset #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::NavigationReset msg;
    msg.setTimeStamp(0.515534231663);
    msg.setSource(39086U);
    msg.setSourceEntity(47U);
    msg.setDestination(35202U);
    msg.setDestinationEntity(65U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("NavigationReset #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblEstimate msg;
    msg.setTimeStamp(0.45147069365);
    msg.setSource(28639U);
    msg.setSourceEntity(176U);
    msg.setDestination(16241U);
    msg.setDestinationEntity(253U);
    IMC::LblBeacon tmp_msg_0;
    tmp_msg_0.beacon.assign("UCHZFIESPVIITLLETAQBZUCKNAWF");
    tmp_msg_0.lat = 0.608868784152;
    tmp_msg_0.lon = 0.325459914921;
    tmp_msg_0.depth = 0.277341712188;
    tmp_msg_0.query_channel = 86U;
    tmp_msg_0.reply_channel = 107U;
    tmp_msg_0.transponder_delay = 9U;
    msg.beacon.set(tmp_msg_0);
    msg.x = 0.908886244187;
    msg.y = 0.0282947156936;
    msg.var_x = 0.84512275739;
    msg.var_y = 0.235752182866;
    msg.distance = 0.573756666233;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblEstimate #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblEstimate msg;
    msg.setTimeStamp(0.125617315264);
    msg.setSource(48215U);
    msg.setSourceEntity(126U);
    msg.setDestination(1245U);
    msg.setDestinationEntity(16U);
    IMC::LblBeacon tmp_msg_0;
    tmp_msg_0.beacon.assign("VAKBBFASBRURCYZUDKZDGTXEAHVOWHGZAUMSQQCFIBIQZDTHGNRJTXCIRGTQRBWVMJMRPINZLQWSRKFRLYDWFWOUNJKQPCPJMMYFJXBSMOCONYLALGKEHIDBMYQVELVFJEXOOKCKUAAFVTYYVWPDEWEYHXLNQDITT");
    tmp_msg_0.lat = 0.248033187727;
    tmp_msg_0.lon = 0.910737180438;
    tmp_msg_0.depth = 0.0258864760875;
    tmp_msg_0.query_channel = 25U;
    tmp_msg_0.reply_channel = 9U;
    tmp_msg_0.transponder_delay = 93U;
    msg.beacon.set(tmp_msg_0);
    msg.x = 0.721628755659;
    msg.y = 0.491596576444;
    msg.var_x = 0.47744391131;
    msg.var_y = 0.830756885304;
    msg.distance = 0.302179784263;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblEstimate #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LblEstimate msg;
    msg.setTimeStamp(0.54119408944);
    msg.setSource(34504U);
    msg.setSourceEntity(87U);
    msg.setDestination(31162U);
    msg.setDestinationEntity(237U);
    IMC::LblBeacon tmp_msg_0;
    tmp_msg_0.beacon.assign("PMZGCPYULHRPRCZTAPACXGCTBXWHBVQDMZUOLUNR");
    tmp_msg_0.lat = 0.937874584862;
    tmp_msg_0.lon = 0.961676488459;
    tmp_msg_0.depth = 0.17039754778;
    tmp_msg_0.query_channel = 125U;
    tmp_msg_0.reply_channel = 15U;
    tmp_msg_0.transponder_delay = 174U;
    msg.beacon.set(tmp_msg_0);
    msg.x = 0.819850357466;
    msg.y = 0.517516226653;
    msg.var_x = 0.620817830693;
    msg.var_y = 0.41070456976;
    msg.distance = 0.676791853103;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LblEstimate #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AlignmentState msg;
    msg.setTimeStamp(0.62531066023);
    msg.setSource(2803U);
    msg.setSourceEntity(142U);
    msg.setDestination(26811U);
    msg.setDestinationEntity(16U);
    msg.state = 147U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AlignmentState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AlignmentState msg;
    msg.setTimeStamp(0.326882077692);
    msg.setSource(3811U);
    msg.setSourceEntity(241U);
    msg.setDestination(42519U);
    msg.setDestinationEntity(152U);
    msg.state = 85U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AlignmentState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AlignmentState msg;
    msg.setTimeStamp(0.669935344996);
    msg.setSource(32603U);
    msg.setSourceEntity(220U);
    msg.setDestination(56038U);
    msg.setDestinationEntity(186U);
    msg.state = 174U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AlignmentState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GroupStreamVelocity msg;
    msg.setTimeStamp(0.103490257956);
    msg.setSource(11319U);
    msg.setSourceEntity(138U);
    msg.setDestination(5012U);
    msg.setDestinationEntity(224U);
    msg.x = 0.027788758895;
    msg.y = 0.762967020268;
    msg.z = 0.860703413073;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GroupStreamVelocity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GroupStreamVelocity msg;
    msg.setTimeStamp(0.488119771857);
    msg.setSource(17734U);
    msg.setSourceEntity(226U);
    msg.setDestination(25723U);
    msg.setDestinationEntity(95U);
    msg.x = 0.108131073101;
    msg.y = 0.93703632093;
    msg.z = 0.606875238114;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GroupStreamVelocity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GroupStreamVelocity msg;
    msg.setTimeStamp(0.165717790051);
    msg.setSource(37568U);
    msg.setSourceEntity(6U);
    msg.setDestination(28231U);
    msg.setDestinationEntity(194U);
    msg.x = 0.139392722422;
    msg.y = 0.319725283704;
    msg.z = 0.886647885262;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GroupStreamVelocity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredHeading msg;
    msg.setTimeStamp(0.877748446908);
    msg.setSource(52841U);
    msg.setSourceEntity(135U);
    msg.setDestination(43463U);
    msg.setDestinationEntity(234U);
    msg.value = 0.897677144542;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredHeading #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredHeading msg;
    msg.setTimeStamp(0.745950277699);
    msg.setSource(16015U);
    msg.setSourceEntity(102U);
    msg.setDestination(56630U);
    msg.setDestinationEntity(40U);
    msg.value = 0.860265874378;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredHeading #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredHeading msg;
    msg.setTimeStamp(0.80494227289);
    msg.setSource(39404U);
    msg.setSourceEntity(220U);
    msg.setDestination(18551U);
    msg.setDestinationEntity(92U);
    msg.value = 0.657563426147;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredHeading #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredZ msg;
    msg.setTimeStamp(0.0146563644939);
    msg.setSource(26855U);
    msg.setSourceEntity(110U);
    msg.setDestination(25260U);
    msg.setDestinationEntity(88U);
    msg.value = 0.365771207796;
    msg.z_units = 250U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredZ #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredZ msg;
    msg.setTimeStamp(0.489361837249);
    msg.setSource(61027U);
    msg.setSourceEntity(12U);
    msg.setDestination(60400U);
    msg.setDestinationEntity(224U);
    msg.value = 0.548079858973;
    msg.z_units = 7U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredZ #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredZ msg;
    msg.setTimeStamp(0.292590346377);
    msg.setSource(45487U);
    msg.setSourceEntity(205U);
    msg.setDestination(51711U);
    msg.setDestinationEntity(151U);
    msg.value = 0.847053968056;
    msg.z_units = 19U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredZ #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredSpeed msg;
    msg.setTimeStamp(0.499491525166);
    msg.setSource(32354U);
    msg.setSourceEntity(102U);
    msg.setDestination(13774U);
    msg.setDestinationEntity(80U);
    msg.value = 0.0628777252638;
    msg.speed_units = 38U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredSpeed #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredSpeed msg;
    msg.setTimeStamp(0.179052691932);
    msg.setSource(29439U);
    msg.setSourceEntity(113U);
    msg.setDestination(481U);
    msg.setDestinationEntity(123U);
    msg.value = 0.125637937553;
    msg.speed_units = 175U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredSpeed #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredSpeed msg;
    msg.setTimeStamp(0.682681348464);
    msg.setSource(15302U);
    msg.setSourceEntity(59U);
    msg.setDestination(50342U);
    msg.setDestinationEntity(146U);
    msg.value = 0.864553596077;
    msg.speed_units = 17U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredSpeed #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredRoll msg;
    msg.setTimeStamp(0.688897496697);
    msg.setSource(33353U);
    msg.setSourceEntity(139U);
    msg.setDestination(61621U);
    msg.setDestinationEntity(253U);
    msg.value = 0.884310338421;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredRoll #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredRoll msg;
    msg.setTimeStamp(0.141978427762);
    msg.setSource(25480U);
    msg.setSourceEntity(111U);
    msg.setDestination(59503U);
    msg.setDestinationEntity(205U);
    msg.value = 0.155985451027;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredRoll #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredRoll msg;
    msg.setTimeStamp(0.621928012995);
    msg.setSource(61490U);
    msg.setSourceEntity(110U);
    msg.setDestination(40800U);
    msg.setDestinationEntity(92U);
    msg.value = 0.283473724123;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredRoll #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredPitch msg;
    msg.setTimeStamp(0.280228793803);
    msg.setSource(6913U);
    msg.setSourceEntity(161U);
    msg.setDestination(58252U);
    msg.setDestinationEntity(196U);
    msg.value = 0.537616359869;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredPitch #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredPitch msg;
    msg.setTimeStamp(0.218731372188);
    msg.setSource(9812U);
    msg.setSourceEntity(151U);
    msg.setDestination(39930U);
    msg.setDestinationEntity(45U);
    msg.value = 0.760438312999;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredPitch #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredPitch msg;
    msg.setTimeStamp(0.767770884649);
    msg.setSource(35932U);
    msg.setSourceEntity(103U);
    msg.setDestination(24567U);
    msg.setDestinationEntity(229U);
    msg.value = 0.937955247778;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredPitch #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredVerticalRate msg;
    msg.setTimeStamp(0.326490061437);
    msg.setSource(15241U);
    msg.setSourceEntity(43U);
    msg.setDestination(7194U);
    msg.setDestinationEntity(192U);
    msg.value = 0.226511546024;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredVerticalRate #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredVerticalRate msg;
    msg.setTimeStamp(0.332980545704);
    msg.setSource(51786U);
    msg.setSourceEntity(119U);
    msg.setDestination(47138U);
    msg.setDestinationEntity(219U);
    msg.value = 0.260369098967;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredVerticalRate #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredVerticalRate msg;
    msg.setTimeStamp(0.39397871593);
    msg.setSource(29406U);
    msg.setSourceEntity(121U);
    msg.setDestination(21423U);
    msg.setDestinationEntity(248U);
    msg.value = 0.106180806726;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredVerticalRate #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredPath msg;
    msg.setTimeStamp(0.772641382224);
    msg.setSource(8730U);
    msg.setSourceEntity(2U);
    msg.setDestination(20744U);
    msg.setDestinationEntity(243U);
    msg.start_lat = 0.979510599461;
    msg.start_lon = 0.974332507117;
    msg.start_z = 0.0122064264464;
    msg.start_z_units = 157U;
    msg.end_lat = 0.817648115399;
    msg.end_lon = 0.329157640359;
    msg.end_z = 0.133951670471;
    msg.end_z_units = 195U;
    msg.speed = 0.922286902643;
    msg.speed_units = 79U;
    msg.lradius = 0.425654705291;
    msg.flags = 172U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredPath #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredPath msg;
    msg.setTimeStamp(0.0719332458003);
    msg.setSource(21968U);
    msg.setSourceEntity(131U);
    msg.setDestination(46655U);
    msg.setDestinationEntity(51U);
    msg.start_lat = 0.999703791241;
    msg.start_lon = 0.647754387121;
    msg.start_z = 0.313522001224;
    msg.start_z_units = 4U;
    msg.end_lat = 0.388751866425;
    msg.end_lon = 0.646526435694;
    msg.end_z = 0.996873510353;
    msg.end_z_units = 118U;
    msg.speed = 0.119187369605;
    msg.speed_units = 201U;
    msg.lradius = 0.0748894835815;
    msg.flags = 161U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredPath #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredPath msg;
    msg.setTimeStamp(0.166165855033);
    msg.setSource(16502U);
    msg.setSourceEntity(218U);
    msg.setDestination(6156U);
    msg.setDestinationEntity(29U);
    msg.start_lat = 0.342426146542;
    msg.start_lon = 0.665184020733;
    msg.start_z = 0.0693579626266;
    msg.start_z_units = 167U;
    msg.end_lat = 0.400418864691;
    msg.end_lon = 0.183238366498;
    msg.end_z = 0.613298890749;
    msg.end_z_units = 221U;
    msg.speed = 0.64077598411;
    msg.speed_units = 251U;
    msg.lradius = 0.937009280849;
    msg.flags = 245U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredPath #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredControl msg;
    msg.setTimeStamp(0.991329822883);
    msg.setSource(32690U);
    msg.setSourceEntity(199U);
    msg.setDestination(50949U);
    msg.setDestinationEntity(96U);
    msg.x = 0.312649974324;
    msg.y = 0.245389037632;
    msg.z = 0.957218193746;
    msg.k = 0.0052458809897;
    msg.m = 0.169306537843;
    msg.n = 0.490975332512;
    msg.flags = 172U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredControl msg;
    msg.setTimeStamp(0.94957926816);
    msg.setSource(52981U);
    msg.setSourceEntity(250U);
    msg.setDestination(27299U);
    msg.setDestinationEntity(205U);
    msg.x = 0.940102526467;
    msg.y = 0.221264840904;
    msg.z = 0.905270105512;
    msg.k = 0.615361495727;
    msg.m = 0.423806109562;
    msg.n = 0.938079689328;
    msg.flags = 64U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredControl msg;
    msg.setTimeStamp(0.800190206637);
    msg.setSource(52361U);
    msg.setSourceEntity(43U);
    msg.setDestination(32428U);
    msg.setDestinationEntity(138U);
    msg.x = 0.565331102015;
    msg.y = 0.0830522576504;
    msg.z = 0.249031026352;
    msg.k = 0.692381824988;
    msg.m = 0.817407201219;
    msg.n = 0.702589542041;
    msg.flags = 15U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredHeadingRate msg;
    msg.setTimeStamp(0.597068096977);
    msg.setSource(44372U);
    msg.setSourceEntity(225U);
    msg.setDestination(20173U);
    msg.setDestinationEntity(95U);
    msg.value = 0.0785094869687;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredHeadingRate #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredHeadingRate msg;
    msg.setTimeStamp(0.457824740947);
    msg.setSource(61221U);
    msg.setSourceEntity(162U);
    msg.setDestination(63645U);
    msg.setDestinationEntity(196U);
    msg.value = 0.710421502065;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredHeadingRate #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredHeadingRate msg;
    msg.setTimeStamp(0.981672514495);
    msg.setSource(62177U);
    msg.setSourceEntity(86U);
    msg.setDestination(62557U);
    msg.setDestinationEntity(72U);
    msg.value = 0.11571892941;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredHeadingRate #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredVelocity msg;
    msg.setTimeStamp(0.718108104926);
    msg.setSource(19690U);
    msg.setSourceEntity(206U);
    msg.setDestination(29561U);
    msg.setDestinationEntity(97U);
    msg.u = 0.884847524202;
    msg.v = 0.654027693715;
    msg.w = 0.289304434127;
    msg.p = 0.940850955049;
    msg.q = 0.532224928121;
    msg.r = 0.177577508207;
    msg.flags = 137U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredVelocity #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredVelocity msg;
    msg.setTimeStamp(0.271366806145);
    msg.setSource(31055U);
    msg.setSourceEntity(79U);
    msg.setDestination(46298U);
    msg.setDestinationEntity(96U);
    msg.u = 0.160205947166;
    msg.v = 0.342108391238;
    msg.w = 0.877583182486;
    msg.p = 0.300213616456;
    msg.q = 0.374439415963;
    msg.r = 0.642556613377;
    msg.flags = 54U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredVelocity #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredVelocity msg;
    msg.setTimeStamp(0.904992328568);
    msg.setSource(22650U);
    msg.setSourceEntity(50U);
    msg.setDestination(29273U);
    msg.setDestinationEntity(135U);
    msg.u = 0.477290152612;
    msg.v = 0.526345955513;
    msg.w = 0.449590026003;
    msg.p = 0.102541628649;
    msg.q = 0.686767981724;
    msg.r = 0.472370451943;
    msg.flags = 64U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredVelocity #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PathControlState msg;
    msg.setTimeStamp(0.869685477868);
    msg.setSource(53702U);
    msg.setSourceEntity(90U);
    msg.setDestination(4572U);
    msg.setDestinationEntity(1U);
    msg.start_lat = 0.631374022627;
    msg.start_lon = 0.142325721459;
    msg.start_z = 0.869955786165;
    msg.start_z_units = 153U;
    msg.end_lat = 0.913102271106;
    msg.end_lon = 0.981051574955;
    msg.end_z = 0.521084216965;
    msg.end_z_units = 16U;
    msg.lradius = 0.0377293702033;
    msg.flags = 159U;
    msg.x = 0.330226674931;
    msg.y = 0.0112691355736;
    msg.z = 0.457429222159;
    msg.vx = 0.266817739382;
    msg.vy = 0.591630345753;
    msg.vz = 0.0664526509045;
    msg.course_error = 0.70906023651;
    msg.eta = 52230U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PathControlState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PathControlState msg;
    msg.setTimeStamp(0.731476768619);
    msg.setSource(48190U);
    msg.setSourceEntity(133U);
    msg.setDestination(39983U);
    msg.setDestinationEntity(160U);
    msg.start_lat = 0.869523172484;
    msg.start_lon = 0.201422767485;
    msg.start_z = 0.389924270994;
    msg.start_z_units = 167U;
    msg.end_lat = 0.495830414629;
    msg.end_lon = 0.734363133038;
    msg.end_z = 0.500294091183;
    msg.end_z_units = 188U;
    msg.lradius = 0.0738239152967;
    msg.flags = 19U;
    msg.x = 0.976310790824;
    msg.y = 0.703806756073;
    msg.z = 0.776928695042;
    msg.vx = 0.650382102097;
    msg.vy = 0.774413362684;
    msg.vz = 0.127911623845;
    msg.course_error = 0.0163693229857;
    msg.eta = 9994U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PathControlState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PathControlState msg;
    msg.setTimeStamp(0.0515439756217);
    msg.setSource(58671U);
    msg.setSourceEntity(87U);
    msg.setDestination(48634U);
    msg.setDestinationEntity(159U);
    msg.start_lat = 0.832795514957;
    msg.start_lon = 0.32360528223;
    msg.start_z = 0.132421345037;
    msg.start_z_units = 202U;
    msg.end_lat = 0.289469120839;
    msg.end_lon = 0.0378375073515;
    msg.end_z = 0.399546973086;
    msg.end_z_units = 121U;
    msg.lradius = 0.66536769187;
    msg.flags = 136U;
    msg.x = 0.978716116559;
    msg.y = 0.733568989876;
    msg.z = 0.773315951811;
    msg.vx = 0.568535199668;
    msg.vy = 0.632868005027;
    msg.vz = 0.751625662803;
    msg.course_error = 0.456960691396;
    msg.eta = 54506U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PathControlState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AllocatedControlTorques msg;
    msg.setTimeStamp(0.317429171627);
    msg.setSource(41119U);
    msg.setSourceEntity(126U);
    msg.setDestination(26441U);
    msg.setDestinationEntity(41U);
    msg.k = 0.14970132793;
    msg.m = 0.302140675275;
    msg.n = 0.842709532221;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AllocatedControlTorques #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AllocatedControlTorques msg;
    msg.setTimeStamp(0.0277363759714);
    msg.setSource(55094U);
    msg.setSourceEntity(35U);
    msg.setDestination(16435U);
    msg.setDestinationEntity(221U);
    msg.k = 0.127877749361;
    msg.m = 0.59679342599;
    msg.n = 0.0441765259446;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AllocatedControlTorques #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AllocatedControlTorques msg;
    msg.setTimeStamp(0.42891890983);
    msg.setSource(34407U);
    msg.setSourceEntity(124U);
    msg.setDestination(57808U);
    msg.setDestinationEntity(148U);
    msg.k = 0.394310230644;
    msg.m = 0.290872709805;
    msg.n = 0.277668247907;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AllocatedControlTorques #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ControlParcel msg;
    msg.setTimeStamp(0.116792211289);
    msg.setSource(17574U);
    msg.setSourceEntity(229U);
    msg.setDestination(64436U);
    msg.setDestinationEntity(234U);
    msg.p = 0.161981955954;
    msg.i = 0.779449041155;
    msg.d = 0.417035912818;
    msg.a = 0.329615284548;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ControlParcel #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ControlParcel msg;
    msg.setTimeStamp(0.696113050891);
    msg.setSource(14769U);
    msg.setSourceEntity(221U);
    msg.setDestination(2600U);
    msg.setDestinationEntity(76U);
    msg.p = 0.718854130128;
    msg.i = 0.310791473405;
    msg.d = 0.372919251198;
    msg.a = 0.243896034085;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ControlParcel #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ControlParcel msg;
    msg.setTimeStamp(0.746439730797);
    msg.setSource(64758U);
    msg.setSourceEntity(204U);
    msg.setDestination(1253U);
    msg.setDestinationEntity(144U);
    msg.p = 0.415181311432;
    msg.i = 0.917966605796;
    msg.d = 0.621783798601;
    msg.a = 0.761915756861;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ControlParcel #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Brake msg;
    msg.setTimeStamp(0.577013461641);
    msg.setSource(62861U);
    msg.setSourceEntity(201U);
    msg.setDestination(26685U);
    msg.setDestinationEntity(178U);
    msg.op = 109U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Brake #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Brake msg;
    msg.setTimeStamp(0.525532200746);
    msg.setSource(45043U);
    msg.setSourceEntity(252U);
    msg.setDestination(12887U);
    msg.setDestinationEntity(59U);
    msg.op = 57U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Brake #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Brake msg;
    msg.setTimeStamp(0.436677631378);
    msg.setSource(59411U);
    msg.setSourceEntity(14U);
    msg.setDestination(57566U);
    msg.setDestinationEntity(91U);
    msg.op = 49U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Brake #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredYaw msg;
    msg.setTimeStamp(0.595485762697);
    msg.setSource(32363U);
    msg.setSourceEntity(46U);
    msg.setDestination(42217U);
    msg.setDestinationEntity(145U);
    msg.value = 0.172858151171;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredYaw #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredYaw msg;
    msg.setTimeStamp(0.92131398792);
    msg.setSource(60395U);
    msg.setSourceEntity(50U);
    msg.setDestination(11682U);
    msg.setDestinationEntity(239U);
    msg.value = 0.40020357194;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredYaw #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DesiredYaw msg;
    msg.setTimeStamp(0.155918987366);
    msg.setSource(36898U);
    msg.setSourceEntity(224U);
    msg.setDestination(14336U);
    msg.setDestinationEntity(186U);
    msg.value = 0.724101946208;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DesiredYaw #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GuidanceControlState msg;
    msg.setTimeStamp(0.582295352318);
    msg.setSource(16238U);
    msg.setSourceEntity(241U);
    msg.setDestination(42231U);
    msg.setDestinationEntity(142U);
    msg.start_lat = 0.229241844938;
    msg.start_lon = 0.790868955574;
    msg.lradius = 0.953018117865;
    msg.flags = 88U;
    msg.eta = 25755U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GuidanceControlState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GuidanceControlState msg;
    msg.setTimeStamp(0.265655903249);
    msg.setSource(12312U);
    msg.setSourceEntity(30U);
    msg.setDestination(28829U);
    msg.setDestinationEntity(199U);
    msg.start_lat = 0.104750853004;
    msg.start_lon = 0.00727111445947;
    msg.lradius = 0.0505012332858;
    msg.flags = 16U;
    msg.eta = 64248U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GuidanceControlState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GuidanceControlState msg;
    msg.setTimeStamp(0.130175110674);
    msg.setSource(13138U);
    msg.setSourceEntity(97U);
    msg.setDestination(49891U);
    msg.setDestinationEntity(210U);
    msg.start_lat = 0.219892786484;
    msg.start_lon = 0.872687607815;
    msg.lradius = 0.768281366996;
    msg.flags = 143U;
    msg.eta = 2808U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GuidanceControlState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Goto msg;
    msg.setTimeStamp(0.989416641974);
    msg.setSource(2448U);
    msg.setSourceEntity(48U);
    msg.setDestination(41256U);
    msg.setDestinationEntity(231U);
    msg.timeout = 61841U;
    msg.lat = 0.695759343434;
    msg.lon = 0.436208058047;
    msg.z = 0.691361904446;
    msg.z_units = 224U;
    msg.speed = 0.762912247313;
    msg.speed_units = 154U;
    msg.roll = 0.927488156673;
    msg.pitch = 0.0916366800454;
    msg.yaw = 0.346570905108;
    msg.custom.assign("JQSPIAZDEUTSGHZVWXJURHRCIFJPFCIUFMLUBTTNXMWJDGQONTFPKRUYHVOLPXCBYNOVTZDDJFLCZMMGFROWQGMVOZCHREZUADPKONQHSSBLDYLIRWNUBHBIXSWIGRNVATEHKPXQWCOELKBEDNSJHXVHRIDAYADKJUPYBQKXELWZABPVIOKTIFZWASOENUTLLDGXQFXBMZPQAJVYFSWJUEERHJLWCTGMKMCASVNBXAOKMQCFMCGYEYSGGZQNV");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Goto #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Goto msg;
    msg.setTimeStamp(0.0211809441449);
    msg.setSource(47702U);
    msg.setSourceEntity(236U);
    msg.setDestination(57351U);
    msg.setDestinationEntity(100U);
    msg.timeout = 46456U;
    msg.lat = 0.0984172331474;
    msg.lon = 0.638190726456;
    msg.z = 0.22343848827;
    msg.z_units = 169U;
    msg.speed = 0.57508725723;
    msg.speed_units = 132U;
    msg.roll = 0.984424917461;
    msg.pitch = 0.0489240803348;
    msg.yaw = 0.767433130977;
    msg.custom.assign("QPTSIKMMYGORZJNXDBPYUFCGHRHQYCQRVLPYNLFYEXTDTGFSCFMDTAANQRQCSKTZXNXIEJDJNWBYBGWCVPEXAHJRKLQQGBGMPTONYUFLWDTUAWERNSZBVPOFMCUAZBNGXRNGMIBJFGZJLOJLSKYTKUIVHJITKIMLLSFOXMSEUKUXVBHPOHHHUURAJIOIZNVWHKBIDDIFRWSMXYPUCVCCRQEELAYZPFWOQAJSDMVKESVVBD");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Goto #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Goto msg;
    msg.setTimeStamp(0.307125039014);
    msg.setSource(26296U);
    msg.setSourceEntity(49U);
    msg.setDestination(13491U);
    msg.setDestinationEntity(246U);
    msg.timeout = 29012U;
    msg.lat = 0.54275029638;
    msg.lon = 0.870905778441;
    msg.z = 0.732551353309;
    msg.z_units = 13U;
    msg.speed = 0.826936273987;
    msg.speed_units = 59U;
    msg.roll = 0.982461881696;
    msg.pitch = 0.847962506142;
    msg.yaw = 0.252659191581;
    msg.custom.assign("EZRKFKNNISSDJCACOTEKFCVUYGMMNPJKEONIBTARRHOCJDFVUUBBLBGSDLDEMKGJWDGQAXVXZAFXYZORBPQIZNCFDLSMTWEUQRZEGHSSGXFPZTIIWIXLXNLPQCPMUQURWOJWHYTWGVTKSUWGKLYPMYHBYPKLENRBRBFOHWMHOVTFYVFQMPDNLLYSDXNRNXATAMTAQXC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Goto #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PopUp msg;
    msg.setTimeStamp(0.920509600704);
    msg.setSource(51667U);
    msg.setSourceEntity(82U);
    msg.setDestination(33577U);
    msg.setDestinationEntity(168U);
    msg.timeout = 20012U;
    msg.lat = 0.604761212981;
    msg.lon = 0.561511008042;
    msg.z = 0.0944037416387;
    msg.z_units = 142U;
    msg.speed = 0.419918912507;
    msg.speed_units = 175U;
    msg.duration = 19751U;
    msg.radius = 0.13220351234;
    msg.flags = 50U;
    msg.custom.assign("YHTQMIIOPKIRPTMUFWERTLKKMSYYZZNEOSCVTWBRTMAXIFQTYQHAPAJOWLSGDGUFKRIEDDJYXSWUKHQLMNLXBFAGTVHQSPGDAUBYWBFUNBWCCSMPUPVRBVBVZAPHDFOCKATNJCBHPQCYXJRJEJREAFLUQZGWKEEGZFOURJCLNPVTXVRJONUDYPDVEINMZYL");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PopUp #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PopUp msg;
    msg.setTimeStamp(0.361955470841);
    msg.setSource(22374U);
    msg.setSourceEntity(181U);
    msg.setDestination(8087U);
    msg.setDestinationEntity(54U);
    msg.timeout = 38896U;
    msg.lat = 0.221451439634;
    msg.lon = 0.383595562899;
    msg.z = 0.80979858393;
    msg.z_units = 65U;
    msg.speed = 0.511105095929;
    msg.speed_units = 95U;
    msg.duration = 64047U;
    msg.radius = 0.727750179826;
    msg.flags = 63U;
    msg.custom.assign("AHMADDJJECVGLPOBXLUIQVXURPZYJVBBVOOUMPGTERCOQGWLWLWPCMEMVIYTMZPGWRGHOKSGYBJIOALACUAEGHCYLIFTQWFXYYXUEXWKRKNDGQHCUDUITRINNXNZVHNFEJZXBFHLWSKYISXAXOVU");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PopUp #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PopUp msg;
    msg.setTimeStamp(0.5661410872);
    msg.setSource(62338U);
    msg.setSourceEntity(184U);
    msg.setDestination(27220U);
    msg.setDestinationEntity(105U);
    msg.timeout = 43879U;
    msg.lat = 0.324929703572;
    msg.lon = 0.257765806545;
    msg.z = 0.981614628723;
    msg.z_units = 70U;
    msg.speed = 0.634030869343;
    msg.speed_units = 190U;
    msg.duration = 36649U;
    msg.radius = 0.857320965584;
    msg.flags = 168U;
    msg.custom.assign("UPEFNPXNDSSZQEQBXOPZBNJAJRXVDTXORQYAHKGJZTCCQNVADKMUSKJVCPKMJUGMHMPERFIDRSZQKOXRWROKTFNVTWOTZTVMTSYQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PopUp #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Teleoperation msg;
    msg.setTimeStamp(0.133176402268);
    msg.setSource(18751U);
    msg.setSourceEntity(131U);
    msg.setDestination(53850U);
    msg.setDestinationEntity(92U);
    msg.custom.assign("CUEGIBEFDXZCAQJGAZOEVSSMWGPCLTIBZPSRMYWRVPELOWBNQXTJLULCTPKIMXQKP");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Teleoperation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Teleoperation msg;
    msg.setTimeStamp(0.839578460683);
    msg.setSource(36176U);
    msg.setSourceEntity(41U);
    msg.setDestination(15303U);
    msg.setDestinationEntity(56U);
    msg.custom.assign("JGSPQGPVAKSIAAWDKNTZSWTVZZWQQAYNPAFEAJBOQDNMSJYWKHNRYZEHRTKYZEIVPKMMEUETFOTIHLISDXFGCVQOOQDDDUURNHUMJLLQNXDCZCDJYEZUFPJVKAAXUCLAZBRXQUKWOMZDDFTBFAWEBGMOGIPGLTVXIFREXTBSCRXPERJXPXOLONFMVTWLEYBRNVMWSCHOUBBYXJIQHF");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Teleoperation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Teleoperation msg;
    msg.setTimeStamp(0.169823365985);
    msg.setSource(56927U);
    msg.setSourceEntity(15U);
    msg.setDestination(45370U);
    msg.setDestinationEntity(109U);
    msg.custom.assign("AEDZQJRMQSUVCIJNJISGUHRUFMIXOONM");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Teleoperation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Loiter msg;
    msg.setTimeStamp(0.0286171775869);
    msg.setSource(18892U);
    msg.setSourceEntity(80U);
    msg.setDestination(1516U);
    msg.setDestinationEntity(128U);
    msg.timeout = 59173U;
    msg.lat = 0.202344690371;
    msg.lon = 0.0541445854475;
    msg.z = 0.589763754338;
    msg.z_units = 26U;
    msg.duration = 37060U;
    msg.speed = 0.975292751537;
    msg.speed_units = 125U;
    msg.type = 46U;
    msg.radius = 0.611531377339;
    msg.length = 0.909013040934;
    msg.bearing = 0.0273042227395;
    msg.direction = 5U;
    msg.custom.assign("RHNCUXTOXEFIFKNFBNFLPQMNDSIPYTVBIUKTKXRAEODMOZMWWESYDJRJYKURGALJTFPYUSQLCSIW");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Loiter #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Loiter msg;
    msg.setTimeStamp(0.637023776396);
    msg.setSource(44198U);
    msg.setSourceEntity(91U);
    msg.setDestination(14072U);
    msg.setDestinationEntity(28U);
    msg.timeout = 42235U;
    msg.lat = 0.109715802475;
    msg.lon = 0.555808813055;
    msg.z = 0.881617228014;
    msg.z_units = 159U;
    msg.duration = 6669U;
    msg.speed = 0.884199610698;
    msg.speed_units = 50U;
    msg.type = 23U;
    msg.radius = 0.912120035883;
    msg.length = 0.0567046127877;
    msg.bearing = 0.088271507012;
    msg.direction = 22U;
    msg.custom.assign("KTBOUQDYPETYFFDWYCHRSBSNJMYIVRUMICDXXPXOWOHOXTWEYXNGWEMEVSZBPSFQOCKRNA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Loiter #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Loiter msg;
    msg.setTimeStamp(0.53867149626);
    msg.setSource(47317U);
    msg.setSourceEntity(128U);
    msg.setDestination(56030U);
    msg.setDestinationEntity(74U);
    msg.timeout = 63545U;
    msg.lat = 0.784610902863;
    msg.lon = 0.180619891771;
    msg.z = 0.127998764496;
    msg.z_units = 94U;
    msg.duration = 13218U;
    msg.speed = 0.446030645574;
    msg.speed_units = 0U;
    msg.type = 132U;
    msg.radius = 0.39263568487;
    msg.length = 0.619613446864;
    msg.bearing = 0.743453055678;
    msg.direction = 197U;
    msg.custom.assign("JNBJNLFUZWKDUOITLPYXPJLGVYLCNQRPVAQJNCOUPGBWCAOHDLMIBMZU");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Loiter #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IdleManeuver msg;
    msg.setTimeStamp(0.647799963122);
    msg.setSource(16167U);
    msg.setSourceEntity(227U);
    msg.setDestination(55816U);
    msg.setDestinationEntity(11U);
    msg.duration = 1828U;
    msg.custom.assign("CQALIXLNBWMLNUJPJUYWWFQFOEREICMDFFNFOQBYDNVRXTZMMMTPOVVDIMPPZEIJCUINZCYVJWQXZXVWFKXCUUQIAOSSSZVOHMDOCFPBE");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IdleManeuver #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IdleManeuver msg;
    msg.setTimeStamp(0.593583859279);
    msg.setSource(34688U);
    msg.setSourceEntity(124U);
    msg.setDestination(61512U);
    msg.setDestinationEntity(213U);
    msg.duration = 13392U;
    msg.custom.assign("PMIVZJHVQBWINNKXIEEMEEWMBLJHXWTEMRSHKBNWHXUDRNATEGIOLMOGYQDDRHIAKSUVNQXACRLURVLVFXEFIDHNOZCWNWCHMCGBDOYSMFKIVUFOFPTEKJNARTIPRDTZTBBUOTJKBLRSPQWAZRPGGJGLJTCMGLYQCSASPUUIFAGWWSZONTDXVJOUKOZPDJFRZKVLQEHLCWYHMMKBFSGYXZBFOSCQUJZYUKL");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IdleManeuver #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IdleManeuver msg;
    msg.setTimeStamp(0.103626613155);
    msg.setSource(23922U);
    msg.setSourceEntity(178U);
    msg.setDestination(29751U);
    msg.setDestinationEntity(41U);
    msg.duration = 5735U;
    msg.custom.assign("FHYEATXNQFGPFCXKHPQAONQWSXMEWXJWNPMZKDNTGUULGETNDRPKTQBWLJ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IdleManeuver #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LowLevelControl msg;
    msg.setTimeStamp(0.242222319184);
    msg.setSource(39299U);
    msg.setSourceEntity(235U);
    msg.setDestination(48318U);
    msg.setDestinationEntity(93U);
    IMC::DesiredPath tmp_msg_0;
    tmp_msg_0.start_lat = 0.216694772592;
    tmp_msg_0.start_lon = 0.313762742282;
    tmp_msg_0.start_z = 0.607125251637;
    tmp_msg_0.start_z_units = 7U;
    tmp_msg_0.end_lat = 0.240767451102;
    tmp_msg_0.end_lon = 0.210999162444;
    tmp_msg_0.end_z = 0.463047767098;
    tmp_msg_0.end_z_units = 105U;
    tmp_msg_0.speed = 0.960782719592;
    tmp_msg_0.speed_units = 5U;
    tmp_msg_0.lradius = 0.647092619824;
    tmp_msg_0.flags = 32U;
    msg.control.set(tmp_msg_0);
    msg.duration = 51648U;
    msg.custom.assign("FAJJUTUNXWGTEVNSWQMBVPXUBHOZDEIKRGKWZISEEPKHJACHQNMXQIDORDSUXWVYQLIMMSVWJLDAFCJUGTIVPJPMCKDBZWDTXOXOZVPGHCXOSKHFQFGOEPWTAMKFVFYBTKJKWRWSGUMEHHMAEDGEBLLTHYRCBWNYHFLQFYZNNLAYQOZJCLSUCFTGDNL");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LowLevelControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LowLevelControl msg;
    msg.setTimeStamp(0.244163817818);
    msg.setSource(43435U);
    msg.setSourceEntity(236U);
    msg.setDestination(23020U);
    msg.setDestinationEntity(184U);
    IMC::DesiredPath tmp_msg_0;
    tmp_msg_0.start_lat = 0.906220916865;
    tmp_msg_0.start_lon = 0.0352075745466;
    tmp_msg_0.start_z = 0.11591199466;
    tmp_msg_0.start_z_units = 250U;
    tmp_msg_0.end_lat = 0.938422789554;
    tmp_msg_0.end_lon = 0.841326530338;
    tmp_msg_0.end_z = 0.892065814721;
    tmp_msg_0.end_z_units = 174U;
    tmp_msg_0.speed = 0.0370994641478;
    tmp_msg_0.speed_units = 247U;
    tmp_msg_0.lradius = 0.424138589673;
    tmp_msg_0.flags = 91U;
    msg.control.set(tmp_msg_0);
    msg.duration = 52664U;
    msg.custom.assign("KWOQUPDZDXGYA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LowLevelControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LowLevelControl msg;
    msg.setTimeStamp(0.260256697243);
    msg.setSource(30544U);
    msg.setSourceEntity(137U);
    msg.setDestination(42095U);
    msg.setDestinationEntity(144U);
    IMC::DesiredZ tmp_msg_0;
    tmp_msg_0.value = 0.0647267296782;
    tmp_msg_0.z_units = 156U;
    msg.control.set(tmp_msg_0);
    msg.duration = 49260U;
    msg.custom.assign("DUCSOHNIVLWXVCAFTPBYSQVSGNZTTATXMARYCWHLIMMOFKVLDDIUHRQOVWGZTKIXHVJQFSAHZKTBYERYYOPAONMQPGNVCBUBETBGOHPKEZJYEWSOJZQWKALNYQFPZUMRKGBEXFRJLEXUEJVYUIGAGUYHJZUMQINLMIBPCKWHFYTJIIVNPSMSSDQZHWKOLDEPCEQW");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LowLevelControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Rows msg;
    msg.setTimeStamp(0.340250541972);
    msg.setSource(35370U);
    msg.setSourceEntity(170U);
    msg.setDestination(61214U);
    msg.setDestinationEntity(77U);
    msg.timeout = 41494U;
    msg.lat = 0.0909074042555;
    msg.lon = 0.395788822443;
    msg.z = 0.685084259224;
    msg.z_units = 252U;
    msg.speed = 0.721474095805;
    msg.speed_units = 199U;
    msg.bearing = 0.350294158822;
    msg.cross_angle = 0.21564602417;
    msg.width = 0.0524324012705;
    msg.length = 0.986765796222;
    msg.hstep = 0.774538138286;
    msg.coff = 103U;
    msg.alternation = 95U;
    msg.flags = 33U;
    msg.custom.assign("FBJOEILMAJUHPSIMSYZAWSYHZVSKXINPKWDXDTPLIYZQUAPJQBAJWKODOCSTFVCHLDKNXXWTVCMUQVFZTELKWDNBACPZZMRMZYRJWUBGTGXKDCLXZLFLHSEFPRTCYAVSFJYSJOPERGGNVFXGYNREJTICBQVPYJOHMQP");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Rows #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Rows msg;
    msg.setTimeStamp(0.147138551028);
    msg.setSource(9580U);
    msg.setSourceEntity(66U);
    msg.setDestination(2975U);
    msg.setDestinationEntity(129U);
    msg.timeout = 35027U;
    msg.lat = 0.750551365568;
    msg.lon = 0.457013475707;
    msg.z = 0.353857761336;
    msg.z_units = 176U;
    msg.speed = 0.811948994043;
    msg.speed_units = 184U;
    msg.bearing = 0.406820976893;
    msg.cross_angle = 0.854288141625;
    msg.width = 0.641656896886;
    msg.length = 0.764354916463;
    msg.hstep = 0.199083591519;
    msg.coff = 9U;
    msg.alternation = 139U;
    msg.flags = 201U;
    msg.custom.assign("RAJBSYYZGUAAJWNSJFJCXIHRYJLXROUPSGIMAEGTKJMLFQZPCMZIAPNKNMGOFFVMIDGCURYAHIRXSEQVVLFQKHOTQCEVNCNQBVUACGYIEUCELCKGJWQLXVIFOGCWNTTLDYHXRYPTMQXMWFRVUHSTZOPEHSJUBWETTYKNODZBQLSDAVRADUCVINXKMHNEKZZTWOBBDDWTZPPPIPGKMBSNERLGOSHZWKMHABORWLU");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Rows #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Rows msg;
    msg.setTimeStamp(0.448115190076);
    msg.setSource(7184U);
    msg.setSourceEntity(216U);
    msg.setDestination(6475U);
    msg.setDestinationEntity(254U);
    msg.timeout = 18453U;
    msg.lat = 0.466595477776;
    msg.lon = 0.394588926959;
    msg.z = 0.121092492438;
    msg.z_units = 36U;
    msg.speed = 0.10096140318;
    msg.speed_units = 89U;
    msg.bearing = 0.619294097971;
    msg.cross_angle = 0.493589601745;
    msg.width = 0.394838386153;
    msg.length = 0.293590611365;
    msg.hstep = 0.883891316738;
    msg.coff = 213U;
    msg.alternation = 152U;
    msg.flags = 242U;
    msg.custom.assign("LRZOOPYWBOYTQBTFWGCOSJNCACYQDNRWZIFRDITDHEPYHPSUTFHDRGMVAABQAYQUKNJCEXFBGACNMRKXPUHQTTGUBTJLDSNPZHCDJCOMZGGZVJHCKWUEYKUSPKFRTVLGPDXJSVPADFKWMLKKOWECVHJHVMDXUGIQGENRSXEXWZZEELJXSFQMCLQNIBBJHRILETFLVA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Rows #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowPath msg;
    msg.setTimeStamp(0.915391100039);
    msg.setSource(33091U);
    msg.setSourceEntity(193U);
    msg.setDestination(55501U);
    msg.setDestinationEntity(179U);
    msg.timeout = 26429U;
    msg.lat = 0.751567197098;
    msg.lon = 0.0520930876458;
    msg.z = 0.945531225421;
    msg.z_units = 119U;
    msg.speed = 0.908415426889;
    msg.speed_units = 109U;
    msg.custom.assign("AMHXKQMFHCPWQBRIOOWMXEUCQIJLRSWYVJXTRYKJNBFARXVMVFMPGTVABXNTGMOTBEJJNSHGKXIKHSFVEWTBVCTUBOQJNODADSXWIGKYSWAHSCLZKTHMLLZULXFMODBDPYIKGIOJBRCNRCAZPNIQLDVECAPGQEPZDHEPWTYLRWZSZUYUHAUDFDGY");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowPath #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowPath msg;
    msg.setTimeStamp(0.233336698482);
    msg.setSource(35044U);
    msg.setSourceEntity(140U);
    msg.setDestination(35462U);
    msg.setDestinationEntity(192U);
    msg.timeout = 26945U;
    msg.lat = 0.322294846298;
    msg.lon = 0.519852242349;
    msg.z = 0.174935359044;
    msg.z_units = 189U;
    msg.speed = 0.682612124081;
    msg.speed_units = 250U;
    IMC::PathPoint tmp_msg_0;
    tmp_msg_0.x = 0.968439217653;
    tmp_msg_0.y = 0.806385496633;
    tmp_msg_0.z = 0.0428839890128;
    msg.points.push_back(tmp_msg_0);
    msg.custom.assign("CBWGIMPDNTSUVYWXADGSILNVASOYLNGOEAVNCZAMSKEBDSNUMMQJKIQGRIREJGTXZCWNWWTMORVTMVIBBIDMGZPQFKEXFMASFWCGXHYGEKOJXYFEYLOUFRAUZHRKJDGBZXYMWHOLLHHSWVIQYPCKQUDFHEXKKJJNRCEPDDSYAFLBQFWBLIXBCIRZKJNQKIBFVOWSXXAVQOUHDPTJQY");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowPath #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowPath msg;
    msg.setTimeStamp(0.0203612363235);
    msg.setSource(30525U);
    msg.setSourceEntity(62U);
    msg.setDestination(15031U);
    msg.setDestinationEntity(36U);
    msg.timeout = 17878U;
    msg.lat = 0.229768376111;
    msg.lon = 0.248072785126;
    msg.z = 0.73903609683;
    msg.z_units = 79U;
    msg.speed = 0.530409866003;
    msg.speed_units = 202U;
    IMC::PathPoint tmp_msg_0;
    tmp_msg_0.x = 0.903991949964;
    tmp_msg_0.y = 0.901447850489;
    tmp_msg_0.z = 0.987962200367;
    msg.points.push_back(tmp_msg_0);
    msg.custom.assign("EKIUTDTSBVKCRUXCSSEFVXVGUIUBGDQUYHPQLZNHMCDEQMEFZAKHYYJGCMPZJSIOFGZWGQUVHRNELNRLXIKXMWWSCSHRXGFOWYCMKKPFWKVKQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowPath #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PathPoint msg;
    msg.setTimeStamp(0.301546801025);
    msg.setSource(59735U);
    msg.setSourceEntity(191U);
    msg.setDestination(33813U);
    msg.setDestinationEntity(87U);
    msg.x = 0.647754244963;
    msg.y = 0.551807614498;
    msg.z = 0.962115897111;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PathPoint #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PathPoint msg;
    msg.setTimeStamp(0.686052448458);
    msg.setSource(6559U);
    msg.setSourceEntity(92U);
    msg.setDestination(56526U);
    msg.setDestinationEntity(21U);
    msg.x = 0.475287568874;
    msg.y = 0.618251651163;
    msg.z = 0.646266541364;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PathPoint #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PathPoint msg;
    msg.setTimeStamp(0.957815130557);
    msg.setSource(61850U);
    msg.setSourceEntity(124U);
    msg.setDestination(46440U);
    msg.setDestinationEntity(230U);
    msg.x = 0.428821671687;
    msg.y = 0.145904838708;
    msg.z = 0.954599197423;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PathPoint #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::YoYo msg;
    msg.setTimeStamp(0.840756524038);
    msg.setSource(65336U);
    msg.setSourceEntity(252U);
    msg.setDestination(50121U);
    msg.setDestinationEntity(220U);
    msg.timeout = 32717U;
    msg.lat = 0.636896228693;
    msg.lon = 0.0522908757279;
    msg.z = 0.304697372957;
    msg.z_units = 85U;
    msg.amplitude = 0.683410816223;
    msg.pitch = 0.964232687153;
    msg.speed = 0.129541436045;
    msg.speed_units = 95U;
    msg.custom.assign("IKYGPSLXPAORAJWZRQJVXPIWBUMDSMCTINSLRLKBMHRCOKMVDHGCYXITTQFSWXNROTBPDEQLCBZYYNUOXJFVFIRHGPNWCXPIGWQSTIAXKNUKSSVSNOQUENRDHAFAIEFYEWNEGOCMJGGVFITEZOJZPHQZLABEFQRCDADJWHHHLGPXXBGKCYIPJUEENFHEZOTMAFUUX");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("YoYo #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::YoYo msg;
    msg.setTimeStamp(0.897597118133);
    msg.setSource(16701U);
    msg.setSourceEntity(188U);
    msg.setDestination(12420U);
    msg.setDestinationEntity(126U);
    msg.timeout = 36987U;
    msg.lat = 0.0899320729531;
    msg.lon = 0.687394511675;
    msg.z = 0.726541015459;
    msg.z_units = 92U;
    msg.amplitude = 0.780310004076;
    msg.pitch = 0.647206653888;
    msg.speed = 0.510576821275;
    msg.speed_units = 124U;
    msg.custom.assign("NORFSRCGBISMEBRM");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("YoYo #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::YoYo msg;
    msg.setTimeStamp(0.493867750171);
    msg.setSource(30986U);
    msg.setSourceEntity(108U);
    msg.setDestination(36265U);
    msg.setDestinationEntity(110U);
    msg.timeout = 36879U;
    msg.lat = 0.315209681112;
    msg.lon = 0.268968645917;
    msg.z = 0.528243930811;
    msg.z_units = 72U;
    msg.amplitude = 0.657682942981;
    msg.pitch = 0.71656055295;
    msg.speed = 0.984881902082;
    msg.speed_units = 82U;
    msg.custom.assign("GJUHDEFBCBLVSFVGPKCQLRUXZBNXJCBHWEZHDMBNRNLHQMUBLGZXJGPCWVIWYVYTAIZILQYYLDBCHFFTOWAKJNJKSAIIZFRXSGQMEREGXLWETQWEZVQPPHZOKAGHAUCDQFYUODEADBFNEOJFHZYJCZNAGGRDXAROVKQTRUFKPIKKWOMJMUQCSOISLTLUMSRKXDCTMXPPWONUBQZMVIKS");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("YoYo #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TeleoperationDone msg;
    msg.setTimeStamp(0.787912612034);
    msg.setSource(50592U);
    msg.setSourceEntity(40U);
    msg.setDestination(32560U);
    msg.setDestinationEntity(88U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TeleoperationDone #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TeleoperationDone msg;
    msg.setTimeStamp(0.307122516474);
    msg.setSource(27992U);
    msg.setSourceEntity(61U);
    msg.setDestination(9100U);
    msg.setDestinationEntity(178U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TeleoperationDone #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TeleoperationDone msg;
    msg.setTimeStamp(0.916080174016);
    msg.setSource(32749U);
    msg.setSourceEntity(160U);
    msg.setDestination(16604U);
    msg.setDestinationEntity(253U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TeleoperationDone #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::StationKeeping msg;
    msg.setTimeStamp(0.467014256213);
    msg.setSource(16239U);
    msg.setSourceEntity(142U);
    msg.setDestination(35378U);
    msg.setDestinationEntity(8U);
    msg.lat = 0.347201561352;
    msg.lon = 0.985254287041;
    msg.z = 0.489042550672;
    msg.z_units = 228U;
    msg.radius = 0.39693775762;
    msg.duration = 63312U;
    msg.speed = 0.598056727956;
    msg.speed_units = 153U;
    msg.custom.assign("MUGEPPHEQKUNVXTZBVKZUEPJVQSLZXZHNNJLYGRONFDFIBWTNUHMOUSXARABZHBLIYUDIDXJWDPEVNQHSGKIFAHQBVMOKVTAKCMRZIRWESOEWLFDQXSIDOYUOKMJIHIQJWFCPDCYFAOAYLLY");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("StationKeeping #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::StationKeeping msg;
    msg.setTimeStamp(0.0906606341338);
    msg.setSource(48306U);
    msg.setSourceEntity(89U);
    msg.setDestination(46403U);
    msg.setDestinationEntity(92U);
    msg.lat = 0.788373310942;
    msg.lon = 0.268755084998;
    msg.z = 0.43764157378;
    msg.z_units = 33U;
    msg.radius = 0.695369052259;
    msg.duration = 41004U;
    msg.speed = 0.698931350965;
    msg.speed_units = 120U;
    msg.custom.assign("LCCEFNCTXHHMNYLGHNODVQZOQXJYJFRICULLJUDYEBARHOMRBQUMPWFSPCVBKFUFTETZTKTZISEAVQZKFWT");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("StationKeeping #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::StationKeeping msg;
    msg.setTimeStamp(0.275194125978);
    msg.setSource(35444U);
    msg.setSourceEntity(148U);
    msg.setDestination(57952U);
    msg.setDestinationEntity(225U);
    msg.lat = 0.292669482251;
    msg.lon = 0.178886196076;
    msg.z = 0.257766722008;
    msg.z_units = 119U;
    msg.radius = 0.422717912349;
    msg.duration = 56436U;
    msg.speed = 0.951009058839;
    msg.speed_units = 190U;
    msg.custom.assign("WTMXGJQJNCDLZMLJDATPAHFKOEZPLKSVQQAMCORZFYJVSMERMQHBIPSDKL");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("StationKeeping #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Elevator msg;
    msg.setTimeStamp(0.947412092582);
    msg.setSource(31252U);
    msg.setSourceEntity(65U);
    msg.setDestination(32802U);
    msg.setDestinationEntity(79U);
    msg.timeout = 35969U;
    msg.flags = 27U;
    msg.lat = 0.216328460156;
    msg.lon = 0.897387290899;
    msg.start_z = 0.4837312672;
    msg.start_z_units = 8U;
    msg.end_z = 0.731628154184;
    msg.end_z_units = 149U;
    msg.radius = 0.66208421704;
    msg.speed = 0.551045668612;
    msg.speed_units = 133U;
    msg.custom.assign("ZPDDGBFZFRXLBQZOIRWSSHUMDPEV");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Elevator #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Elevator msg;
    msg.setTimeStamp(0.65355798616);
    msg.setSource(46724U);
    msg.setSourceEntity(224U);
    msg.setDestination(42775U);
    msg.setDestinationEntity(221U);
    msg.timeout = 3306U;
    msg.flags = 66U;
    msg.lat = 0.801405335754;
    msg.lon = 0.0744543180224;
    msg.start_z = 0.342659808564;
    msg.start_z_units = 42U;
    msg.end_z = 0.526739635461;
    msg.end_z_units = 125U;
    msg.radius = 0.0250825804402;
    msg.speed = 0.185826794123;
    msg.speed_units = 249U;
    msg.custom.assign("XVZWQNARWCATKM");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Elevator #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Elevator msg;
    msg.setTimeStamp(0.789798370222);
    msg.setSource(55994U);
    msg.setSourceEntity(123U);
    msg.setDestination(13097U);
    msg.setDestinationEntity(94U);
    msg.timeout = 60397U;
    msg.flags = 108U;
    msg.lat = 0.779775781493;
    msg.lon = 0.997748740208;
    msg.start_z = 0.298910316249;
    msg.start_z_units = 223U;
    msg.end_z = 0.351288626121;
    msg.end_z_units = 114U;
    msg.radius = 0.758861195496;
    msg.speed = 0.827672960199;
    msg.speed_units = 37U;
    msg.custom.assign("KMWKHJLSJLQAZNYSZOWKGBQLXHQAUVDEHQWZKPRAMCEILNYSBR");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Elevator #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowTrajectory msg;
    msg.setTimeStamp(0.562907271617);
    msg.setSource(56000U);
    msg.setSourceEntity(224U);
    msg.setDestination(1154U);
    msg.setDestinationEntity(198U);
    msg.timeout = 53192U;
    msg.lat = 0.118313872457;
    msg.lon = 0.883396559305;
    msg.z = 0.537609318729;
    msg.z_units = 161U;
    msg.speed = 0.0543840931831;
    msg.speed_units = 117U;
    msg.custom.assign("ENQDBHYJFMAXFLTCWMKYWXSAKWRXHOMQHXHIFNRKUOCNG");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowTrajectory #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowTrajectory msg;
    msg.setTimeStamp(0.259091961252);
    msg.setSource(25818U);
    msg.setSourceEntity(133U);
    msg.setDestination(55703U);
    msg.setDestinationEntity(106U);
    msg.timeout = 13652U;
    msg.lat = 0.861542051376;
    msg.lon = 0.394694235089;
    msg.z = 0.953595310868;
    msg.z_units = 18U;
    msg.speed = 0.0451783371717;
    msg.speed_units = 170U;
    msg.custom.assign("DUFZOPXZHJXQPTNEGOVHVURNJZWEVMKNUFTXGSYDCKSTTFAFAUBFDJHRTHEGACVRH");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowTrajectory #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowTrajectory msg;
    msg.setTimeStamp(0.760178524634);
    msg.setSource(62541U);
    msg.setSourceEntity(38U);
    msg.setDestination(61406U);
    msg.setDestinationEntity(85U);
    msg.timeout = 61571U;
    msg.lat = 0.716898680742;
    msg.lon = 0.60010236397;
    msg.z = 0.485966467658;
    msg.z_units = 161U;
    msg.speed = 0.339512758657;
    msg.speed_units = 108U;
    msg.custom.assign("ZUGUHXTDZGMHEMJDBFYCBKLSPWQVOILAXOLKPYMLOOHIRSTMWIETHDKTJQTPYTVTZWAMNUQC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowTrajectory #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrajectoryPoint msg;
    msg.setTimeStamp(0.530087272638);
    msg.setSource(3342U);
    msg.setSourceEntity(7U);
    msg.setDestination(19278U);
    msg.setDestinationEntity(144U);
    msg.x = 0.142297396302;
    msg.y = 0.168890204718;
    msg.z = 0.240107662208;
    msg.t = 0.00223942504013;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrajectoryPoint #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrajectoryPoint msg;
    msg.setTimeStamp(0.891584545423);
    msg.setSource(14150U);
    msg.setSourceEntity(27U);
    msg.setDestination(28042U);
    msg.setDestinationEntity(80U);
    msg.x = 0.888626550676;
    msg.y = 0.929905493983;
    msg.z = 0.765944193636;
    msg.t = 0.127856462246;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrajectoryPoint #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrajectoryPoint msg;
    msg.setTimeStamp(0.946571698342);
    msg.setSource(3495U);
    msg.setSourceEntity(112U);
    msg.setDestination(52538U);
    msg.setDestinationEntity(97U);
    msg.x = 0.346889645853;
    msg.y = 0.0457675576234;
    msg.z = 0.0388072799998;
    msg.t = 0.432316175642;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrajectoryPoint #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CustomManeuver msg;
    msg.setTimeStamp(0.0257618736363);
    msg.setSource(12775U);
    msg.setSourceEntity(158U);
    msg.setDestination(50618U);
    msg.setDestinationEntity(117U);
    msg.timeout = 45378U;
    msg.name.assign("WFDTFPGVHNXOQVJAGOYERMNLRWCREVIESVYIEDXVSNWTSBLPGRNIHMPTWNDGACJJLWDQSEINWHFYPXNQVMJXZKDBDSPGIKTDGYPHTTBRKBPHFOUDJBRKEQFAUQUUMUOLAAFBJOBMCARLXAFCAPVUIQZHQZKXOCYJLCBWZIFOYNYEVXZNCNOHKWUTHGHYSEKEFVOFUZMW");
    msg.custom.assign("KNRQXFETZVDWPPGLLSEAEATTXVZUWCJGRNXASUEBISHWFAJSYYDWRFKFMLWJOLJJSGYCMJMSINKRXAVQGDBZAHGQZWHKTUWMVHBUKADPBPUCQRNEXVVZSOMJUBNAPXYIYKYVTOCPBNLNQCOGGGPHPKDTAOXOHRIMAIGUZH");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CustomManeuver #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CustomManeuver msg;
    msg.setTimeStamp(0.889278277851);
    msg.setSource(10072U);
    msg.setSourceEntity(57U);
    msg.setDestination(7736U);
    msg.setDestinationEntity(9U);
    msg.timeout = 62038U;
    msg.name.assign("MVQUNURYOVPXUIBAGFKJKCZZGCQLXVMGQEBUCTXICNOZLKLYQSJQIPPCBRMTXNZSGBWPBLYFWIITCKQFJNMCYDKINFMWRDTVMNBLMERHHAKIBAIAFWUWHACWDSJVFLMOSJBE");
    msg.custom.assign("ZQCTCFABCFXQZGUOPTQQXHQNJQXCUWHZCVISHPWXEHNMCZJUBMZWDLLCUWGNTWVPOXZRWCMQIFHUIRMHPOINJRDKBJGUEDZAREKPKZRIOSFFGSWGEAVRDSZDDUPFYKYSVPBLKEYLJBNJTFPRUOGIMTCSNHHUD");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CustomManeuver #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CustomManeuver msg;
    msg.setTimeStamp(0.672727150061);
    msg.setSource(40416U);
    msg.setSourceEntity(89U);
    msg.setDestination(38002U);
    msg.setDestinationEntity(87U);
    msg.timeout = 16629U;
    msg.name.assign("VWQLBMNGTUGXEUIJHDYEBRAUPZVFSAEWDCYLCDOVXOU");
    msg.custom.assign("NJXLDHVQENBNFVUZKPTXIYRKVUGVKWDQYUROUZIPHGGEZFJOWMKAHM");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CustomManeuver #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleFormation msg;
    msg.setTimeStamp(0.152752182908);
    msg.setSource(5213U);
    msg.setSourceEntity(198U);
    msg.setDestination(52369U);
    msg.setDestinationEntity(121U);
    msg.lat = 0.638415830888;
    msg.lon = 0.226662809889;
    msg.z = 0.761465707518;
    msg.z_units = 177U;
    msg.speed = 0.725405779609;
    msg.speed_units = 250U;
    IMC::VehicleFormationParticipant tmp_msg_0;
    tmp_msg_0.vid = 50306U;
    tmp_msg_0.off_x = 0.712380656941;
    tmp_msg_0.off_y = 0.310355520101;
    tmp_msg_0.off_z = 0.642954425452;
    msg.participants.push_back(tmp_msg_0);
    msg.start_time = 0.568544649762;
    msg.custom.assign("HTVRNSEETDRBK");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleFormation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleFormation msg;
    msg.setTimeStamp(0.259774207629);
    msg.setSource(43101U);
    msg.setSourceEntity(36U);
    msg.setDestination(12307U);
    msg.setDestinationEntity(26U);
    msg.lat = 0.903962204228;
    msg.lon = 0.0793246724909;
    msg.z = 0.0890119734004;
    msg.z_units = 27U;
    msg.speed = 0.554141032138;
    msg.speed_units = 217U;
    IMC::VehicleFormationParticipant tmp_msg_0;
    tmp_msg_0.vid = 44829U;
    tmp_msg_0.off_x = 0.0480655170117;
    tmp_msg_0.off_y = 0.443327619509;
    tmp_msg_0.off_z = 0.541582981379;
    msg.participants.push_back(tmp_msg_0);
    msg.start_time = 0.872021356122;
    msg.custom.assign("FERMAKMLGZYIWQQFOCSVGKHRTKCHDRBMJGNJAJGTUHADCNROZTQSAGAANOOFIQZOJLDBHJPUGVHYFSOREIAFARJYZINPBTTMWBCLEYWACWZHNGPQNIDPQKFXPYVTZFQGBTIBSDIOFDEELTXLZWJKVBRJWVXZDXCSMSVEJLPSYNHKZ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleFormation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleFormation msg;
    msg.setTimeStamp(0.492251739065);
    msg.setSource(15962U);
    msg.setSourceEntity(86U);
    msg.setDestination(40402U);
    msg.setDestinationEntity(6U);
    msg.lat = 0.644001725145;
    msg.lon = 0.74592304458;
    msg.z = 0.519694357976;
    msg.z_units = 152U;
    msg.speed = 0.326446460728;
    msg.speed_units = 26U;
    IMC::TrajectoryPoint tmp_msg_0;
    tmp_msg_0.x = 0.335687053653;
    tmp_msg_0.y = 0.878123644303;
    tmp_msg_0.z = 0.35669309644;
    tmp_msg_0.t = 0.178868375829;
    msg.points.push_back(tmp_msg_0);
    IMC::VehicleFormationParticipant tmp_msg_1;
    tmp_msg_1.vid = 17411U;
    tmp_msg_1.off_x = 0.574606807659;
    tmp_msg_1.off_y = 0.431197941321;
    tmp_msg_1.off_z = 0.202523244856;
    msg.participants.push_back(tmp_msg_1);
    msg.start_time = 0.947345501825;
    msg.custom.assign("VLMFMGJORHTEVEQAORHPWDGULXYCFUMJXBXHSTCDADFAVYTTKHIJPUDCZUVIBCLUYNFCRKFRTLDPIFQRMMOWFARIQBARMTLPUXAVGGVKAMEXNJINAVYKOJXGVGWKSNPNIXOLITAEQXSFESZYQXPUKEJSZOLOCJONGHCJZBHIWOWEBDYPRMGKWPSTNSDCZQACJBDWNLVKZYPIQZOZGHRHEIHKBNCYFURUXJQEDSVELKYPQBGYQDMWUZT");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleFormation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleFormationParticipant msg;
    msg.setTimeStamp(0.163440038734);
    msg.setSource(26043U);
    msg.setSourceEntity(179U);
    msg.setDestination(5034U);
    msg.setDestinationEntity(177U);
    msg.vid = 61031U;
    msg.off_x = 0.180643337061;
    msg.off_y = 0.999468413114;
    msg.off_z = 0.236589791408;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleFormationParticipant #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleFormationParticipant msg;
    msg.setTimeStamp(0.247327207719);
    msg.setSource(13555U);
    msg.setSourceEntity(221U);
    msg.setDestination(59796U);
    msg.setDestinationEntity(67U);
    msg.vid = 17745U;
    msg.off_x = 0.277452845668;
    msg.off_y = 0.933360377242;
    msg.off_z = 0.28211221618;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleFormationParticipant #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleFormationParticipant msg;
    msg.setTimeStamp(0.0590304688559);
    msg.setSource(55981U);
    msg.setSourceEntity(67U);
    msg.setDestination(13602U);
    msg.setDestinationEntity(191U);
    msg.vid = 18457U;
    msg.off_x = 0.795849027982;
    msg.off_y = 0.312374493782;
    msg.off_z = 0.201371291378;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleFormationParticipant #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::StopManeuver msg;
    msg.setTimeStamp(0.160003911851);
    msg.setSource(9111U);
    msg.setSourceEntity(254U);
    msg.setDestination(59707U);
    msg.setDestinationEntity(137U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("StopManeuver #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::StopManeuver msg;
    msg.setTimeStamp(0.413225012397);
    msg.setSource(65487U);
    msg.setSourceEntity(187U);
    msg.setDestination(38847U);
    msg.setDestinationEntity(102U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("StopManeuver #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::StopManeuver msg;
    msg.setTimeStamp(0.0802007077744);
    msg.setSource(21238U);
    msg.setSourceEntity(209U);
    msg.setDestination(58487U);
    msg.setDestinationEntity(82U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("StopManeuver #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RegisterManeuver msg;
    msg.setTimeStamp(0.534894660283);
    msg.setSource(22409U);
    msg.setSourceEntity(230U);
    msg.setDestination(38634U);
    msg.setDestinationEntity(15U);
    msg.mid = 50529U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RegisterManeuver #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RegisterManeuver msg;
    msg.setTimeStamp(0.512908399953);
    msg.setSource(63624U);
    msg.setSourceEntity(213U);
    msg.setDestination(27647U);
    msg.setDestinationEntity(39U);
    msg.mid = 3995U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RegisterManeuver #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RegisterManeuver msg;
    msg.setTimeStamp(0.525705772097);
    msg.setSource(60050U);
    msg.setSourceEntity(114U);
    msg.setDestination(2148U);
    msg.setDestinationEntity(120U);
    msg.mid = 41281U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RegisterManeuver #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ManeuverControlState msg;
    msg.setTimeStamp(0.696949313271);
    msg.setSource(4971U);
    msg.setSourceEntity(240U);
    msg.setDestination(24961U);
    msg.setDestinationEntity(71U);
    msg.state = 120U;
    msg.eta = 55368U;
    msg.info.assign("KTASUBWLALVDSBHWNUFGFWOMIMCQZIGIJEWZUTXQKCHZZNEBZEIKBMXYPYHLKVXYVMWLLNFGWEDJJJRPDVSABESCAXFTAAPNJAGZPTTXJZLRYL");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ManeuverControlState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ManeuverControlState msg;
    msg.setTimeStamp(0.39681757747);
    msg.setSource(48099U);
    msg.setSourceEntity(117U);
    msg.setDestination(43823U);
    msg.setDestinationEntity(235U);
    msg.state = 252U;
    msg.eta = 36168U;
    msg.info.assign("GUEYQMWVLDNEFPBXVQDBMJVETQUJKJRHHYHYPXCAXZDBJCMNVKHRYHAZSZDENWNKGKGXHDHOJWEZUOCQYILUKSGOAIUGOMIECKQADTJSXTFXUYJHZVXVG");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ManeuverControlState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ManeuverControlState msg;
    msg.setTimeStamp(0.609596276446);
    msg.setSource(60143U);
    msg.setSourceEntity(175U);
    msg.setDestination(48639U);
    msg.setDestinationEntity(119U);
    msg.state = 19U;
    msg.eta = 41850U;
    msg.info.assign("QKTIQEERAJZKOLIUIMYIHJQIUXSDBMNRKDBXHFLWWRBJWFPYCDCBYQSGHRQE");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ManeuverControlState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowSystem msg;
    msg.setTimeStamp(0.888908788554);
    msg.setSource(64840U);
    msg.setSourceEntity(229U);
    msg.setDestination(15511U);
    msg.setDestinationEntity(138U);
    msg.system = 42853U;
    msg.duration = 11985U;
    msg.speed = 0.602316128833;
    msg.speed_units = 116U;
    msg.x = 0.363098427908;
    msg.y = 0.0619975290121;
    msg.z = 0.912723816899;
    msg.z_units = 150U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowSystem #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowSystem msg;
    msg.setTimeStamp(0.295771391601);
    msg.setSource(10739U);
    msg.setSourceEntity(58U);
    msg.setDestination(26195U);
    msg.setDestinationEntity(183U);
    msg.system = 35596U;
    msg.duration = 42494U;
    msg.speed = 0.839258469473;
    msg.speed_units = 203U;
    msg.x = 0.982964911228;
    msg.y = 0.313981738398;
    msg.z = 0.63924885481;
    msg.z_units = 82U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowSystem #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowSystem msg;
    msg.setTimeStamp(0.470104225395);
    msg.setSource(16192U);
    msg.setSourceEntity(135U);
    msg.setDestination(55577U);
    msg.setDestinationEntity(39U);
    msg.system = 33958U;
    msg.duration = 56269U;
    msg.speed = 0.667543252583;
    msg.speed_units = 83U;
    msg.x = 0.246549089947;
    msg.y = 0.221662284313;
    msg.z = 0.141267834243;
    msg.z_units = 185U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowSystem #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CommsRelay msg;
    msg.setTimeStamp(0.170389324235);
    msg.setSource(26289U);
    msg.setSourceEntity(230U);
    msg.setDestination(14395U);
    msg.setDestinationEntity(87U);
    msg.lat = 0.333553014274;
    msg.lon = 0.772074136602;
    msg.speed = 0.668562765916;
    msg.speed_units = 212U;
    msg.duration = 3549U;
    msg.sys_a = 5607U;
    msg.sys_b = 27107U;
    msg.move_threshold = 0.553744034938;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CommsRelay #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CommsRelay msg;
    msg.setTimeStamp(0.592302064015);
    msg.setSource(36061U);
    msg.setSourceEntity(161U);
    msg.setDestination(43820U);
    msg.setDestinationEntity(49U);
    msg.lat = 0.775099595141;
    msg.lon = 0.142192149661;
    msg.speed = 0.781995053192;
    msg.speed_units = 162U;
    msg.duration = 19849U;
    msg.sys_a = 18300U;
    msg.sys_b = 19467U;
    msg.move_threshold = 0.920125470559;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CommsRelay #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CommsRelay msg;
    msg.setTimeStamp(0.205578359823);
    msg.setSource(28461U);
    msg.setSourceEntity(139U);
    msg.setDestination(15793U);
    msg.setDestinationEntity(195U);
    msg.lat = 0.0407700419773;
    msg.lon = 0.536634409095;
    msg.speed = 0.261070111605;
    msg.speed_units = 60U;
    msg.duration = 12051U;
    msg.sys_a = 64062U;
    msg.sys_b = 54881U;
    msg.move_threshold = 0.366023424642;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CommsRelay #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CoverArea msg;
    msg.setTimeStamp(0.912735388077);
    msg.setSource(1111U);
    msg.setSourceEntity(163U);
    msg.setDestination(5125U);
    msg.setDestinationEntity(101U);
    msg.lat = 0.163880931423;
    msg.lon = 0.663084409366;
    msg.z = 0.332784951816;
    msg.z_units = 134U;
    msg.speed = 0.462448106011;
    msg.speed_units = 233U;
    IMC::PolygonVertex tmp_msg_0;
    tmp_msg_0.lat = 0.864447933987;
    tmp_msg_0.lon = 0.301646548412;
    msg.polygon.push_back(tmp_msg_0);
    msg.custom.assign("MKFQKYOCDYTFFEIKABGFLSHNMJWTCJBQUZJLUMWDGVH");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CoverArea #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CoverArea msg;
    msg.setTimeStamp(0.35854097655);
    msg.setSource(44629U);
    msg.setSourceEntity(68U);
    msg.setDestination(10235U);
    msg.setDestinationEntity(191U);
    msg.lat = 0.314979915218;
    msg.lon = 0.910050702873;
    msg.z = 0.10638455962;
    msg.z_units = 147U;
    msg.speed = 0.514307671198;
    msg.speed_units = 57U;
    IMC::PolygonVertex tmp_msg_0;
    tmp_msg_0.lat = 0.502801731895;
    tmp_msg_0.lon = 0.735165227674;
    msg.polygon.push_back(tmp_msg_0);
    msg.custom.assign("IDBULEACSRLKNHZDHTMFOOZNSY");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CoverArea #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CoverArea msg;
    msg.setTimeStamp(0.883956215185);
    msg.setSource(25060U);
    msg.setSourceEntity(218U);
    msg.setDestination(64109U);
    msg.setDestinationEntity(40U);
    msg.lat = 0.327479319836;
    msg.lon = 0.998223348559;
    msg.z = 0.121647484441;
    msg.z_units = 80U;
    msg.speed = 0.582694463932;
    msg.speed_units = 201U;
    msg.custom.assign("MHLYGKXVLJORPYGWKPEYTESATFDWJAGZGULDQPOIRCTOOXKBAUBEEMDYBQMOMXIXKWVVWNYFTIAGSWLCHIBDCNITQWHSJTQFDMXJPHEKAZYGRCPLZJLUWONRUSBGZJFGXIYOZLHYIVNIUBSHSDLTZXASRUCRUBGOFCHDNCVRSMYNAXBDAR");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CoverArea #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PolygonVertex msg;
    msg.setTimeStamp(0.135299394885);
    msg.setSource(20973U);
    msg.setSourceEntity(171U);
    msg.setDestination(8324U);
    msg.setDestinationEntity(138U);
    msg.lat = 0.729776696159;
    msg.lon = 0.208272062316;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PolygonVertex #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PolygonVertex msg;
    msg.setTimeStamp(0.780780292048);
    msg.setSource(23114U);
    msg.setSourceEntity(82U);
    msg.setDestination(4293U);
    msg.setDestinationEntity(2U);
    msg.lat = 0.780145077501;
    msg.lon = 0.263264927073;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PolygonVertex #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PolygonVertex msg;
    msg.setTimeStamp(0.708637473606);
    msg.setSource(38906U);
    msg.setSourceEntity(93U);
    msg.setDestination(22080U);
    msg.setDestinationEntity(180U);
    msg.lat = 0.420206130887;
    msg.lon = 0.0496336348047;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PolygonVertex #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CompassCalibration msg;
    msg.setTimeStamp(0.585922545873);
    msg.setSource(32112U);
    msg.setSourceEntity(48U);
    msg.setDestination(8464U);
    msg.setDestinationEntity(157U);
    msg.timeout = 52465U;
    msg.lat = 0.698869271315;
    msg.lon = 0.468636251069;
    msg.z = 0.714266888765;
    msg.z_units = 69U;
    msg.pitch = 0.234751934224;
    msg.amplitude = 0.99911756553;
    msg.duration = 22700U;
    msg.speed = 0.0285139970911;
    msg.speed_units = 234U;
    msg.radius = 0.436864798167;
    msg.direction = 117U;
    msg.custom.assign("GHGUCCYSNYLEURDHDXETHDTEQDMSZEVIHGIXWWTHUWBZVZAJOPNCFTFSLRKOSELULIACVFGOHRCMQNHPLUZZYIBSIKGTDYEHQNZJTTKOS");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CompassCalibration #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CompassCalibration msg;
    msg.setTimeStamp(0.697148879518);
    msg.setSource(42685U);
    msg.setSourceEntity(31U);
    msg.setDestination(12030U);
    msg.setDestinationEntity(36U);
    msg.timeout = 56213U;
    msg.lat = 0.957736495289;
    msg.lon = 0.0265568035144;
    msg.z = 0.289413642226;
    msg.z_units = 27U;
    msg.pitch = 0.148593071722;
    msg.amplitude = 0.974127867155;
    msg.duration = 57735U;
    msg.speed = 0.358436243739;
    msg.speed_units = 160U;
    msg.radius = 0.872359748134;
    msg.direction = 246U;
    msg.custom.assign("AAIJJKDCPGIRFUWJZHQAQNVBWTVGQKMSHPTLJQORSYYBSUZNVO");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CompassCalibration #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CompassCalibration msg;
    msg.setTimeStamp(0.621882955246);
    msg.setSource(47937U);
    msg.setSourceEntity(98U);
    msg.setDestination(40194U);
    msg.setDestinationEntity(119U);
    msg.timeout = 12782U;
    msg.lat = 0.724444061515;
    msg.lon = 0.157738195925;
    msg.z = 0.434226496364;
    msg.z_units = 106U;
    msg.pitch = 0.0334218718504;
    msg.amplitude = 0.340877704301;
    msg.duration = 1878U;
    msg.speed = 0.662676191171;
    msg.speed_units = 64U;
    msg.radius = 0.585485070342;
    msg.direction = 99U;
    msg.custom.assign("XFKQCNDTOEAQGALQUIRTHEDZTVLBSRABKGWRNFDCHUUMZVOUQMWGAJVRIHTJPFOLIYMJRHETMNOUYZYGQOJLMHTTXWBJIF");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CompassCalibration #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormationParameters msg;
    msg.setTimeStamp(0.493889573033);
    msg.setSource(45306U);
    msg.setSourceEntity(233U);
    msg.setDestination(5071U);
    msg.setDestinationEntity(108U);
    msg.formation_name.assign("SBNXCRBTPTSRWHSRGUEVVVRIIVLNLPDMAGYXZQUNBDRWBFMOCUJGRQGUECJNPYKFAWJCMZMDAPDDOMOKCYJGOMRNQKEILZJMVYJCDLHNOKMWLKKXTIXIKABHWRCFSMYNFEBHUWHHYGQDWKITUHWGIYEOJJAZEFZRBVEHASPQPJUTLSJCLRWZLUSTIDWTISTVZBDYXIEEALZXPAXZCQGVA");
    msg.reference_frame = 38U;
    msg.custom.assign("SKJDIIKWMCYAJEHIAZVQCSHMENDDAAZXUCFXVTAWGTEQQCGREHUFITGFMVUHBJQLBJYTCNKJUIGPHOPPUYBWZRVQXDVNPTVZMVLBEONHZLYWZGWCRAWSSOQOANTPHKIBUBXBDSKOCOQSDDWSEPEOQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormationParameters #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormationParameters msg;
    msg.setTimeStamp(0.27870438397);
    msg.setSource(49841U);
    msg.setSourceEntity(51U);
    msg.setDestination(62966U);
    msg.setDestinationEntity(63U);
    msg.formation_name.assign("EXKGYESPXFNWXLPNQHENYYWQGUSSZBSJGCQFTARPYDIHJGPCPEXMKAVZVWXVPIEAHLIBBRJKJGZKFALXLHKCRTPJRCCQUDNRIDTFUCILIFTLJZGOSLMFHILGBANUFLPUOQRTYPUTXSKOVMWEBMZUOSHRZOKVJRQRWDTDWEHTABJGECNATVCMQOYDNUZFWWJMUYBPAZHHSDEDBOIM");
    msg.reference_frame = 212U;
    msg.custom.assign("PFVKVLKSLWZWSIBIHDOESNQCXQBLQPDFPEWSQFADHCJPTLOURZXYPMGTCNRVDMZDRLZKIPHYUMGHRIEEEIUXBYVLUDAXYZWUMUSYPICHLHYGCPDGFKITNZLBAJBTVBKJCFGNFGHDFFJYORFAAXKZCBRJKDNGALSHKMOQNPVUKJGTGJKZVXTTFLGRMHEAEQSWETQTTYOBAIJZVEWWSISCUJDR");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormationParameters #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormationParameters msg;
    msg.setTimeStamp(0.0748712269624);
    msg.setSource(56080U);
    msg.setSourceEntity(85U);
    msg.setDestination(12416U);
    msg.setDestinationEntity(87U);
    msg.formation_name.assign("NZBFQNQALGLPKHBTMVECPIUCHBLLTHDXMFZHKNVLWTZXCYDAQCMNTEAGZNFRGVPCRVJXBCYYYJIBBHIDJJKWLEDTGOEDOPFYKEWCAGYIFQBRARUUNKAOSSUPCNXSQROWIVQOQWEYMZFOMXXTXEGPRRWEJLPMBIDRJDWFCKPSJPATTHKIYSTBVRVQDNZAWQGVGSQEOJZHPARAMBOJCSXLSWJZDVWUONHUIDEYIM");
    msg.reference_frame = 2U;
    IMC::VehicleFormationParticipant tmp_msg_0;
    tmp_msg_0.vid = 30744U;
    tmp_msg_0.off_x = 0.924899873381;
    tmp_msg_0.off_y = 0.636149672633;
    tmp_msg_0.off_z = 0.74117540804;
    msg.participants.push_back(tmp_msg_0);
    msg.custom.assign("ZDEWRPLAVXRNRGEECLVPIFIEGBTZCUKAHHYHBBLUUGMIPH");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormationParameters #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormationPlanExecution msg;
    msg.setTimeStamp(0.252333428923);
    msg.setSource(23407U);
    msg.setSourceEntity(181U);
    msg.setDestination(57860U);
    msg.setDestinationEntity(194U);
    msg.group_name.assign("TBRWQUSSRDFDHOOYCDPLSCLQYJMXJXSRNNJKFVEVFHVNBMWXEJGNEFPIDAOBDFEHLSCMNXEAYRUBYJKLUIXIKGGFKUVWACAMWVPQCHVGVRJYLHGMJIHRIVNBJSWKSNZFADTDLZFAQDQORTIWHUVLQJZTRGWBCTKERHQJMAPOOWHBLXKCHIVMOMSZUPPDTXTEPCYZSYPKOLAQZGEG");
    msg.formation_name.assign("PAIYOTEQBHVASIWOXPKGCBOQCKZEMCISQKZYWBHEGJDDLXDFOCBAUWMQDSMPXODKQHYJINNVXRMXVQFCIJXINEBTKTMJIPEUWVDLQYZVDRTGRZSKRTPHBEWDVVQZZJPUVOTSPUREUGGJSGVFELMAPZLUNHSRWEFYYBUOAYFUAGRHRKZABNVALYFKXMLHDYCIQFXJGCDNQPIUGWXTLOZBBLJHMLHOERSIKGT");
    msg.plan_id.assign("ICQUNHSHOJOBPXTOFWAPWEMYFRSVVGFSZMPWKNJDAKUYIIDABZHAVLDZXQUSGEYXPDSTKUSORHLNYMINDJWRZPUNIFLJBCPFJJHRTRKLYBPELHENKSDVUQYOBDAQKMIXNDHOIVXOOHGWJRYIAZUQBCTGTCIGBRKFTLLCCGWNOIBGRQVMQMSZQASLXEJLATGTCWDEGTCWNYPCA");
    msg.description.assign("ZNLMRAAGIFFCWAXCBZOHRSZSVHGTGNHEBLTZAOGVNWYXMOFMJSZUPZHAJCDWUIXCOLAEVLSMDVEW");
    msg.leader_speed = 0.672778239127;
    msg.leader_bank_lim = 0.718062263775;
    msg.pos_sim_err_lim = 0.47282216943;
    msg.pos_sim_err_wrn = 0.289294839427;
    msg.pos_sim_err_timeout = 24800U;
    msg.converg_max = 0.271607802087;
    msg.converg_timeout = 26247U;
    msg.comms_timeout = 15377U;
    msg.turb_lim = 0.613795728135;
    msg.custom.assign("HVTAAADRILBSFJGQVBFDCJGPXEEKQICJWUONEBNUETZVRPMMCJJWQMPOEDTPJLZWLOKTIKWLMMRCXHRGYRUBNQTCNGSSEXXBK");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormationPlanExecution #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormationPlanExecution msg;
    msg.setTimeStamp(0.457587158309);
    msg.setSource(33159U);
    msg.setSourceEntity(6U);
    msg.setDestination(22322U);
    msg.setDestinationEntity(137U);
    msg.group_name.assign("MAUNPGVZPTRJLNLDUDJWBIPDFURNYOYTCRIWKRUKPIMYBXNCDFPEXEXJJYQFVZYHHUEOTSPHQGSB");
    msg.formation_name.assign("MXSYHZXVACZDOWFBSLKNMRDCVTHGYMOOODPLPXSMJXUZFABAJRNP");
    msg.plan_id.assign("DUIODRDAMPEKDKATLWUY");
    msg.description.assign("JSXEHMPQMWDTVQAGDTDFKIVYYOXKPQWROVYXPSNGUIQIQQERNUPITFRXVZSHSHMOHWLQIGQRNRFWAWCZWZOJYUR");
    msg.leader_speed = 0.866302898119;
    msg.leader_bank_lim = 0.633971972923;
    msg.pos_sim_err_lim = 0.252159336682;
    msg.pos_sim_err_wrn = 0.691225839818;
    msg.pos_sim_err_timeout = 9045U;
    msg.converg_max = 0.154477720588;
    msg.converg_timeout = 45332U;
    msg.comms_timeout = 43396U;
    msg.turb_lim = 0.0882467022303;
    msg.custom.assign("QTQNXVARYCPFNMYKAURENLVSPWOITCMSPMGNYEXHQRZOCLJUNTGJQZHKFXVQPUZHZGYWUPEAJWOQDJCZHSHKDBELGSGJMLMRASIDLKVWUCWUGTGRIDBHZRVCHZEJKJRXRKZHPNHKALVSXRMQCEIDYAPBWXHFGZMXITKPPWZXLNDLOISIONTVOQMINSALPFIBBGBYVRF");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormationPlanExecution #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormationPlanExecution msg;
    msg.setTimeStamp(0.0160878030713);
    msg.setSource(35302U);
    msg.setSourceEntity(56U);
    msg.setDestination(17457U);
    msg.setDestinationEntity(29U);
    msg.group_name.assign("GDZGICNHDNVVFJHTDKNQRCLZZSJJXTCCDMYEIRLOHWVWLKWZSUVFCIKOEGIWXSQOUZOVQKTJKCSVEOMUXEMROWBLSYJRDUNTTFYFNTJ");
    msg.formation_name.assign("HYCSTNIUFYWAWFZBJGXGOPOOHXWIVBLDCKXTNMSRNKICCWUJNEGUUOSNKTXQZXSBPDOQRBLJAZISGIWPQEOWUFHDDXYPZZVCCGAYHQZBHMAMWFAPYECYKQVJVTHJLNTKVVLQGDSARFKRFFEJMBUSZMFUDMUQHNTQOUMECORBVRNHCRNTILJRRAIIHGKTYXPKYEEWZNEFBLEPJMLHATQOXWSVDKQPYBJLFVZPPLLBTDXWEIDAJKUGXC");
    msg.plan_id.assign("UTTOASHQQZBDQEHVZOWJJLMQCRZFLHSITEVANHVGWFFOROFRFIPWFAWWLCHOXSUUKJAEJRDLOSDUMWKIBPUVWBBBJKIZROPUDIRHVZFOEKTYGHGNJXCMPZBRCEGKPK");
    msg.description.assign("MSVJAUFHKELXBCKUGQMNQKJPMOMVDWDVADNVXVXORYNBBLVLZOQSCTMAIYJYGDPTPGODHQUEPCJPJPBWFURDYZOFCCWXCRTWEIGGHAEE");
    msg.leader_speed = 0.560471732395;
    msg.leader_bank_lim = 0.858762872212;
    msg.pos_sim_err_lim = 0.882186466193;
    msg.pos_sim_err_wrn = 0.934955818136;
    msg.pos_sim_err_timeout = 32837U;
    msg.converg_max = 0.475085718732;
    msg.converg_timeout = 2469U;
    msg.comms_timeout = 19358U;
    msg.turb_lim = 0.0139081081523;
    msg.custom.assign("CNXBKKLUVPKYQBKWTPJCFBQSGWDXVEDODXHKWJVHOLNANGXWMNMLYRELLWECJIJLFGTJZJQZWIDSBPMQWQQFXGGLSJEMPPONVRDNDRBOMZOQZVROWISRTRHIVXKMTFITAENRMGQYTHTXFSOVZFFDDIPNHZZISYAXIAHEYGKBLOXSJQEWREYTYTUVYUQOTPVHBRKKZWXCCAAHSIDPJBBHDEZYEVBGLOUNMGFCCCYHSLPFA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormationPlanExecution #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowReference msg;
    msg.setTimeStamp(0.235612436637);
    msg.setSource(31632U);
    msg.setSourceEntity(195U);
    msg.setDestination(38237U);
    msg.setDestinationEntity(58U);
    msg.control_src = 36808U;
    msg.control_ent = 58U;
    msg.timeout = 0.0533748095592;
    msg.loiter_radius = 0.410370475744;
    msg.altitude_interval = 0.0711842264216;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowReference #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowReference msg;
    msg.setTimeStamp(0.980149166991);
    msg.setSource(1303U);
    msg.setSourceEntity(83U);
    msg.setDestination(21320U);
    msg.setDestinationEntity(211U);
    msg.control_src = 64930U;
    msg.control_ent = 163U;
    msg.timeout = 0.0172776007834;
    msg.loiter_radius = 0.300756322626;
    msg.altitude_interval = 0.651913922754;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowReference #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowReference msg;
    msg.setTimeStamp(0.0636690776072);
    msg.setSource(8820U);
    msg.setSourceEntity(60U);
    msg.setDestination(64567U);
    msg.setDestinationEntity(218U);
    msg.control_src = 16414U;
    msg.control_ent = 1U;
    msg.timeout = 0.462209346943;
    msg.loiter_radius = 0.572697349427;
    msg.altitude_interval = 0.287088532681;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowReference #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Reference msg;
    msg.setTimeStamp(0.0353342280268);
    msg.setSource(23122U);
    msg.setSourceEntity(42U);
    msg.setDestination(46656U);
    msg.setDestinationEntity(142U);
    msg.flags = 39U;
    IMC::DesiredSpeed tmp_msg_0;
    tmp_msg_0.value = 0.205193610482;
    tmp_msg_0.speed_units = 17U;
    msg.speed.set(tmp_msg_0);
    IMC::DesiredZ tmp_msg_1;
    tmp_msg_1.value = 0.16730539308;
    tmp_msg_1.z_units = 187U;
    msg.z.set(tmp_msg_1);
    msg.lat = 0.321706293739;
    msg.lon = 0.0813932792203;
    msg.radius = 0.627384145396;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Reference #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Reference msg;
    msg.setTimeStamp(0.0966771053444);
    msg.setSource(48300U);
    msg.setSourceEntity(132U);
    msg.setDestination(50140U);
    msg.setDestinationEntity(4U);
    msg.flags = 19U;
    IMC::DesiredSpeed tmp_msg_0;
    tmp_msg_0.value = 0.81156420806;
    tmp_msg_0.speed_units = 121U;
    msg.speed.set(tmp_msg_0);
    IMC::DesiredZ tmp_msg_1;
    tmp_msg_1.value = 0.0478769902253;
    tmp_msg_1.z_units = 123U;
    msg.z.set(tmp_msg_1);
    msg.lat = 0.153732478231;
    msg.lon = 0.612103046341;
    msg.radius = 0.00335220691837;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Reference #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Reference msg;
    msg.setTimeStamp(0.403222481596);
    msg.setSource(27224U);
    msg.setSourceEntity(43U);
    msg.setDestination(32762U);
    msg.setDestinationEntity(96U);
    msg.flags = 32U;
    IMC::DesiredSpeed tmp_msg_0;
    tmp_msg_0.value = 0.923859560817;
    tmp_msg_0.speed_units = 38U;
    msg.speed.set(tmp_msg_0);
    IMC::DesiredZ tmp_msg_1;
    tmp_msg_1.value = 0.656030219761;
    tmp_msg_1.z_units = 24U;
    msg.z.set(tmp_msg_1);
    msg.lat = 0.496137516225;
    msg.lon = 0.477687598735;
    msg.radius = 0.60620793788;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Reference #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowRefState msg;
    msg.setTimeStamp(0.247731290315);
    msg.setSource(38631U);
    msg.setSourceEntity(71U);
    msg.setDestination(58387U);
    msg.setDestinationEntity(45U);
    msg.control_src = 3870U;
    msg.control_ent = 246U;
    IMC::Reference tmp_msg_0;
    tmp_msg_0.flags = 203U;
    IMC::DesiredSpeed tmp_tmp_msg_0_0;
    tmp_tmp_msg_0_0.value = 0.516462619322;
    tmp_tmp_msg_0_0.speed_units = 118U;
    tmp_msg_0.speed.set(tmp_tmp_msg_0_0);
    IMC::DesiredZ tmp_tmp_msg_0_1;
    tmp_tmp_msg_0_1.value = 0.183308565274;
    tmp_tmp_msg_0_1.z_units = 3U;
    tmp_msg_0.z.set(tmp_tmp_msg_0_1);
    tmp_msg_0.lat = 0.140318894941;
    tmp_msg_0.lon = 0.659176587191;
    tmp_msg_0.radius = 0.394209739365;
    msg.reference.set(tmp_msg_0);
    msg.state = 246U;
    msg.proximity = 110U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowRefState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowRefState msg;
    msg.setTimeStamp(0.908391301056);
    msg.setSource(3880U);
    msg.setSourceEntity(80U);
    msg.setDestination(50215U);
    msg.setDestinationEntity(107U);
    msg.control_src = 54859U;
    msg.control_ent = 196U;
    IMC::Reference tmp_msg_0;
    tmp_msg_0.flags = 50U;
    IMC::DesiredSpeed tmp_tmp_msg_0_0;
    tmp_tmp_msg_0_0.value = 0.895429864577;
    tmp_tmp_msg_0_0.speed_units = 49U;
    tmp_msg_0.speed.set(tmp_tmp_msg_0_0);
    IMC::DesiredZ tmp_tmp_msg_0_1;
    tmp_tmp_msg_0_1.value = 0.915714306051;
    tmp_tmp_msg_0_1.z_units = 221U;
    tmp_msg_0.z.set(tmp_tmp_msg_0_1);
    tmp_msg_0.lat = 0.990502026833;
    tmp_msg_0.lon = 0.98389788768;
    tmp_msg_0.radius = 0.163561235879;
    msg.reference.set(tmp_msg_0);
    msg.state = 197U;
    msg.proximity = 42U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowRefState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FollowRefState msg;
    msg.setTimeStamp(0.88306608564);
    msg.setSource(41756U);
    msg.setSourceEntity(17U);
    msg.setDestination(16342U);
    msg.setDestinationEntity(186U);
    msg.control_src = 40512U;
    msg.control_ent = 24U;
    IMC::Reference tmp_msg_0;
    tmp_msg_0.flags = 127U;
    IMC::DesiredSpeed tmp_tmp_msg_0_0;
    tmp_tmp_msg_0_0.value = 0.15193340832;
    tmp_tmp_msg_0_0.speed_units = 181U;
    tmp_msg_0.speed.set(tmp_tmp_msg_0_0);
    IMC::DesiredZ tmp_tmp_msg_0_1;
    tmp_tmp_msg_0_1.value = 0.129636246746;
    tmp_tmp_msg_0_1.z_units = 97U;
    tmp_msg_0.z.set(tmp_tmp_msg_0_1);
    tmp_msg_0.lat = 0.983658847115;
    tmp_msg_0.lon = 0.603043347173;
    tmp_msg_0.radius = 0.0564067745652;
    msg.reference.set(tmp_msg_0);
    msg.state = 164U;
    msg.proximity = 227U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FollowRefState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleState msg;
    msg.setTimeStamp(0.564369306463);
    msg.setSource(27321U);
    msg.setSourceEntity(112U);
    msg.setDestination(62257U);
    msg.setDestinationEntity(240U);
    msg.op_mode = 252U;
    msg.error_count = 155U;
    msg.error_ents.assign("UDCIQTOFQXXTAEYXZJAFOJCLVJKALEFRBZBEDSNTFSEHMVYKKHWPMTAJXBFSMSCQY");
    msg.maneuver_type = 12946U;
    msg.maneuver_stime = 0.167711858502;
    msg.maneuver_eta = 51103U;
    msg.control_loops = 823187601U;
    msg.flags = 180U;
    msg.last_error.assign("VJFXWJGHRLFVGJTYENNVKDHZLASCFUWEXISGT");
    msg.last_error_time = 0.357345868645;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleState msg;
    msg.setTimeStamp(0.514510555581);
    msg.setSource(55896U);
    msg.setSourceEntity(206U);
    msg.setDestination(44153U);
    msg.setDestinationEntity(229U);
    msg.op_mode = 206U;
    msg.error_count = 212U;
    msg.error_ents.assign("LVDOBTFSBRGYQHHGGPNCIZYZCJEPYUJOBAXADOPRF");
    msg.maneuver_type = 64359U;
    msg.maneuver_stime = 0.405838918427;
    msg.maneuver_eta = 50658U;
    msg.control_loops = 2605041170U;
    msg.flags = 195U;
    msg.last_error.assign("ENBJDALUVJMRMCVYFKILTBOWTNPCIAXVPIXEUGSXYCCIQYKZEYPTWXGQEHQHAELSNNSLAQRAMGHCBZDNJABSORMIMVPEXLNILNOQOCKUUSZSFZUJMQZWNGGQ");
    msg.last_error_time = 0.355272126575;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleState msg;
    msg.setTimeStamp(0.989622125728);
    msg.setSource(58844U);
    msg.setSourceEntity(206U);
    msg.setDestination(16407U);
    msg.setDestinationEntity(62U);
    msg.op_mode = 193U;
    msg.error_count = 97U;
    msg.error_ents.assign("GPUEBZILTIXJYXTHJGGZDTYZDBYCTBCQBMDGILCYKNHNAJHO");
    msg.maneuver_type = 11946U;
    msg.maneuver_stime = 0.593947844682;
    msg.maneuver_eta = 40713U;
    msg.control_loops = 4034181466U;
    msg.flags = 199U;
    msg.last_error.assign("OEOYVGCLNFHACSABTOLVFJRJBUFDXYPABUBHSZROZPOSLCNBJDOLJUASQQNHWMIFJMNQXAIXZNCL");
    msg.last_error_time = 0.874537360713;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleCommand msg;
    msg.setTimeStamp(0.632949682804);
    msg.setSource(48496U);
    msg.setSourceEntity(161U);
    msg.setDestination(12569U);
    msg.setDestinationEntity(6U);
    msg.type = 12U;
    msg.request_id = 9533U;
    msg.command = 80U;
    IMC::PopUp tmp_msg_0;
    tmp_msg_0.timeout = 64885U;
    tmp_msg_0.lat = 0.419341294772;
    tmp_msg_0.lon = 0.867352142341;
    tmp_msg_0.z = 0.344289832009;
    tmp_msg_0.z_units = 79U;
    tmp_msg_0.speed = 0.448311395784;
    tmp_msg_0.speed_units = 107U;
    tmp_msg_0.duration = 28095U;
    tmp_msg_0.radius = 0.179964675107;
    tmp_msg_0.flags = 129U;
    tmp_msg_0.custom.assign("AWQYDXYLOMTABBOLKPYTVAJXCQFWWGMGZSVJHOVHXJZUFDHPFEDXTLJGMTNRCANMMBNTUXBFKWCUJECIRHVWEHJQSTSUWNWEWVYSKEJYIGPGIEYPWKKGZDKARJIPKSXAUKHGZOGQXFLBTDNALNGVIFEEBITVLRFEUBZNPQ");
    msg.maneuver.set(tmp_msg_0);
    msg.calib_time = 20326U;
    msg.info.assign("QGKTXGIPRNJDBNMLFUAUNGIOIYNSLHMNYQCAYAOSMZXZAVUSEYNRWWULALDBCAGAORCJPFOOHCNKQQJEJQVSPNXAGRRCYGJTUIVKWMGFBKOWTRHXBKOMZYDYKLBJUWKMTFIEVIWFICDCIFZXCQNRVQSBADSSHJBVTWJLCDFUPTNOUEXVLLEOZLTMFRZDPDXMVKHQKYGESYIHCEEWARZPFHWETUSHXPIHKXZXPVMOTPGBUMWHGTDEJJQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleCommand #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleCommand msg;
    msg.setTimeStamp(0.806244122789);
    msg.setSource(10167U);
    msg.setSourceEntity(27U);
    msg.setDestination(41626U);
    msg.setDestinationEntity(80U);
    msg.type = 213U;
    msg.request_id = 26174U;
    msg.command = 78U;
    IMC::PopUp tmp_msg_0;
    tmp_msg_0.timeout = 59534U;
    tmp_msg_0.lat = 0.514442190121;
    tmp_msg_0.lon = 0.384866727868;
    tmp_msg_0.z = 0.817760720235;
    tmp_msg_0.z_units = 137U;
    tmp_msg_0.speed = 0.0637225200968;
    tmp_msg_0.speed_units = 150U;
    tmp_msg_0.duration = 21687U;
    tmp_msg_0.radius = 0.942718496156;
    tmp_msg_0.flags = 145U;
    tmp_msg_0.custom.assign("KYUFZFYDPHVNVVVEFTGOOLBQUCXFDZSTHVGCAZXCUPRHMMUBTNWRALDGDBQNISVX");
    msg.maneuver.set(tmp_msg_0);
    msg.calib_time = 5184U;
    msg.info.assign("HUQVNNOCENHSVTYFDCXJMNXGRMGUOKZCXQBJCIBOZNPGXCQMRVXYCMVGOYEWMPESKQVFKEJZDPYSQWUUKKASDSLMLHCLKWWYDAILAHWXVGFROSYBEUMJGENQGBPTIKLYROHGWVRUOMZXAAFZCBHUSABDPANSWAIJWUTATNUKBZDSVFJRPESAOXZMYIFTDZP");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleCommand #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleCommand msg;
    msg.setTimeStamp(0.371181504804);
    msg.setSource(33865U);
    msg.setSourceEntity(178U);
    msg.setDestination(24728U);
    msg.setDestinationEntity(181U);
    msg.type = 187U;
    msg.request_id = 45917U;
    msg.command = 81U;
    IMC::FollowTrajectory tmp_msg_0;
    tmp_msg_0.timeout = 54259U;
    tmp_msg_0.lat = 0.320665003447;
    tmp_msg_0.lon = 0.63068975554;
    tmp_msg_0.z = 0.577696029116;
    tmp_msg_0.z_units = 186U;
    tmp_msg_0.speed = 0.780543595932;
    tmp_msg_0.speed_units = 242U;
    tmp_msg_0.custom.assign("PKAKUNFTSATEJKJOFENODFNUMKRNXHYJEADIQYYAFVKEECBAGYRYQFCRPMXHWGWOJZCWBSHZSZLISLIRTUHWRZSBVEOVLTDDAOBXDGIYOLSXMJBTNITRSBGVRDQHGJUCDTRKWM");
    msg.maneuver.set(tmp_msg_0);
    msg.calib_time = 6930U;
    msg.info.assign("FOIEPQTASNJYKHPGHXLJOTPSJMIEJPXJJUIRLALOMYHXRMDMRWDUXVNMJFLGXIKCPZGZRUZVWSAF");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleCommand #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MonitorEntityState msg;
    msg.setTimeStamp(0.934596779011);
    msg.setSource(21734U);
    msg.setSourceEntity(59U);
    msg.setDestination(62506U);
    msg.setDestinationEntity(88U);
    msg.command = 87U;
    msg.entities.assign("HZVIKITTAONVPFOPTTARIWPUVNJAUBPFGRAYKHSTZFYWDDHFDFYIHDKOJHVPGGEVEUKQBBIKVLJREORSNIIMJQTLFSDSZSDGUXCVLNXNSKMNJRWCYRAGPLLQGEVZOAXBWDOISUBBDRLPPCOEFLDQWKXSEVCFMHEQQATZEYWKZNJHGXVWWPCUBGYWTYCDBFLYGAQXEOIYJNEHZCSX");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MonitorEntityState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MonitorEntityState msg;
    msg.setTimeStamp(0.428685663346);
    msg.setSource(35600U);
    msg.setSourceEntity(150U);
    msg.setDestination(50146U);
    msg.setDestinationEntity(228U);
    msg.command = 122U;
    msg.entities.assign("RSAKUBUPSQDDVMIQCMQRIKEEHZRIGZNNOJBFKRCQFRWPZBSGYWCDXSUMCOWDVGEEEGKEHENKVVVYLGFQZULLTSNMAOKMMOYTXNWCFAHCJEYZRAQUPWATHMJMOGVIUSYCYRSUCJOJPHHXPRNZQSNZIDXUXBXYABUBFRHNBJAVDPVMXDKZAXQFXPQFHIPONGITRTCDBJHSMGFVJHJCZYWLSAYKOAKWJULVWWNTPT");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MonitorEntityState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MonitorEntityState msg;
    msg.setTimeStamp(0.452404673999);
    msg.setSource(27551U);
    msg.setSourceEntity(72U);
    msg.setDestination(3174U);
    msg.setDestinationEntity(110U);
    msg.command = 187U;
    msg.entities.assign("NKGIJKPREZACOBPTTVNDAWYVIQNSYXKOBFWKMRRRYOUKXWZBMUTOXUGUSIJFBHOMEPUTDVLDAFBFBFNZNQLJLMKLWLCJCRHKHKYGTZKQTBQNRXGSJTQECLQDESAPYCXWBLYTZBZOHXRDCYZMRAIESMHJSPVOUGCXUFIAFVQBDYPESGDTVENQOPIAGZNRXVAOFLRMCPHONWPWYHKHCEQLNWDYMWZGIULZQCTFWMJ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MonitorEntityState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityMonitoringState msg;
    msg.setTimeStamp(0.81080181237);
    msg.setSource(61091U);
    msg.setSourceEntity(132U);
    msg.setDestination(8419U);
    msg.setDestinationEntity(168U);
    msg.mcount = 231U;
    msg.mnames.assign("EBNQMXCLOPHGOWCWZIQFRUCGGRLZMAIZXRZLOVAERWJDVTZLPJPXPZCMTKLEJSHMQYENPWJVSGBAMUXNTHR");
    msg.ecount = 58U;
    msg.enames.assign("FCEKIZWFUTMBDR");
    msg.ccount = 246U;
    msg.cnames.assign("TNITSANUPXZVADLYECVDDMEMBPUTQCOJWMMFREPGNJBPT");
    msg.last_error.assign("EISVLKERJNSFWGFQCZSTAZECOAYJUBIHBQNUHXMYTXTBUXIYNEWJVJBIERFMKYQQNPNWFOIQISVNGRHCDQLZDDQCYHHULBRGEFCSMYXZAADEZMKWCMNWYUCUFJOWKVRMCXQNGSKZOUSROHVMUTZGLDSETHCVWLVADNSJIEJBORPNGQIIVYZJXRBXEPTKPWIXOTTTWVGPKAVQZRLABFFJZRFJHPHGUGDYUBKDLDSBPLMKFXYMA");
    msg.last_error_time = 0.922884363199;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityMonitoringState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityMonitoringState msg;
    msg.setTimeStamp(0.0311126552543);
    msg.setSource(22691U);
    msg.setSourceEntity(212U);
    msg.setDestination(5463U);
    msg.setDestinationEntity(73U);
    msg.mcount = 232U;
    msg.mnames.assign("FCJDSQOVQNNZUHCWZNHVWTFPEXHZFPVKURXJOLXFAEBMFYFCMEWDDNPLJIADGQQMIXKTMYYKAEGGSLEIQYSSLRJPRAJBNAAPOAMCVXOHSMVYOGTRHWMGNSYFLQPBGCCBJRCVIJWFSKVWZGXTUDLZKISQYZPPXOTUJEYBDPBXWBIGGBRTEZMIXKRVTUMFFTLADQTNUAURVZBKDUHCNYOQKZTSKAHCLBDSMIUIHQECZVRENGUDKXH");
    msg.ecount = 22U;
    msg.enames.assign("FICVGGAJKAUHTKIJEENEATLKRNXIPQRUUZXQSBHCRTBDEYEXGWAMWKLMZQMVFWNBOEBUHTSUOEBVDNQKIRVNFHZJJZTR");
    msg.ccount = 195U;
    msg.cnames.assign("RAVKQKINIIVXXWRGWEYUFLMFPBNADMWDWLUQARVMJKGYKZSCMGTXSKUCIAGOFPJAEFNPATN");
    msg.last_error.assign("VUQQPEBUCJEBJXMCOONFBCADQRGMQYKHKRKGPJIXYWFWQBWCCAXATLOSKVWOLAIRQIENF");
    msg.last_error_time = 0.764683104113;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityMonitoringState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityMonitoringState msg;
    msg.setTimeStamp(0.336657616721);
    msg.setSource(32825U);
    msg.setSourceEntity(22U);
    msg.setDestination(763U);
    msg.setDestinationEntity(120U);
    msg.mcount = 156U;
    msg.mnames.assign("DOXSKVNRCOTWRPGBUOMHTGETNNOWZQGJNZMUXKYZLWPLVVFBKQPAPSJXITKVFMPRMEJIROULBPYNJJLKPSUBUQZVHAMDTFFFWEAHSTDEHBOSKGURGFBEIJNOTFSXYFVRAVLVVXXOEXQGFRBBRWWSHYDMLAKNJWEAY");
    msg.ecount = 6U;
    msg.enames.assign("TUGUOFXBINGSRVGFXDRBITQLFZTTAOAWXKNVZOXNPVJTHWULOMTVWAWXOKBABVMPSSBMNEZFZVAZSAOVJKCYRXPREUUDPJNXRVLFCCTQNSNJHJPKLQWTOUCFYHEDMIHSIEKMDENGEOMDECAVXJZRSCDQIMYDQAJZPFPHI");
    msg.ccount = 62U;
    msg.cnames.assign("LHDFJMPKLSAOPSAHDGFQZXYURZCBXLFMECEZCBTLFGPRWGJBNUIOAHHUMCTKEOPXCAYLPUPXHRNKJNQTSFMGWVEXTUWRTIDUSFOBVHNMSKGPYCLYLGJSCJJBWXYRVXIXOFKHUZJETNSJBTOGDKFGIQBXZVYAJWERBQIQZNIRQROQMWAEMWOFLVVFDCUQHEIADKVYYDKUVLVMCIAZNSDCWMDE");
    msg.last_error.assign("MKRCSQEZBOFFOHLQIFAGOTXFQUEBXWETXVGBORJBMUTCZHKFIJDXSZGNDZEHIAASMYFANQJTWCKIHVPCYUCKGEFNQALEVVDBGSSJUMXINGTYBLNDKRPNXITWLDYKJDJCEGDHRHGVQUYCCHABTJPUKZTEVZSBQIPVRRSNORSILDVVYHPRSJFT");
    msg.last_error_time = 0.211811756913;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityMonitoringState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::OperationalLimits msg;
    msg.setTimeStamp(0.628710837615);
    msg.setSource(708U);
    msg.setSourceEntity(117U);
    msg.setDestination(33970U);
    msg.setDestinationEntity(78U);
    msg.mask = 36U;
    msg.max_depth = 0.456436120707;
    msg.min_altitude = 0.587025638386;
    msg.max_altitude = 0.881477705337;
    msg.min_speed = 0.113343552289;
    msg.max_speed = 0.286082596198;
    msg.max_vrate = 0.46086921951;
    msg.lat = 0.413643590741;
    msg.lon = 0.182268940394;
    msg.orientation = 0.187765949759;
    msg.width = 0.964389301883;
    msg.length = 0.124235261269;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("OperationalLimits #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::OperationalLimits msg;
    msg.setTimeStamp(0.242410160527);
    msg.setSource(3831U);
    msg.setSourceEntity(133U);
    msg.setDestination(49564U);
    msg.setDestinationEntity(235U);
    msg.mask = 214U;
    msg.max_depth = 0.0622194674161;
    msg.min_altitude = 0.669289973763;
    msg.max_altitude = 0.764766801257;
    msg.min_speed = 0.671770165525;
    msg.max_speed = 0.911301435771;
    msg.max_vrate = 0.346784766991;
    msg.lat = 0.58210438962;
    msg.lon = 0.341481989557;
    msg.orientation = 0.106742584771;
    msg.width = 0.924295643912;
    msg.length = 0.0389254231801;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("OperationalLimits #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::OperationalLimits msg;
    msg.setTimeStamp(0.754608110253);
    msg.setSource(22225U);
    msg.setSourceEntity(187U);
    msg.setDestination(4639U);
    msg.setDestinationEntity(139U);
    msg.mask = 221U;
    msg.max_depth = 0.594402896073;
    msg.min_altitude = 0.559017011907;
    msg.max_altitude = 0.798343606649;
    msg.min_speed = 0.213400717952;
    msg.max_speed = 0.435808528899;
    msg.max_vrate = 0.658446286892;
    msg.lat = 0.231606730413;
    msg.lon = 0.678196762791;
    msg.orientation = 0.405505325029;
    msg.width = 0.960169872199;
    msg.length = 0.39435129821;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("OperationalLimits #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GetOperationalLimits msg;
    msg.setTimeStamp(0.63415198805);
    msg.setSource(42408U);
    msg.setSourceEntity(49U);
    msg.setDestination(15785U);
    msg.setDestinationEntity(30U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GetOperationalLimits #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GetOperationalLimits msg;
    msg.setTimeStamp(0.661879788993);
    msg.setSource(33477U);
    msg.setSourceEntity(156U);
    msg.setDestination(1728U);
    msg.setDestinationEntity(251U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GetOperationalLimits #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::GetOperationalLimits msg;
    msg.setTimeStamp(0.624367323985);
    msg.setSource(58161U);
    msg.setSourceEntity(26U);
    msg.setDestination(54308U);
    msg.setDestinationEntity(144U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("GetOperationalLimits #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Calibration msg;
    msg.setTimeStamp(0.800279778344);
    msg.setSource(46063U);
    msg.setSourceEntity(78U);
    msg.setDestination(18978U);
    msg.setDestinationEntity(139U);
    msg.duration = 9899U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Calibration #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Calibration msg;
    msg.setTimeStamp(0.647233063583);
    msg.setSource(2340U);
    msg.setSourceEntity(137U);
    msg.setDestination(35078U);
    msg.setDestinationEntity(140U);
    msg.duration = 35055U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Calibration #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Calibration msg;
    msg.setTimeStamp(0.153350415671);
    msg.setSource(42282U);
    msg.setSourceEntity(166U);
    msg.setDestination(26509U);
    msg.setDestinationEntity(229U);
    msg.duration = 18148U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Calibration #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ControlLoops msg;
    msg.setTimeStamp(0.431983997145);
    msg.setSource(52703U);
    msg.setSourceEntity(2U);
    msg.setDestination(45482U);
    msg.setDestinationEntity(76U);
    msg.enable = 172U;
    msg.mask = 205318797U;
    msg.scope_ref = 3410843561U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ControlLoops #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ControlLoops msg;
    msg.setTimeStamp(0.814601345959);
    msg.setSource(57551U);
    msg.setSourceEntity(37U);
    msg.setDestination(10331U);
    msg.setDestinationEntity(22U);
    msg.enable = 9U;
    msg.mask = 143576844U;
    msg.scope_ref = 2437935515U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ControlLoops #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ControlLoops msg;
    msg.setTimeStamp(0.68120416945);
    msg.setSource(6404U);
    msg.setSourceEntity(77U);
    msg.setDestination(45378U);
    msg.setDestinationEntity(145U);
    msg.enable = 178U;
    msg.mask = 2117490591U;
    msg.scope_ref = 3923604898U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ControlLoops #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleMedium msg;
    msg.setTimeStamp(0.422232316531);
    msg.setSource(35753U);
    msg.setSourceEntity(60U);
    msg.setDestination(36398U);
    msg.setDestinationEntity(186U);
    msg.medium = 216U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleMedium #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleMedium msg;
    msg.setTimeStamp(0.21990846807);
    msg.setSource(52506U);
    msg.setSourceEntity(167U);
    msg.setDestination(32020U);
    msg.setDestinationEntity(104U);
    msg.medium = 135U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleMedium #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleMedium msg;
    msg.setTimeStamp(0.23102590805);
    msg.setSource(38018U);
    msg.setSourceEntity(18U);
    msg.setDestination(27747U);
    msg.setDestinationEntity(13U);
    msg.medium = 56U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleMedium #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Collision msg;
    msg.setTimeStamp(0.649488236089);
    msg.setSource(11964U);
    msg.setSourceEntity(86U);
    msg.setDestination(41092U);
    msg.setDestinationEntity(3U);
    msg.value = 0.0942897546076;
    msg.type = 241U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Collision #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Collision msg;
    msg.setTimeStamp(0.652333669986);
    msg.setSource(19390U);
    msg.setSourceEntity(161U);
    msg.setDestination(58371U);
    msg.setDestinationEntity(207U);
    msg.value = 0.441017655958;
    msg.type = 40U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Collision #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Collision msg;
    msg.setTimeStamp(0.610819059);
    msg.setSource(4117U);
    msg.setSourceEntity(233U);
    msg.setDestination(54377U);
    msg.setDestinationEntity(38U);
    msg.value = 0.616079685386;
    msg.type = 23U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Collision #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormState msg;
    msg.setTimeStamp(0.243953945862);
    msg.setSource(45711U);
    msg.setSourceEntity(76U);
    msg.setDestination(59250U);
    msg.setDestinationEntity(196U);
    msg.possimerr = 0.994887409572;
    msg.converg = 0.898257589342;
    msg.turbulence = 0.651006640015;
    msg.possimmon = 68U;
    msg.commmon = 127U;
    msg.convergmon = 116U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormState msg;
    msg.setTimeStamp(0.231419802489);
    msg.setSource(60995U);
    msg.setSourceEntity(231U);
    msg.setDestination(3256U);
    msg.setDestinationEntity(140U);
    msg.possimerr = 0.482446093642;
    msg.converg = 0.515048674238;
    msg.turbulence = 0.915620948415;
    msg.possimmon = 42U;
    msg.commmon = 90U;
    msg.convergmon = 175U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormState msg;
    msg.setTimeStamp(0.436209169543);
    msg.setSource(62789U);
    msg.setSourceEntity(13U);
    msg.setDestination(49027U);
    msg.setDestinationEntity(83U);
    msg.possimerr = 0.905403314007;
    msg.converg = 0.882749887537;
    msg.turbulence = 0.262491733134;
    msg.possimmon = 206U;
    msg.commmon = 242U;
    msg.convergmon = 117U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ArduPilotState msg;
    msg.setTimeStamp(0.458457434637);
    msg.setSource(62968U);
    msg.setSourceEntity(245U);
    msg.setDestination(55914U);
    msg.setDestinationEntity(6U);
    msg.type = 146U;
    msg.mode.assign("BNRGYEBTLIWAHTXBWQQTS");
    msg.armed = 79U;
    msg.other.assign("SGZOQCVULMZMDQZGWJC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ArduPilotState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ArduPilotState msg;
    msg.setTimeStamp(0.980510406062);
    msg.setSource(31831U);
    msg.setSourceEntity(42U);
    msg.setDestination(48454U);
    msg.setDestinationEntity(244U);
    msg.type = 203U;
    msg.mode.assign("XTRANWZEZPPIGKOMRITNHVTUCIYPNTJBKZZQFQGMDHZVEXPTVRDTYZFLAJKKUNDSMZXU");
    msg.armed = 56U;
    msg.other.assign("THKFIADPRNDWRRMNTNWVHALAKESJOVXYTZSGWMZRZFRQZWIGFVCJNRISDOJUUAOUCMMJANXGMGTJQCQSIHLEDKISPBFDIDRXTFKBUXITIEECKHCFDBRRCFSPBOEWGYKLOHQOPFNBWZGIHOTZVBKAPQYJWWZUQCFLLEL");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ArduPilotState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ArduPilotState msg;
    msg.setTimeStamp(0.370638195307);
    msg.setSource(59019U);
    msg.setSourceEntity(145U);
    msg.setDestination(29350U);
    msg.setDestinationEntity(180U);
    msg.type = 217U;
    msg.mode.assign("CPZRXNWVVZRMZJMBLZHYKWEXKADTNQOCYWZQFMIQZCGOPUBFHQHGKUVPOHJDOASUFCVLUYNJHKEOPKCFRCEZZDUOABSXZHAABLUGYIRSLPLXLYYGWKMDIFEWSFKBBLHNOSIUNIWJDJDLRETWWPAVOYRJMADISKUXFVSBMQXDNBOSTNCRJNQHYAGHVQNJWPYRTR");
    msg.armed = 78U;
    msg.other.assign("GEUGCCYHYNGQYSPDMIWBEMBRYJOMOUASPXTQMNFFWSDJXKDTYNIUBHEORHUXCTJQSPBXELFFVIOCLSKAJVVCQLGFLNZCUDQNT");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ArduPilotState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Abort msg;
    msg.setTimeStamp(0.304081208387);
    msg.setSource(57776U);
    msg.setSourceEntity(191U);
    msg.setDestination(39218U);
    msg.setDestinationEntity(166U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Abort #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Abort msg;
    msg.setTimeStamp(0.00852198034093);
    msg.setSource(28609U);
    msg.setSourceEntity(181U);
    msg.setDestination(47139U);
    msg.setDestinationEntity(125U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Abort #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Abort msg;
    msg.setTimeStamp(0.262725382425);
    msg.setSource(13858U);
    msg.setSourceEntity(248U);
    msg.setDestination(25408U);
    msg.setDestinationEntity(170U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Abort #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanSpecification msg;
    msg.setTimeStamp(0.729688966545);
    msg.setSource(41484U);
    msg.setSourceEntity(231U);
    msg.setDestination(47090U);
    msg.setDestinationEntity(216U);
    msg.plan_id.assign("LTAMIOSDQRZGRNZPYEOIOXQPGTXAGVLUZQFIHHYBKQDMOMZJBCXJPFBVESSWRURFJTXKBICRZYWZYMOMVZGBTUHQWAAIUDHCEQNAMXGUEQRPULESKKHWOKSGGJHPDVJDJGTWCN");
    msg.description.assign("KWDIXSYLAKKZTABGFNRZHSWSRGIHIJULNVTGHQUVAIHACZFHKMQXPQKGNMLRUBYPKJKCNLFUGBIVJWSVSEWHZWBNOGSZOLCCWYXJHR");
    msg.vnamespace.assign("EEYJDVVEOAAKRVBHXLJBWHMTMXXZKGVRSHOMXBKPTBXBDGRLYV");
    IMC::PlanVariable tmp_msg_0;
    tmp_msg_0.name.assign("NWVMSYDRMZTZIYVOBRNNLKWXOLRCOXHOMAJCGJLHJEBUSWODDWSOYFZNPXRCHTFEEBBTDFTTEEKISNZMRNQRVPFOGQCWXMOQXLXBZWILVEZYJWGBVGUXAXILDPKPMJFAAIYQEAWVKBFQRKLLUPPGCVNQYHVANEZEFUPDTHYSNSNFHKOZZAMEHTJGLXAGYBLFIUOPRYDBMMBYIXSMCJTKPCQAIKVDSGDFRUUWSUCZIKHUTC");
    tmp_msg_0.value.assign("QYHXZOGOSNJKAJPLOXWKSHYLIWMRMTFSKDGUGIJLLHUCAFIRJZZOOPWXVMCOFVDGQCELIMJTIAZNSBPCPMCNIJVPHRSLQHEXGPTRWWXDHWAUEXYNFPVNCUQJRKNFSZQIBBNYAVZUNMOVGSUKLRBDTRZBDD");
    tmp_msg_0.type = 207U;
    tmp_msg_0.access = 149U;
    msg.variables.push_back(tmp_msg_0);
    msg.start_man_id.assign("JOYEZLZGYELGYAHWLKMHSTOKBOIPCFQFORNNKLUTRUCCSUQSVETPWBKAXCVFUZITHSSEKYTDBRBUJDNTHAJKSHAOSPRCIVDIHJUICCQPGDVYEMIUMMDUYBNQMBTHQZ");
    IMC::RemoteSensorInfo tmp_msg_1;
    tmp_msg_1.id.assign("YTKRMRJHSYFMREWGUAORZGOVQDGIOCQDEVXVIOGZUYIETNANLXIGSXVZQIDJGHKWEKYSTULFKRVYSFLYYNXPVSXTABNVBSOGIJDCHK");
    tmp_msg_1.sensor_class.assign("NMIVCPAWLWITMLGLTNYCKEZKMIUGKDLIZZRDXXDZFVLHUSOHZUNIMXER");
    tmp_msg_1.lat = 0.384397402866;
    tmp_msg_1.lon = 0.32730532402;
    tmp_msg_1.alt = 0.881620543316;
    tmp_msg_1.heading = 0.622040708198;
    tmp_msg_1.data.assign("HFYAIVPWATDPUVUWXSKSRGWPCDHQVLZRRDMONMNBMOIJHBFFIVXEYFLMPSLSESQCSDUZEQECKVQDRVGXEJOFKYSHWNNIZQZBGEFTEQBPXCWJYAJBLTHBIAODYGLZVEAQLNNYJPFLGOUZWHNFVYXXVEAKIGTIOHLJUOSYLICBEGWFZKTCRKYMQKCKAZFCTOXX");
    msg.start_actions.push_back(tmp_msg_1);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanSpecification #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanSpecification msg;
    msg.setTimeStamp(0.237425217487);
    msg.setSource(64771U);
    msg.setSourceEntity(191U);
    msg.setDestination(1377U);
    msg.setDestinationEntity(136U);
    msg.plan_id.assign("KGKTGLFMWLLCIIOFYGBCLGUTTHSMGZSOAZEYCWSFUDDWCLCFZODDBMYPIJJXNDQBDBNPZDVNUAQLHNIZXYKWIZEMDBHINPBWUZFWYRPLBWSUJAAOGTGKPQHQAOICXQXBMTDORWOFXCESTEWYZRQRYVDBFOMXKRVUNNYFAHHHAHMCLIPIULXLEVJSMKRGCFUEKSJERAGM");
    msg.description.assign("BYXOKGTASBSWXMGIIQZPQTEMYJFQZUNSWSTPUUFPZWBOCYIEOUVOMQFXACLQDHTPGBXTROCFFHLWERPROJOGJDQHWCWGIPNTRIVFYMYVLKFLZGDAMXXAKIGJHNEUJGFCKPDUAQIPMLSLVBWKALDZQEGEZTYSHFAWVBRUJJXDCQOZK");
    msg.vnamespace.assign("FVWWDHUIEMHXAHNKSKQVDAVLCOIYPAQXQUNZOWTGBOXUFFXZRJQOJNAKDMEUAERZBNFOWPREACMEGJRGLZBSTCVDSVOMWPWSKSLOKGBEHEVHITHOMJFYIUTGWCJZZGURDLDTMZJEXZNFINUYQPLHCTZXWIYPNQBJVPLZMEDWFSRVHATAXIIPGLGFSKKAOCLKCUQNTPVGYOXLBWBRRIYRKGDRSQBCSYUCDMBMXINSDNTQPKHCLUYBVQXHJ");
    msg.start_man_id.assign("XJUCHYJQJSJPXEHHPMWTRWROQKQAVCWAHAHTLJMZSECDOECWUSNBBBN");
    IMC::EstimatedStreamVelocity tmp_msg_0;
    tmp_msg_0.x = 0.699620385534;
    tmp_msg_0.y = 0.357955614117;
    tmp_msg_0.z = 0.159756370265;
    msg.start_actions.push_back(tmp_msg_0);
    IMC::LeakSimulation tmp_msg_1;
    tmp_msg_1.op = 201U;
    tmp_msg_1.entities.assign("YDURFGMCNXFIPXRKTCRGRGCSJSKNIQLYMDDPWESFEWPPZSGWMONFKIMQBLYAIITWTGYBUTOUZRCVJCRIP");
    msg.end_actions.push_back(tmp_msg_1);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanSpecification #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanSpecification msg;
    msg.setTimeStamp(0.557273514492);
    msg.setSource(27673U);
    msg.setSourceEntity(115U);
    msg.setDestination(49182U);
    msg.setDestinationEntity(146U);
    msg.plan_id.assign("WHYBBWGRPVGYXBRQMXILYBRHWJYWDIUASOQLBUNAZCHAVQDPZBNENTWDAOTDDODUSJTJVLQYURKTOHLGOEXVVPHEAZZTJOPMVAULOMGSOZRHPJDQHCYNMLKRFNJLYDEKRYBZFZXDQUNNQKIJJQCILLFSZCEPFCSIPIYXAKWEANFMCWFEXENOVCSRMGVOZKVYAEEKCGUTCSHXMJTGCQHUITPWXXPUSUGBSHJTGDWBPMTXVNFBKKRLRIFFFIWQG");
    msg.description.assign("ZSMBEPABFFETVFJIPOUMWDXRKSHKILHIGQ");
    msg.vnamespace.assign("SFVLDTQIOLMVUUZBLGRMXXKVNXSTYEIDQOENJFCAXLHCWZRIGKXSIUIMGWFCCWYJSCGNXGKUADCJOCMJOQYXYHKJDZQNAJNSBDLKGZMIZDBEBRLVSKUTHVTJEQZGIFBCGULTDOULUHOFOAONHZZTEADPEF");
    msg.start_man_id.assign("EVZDNBBIDUXAGQDAGPUTTKQRINSWMRCUGCFKQNDCFVUZRBRPWPRMDJKDQSXOZETEOAUDXKVNIWKJRMLEYQCLQKQTFLHJWBBJFCZEBDLSWGWYYFRSIJNCWTAEXSLPGYENLUBNYHZYEAGFUSHGFVHVZVVIFHVPGFZJOXSSIIATNCPDO");
    IMC::PlanManeuver tmp_msg_0;
    tmp_msg_0.maneuver_id.assign("LSRMWPDNBNG");
    IMC::LowLevelControl tmp_tmp_msg_0_0;
    IMC::DesiredRoll tmp_tmp_tmp_msg_0_0_0;
    tmp_tmp_tmp_msg_0_0_0.value = 0.298748269838;
    tmp_tmp_msg_0_0.control.set(tmp_tmp_tmp_msg_0_0_0);
    tmp_tmp_msg_0_0.duration = 39651U;
    tmp_tmp_msg_0_0.custom.assign("OVQXPMVEUACEQGXEIEYOQDXUNKFKMLHBQBFTQZOTUVFJUPMXBPSRWNNISRGDCMGQIRIAGFDOWRNKLKHHWJOOUCK");
    tmp_msg_0.data.set(tmp_tmp_msg_0_0);
    IMC::QueryEntityState tmp_tmp_msg_0_1;
    tmp_msg_0.start_actions.push_back(tmp_tmp_msg_0_1);
    IMC::LblDetection tmp_tmp_msg_0_2;
    tmp_tmp_msg_0_2.tx = 14U;
    tmp_tmp_msg_0_2.channel = 215U;
    tmp_tmp_msg_0_2.timer = 6704U;
    tmp_msg_0.end_actions.push_back(tmp_tmp_msg_0_2);
    msg.maneuvers.push_back(tmp_msg_0);
    IMC::AllocatedControlTorques tmp_msg_1;
    tmp_msg_1.k = 0.817948462227;
    tmp_msg_1.m = 0.70398624345;
    tmp_msg_1.n = 0.348852193636;
    msg.start_actions.push_back(tmp_msg_1);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanSpecification #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanManeuver msg;
    msg.setTimeStamp(0.832030029887);
    msg.setSource(57763U);
    msg.setSourceEntity(231U);
    msg.setDestination(3722U);
    msg.setDestinationEntity(63U);
    msg.maneuver_id.assign("DLDEZYMPCVIPSGWIOXEPQOJGODJZBHWTPKGZXDCXTETZUWMMZSKAWBSJPYXFYKUGFFITRJDIYWUTRTSDBVHARHHKFEMBILELDDCYQRIMISVVGASTTUDBNHUSHUJBENIUEAWBRNNGCZFJXYXWTFHFNCPILXAOCVQFBMXQGHVQPLROMCORKJKKACOYCBAJLQOVONLSPNNPMGAQQDOCSHYYNVEVGLAXZA");
    IMC::Elevator tmp_msg_0;
    tmp_msg_0.timeout = 36650U;
    tmp_msg_0.flags = 130U;
    tmp_msg_0.lat = 0.387953760267;
    tmp_msg_0.lon = 0.608241228277;
    tmp_msg_0.start_z = 0.257295439101;
    tmp_msg_0.start_z_units = 49U;
    tmp_msg_0.end_z = 0.732121126134;
    tmp_msg_0.end_z_units = 12U;
    tmp_msg_0.radius = 0.474810431894;
    tmp_msg_0.speed = 0.670449194671;
    tmp_msg_0.speed_units = 150U;
    tmp_msg_0.custom.assign("FPIWJEBMYXRJJRDKOJOVWENULBSCEILVPMBJFZXXVYLYSQUAQUKAFXLTLAJVDLYKOXAGRMIPSDRMBHPETGHAOIJYFXCPHTEQOGCBZMIKEJUBGCHKZSOAGYNOKCRGQWUSCUGQNKMOHOKUVPVHTWDYGJBSVNVWRJPWHHMWLXSEIKTXRQYPNZCEDTPDAXTQDBUEUWTN");
    msg.data.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanManeuver #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanManeuver msg;
    msg.setTimeStamp(0.210862347484);
    msg.setSource(13976U);
    msg.setSourceEntity(5U);
    msg.setDestination(29388U);
    msg.setDestinationEntity(139U);
    msg.maneuver_id.assign("NUFYQQFHBMLPWOJZNXMMBXKKOXNRXLTBDIEYWISBCZGPUSTKAVHCHDWHAUIANJAQOQPXMTAECRNUNFCEZUTXKGFLTABSDDSQMHOPOVHZVWXKVQQIZCIQUWSGJCGINGHLUMGPXPMHQTFATUJZDSDFTDCBJLMBEWFRFKXNICDKYJYUYGRFPKTNGVJEVE");
    IMC::Teleoperation tmp_msg_0;
    tmp_msg_0.custom.assign("CSPMYFZMADKI");
    msg.data.set(tmp_msg_0);
    IMC::QueryEntityParameters tmp_msg_1;
    tmp_msg_1.name.assign("EXYDUIZCKZXTCZJTSFAUKVVQACROWFSWMIDPDMNDMWVWNMIOOHSXQOHRRBBNPNNOCCINNAMAGW");
    tmp_msg_1.visibility.assign("ZKHYBRPKUBCVTTEXFFQBEYUSTDQAOEGUPVPJSJXGCWYYJIMEQRAUJROZXMRVQGSKRARKUNWQWNUKIVYEFFJYTHSDCIQADGOOZGGBXPAIFWYXJNEFBQLZOXDHCZZQRKDHBUCWLLNWRWBIZLOKS");
    tmp_msg_1.scope.assign("EFZMQBSAJLEGTYVUBUULBPXOPDABPPYLCNCWAOCXHHFQMHMPFOIYZRYXVFPOEZKDHAVCRROIOFHDNBIWKRLJFTUXFCEFJGSVADLRXYEWNPSMSUKJVYDIYMDEOGPNBROWVQGZLFQKQBGOHTFJYUT");
    msg.end_actions.push_back(tmp_msg_1);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanManeuver #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanManeuver msg;
    msg.setTimeStamp(0.158322341327);
    msg.setSource(62407U);
    msg.setSourceEntity(150U);
    msg.setDestination(15219U);
    msg.setDestinationEntity(196U);
    msg.maneuver_id.assign("ELMPCEJVQIGJPILSUCVIMEWLBOVYKQRBLIXCNZXFLKINZRKTTMPDKFHOFIZBTFSIJPXEZOFCNPHXDOYPJUAOFTGWQOTDMVZPRCUXLARTULISVWBPMAZUGHWCXZHYOATNYMZXGMAHIPRIJQNTYXSAJNEWNSZBGHKULRBMNWVDDWZQUDDGKSSUJBWJNWLKSHEADEHMXCFVYUGGRDFQKQALHSYVHSKJDCOBBAFFQVGEBTXN");
    IMC::CommsRelay tmp_msg_0;
    tmp_msg_0.lat = 0.689616268315;
    tmp_msg_0.lon = 0.93801928794;
    tmp_msg_0.speed = 0.841652112365;
    tmp_msg_0.speed_units = 49U;
    tmp_msg_0.duration = 52126U;
    tmp_msg_0.sys_a = 3923U;
    tmp_msg_0.sys_b = 54332U;
    tmp_msg_0.move_threshold = 0.432752457322;
    msg.data.set(tmp_msg_0);
    IMC::AcousticPingReply tmp_msg_1;
    msg.start_actions.push_back(tmp_msg_1);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanManeuver #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanTransition msg;
    msg.setTimeStamp(0.0603073205289);
    msg.setSource(62111U);
    msg.setSourceEntity(240U);
    msg.setDestination(10106U);
    msg.setDestinationEntity(228U);
    msg.source_man.assign("QSGFXVQTNEQTLZSMBJWNWDENCOGLYBHDMUHZNRTLIDFXFEIOITRQRVNDCRYBWYWAKSJAQIYWSLXJOIKVTKHLNTBUXAXURYVXZABCMOPEUPXKCWZZJLGACNKYOPJABCFVDABHUTQOSSIQPDNWKHOUVRJZGEHGSKZNRPIOWRHAKERNPUTFZWFJBEIMUVKECMOPYSDFVQLCLWKRAIGVCXYHSMIPLXYHQ");
    msg.dest_man.assign("WAXFPYWPJFOPRUZGZDKKHOXRALPOKDFFGLGDCQEOUZTIRCEYGDQCVQLPYICVKTEXEYKXHTJGSXIVZLVASNQQCHKFRUBMIARUQIEJSBRXQDNVDOBLFMYIOLDQPJCASPFUJXLNVTNUSHLTWBVYQSMGONLPPBNAZOUTTZHZWGGMFKGYTEWVBSMDDTMSHWHIUWBR");
    msg.conditions.assign("GKLHIOCNJSQLRRIQNMVYDEXIWTUZUTWFTBYFXYNSQPRJGNOBXKWDFBGUSUJPVXDMAQVCGEWRZWRWRCFQLAAZDROVBUJELQGCOMZESDHGZDIPSKIBVVAYKBBTGJARUZLHMYLBQSHXZWSKUNHEHQFVVRPDTNDQIAGGNIMATPCHMYKPSJEPIIZMAJCVOUSOJEFWLKYYTTECPCNXBMOPQIODMYOWGKWTCUJOFZTKMFVRDCFZNFA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanTransition #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanTransition msg;
    msg.setTimeStamp(0.0278179463232);
    msg.setSource(9942U);
    msg.setSourceEntity(60U);
    msg.setDestination(55814U);
    msg.setDestinationEntity(204U);
    msg.source_man.assign("YFYNUAXVGGJUOGJWNDVTAYRLJOBKLERXLMFIWPNZZRIDCWPTWRQQJOUQJPCYFRCGNIKGDSOCVEGXHLUWBMMJMTGJYIOXRMQNNDUSZSRCOKUZQTNMPUKEVJMBYPRQCSALXTOAUKVDDUXBGHXXATHFNZSQEVWVZANFLBSQLUNHMTZBEHZFHOTVFYGHCMAZRKKWFWBEABCEXYGEJBABLKSPIDJHPMILOPCOSQLQFVWXETPWITVDAIHDEPKHKSDYSI");
    msg.dest_man.assign("GTWQPCNZGVQJSJKGLHLAGROTUKSXIQBYCXAPABWSOIFOBUMTIOOSL");
    msg.conditions.assign("MUDSSTFUYBHAHFTYXZCKLCATLDOVRVEHZWGDNEDXGWKPGSZRMWQBGQYHLFZENTXXUNE");
    IMC::Abort tmp_msg_0;
    msg.actions.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanTransition #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanTransition msg;
    msg.setTimeStamp(0.387838046519);
    msg.setSource(11365U);
    msg.setSourceEntity(21U);
    msg.setDestination(15769U);
    msg.setDestinationEntity(191U);
    msg.source_man.assign("NAMJUKXNCNHIZAMSGRVMVU");
    msg.dest_man.assign("WNLCHAVSJJYWVKIIKDTLDJVOOWWIOKHXUFPEJEDXIKZJFNUORZXSIPCBTMOMPJFPOFKQZ");
    msg.conditions.assign("XUGTCYDCBLLP");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanTransition #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EmergencyControl msg;
    msg.setTimeStamp(0.807099658824);
    msg.setSource(23105U);
    msg.setSourceEntity(246U);
    msg.setDestination(25937U);
    msg.setDestinationEntity(44U);
    msg.command = 20U;
    IMC::PlanSpecification tmp_msg_0;
    tmp_msg_0.plan_id.assign("GIYOOBMXGNEQLBOZRKAFZUIRZVSFYUGKCJSSWQFNUJLQSHXNUGQLJCMAFKNUDJHNVAABIVDWJXVQYWCQBRMMENT");
    tmp_msg_0.description.assign("CEGUMARYPUCIGFTPIQJOEZFKQFZRDGTZUQJAMDJRRYBUNUMDKHWSVIPHRODESTEABQUBSLMTBWXNKUYJOWWYAQWZSINZJVKYWLLTBYCUJBIFNBRPDEPNDJYNNPAQGXXHOLFXIKKVUJVFFAOCTALNMPLGAEAHITCLZHWUFSHCCZDODIEFWVKXTZVYXPGPGCEEWPQYYDV");
    tmp_msg_0.vnamespace.assign("XBZRAFHGLQCQZVWYQVSLDHPPYRLOSOKIZZSTTWZLTSUZNPAJMARNTUVXCNBIGGFQVKAULGCORHHVZBULGJGCQYBDECIOYNBXELEBOVOYJFAXKHBTMINFNPEXEQPYWWMXFTDMYKMWIGESWXRVMRGPSHICMDFPRJXPSDVRBNVHLHKAECGKCYWUJITUGSQJPFMSQHZUDHSLTWDFDJQKEULICEYWWRYEXXOAKOFNKKAJVCFMQ");
    IMC::PlanVariable tmp_tmp_msg_0_0;
    tmp_tmp_msg_0_0.name.assign("DBXNMIAPBTHGILFATNRJWINRFPWGMSPCFTOIFXHRVVSOPYUTWHTNKOEQHCZHZGFNDKXKCLBQTTZEJUPQXVKLWBRDJMRWSCXCHUUKMTTXSLYIVZDCGAOIYOUCSULQWEVPRYZJIGAPNDHLWEFVBYLRUKJBMHSPAWFYJQGYDJKKIQAEMYDICGZOSNZJBROOSBPEMFDQQVQYMSEVOHWG");
    tmp_tmp_msg_0_0.value.assign("DIRLWATIVYLIDMUGWIMVCKDFHTQ");
    tmp_tmp_msg_0_0.type = 81U;
    tmp_tmp_msg_0_0.access = 223U;
    tmp_msg_0.variables.push_back(tmp_tmp_msg_0_0);
    tmp_msg_0.start_man_id.assign("GGJUKMHJTWINJCWYZVUNPPNJQGOATSYNRBMNQZOTCQGWLQXPDVIXLVVKIFCCHACZFAKNSRBISDTTNFAZEEXFYJSFDZSXRUGJHSBBUIJKCGWDPOGLGKGHFDAXEJRHMQRWCEKLWMFVRUXTYHOPUMXOHEXKLLXICKMIVYSMAZPQSAETPQIWBOLLSYKDQHQJMUKOZCPNPBOHQREULNZDLZIODFMYBNECWOTPTYWVARFAMTJVBBV");
    IMC::PlanManeuver tmp_tmp_msg_0_1;
    tmp_tmp_msg_0_1.maneuver_id.assign("ZKGJNMHWHUDZTPXCPLVJ");
    IMC::LowLevelControl tmp_tmp_tmp_msg_0_1_0;
    IMC::DesiredHeading tmp_tmp_tmp_tmp_msg_0_1_0_0;
    tmp_tmp_tmp_tmp_msg_0_1_0_0.value = 0.452237767827;
    tmp_tmp_tmp_msg_0_1_0.control.set(tmp_tmp_tmp_tmp_msg_0_1_0_0);
    tmp_tmp_tmp_msg_0_1_0.duration = 30996U;
    tmp_tmp_tmp_msg_0_1_0.custom.assign("JGMNXVUMAKTHDGBKWMNXLLUHGDYFPDJMMVNOIYQCPDGCTNSOMEVVMBTUIAWVAWBALSVJPFZSOVELIAREFQAMCKTJSOGFVUTFUEKQPLTPEEQRYSAIPHRBUHODEVIYBQGWRDFCTKZOXSWSRTNTUFYCPWIHWECQDAFWMINOIOFNEOHDLIXSZDBUKZZ");
    tmp_tmp_msg_0_1.data.set(tmp_tmp_tmp_msg_0_1_0);
    IMC::Elevator tmp_tmp_tmp_msg_0_1_1;
    tmp_tmp_tmp_msg_0_1_1.timeout = 34573U;
    tmp_tmp_tmp_msg_0_1_1.flags = 213U;
    tmp_tmp_tmp_msg_0_1_1.lat = 0.373404987824;
    tmp_tmp_tmp_msg_0_1_1.lon = 0.836495740699;
    tmp_tmp_tmp_msg_0_1_1.start_z = 0.701730613093;
    tmp_tmp_tmp_msg_0_1_1.start_z_units = 101U;
    tmp_tmp_tmp_msg_0_1_1.end_z = 0.247235187074;
    tmp_tmp_tmp_msg_0_1_1.end_z_units = 230U;
    tmp_tmp_tmp_msg_0_1_1.radius = 0.206480496677;
    tmp_tmp_tmp_msg_0_1_1.speed = 0.77631429549;
    tmp_tmp_tmp_msg_0_1_1.speed_units = 42U;
    tmp_tmp_tmp_msg_0_1_1.custom.assign("IPDXBLAICDRVVGOKRRWKUGCTJSQSQMBXWAMD");
    tmp_tmp_msg_0_1.start_actions.push_back(tmp_tmp_tmp_msg_0_1_1);
    IMC::TrexOperation tmp_tmp_tmp_msg_0_1_2;
    tmp_tmp_tmp_msg_0_1_2.op = 91U;
    tmp_tmp_tmp_msg_0_1_2.goal_id.assign("SUHTQKTLECWCFYNBYB");
    IMC::TrexToken tmp_tmp_tmp_tmp_msg_0_1_2_0;
    tmp_tmp_tmp_tmp_msg_0_1_2_0.timeline.assign("CFMIPDCMEUAXDTEAJKTQWHPNNWEKC");
    tmp_tmp_tmp_tmp_msg_0_1_2_0.predicate.assign("DVZPUEMAXZFVHOPIELNCTSMJDLSWVQASSOIQAXOTMEHYAOAQFVGRQULQSCKZLYBHBVZPKQXMWAUFSQMWJNRDKRKVIHKWHDJBBSSGFGMZFNLXICWJZLOHKDJEYGFMRYPJMJO");
    IMC::TrexAttribute tmp_tmp_tmp_tmp_tmp_msg_0_1_2_0_0;
    tmp_tmp_tmp_tmp_tmp_msg_0_1_2_0_0.name.assign("ODRDHOEWLDXPPHVXHZMMCHJCGGVPFEWCWZCNCRLEBIGAQPCLFMLWTJSPEJHOXSGICJYYDUX");
    tmp_tmp_tmp_tmp_tmp_msg_0_1_2_0_0.attr_type = 188U;
    tmp_tmp_tmp_tmp_tmp_msg_0_1_2_0_0.min.assign("GWJOTFJRLTSIUCZCBAHSTZKMJEQAKQXOLMBTDOPCNWWCRQZLDPRADLAQ");
    tmp_tmp_tmp_tmp_tmp_msg_0_1_2_0_0.max.assign("YWAPGCTIECONQDHPUUSMFOIFNPBCZLLUZXVLPMMVBROBKZEEFJMQGTVBPAIYASUNOJAKNQPSBEMRGOVBSDTOXYXXPMIHCJIOBORZQJLIVKHYDKFWEUNZTLUNEKRJLPYKRWCRDWXWSKNQGNVHITAYHMSVDGQAQPI");
    tmp_tmp_tmp_tmp_msg_0_1_2_0.attributes.push_back(tmp_tmp_tmp_tmp_tmp_msg_0_1_2_0_0);
    tmp_tmp_tmp_msg_0_1_2.token.set(tmp_tmp_tmp_tmp_msg_0_1_2_0);
    tmp_tmp_msg_0_1.end_actions.push_back(tmp_tmp_tmp_msg_0_1_2);
    tmp_msg_0.maneuvers.push_back(tmp_tmp_msg_0_1);
    IMC::PlanTransition tmp_tmp_msg_0_2;
    tmp_tmp_msg_0_2.source_man.assign("RBBTNDUJZGPMSRS");
    tmp_tmp_msg_0_2.dest_man.assign("TFFETANILSJXLNFXBLWPGUBFOEXQEQPZCVVTGEZXKAAJNWEIWRDQFLKBAQRGDVDLIIKRVKWZUNNLYPDDNBJIFZL");
    tmp_tmp_msg_0_2.conditions.assign("ODYYTTXGJLZYGQUXVINDHYMIKXOZPXCEVFALUAEQWNKWR");
    tmp_msg_0.transitions.push_back(tmp_tmp_msg_0_2);
    IMC::CcuEvent tmp_tmp_msg_0_3;
    tmp_tmp_msg_0_3.type = 249U;
    tmp_tmp_msg_0_3.id.assign("RMUUJJDXMQPFLYKFGWFUUUGAYWSKZCPFUIZOENKHJMQJEKJOLFTQHBDETWHIJCYCYJZGAPTMTTNDBRISNEVEGMGSBRMFSWAL");
    IMC::TrexObservation tmp_tmp_tmp_msg_0_3_0;
    tmp_tmp_tmp_msg_0_3_0.timeline.assign("DJQPGNGGGRHWBCPKRJCFMQLMFQHVVSOXWWWSLAMIDWNCAYJHOSALDUFMFSDZRHIGLIJELRVZFEYKFBIEPTZDBVPTZDHHYOEUUNOCFLQQBNVGIUCALKIQKWCIJACMQRDBS");
    tmp_tmp_tmp_msg_0_3_0.predicate.assign("JZZYBXNQWKHGYMBLSSFASTMAYWFFQSPLUEUWFLXRNRDEQKZMPUPSVWPAVZFRQGXYZSOSLWMDPZODAHIVUDQX");
    tmp_tmp_tmp_msg_0_3_0.attributes.assign("DBEQAHTJCRROWNNFQRXDRJQYYXEDFBCPJYAFBOHBCMMXGPOXHJHMOLBPZQVVE");
    tmp_tmp_msg_0_3.arg.set(tmp_tmp_tmp_msg_0_3_0);
    tmp_msg_0.start_actions.push_back(tmp_tmp_msg_0_3);
    IMC::DesiredVelocity tmp_tmp_msg_0_4;
    tmp_tmp_msg_0_4.u = 0.382964093015;
    tmp_tmp_msg_0_4.v = 0.420660245201;
    tmp_tmp_msg_0_4.w = 0.909646772713;
    tmp_tmp_msg_0_4.p = 0.980688471827;
    tmp_tmp_msg_0_4.q = 0.240776214213;
    tmp_tmp_msg_0_4.r = 0.338585810684;
    tmp_tmp_msg_0_4.flags = 238U;
    tmp_msg_0.end_actions.push_back(tmp_tmp_msg_0_4);
    msg.plan.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EmergencyControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EmergencyControl msg;
    msg.setTimeStamp(0.529858972274);
    msg.setSource(48021U);
    msg.setSourceEntity(81U);
    msg.setDestination(24388U);
    msg.setDestinationEntity(96U);
    msg.command = 203U;
    IMC::PlanSpecification tmp_msg_0;
    tmp_msg_0.plan_id.assign("ITEHPPCNOUJEQUILGLYTCEJEYWAVPKZPCANBOFHUCWW");
    tmp_msg_0.description.assign("ZSVBPUAYQDSPPZLZPEZDILALRFFXCIEQVWYAJZNRFUXLTLMBEBATHSGRGEONMGVIEOXDTBPSWRJQDSIWUYUSPRTXXVVNNMCKFUPOAJRJVATTZBOQXEKIYZHMMUZHIFITYFYWWGORMLYNTHXHWOIKWACPDADZWNHGVAXYQDCLMOHYKRVQBMKFCECNPINNDJTGKSD");
    tmp_msg_0.vnamespace.assign("PRSXVOYVTYMBJNUGCLXBCXFOQXSSGYHKMRMTBDNFAXIHLXEVOSFUHZJTCONPYBHDSUKQRFKSWUTGMRBEOFSONWJMPRIIFCWDSTEMNCFAGVUXADYIQZIOVJVIBXOJEPVDKJZUQJPHYUCFCEHCNUGCLQKNGWLZTLILWWLYWLTQTGADVPUXJFPDTBDMEZPEKZEKAVGGLNZRSBIUWYSYANPKXBKAMEECTABDYJLHMQWPRZ");
    tmp_msg_0.start_man_id.assign("KVZZAMHBXDQOUOVJFNKEYAGDVWYCHYTQRKPIUGUKNFVJGMNMDSBSDILHBMQZEHVWLUGOUCEDHRUQKNDVRDANWPPRQPQJLXJCCKADXTRLTBPKXFNYLJTVIXZYCVPYFWBCEBSQEBNTSAPGDTSGAIIJOFMIESTIALWRXXJHDQHVEGSJXMNPXUPSFOKSBZWOOSZZUIGJVUBPXFOFEYHZHKARIIUYCZCLFJ");
    IMC::Heartbeat tmp_tmp_msg_0_0;
    tmp_msg_0.start_actions.push_back(tmp_tmp_msg_0_0);
    IMC::PopUp tmp_tmp_msg_0_1;
    tmp_tmp_msg_0_1.timeout = 53749U;
    tmp_tmp_msg_0_1.lat = 0.363418284171;
    tmp_tmp_msg_0_1.lon = 0.368008770233;
    tmp_tmp_msg_0_1.z = 0.25062909349;
    tmp_tmp_msg_0_1.z_units = 71U;
    tmp_tmp_msg_0_1.speed = 0.781970488788;
    tmp_tmp_msg_0_1.speed_units = 86U;
    tmp_tmp_msg_0_1.duration = 30405U;
    tmp_tmp_msg_0_1.radius = 0.107326291511;
    tmp_tmp_msg_0_1.flags = 150U;
    tmp_tmp_msg_0_1.custom.assign("QCMKXQIRPLOFWUSTUEEFHTBNXPTHJSMEME");
    tmp_msg_0.end_actions.push_back(tmp_tmp_msg_0_1);
    msg.plan.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EmergencyControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EmergencyControl msg;
    msg.setTimeStamp(0.101123306501);
    msg.setSource(35738U);
    msg.setSourceEntity(127U);
    msg.setDestination(63147U);
    msg.setDestinationEntity(201U);
    msg.command = 241U;
    IMC::PlanSpecification tmp_msg_0;
    tmp_msg_0.plan_id.assign("OUKDAHOZNZUKHBDIUPBXIBEBWRBIPDWXLEWBXPAQMDJGFDNEWYPCWJVKKHHKTSPZRZHKICGYVWYQARGTJFWSLHPMZVSQKVVIJOZTFLAZQAJEXWIDQF");
    tmp_msg_0.description.assign("STJXDPHGSBDGDGRUCCUACCUQRLXAHLDNRQSCTYBXNFFEBGREEXQGQHAHLRXCYEQMAZNSMYISPKMFWMCJPAGYPKNTWLCZEKNEFAKUZRMHYYPKHOQOUCOGFHKXVMLBWRPKBIEWRATYWB");
    tmp_msg_0.vnamespace.assign("KUVSBPVAQTHPUHVUPENAXIEDPVPFNLAZUDWABHHNKHLECWBMKWEUDIIFRWMOPLTJGTSNXPWDVGNAOFQFEFCEDBBSFGWQQQNDJCIOCZCNQQVAYSOO");
    IMC::PlanVariable tmp_tmp_msg_0_0;
    tmp_tmp_msg_0_0.name.assign("QREOBJSMKCNOPGBTTQHEXIGVVTAFJU");
    tmp_tmp_msg_0_0.value.assign("LSLGVKLZRUWQPZFGOZFODUUMNCSEBWMNKBZCLKPIYTOLHIALFQBIBHZRQECNPYUVTONJIDBKYQPFVPUAJIWGZGDAZEESRXFCXUWXGOAXSBMFIQXNXSVZXYRJXFMTDNZFXRATVPCQJUCWRJYTRDNDKEKHMUBHSHYYIJLOMRBWHQGALCQGSPWVWMSTTTRYCDKSYDBKDPBO");
    tmp_tmp_msg_0_0.type = 179U;
    tmp_tmp_msg_0_0.access = 178U;
    tmp_msg_0.variables.push_back(tmp_tmp_msg_0_0);
    tmp_msg_0.start_man_id.assign("KSEDRKJOYICQLWZBDMQHJSC");
    IMC::PlanManeuver tmp_tmp_msg_0_1;
    tmp_tmp_msg_0_1.maneuver_id.assign("TUGBPWQVMKATXEYUDAXIQWZFRSJKLRRBTYPNIGDZXLVKHPVDJPZHUAYOJVTVSCSIBTXIWFUTUABNKKQKJLCBPNQWJBLLLGPAZOQIESYCGGOTNWHFXMOZSFMRRNREPUHJMVOQRSMBUJGBMYGLXIUSETDOHFBJFYFJWECMURFKNKPB");
    IMC::FollowPath tmp_tmp_tmp_msg_0_1_0;
    tmp_tmp_tmp_msg_0_1_0.timeout = 51288U;
    tmp_tmp_tmp_msg_0_1_0.lat = 0.504340467973;
    tmp_tmp_tmp_msg_0_1_0.lon = 0.687068591758;
    tmp_tmp_tmp_msg_0_1_0.z = 0.0887051342089;
    tmp_tmp_tmp_msg_0_1_0.z_units = 21U;
    tmp_tmp_tmp_msg_0_1_0.speed = 0.44908381213;
    tmp_tmp_tmp_msg_0_1_0.speed_units = 248U;
    IMC::PathPoint tmp_tmp_tmp_tmp_msg_0_1_0_0;
    tmp_tmp_tmp_tmp_msg_0_1_0_0.x = 0.936922723927;
    tmp_tmp_tmp_tmp_msg_0_1_0_0.y = 0.337928759289;
    tmp_tmp_tmp_tmp_msg_0_1_0_0.z = 0.62400097385;
    tmp_tmp_tmp_msg_0_1_0.points.push_back(tmp_tmp_tmp_tmp_msg_0_1_0_0);
    tmp_tmp_tmp_msg_0_1_0.custom.assign("DHGODXLBDINSIFLSOYPIUWBNWYMIKAOLCOHXFRMNZZQWTMGMNCXZJOZGWZINDGKWVVYGNXJCCMSVSGJKEMASEPVFKONGQRJIDJGZMFUTOTTOJQKTTUK");
    tmp_tmp_msg_0_1.data.set(tmp_tmp_tmp_msg_0_1_0);
    tmp_msg_0.maneuvers.push_back(tmp_tmp_msg_0_1);
    IMC::PlanTransition tmp_tmp_msg_0_2;
    tmp_tmp_msg_0_2.source_man.assign("BPNIWVVFLTVOGSQJBDRITWOOAUUNYQ");
    tmp_tmp_msg_0_2.dest_man.assign("YKKKKMWQVMMGQPQMDVGOWAOTHIGQKEBPWJBTUAWZYHFJPDLXTVNLFPTWIPGQNKEINZPOAXUEUUHVUXFIBCLJGLGMLTZRXNJODVWUFHYOUABIAFQNNESSSCYWSMEIMRXSOPITRFVSEWFCUKCFMVQLJIAZQSBWSTOEVIXEBJEBHQCLMABACZVWXDCAKIHSLPRGUOYJCZSJKNDECYPXZKHLHDRVGQJHRDNHRXBFYRTPDJGBYMRNYZZZUXTLATOGDC");
    tmp_tmp_msg_0_2.conditions.assign("WPGCMINOANGAIDHJVRKMJLHYAPVUYBTQUHBNPTSERHZQFVUKFVYSIDGIGJSZQQTBFVWCYKUXDYVMTYFBCEEMUPPMSPERXUZPTYBEOAXJVAOCDYGIQUHROXDLHVLLNGAMCNUBJWXLXRGZWICFGPOXWXEWKEEQRXBZOSHZMQPWRBDZNBFIHLLKEJHBNKYJJNZFESNTUMLGWQIUCKATSHQWOYXFKGFRCZPLTCDDRQOJZADSLINSDKVC");
    IMC::MapFeature tmp_tmp_tmp_msg_0_2_0;
    tmp_tmp_tmp_msg_0_2_0.id.assign("QVTWMCEMNXBICKQHPQUWJVFIIGENROPBZNPVODXKGMYGUGZSIXIAGICIDBRZKLNAVBKFBTXCJNLTXJFFOPQDAHCSBCRPWYZPSDEOSEKJTOKYWECAZTEWAVHPKDLSLFAHREFETMNRUJIVYYNMJMRGJOPYVXQLLLGZPCINKHXBHASXAZVUZZNXMVUVQFBGHAJEYTOULJQUKNGUIWRLDOJHRYBWEDRAWUDYQSLQ");
    tmp_tmp_tmp_msg_0_2_0.feature_type = 245U;
    tmp_tmp_tmp_msg_0_2_0.rgb_red = 83U;
    tmp_tmp_tmp_msg_0_2_0.rgb_green = 115U;
    tmp_tmp_tmp_msg_0_2_0.rgb_blue = 216U;
    tmp_tmp_msg_0_2.actions.push_back(tmp_tmp_tmp_msg_0_2_0);
    tmp_msg_0.transitions.push_back(tmp_tmp_msg_0_2);
    msg.plan.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EmergencyControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EmergencyControlState msg;
    msg.setTimeStamp(0.872141506835);
    msg.setSource(64U);
    msg.setSourceEntity(92U);
    msg.setDestination(52840U);
    msg.setDestinationEntity(111U);
    msg.state = 244U;
    msg.plan_id.assign("NGHUCGYEVEWNTAYSSGEQDWGSAWVYBBHLDQYV");
    msg.comm_level = 57U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EmergencyControlState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EmergencyControlState msg;
    msg.setTimeStamp(0.012870145421);
    msg.setSource(58391U);
    msg.setSourceEntity(62U);
    msg.setDestination(54579U);
    msg.setDestinationEntity(244U);
    msg.state = 50U;
    msg.plan_id.assign("SFEFGZWQXAJULIFYMMZYTNXERMZSZZSDACEXQZRXNWKIXQKJOBIOYLUMGMFPTBPEWXXGUTQEHAVHFSDYZLVCIDYBRGJDJASZILTVQIWEAOEBUDSHHRWOUHIOTBKWZGNBLYNGQEFLCWJCGCFR");
    msg.comm_level = 148U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EmergencyControlState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EmergencyControlState msg;
    msg.setTimeStamp(0.075897640765);
    msg.setSource(43837U);
    msg.setSourceEntity(193U);
    msg.setDestination(18821U);
    msg.setDestinationEntity(6U);
    msg.state = 177U;
    msg.plan_id.assign("CRMACMRMPGYKQSOGPEDEAZDBUULECEJPEJCLLPFBFDBSTNMOVBMRIKEXLZZEFVDIJQAJDSEOTFLDFJRXNTYXSUARSWKWHOJVLQUHSLDAXMXCAHDZUPKRWONHZQTSYPMERTYKQOFSHNYHVBZNMEGCFTYDGFGNBOXLQBWVLWZISRZJIUUKGSGIHQYPTWVHAMFYNWZQARZIXPUGWVDXFNIAPOMOHCGOTQWVVCKXATBIBKBYVCWKNC");
    msg.comm_level = 4U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EmergencyControlState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanDB msg;
    msg.setTimeStamp(0.551681274932);
    msg.setSource(15632U);
    msg.setSourceEntity(79U);
    msg.setDestination(21525U);
    msg.setDestinationEntity(171U);
    msg.type = 12U;
    msg.op = 46U;
    msg.request_id = 12714U;
    msg.plan_id.assign("EDQOSKREPBHDIGOJUTREXDDPBVRNIPGFHDFIFUIAXFZXBWJHLZRTELGGKSRFREAXRLJGLKMSXNJDMYKKIUMBWYSJYWQNWISHBISPBGBYWWLTOCDXJPXSGLKHLUYMAQCNUWBDATRFNEUYPVXTCIGWLOFEPMNTNJZTZOAKMTGKOYQEHAZVZOPIXCNWQUMCAPFCKVYEAMOROCZYDMYCNH");
    IMC::GpsNavData tmp_msg_0;
    tmp_msg_0.itow = 322858189U;
    tmp_msg_0.lat = 0.992730453349;
    tmp_msg_0.lon = 0.376840133399;
    tmp_msg_0.height_ell = 0.326360683985;
    tmp_msg_0.height_sea = 0.645291589462;
    tmp_msg_0.hacc = 0.405285512017;
    tmp_msg_0.vacc = 0.168343343588;
    tmp_msg_0.vel_n = 0.960480595602;
    tmp_msg_0.vel_e = 0.999364750377;
    tmp_msg_0.vel_d = 0.0517824602556;
    tmp_msg_0.speed = 0.822734902229;
    tmp_msg_0.gspeed = 0.346680625418;
    tmp_msg_0.heading = 0.396684138953;
    tmp_msg_0.sacc = 0.848157739062;
    tmp_msg_0.cacc = 0.193284930095;
    msg.arg.set(tmp_msg_0);
    msg.info.assign("VZYZZYOFLAYQSAOZAYUKTZNWCTNOVPSHKEXBPIQBPHLCCAWJUXTBDTICUEFUUXOURQDMTCWRDESAZKYAGXNKRZIVNRRUDNFDOKPNLPMKEWLJZHVRHXJGMRBZNQHPUCWIVVCBFTSEIGNFBFMYIOFTWKIGMVJJPBSQQHDWEGYHTGOIFBSVDHEAPMHSEWGMHDJC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanDB #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanDB msg;
    msg.setTimeStamp(0.624509559989);
    msg.setSource(61956U);
    msg.setSourceEntity(189U);
    msg.setDestination(36060U);
    msg.setDestinationEntity(151U);
    msg.type = 87U;
    msg.op = 127U;
    msg.request_id = 60848U;
    msg.plan_id.assign("TJGZZLQMBCDYJAOTXNFVNOKXEHEXTYKRANGXYIYWNPRPIVCGNAIJBFLQRXIPIJFMEVRBBACZMNJDVIUHYBLPHEQARKJRZOPKIVJWUXASGDHNKIDNUVLZMTGUHKZJRTWWLF");
    IMC::IridiumTxStatus tmp_msg_0;
    tmp_msg_0.req_id = 33237U;
    tmp_msg_0.status = 232U;
    tmp_msg_0.text.assign("EDOCOABSUVRJLLXFLXLMBRIELRWAXGPHFXJFPYIRCTJDHPQSWFJBSBCHRDLCVWHIYVVPEIURPAXZTMVMVGUSMKYEUEPFKLAGS");
    msg.arg.set(tmp_msg_0);
    msg.info.assign("SUDNBGGAMNCKOATIUIHLTGQOWMOEEBOVFBRJQZDWLKHIMOYAVYWOZCJCHSCPRPNGCZQJRTUJEUCJPYFTWZX");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanDB #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanDB msg;
    msg.setTimeStamp(0.754928002603);
    msg.setSource(34676U);
    msg.setSourceEntity(73U);
    msg.setDestination(5145U);
    msg.setDestinationEntity(54U);
    msg.type = 43U;
    msg.op = 206U;
    msg.request_id = 30329U;
    msg.plan_id.assign("IYYIVVLWLEUXPTGZAOJCREOUXIIMTSZKTEUEVCXNWUUMIBTVWLQRR");
    IMC::EntityParameter tmp_msg_0;
    tmp_msg_0.name.assign("EPRMHKPXNABYCAAVDNVFKXWVDTPYLCIYCGVETDVLJMXGGAMUEHIEBQDHHZHYFTJVXUOBSHCTBZDRYFJJQOYGZRMXMRSQQSRUINBARDPQGWWEWTXBANCDYNJWKATFRXFPXUTLKUPQJEVUTBROMORK");
    tmp_msg_0.value.assign("NQYNUMBOWEYHRNSREDQEQYNQY");
    msg.arg.set(tmp_msg_0);
    msg.info.assign("VBYZQMTOXTLCTZDKEJQQMHLFICAOAANSTYXLHPDZKVNSEZCONERNFCTBSKGPD");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanDB #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanDBState msg;
    msg.setTimeStamp(0.483970914822);
    msg.setSource(48869U);
    msg.setSourceEntity(86U);
    msg.setDestination(62672U);
    msg.setDestinationEntity(194U);
    msg.plan_count = 55209U;
    msg.plan_size = 2121443422U;
    msg.change_time = 0.00097044110863;
    msg.change_sid = 3509U;
    msg.change_sname.assign("TEMTHNNSVMJJQWMFZOJSNFJAMGFBMSPINWRAZAEPHVSGBYJSRKLIOFEEDEXYHLWBAIJVIRVLRJOCORBULZOWFQUDEBUNCUOBBWEZCQVYDAVZCJSZQHWQLVUHMHPNVLTUDWNXQTQEPRDGQLPCBUKFMXTDGPPHUGSQAFCMKHGRONPMBLTVHJKAZCICIGUXYADFZQODYYIOTYCUATXIHKKEOXCDGTXMRKNPYIKZLPSY");
    const char tmp_msg_0[] = {-7, 29, -95, 111, 49, -62, -24, 20, 66, -30, -91, -8, -42, -50, -119, -126, 46, 37, 86, 35, 53, -49, 17, -6, -100, -105, 19, 19, 1, -77, -116, 86, 43, 26, 107, 123, 102, -13, 106, 102, 56, -13, 28, 111, -30, 79, 83, -106, -15, 56, -14, -77, -112, -34, -27, 101, 41, 124, 53, -4, 50, 120, -42, 9, 25, 81, -34, 33, -23, 98, -73, 76, 101, -57, 16, 78, -49, 92, 31, 30, 96, -63, 0, 122, 20, -45, -96, 20, -46, 39, 76, 7, -43, 68, 34, -128, 10, 89, -15, -61, 116, 65, 81, 33, 8, -85, -123, 109, 44, 114, -35, 37, -115, -123, -47, 80, 84, -6, 86, 113, -61, 2, 103, 111, -35, 7, -106, -104, 16, 104, 75, 0, 73, 73, -17, -84, -117, 88, 2, -111, -3, 87, -71, 105, -34, 69, -50, 66, -54, -63, 48, -65, 44, 103, 93, -76, -119};
    msg.md5.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanDBState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanDBState msg;
    msg.setTimeStamp(0.471285329319);
    msg.setSource(57439U);
    msg.setSourceEntity(82U);
    msg.setDestination(50307U);
    msg.setDestinationEntity(152U);
    msg.plan_count = 3460U;
    msg.plan_size = 3752171346U;
    msg.change_time = 0.331854456239;
    msg.change_sid = 56950U;
    msg.change_sname.assign("TMYBEJGZCQOWZIGONEFCQOJLCDWQEGDEOODFUJFNLBPKRIAQRPIXHSCRDWAEUOIATMDWRAFKFDLDXJKSQSWSCARVLIGPXLXRKXWYNNINQPNHSEKBZXYEYHHWJYXDJIYLBMZPNPFISMFWESNZUVQMTAVPZJUEHXVFYNAOTLSDAXBCKTMHYKMRVOZPU");
    const char tmp_msg_0[] = {9, -73, -117, -33, -74, -78, -28, -103, 98, 34, -67, 88, 100, -15, -108, 11, -115, -87, -53, 31, -56, 48, 101, 109, 6, -70, 100, -105, -16, 84, -46, -5, 2, -19, -94, 55, -89, 8, -56, -90, -6, 0, 83, -32, 90, 79, 113, -35, -28, -93, 56, 74, 45, 33, 55, 10, -127, -123, 20, -105, 24, -88, -126, -89, 8, -33, 42, 6, -83, -120, -126, 13, -75, -58, -79, 42, -56, 115, -22, 49, 31, 28, -27, -7, -66, 114, -74, -81, 113, 47, 36, 67, -90, -61, -78, 114, -125, 69, 61, -68, 19, 101, -79, -106, 40, -45, 46, 67, 123, 73, 59, 26, -2, 49, 97, 4, -25, 34, 75, 27, -83, -82, -88, 123, 88, 71, -69, 53, -28, -95, 109, -58, 122, -43, -126, 29, 99, 37, 72, 34, 110, -10, -117, 98, 49, 124, 36, -107, -125, -4, -128, 97, -95, -43, -1, 24, -65, -60, -106, 24, -73, -127, -49, 101, -2, -104, -60, 93, 59, 70, -9, 106, -126, -51, 42, -107, 123, 103, 85, -114, 51, 45, 100, -21, -11, 14, 14, 73, -90, -5, -113, 41, -31, 38, 96, -98, -124, -95, 125, -66, 23, -126, -65, -72, -128, -47, -87, -32, -121, 99, -107, 0, -61, -4, 93, -80, 6, 124, -117, -69, -113, 11, -124, 61, 87, 3, 63, -32, 63, -83, -66, 74, 94, 111, 42, -64};
    msg.md5.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanDBState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanDBState msg;
    msg.setTimeStamp(0.548400911634);
    msg.setSource(2668U);
    msg.setSourceEntity(89U);
    msg.setDestination(9675U);
    msg.setDestinationEntity(23U);
    msg.plan_count = 15275U;
    msg.plan_size = 1976907981U;
    msg.change_time = 0.357866275714;
    msg.change_sid = 10191U;
    msg.change_sname.assign("WYBNVXOTZZOFZERTTNXLDYGGRCIYEJRAFRBPSXNZPUZQQAPKMYASPCSFNCKYJONAUDFSMZNHGKITKVDZDUKDVCJFYFAYMMPIWOFMWJVEIVBZQTCYHSUMIVHKEHXTCEKGZARVUAMUVLXBQSDPXONCXMKNRJGDBJJAIBSEILXWQWWXTOHOLVCHSFNAOHWVKGUJTLHCEIZBGMJBWMQFLAWSSEIJGWKLLODEQYDHPCBYNPUQPEQOITULGDHRTPGR");
    const char tmp_msg_0[] = {-111, -27, 89, 29, 29, 67, -5, 13, 121, -36, 124, -27, 124, -20, -34, -10, 84, 79, 91, 23, 82, 102, -55, -120, 93, -30, 32, -102, 78, -6, -20, 16, -27, 2, -51, 34, 30, 46, -114, -57, 48, -120, 27, -57, 11, -79, 12, -115, 19, 20, -33, 31, -19, 21, 32, -18, 18, 112, -28, -118, 12, 91, 120, 10, 80, -92, 91, 40, 101, -124, -128, -128, -78, -108, 5, -122, -53, 16, 106, 77, 59, 116, 99, 29, -51, -110, 7, -83, 125, -57, -113, -23, 123, -126, -9, -86, 97, 61, -92, -73, -81, 13, 28, -10, 66, -58, 34, 89, 46, -87, -86, -75, 104, -29, 103, 80, -103, 23, -75, 27, -40, 102, -73, -121, 80, -7, 26, 99, 113, 21, -75, -86, 78, -30, -110, -114, 24, -86, -95, -26, -57, -128, 32, -117, 43, -55, -104, -17, -47, -98, -13, -42, -108, 55, -126, -70, -8, 41, -29, -54, 90, -96, 76, -27, 93, -84, 71, 51, -36, 64, -49, -97, 124, 63, 52, -41, -83, 42, -28, -63, 47, -5, -87, 40, 71, 5, 32, -28, 83, -60, -124, 115, 68, 10, 78, 72, -32, -17, -107, -43, 89, -116, 71, 78, 89, 64, 78, 11, 13, -119, 61, -34, -55, -68};
    msg.md5.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));
    IMC::PlanDBInformation tmp_msg_1;
    tmp_msg_1.plan_id.assign("UTRRQLEUMSEFOLHJRPDYNDCLSYUPGRFMYCOWFNVVRDINFPTPE");
    tmp_msg_1.plan_size = 2400U;
    tmp_msg_1.change_time = 0.891403567108;
    tmp_msg_1.change_sid = 45704U;
    tmp_msg_1.change_sname.assign("KLQPOXEFNDDJLSQQCFKKBBVVFHWJVZERXREAWCEYIQWYAYENODVISFGTOAFVANQERDSPHPZIDPYLXZQBGRMVFTSEBZOKUUPNUZATVXGELQTZFSHVGSVNMIYXALGOEPPXTANJYNOIUHPRBNJHFMYCDLUFTMXGWGDDXASQZQZITDWJWRHPCUBHILCCLUKS");
    const char tmp_tmp_msg_1_0[] = {89, 42, 63, 88, -35, -1, 68, -116, -38, 57, 78, 33, 27, -57, -66, 35, 75, 46, 90, -68, 53, 64, 23, -15, -33, 100, -24, 32, -92, 64, 41, -113, -94, 82, 102, -23, -80, 27, -85, -93, -7, 126, -19, -44, 121, -81, 73, -108};
    tmp_msg_1.md5.assign(tmp_tmp_msg_1_0, tmp_tmp_msg_1_0 + sizeof(tmp_tmp_msg_1_0));
    msg.plans_info.push_back(tmp_msg_1);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanDBState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanDBInformation msg;
    msg.setTimeStamp(0.904577307023);
    msg.setSource(36513U);
    msg.setSourceEntity(178U);
    msg.setDestination(61178U);
    msg.setDestinationEntity(176U);
    msg.plan_id.assign("WEFGVCTVGPIWXNGRNXJFMFXNAOEGNPJUIVPUWMLYYPFUNGBWHZLZYZSDDHMORWLWASPCZMJAKJORSCONYOX");
    msg.plan_size = 13997U;
    msg.change_time = 0.974938627975;
    msg.change_sid = 29901U;
    msg.change_sname.assign("YMORDKEITXSLITRSUCCWRHROMGGAQSFEFLTADYAIPIFLSYOEUEKTJTCVJBHQKHPXOZFTNZKVRPAWWQCOGQIXHK");
    const char tmp_msg_0[] = {-90, -102, -77, -43, -41, 103, 73, 51, 58, -2, -71, 34, -108, -88, -127, 104, 40, -109, -83, -75, 122, 88, -29, -7, 58, -117, -89};
    msg.md5.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanDBInformation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanDBInformation msg;
    msg.setTimeStamp(0.0843142527955);
    msg.setSource(47988U);
    msg.setSourceEntity(139U);
    msg.setDestination(22633U);
    msg.setDestinationEntity(224U);
    msg.plan_id.assign("QJJDONAPIFNTJQOTNYFXHORZYLKGPTCXMGGBSUWFLZYZACAKGXYFAZWWJONBFUKNERCBNWEQQLVTDJUQFNBDHWPDZSEDVWCEYCLWIGVVMOXQJRELRMATCSRZAKSIMIGFJTBJQMXPGRYBGEACVMIPIHKYDTPJZBPYFLPMSXLFMHBBAQQWZSIYXXTXUJRROKEZK");
    msg.plan_size = 20471U;
    msg.change_time = 0.0561378554383;
    msg.change_sid = 20491U;
    msg.change_sname.assign("IXNGJAZEXAEORTOTKEJLWCMIBUEYKMPRHSPKSLLVLXDHMWXEOLFDXVKSLLIGNORBRHQDTKEKGMYXQQBSANVQNFXNOYJBDMAQQZQUNFFZGBBUHUFAV");
    const char tmp_msg_0[] = {-128, -59, -40, 70, 61, 59, -35, -113, 113, 56, 31, -17, 102, 47, -77, 46, -49, -111, 120, -22, 48, 8, 96, -1, -5, -3, 95, 50, 80, 104, 122, 51, 111, -82, 53, 116, 38, 60, 86, -10, 114, -46, 39, -66, -66, -81, 64, 13, -109, -21, -5, 66, -8, -34, 61, -56, -40, 35, -116, 118, 81, -96, 9, 101, 56, 59, 114, 109, -10, 11, -80, 12, -38, -48, 83, 84, -35, 93, 58, 118, -56, 93, -119, -62, -7, 37, -26, -21, 41, 51, 52, 43, 22, 77, -115, 15, 39, 63, -127, 54, -8, 97, -93, 2, -14, 80, 72, 76, -40, 118, 15, 50, -109, 43, -85, -80, 36, -102, -94, 28, -65, 122, 0, 0, -59, -29, -48, -19, -107, -81, -16, -108, -71, 69, 93, 39, -71, -64, -44};
    msg.md5.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanDBInformation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanDBInformation msg;
    msg.setTimeStamp(0.456946772992);
    msg.setSource(35828U);
    msg.setSourceEntity(8U);
    msg.setDestination(17155U);
    msg.setDestinationEntity(183U);
    msg.plan_id.assign("UBUPKPNQLSCOXGBTQVOZMMRXTBKQBUFOAZRCNMWWFPPTLTGLFDQISGSHREDEASYZTHNNPYUIHDUGRYFZAMIJFAXGOQHSHFSKMKHJGDFWZRWLNLBEUIFCAYBQ");
    msg.plan_size = 29876U;
    msg.change_time = 0.0854088853917;
    msg.change_sid = 6240U;
    msg.change_sname.assign("BMIEGHOLJFYPSOUXDBWXBRJEHAYJOEGDBDZXTAQNBCTTTWESESZQPOHZVJNQSRAFHYDMFIKTPYJILDGNKNUHLROXDBJUHWKCUUSYSYVNFWJRWISMPMUATPDZZZNHIEVSMUTQKYHNEBLNIVRMFCTJVVLKQGKRNZCDJXIVCAUIALQRADIKHYAQRCMGVCMVMWZKLCEQOWDGPZSULTJFBYFXCKBEQBXFONMPPLGH");
    const char tmp_msg_0[] = {39, 11, -93, 65, -39, -38, 40, 108, -82, 42, 66, 72, -8, 44, 24, 67, 5, 14, -82, 15, 124, 97, 56, 97, 59, 84, -53, -1, 94, -65, 62, 53, -77, -14, 2, 60, -35, -75, 44, -95, 19, 76, 79, -100, -50, -90, -106, 89, 24, 20, 44, -121, 108, -56, -46, -80, -123, -6, 52, 41, 81, 86, 11, -6, 41, 10, 76, 62, 70, 82, -34, 36, 61, 25, 96, 90, -43, 15, 15};
    msg.md5.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanDBInformation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanControl msg;
    msg.setTimeStamp(0.298870057836);
    msg.setSource(47617U);
    msg.setSourceEntity(250U);
    msg.setDestination(32293U);
    msg.setDestinationEntity(249U);
    msg.type = 97U;
    msg.op = 243U;
    msg.request_id = 45222U;
    msg.plan_id.assign("BJUTZOYHCIRJXHRDRTRSBTDVDFIYAPYGYHHESJVTFDLIENLWMMNGXANHUMRQGBJLVYKAXNLHFCAJNNZOYKPFZHQUINHXWFTEJQUKDFTERDSXVZWYADOGXXYJZCSVTTZMLELMMSMPNCCWUEAMDBQKUCBOOCIZGQMSZPLBIGGHIPRAXVHVUSNAXTOOBTGGPVWNQUCDPCOUPAVFMJYFWKEVIEFBEDKWPCLARRKESKIQWUG");
    msg.flags = 55295U;
    IMC::ControlLoops tmp_msg_0;
    tmp_msg_0.enable = 149U;
    tmp_msg_0.mask = 1461312729U;
    tmp_msg_0.scope_ref = 2327845397U;
    msg.arg.set(tmp_msg_0);
    msg.info.assign("QXVFATMYSQHZIBGZIIFRMAGTEBTWZNHFOCRYXPNSOVKGKQENPWPBWKOYXBHDNCEWNTUCAASDRJWQEYGUGBPUTKB");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanControl #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanControl msg;
    msg.setTimeStamp(0.910878458982);
    msg.setSource(39841U);
    msg.setSourceEntity(50U);
    msg.setDestination(32726U);
    msg.setDestinationEntity(223U);
    msg.type = 104U;
    msg.op = 27U;
    msg.request_id = 46456U;
    msg.plan_id.assign("GVMBGEGUCZISRIKMWYHTKAAXNWNVGIAKQVIPTUFJLTRWBGTLWZOCLADUSFJZKXINDEBUMHQHVMQRDCZTIELBHRJCSHAFKLPPBNXMJUTMZCUVSDNPXURFSYOKPCIUYGWXEUIKBGMAHUQVJXNJHDAXE");
    msg.flags = 56546U;
    IMC::SmsTx tmp_msg_0;
    tmp_msg_0.seq = 455413284U;
    tmp_msg_0.destination.assign("RTYJYAESBHDPHQQZUROXGDZRNEYAYPLJMPKKIDEKBXMBHGFVWOAEWCCWNMWVPGEFIDXZPCVMEMSWSOPCNZGXANDSSTTLSDTQGKNMHUILFSBINHTVIBMFBSBOLXQGZADEWKQQXCVKRHVIYOWSUNLRLNONWECOAXWRIJBJUCOPZWHUNLTYSYBXZFRDJKUGMQYJLFKTHOAIQCGVJAVUZPZVRX");
    tmp_msg_0.timeout = 59742U;
    const char tmp_tmp_msg_0_0[] = {36, -7, -54, -48, -103, 114, 53, 69, 12, 107, 122, 94, 84, -4, -56, 125, -71, 7, -25, 64, 62, 50, 50, 61, 20, 4, 95, 51, 6, -73, -7, -119, -108, -101, -26, 62, 105, 91, -62, -101, 58, -95, -11, -118, -2, 3, 119, -104, 123, -78, 26, 34, -71, 72, -100, 68, -34, -45, -18, -11, 88, -40, 85, 6, 28, 32, 44, -87, -24, -101, 17, 34, 97, 74, -15, 114, -108, -101, 109, -87, -57, -118, -92, -93, -82, -105, -17, -116, 7, 25, 71, 86, -112, -60, 106, 42, 66, -78, -57, -74, 68, 90, 87, -93, 104, 69, 102, 46, 59, 26, -124, -105, -92, 85, -80, 15, 119, 55, -112, -106, -122, 10, 11, 75, 0, 8, 84, 25, -2, -4, -95, -80, 0, 6, 103, -111, 81, 84, 56, 115, -16, 117, -71, 40, -75, -118, 38, -10, 11, -54, 102, -3, 100, -111, -102, 107, 47, 63, -120, 55, -61, -112, 100, -123, -55, -50, -1, -9, -101, 7, -78, 109, 80, 87, 16, -8, -88, 119, -88, -41, 86, 38, -40, 54, -63, -38, 25, -120, 52, 103, -103, -123, -50, -42, -110, -108, -125, -123, -60, 91, 126, -23, 13, -123, 6, 71, -128, -99, -78, 101, 78, -126, 55, -21, 15, 104, -121, 53, -1, -96, -13, -37, -95, 111, 30, 107, 116, 22, 21, 31, -45, 63, -86, 5, 115, -31, 48, -60, -56, 115, 95, -31, 43, -46};
    tmp_msg_0.data.assign(tmp_tmp_msg_0_0, tmp_tmp_msg_0_0 + sizeof(tmp_tmp_msg_0_0));
    msg.arg.set(tmp_msg_0);
    msg.info.assign("HCXDGSECBBFLVVFGUNMQKTTMQYFMDWQJWNRDXMMQLTSTGSITIJFKUCLKRJHAIOPVNXBLTKUNZQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanControl #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanControl msg;
    msg.setTimeStamp(0.400134033352);
    msg.setSource(15668U);
    msg.setSourceEntity(56U);
    msg.setDestination(25877U);
    msg.setDestinationEntity(188U);
    msg.type = 71U;
    msg.op = 179U;
    msg.request_id = 55637U;
    msg.plan_id.assign("WJORHNUQVXAZMOWNHJCXTQYINKURXADEFCHFITDVUCTSYPGDAHDNVVTPKLAT");
    msg.flags = 51721U;
    IMC::CreateSession tmp_msg_0;
    tmp_msg_0.timeout = 962363475U;
    msg.arg.set(tmp_msg_0);
    msg.info.assign("OAIHYGWNGBMPNJBUABPLFIPGUFBJHDOLPRAGISXLUXIYJNGZWRKZBCAVGRLPSESLCUWCHCTEIYVVGVIAPHZWEZUWQEUOWFWESDHVGFXZHUGTFLYKNRCKYOCYSQQUZFLWTWPZ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanControl #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanControlState msg;
    msg.setTimeStamp(0.770695980644);
    msg.setSource(19124U);
    msg.setSourceEntity(84U);
    msg.setDestination(59267U);
    msg.setDestinationEntity(153U);
    msg.state = 132U;
    msg.plan_id.assign("QXAJRKOZVHIERTOQSUGFLWDRQVLXFYKPLHBLAIVIYYZUXCSEZCMPCUYBJFIVOOMMNKSFVMEBGJOLNHFITYYNJCEGAWSIKLGEAOJDQIBYRQCPCCDDHQVAKCLWYNMPXAEAKKDMBZSBNXJDHTCEFJAEXXTNZNLBRHXRFUHUMLBYPOSGG");
    msg.plan_eta = -1203522146;
    msg.plan_progress = 0.1885606853;
    msg.man_id.assign("EIKLHIMBMNELTMFSCAEDHTL");
    msg.man_type = 22641U;
    msg.man_eta = -403669665;
    msg.last_outcome = 167U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanControlState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanControlState msg;
    msg.setTimeStamp(0.872839926321);
    msg.setSource(25783U);
    msg.setSourceEntity(1U);
    msg.setDestination(9969U);
    msg.setDestinationEntity(40U);
    msg.state = 225U;
    msg.plan_id.assign("CRNBKDMRNYZYMDWPVTFDBNJKDPNEAZORAOJPXIJOSFWTWYUGJAIECAQWOVKEXJIQABOHBWSOZXGRCGTGZUWKFQSPZLKMNWQKNYRUVQCYTVFUFTDSPLHCTSPOSLVNXLYIYPUMZKRBGAZUFEGMDNHQRQHMHDLULJKYWGOAETXMMELRBGHMCINEUCTAXBAXRHBRQFOYVESBIFVVXZPIIILF");
    msg.plan_eta = -1478211664;
    msg.plan_progress = 0.339625614356;
    msg.man_id.assign("VTNLYVWOXQEGJNXFZBNIPKVNNUJQYVEKGICHAXSLKAJVWLEBUCAQFQKJMIDOFWBVOPGILAMHEYGONFBPCBZUJXOXZTUUMJITDSDDMSFYZHIHWGHZZIFQENQZHGVHUTNZCSKWADRIYOHXMMATWTIECTVKURRWSKDXPUJQFFENNOROPYYVMKDGSABTZFVSL");
    msg.man_type = 46088U;
    msg.man_eta = -1076798706;
    msg.last_outcome = 159U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanControlState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanControlState msg;
    msg.setTimeStamp(0.517839288246);
    msg.setSource(10129U);
    msg.setSourceEntity(194U);
    msg.setDestination(16437U);
    msg.setDestinationEntity(210U);
    msg.state = 12U;
    msg.plan_id.assign("BLPTVVDQYYAGQRIDWYWYXEPCFYWIPCPEAVNZIDUEJORRLONANAKCPGZRCJLHEJHEYOLGZHWKKLJBFQFEUVHUXEWARMJWQTVIFOVJDPYFLCZLMWOXGFDGTLVYXSISUHZJXPNSPMDYSUFZFDMTNTJSYRIOSVXQQCSMWTVTGNFMDXLUASGNKHQDKCCPDMXISHWRRKMPNOCBNTHUREZXNUWGQQBGKBOCIBZHABARJG");
    msg.plan_eta = -1501050843;
    msg.plan_progress = 0.23124442922;
    msg.man_id.assign("LUCONKGSCSSTHVSQXDMLVTYDMPGUUXEFLZMGFVCAUNVEMODOCPM");
    msg.man_type = 2903U;
    msg.man_eta = 354944804;
    msg.last_outcome = 9U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanControlState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanVariable msg;
    msg.setTimeStamp(0.448303086927);
    msg.setSource(14905U);
    msg.setSourceEntity(150U);
    msg.setDestination(7512U);
    msg.setDestinationEntity(248U);
    msg.name.assign("DMAIVBYHZQGEQZMIPRCRLKQCVNJATFGFGXVXFXJNKFNUCTCPSUXMINTEXEJTZRNBDPAJKJCSIYYPYESELGPQHTLWVVLQPTSUDUDMTTEQRUDVODUUWHFKSUEROLWGADYCZBHCNZAZLOCNRAMZVSNJWIPBWFOSGBGCKJIYMXHPOREOXBDYZLOXHWTRKZBPWDONBHXCAHGXAAFLUVQEWKWFMFYZAHSI");
    msg.value.assign("HVKMVCMBBTJVVWAGHXGVSTVICJGEMQOLMLISBBEOXFIAEKJSSMTIEREBENSISTEFANYFJRWYWKBWUGLYJKASUSZDYEYQTNBBAQZQJGCUDXLMMSAFMPCUZWOVBPHYYXGUHKGOAXUQPIBRTQHRQDCPHGJNWOZQNFNYHQRCHROCPCHRGWKGXUZZKKIPMJXPRNVIICXRZDLJWDFQFDAFYUWKDLTZZNVLFWONTYTSJOOLAMN");
    msg.type = 203U;
    msg.access = 56U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanVariable #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanVariable msg;
    msg.setTimeStamp(0.0459445155034);
    msg.setSource(26605U);
    msg.setSourceEntity(212U);
    msg.setDestination(8043U);
    msg.setDestinationEntity(143U);
    msg.name.assign("TUDRIAHQSZGMUSYDOIQVEIYWCFAGCQBWAHDFPMPBVFUVJVFSEGRLKQWPDYGBOZXULHXUSQVDQTMIPKUUERJZAHLOPOBL");
    msg.value.assign("ULQKIYXNHFUTJINRVXKHICAJOOVRHLUNWPYHNGDKCCBDWROFPEXVJXCUTUNVBIOGQXRMISXSUOUNAPIEBOHFLFUAEGJHGZQOZXSIHKEDBNGMKJJGVDNNHZQYUSOWSSKPDKFIAWCJPLRDKJVYCTSZBGM");
    msg.type = 26U;
    msg.access = 133U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanVariable #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanVariable msg;
    msg.setTimeStamp(0.017369739664);
    msg.setSource(31742U);
    msg.setSourceEntity(5U);
    msg.setDestination(21406U);
    msg.setDestinationEntity(246U);
    msg.name.assign("ILMWYPUZPXHPYHPSWFQKHAWCJPODEXNQRTTNIYXXLTXVCTFHMLGJLUDGFFSJKHWFXMHNEUTPOYYCTDWMRYNDAAUYTVEEBINSCBGHCWBXGSROSLVVFN");
    msg.value.assign("AOSVFUNKUEPUQCXYFKOWVDAXRVYGVLTZPODMDVIRPOGNOVONHRAHZKTWBHKZPHTTQMQZWWIYGJYFWQEBMUZMAQYUYTSNHUJWFSKVVWHNJMFJCCEIXEQYFCLBZJEAYLBCRWMEWUZHGMZUXGRADSDAAILKHILQGBJEZGHRSFSRNDUSOYRSCPVCJXXLJLCPMATROTKDMUPMEQLIEBFFGSCDENQLLQNIDWXJBBKPX");
    msg.type = 93U;
    msg.access = 30U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanVariable #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanGeneration msg;
    msg.setTimeStamp(0.396473876553);
    msg.setSource(49422U);
    msg.setSourceEntity(200U);
    msg.setDestination(58769U);
    msg.setDestinationEntity(17U);
    msg.cmd = 3U;
    msg.op = 220U;
    msg.plan_id.assign("PVBSLBEVXJIMNUXMFABMSLRCNDLVFKHYJPXEITPAYCISRDMKBDLSZYWPVMWHPTGULDTYSPVKPSBZQMQBCUMIGFOGJKWUZITNQIHXVEHEKXFWNYAXOVRWGWFOORBIDRIJGFBZFVQXJWJKKUAJLCIRPJDKCVNSRKTTCPGRYZZOHUUOELQFMOSNNTZTRNHFZZJUEUJHETSVPQCXDWKOAIWBBHGADGCWFQYHLGY");
    msg.params.assign("HRJTZKPUNAYQBKTCYTAJLIZVBW");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanGeneration #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanGeneration msg;
    msg.setTimeStamp(0.596934208704);
    msg.setSource(51382U);
    msg.setSourceEntity(176U);
    msg.setDestination(11274U);
    msg.setDestinationEntity(45U);
    msg.cmd = 191U;
    msg.op = 25U;
    msg.plan_id.assign("NQJOEIUWHDBINMGGVITYYXFQLTFVRQUACNQMHZXXCDPBYEVJGDQWLMWFDLGZLAYAIOEDCUDVEKUBKSECETKGVHAQPOBVQQUJRWWMXKRFZSRRWPOUZFZCINPTHAKIKOYWEIRVCZNEAWDKYDGZSVLTOOSKXXSBIHPNOJKLHUMU");
    msg.params.assign("AIFODUZXKMVUXHFQKQMQWCOXHKQOIAYEIJBYZZVTFPESLBKBWIXSPRFTBNDSTAEYYDCVLSRYOSTRXWHOBSNBDTGXGLHVWVNFETCGYLGZWPDAMJHZNPOHNYERRJEJMARRAFNOKNVRGOPUNDWIFFMDDJZLYDKJLTKQGEAPWBSQHEYFAKFZCIVNPKTXZEHCPISUXZVXJCGMLCQCJSOQE");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanGeneration #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PlanGeneration msg;
    msg.setTimeStamp(0.0501504659457);
    msg.setSource(62931U);
    msg.setSourceEntity(157U);
    msg.setDestination(37702U);
    msg.setDestinationEntity(229U);
    msg.cmd = 57U;
    msg.op = 163U;
    msg.plan_id.assign("MYZVDKWDWRBGLPMCXHIYQSMFZEQASBOEPOEITCGZUGQKXXPTJLEXUGEWIDBGYITWBJKYNZZNKERBVZRUIWFYYVGXWVSPIUIMNXISTFCOHTABPLRYVU");
    msg.params.assign("YGJZPJGBHWMNKILMSULHRXJZZNBXFYSXDPTTNCPDVBFFRMXNXEORSYIRKEBTXGBHIVDQLDHGWQWNMUFAWKSOULVVFZNVMGEOWPMWSAEXCJTRRVKVCZJWGFRCQDIMIHISNOLAQLWIQOBFLBYVIFUAQKFVCEZMQOTYQMODTPTOEUGBHA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PlanGeneration #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LeaderState msg;
    msg.setTimeStamp(0.726904927322);
    msg.setSource(21696U);
    msg.setSourceEntity(71U);
    msg.setDestination(35954U);
    msg.setDestinationEntity(7U);
    msg.group_name.assign("FJSVYXYBXLIHGZJBPBTIIJXRRJHSZYPNFUJRRFZVLPMVAWHDYLADQWQZTHCMZSEZNGEDYLGVFYUYNDAGN");
    msg.op = 120U;
    msg.lat = 0.113881467472;
    msg.lon = 0.906476789858;
    msg.height = 0.257551454641;
    msg.x = 0.0173834484992;
    msg.y = 0.209804863034;
    msg.z = 0.981818254113;
    msg.phi = 0.989736315655;
    msg.theta = 0.987672703106;
    msg.psi = 0.219622869929;
    msg.vx = 0.000640642613006;
    msg.vy = 0.81545023321;
    msg.vz = 0.804737186885;
    msg.p = 0.991643672791;
    msg.q = 0.931142413206;
    msg.r = 0.293609687991;
    msg.svx = 0.150390356119;
    msg.svy = 0.30901167084;
    msg.svz = 0.612021961231;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LeaderState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LeaderState msg;
    msg.setTimeStamp(0.39898361007);
    msg.setSource(64894U);
    msg.setSourceEntity(12U);
    msg.setDestination(24141U);
    msg.setDestinationEntity(13U);
    msg.group_name.assign("CPEXYTJIVFZRUCPRBQRPIDHAQGDIGQNCOLMCTWGTNDAAFRKJHDBHMUPVVTSTZPEYIUQVYPDEXTAIZJSDKSMBLKJOIUWZHASACDMQBUMQHVBVUNMRGAOVRSKVELWWMKWJSWOBWQPJHZYSYCCJLXCKFYOQKQLKIXUHTSGYDLIZYHNZDWZGJWKEZNBPBXANELDWUNISYPMROEFGURBJFZXSCLVXPNLOUGFVQMMG");
    msg.op = 206U;
    msg.lat = 0.160996249202;
    msg.lon = 0.907596005196;
    msg.height = 0.320824721811;
    msg.x = 0.860558682947;
    msg.y = 0.288128366841;
    msg.z = 0.046517983336;
    msg.phi = 0.233407686757;
    msg.theta = 0.600537591416;
    msg.psi = 0.940704510595;
    msg.vx = 0.129053342107;
    msg.vy = 0.442604342492;
    msg.vz = 0.633925899314;
    msg.p = 0.407254199491;
    msg.q = 0.337004404266;
    msg.r = 0.181643776527;
    msg.svx = 0.0289027064024;
    msg.svy = 0.655580363294;
    msg.svz = 0.0104367101072;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LeaderState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::LeaderState msg;
    msg.setTimeStamp(0.772385738867);
    msg.setSource(62625U);
    msg.setSourceEntity(155U);
    msg.setDestination(52920U);
    msg.setDestinationEntity(172U);
    msg.group_name.assign("IKXBQAGESXEAKPREGNFKDDNTQNMQZCNNXQBAOZDUBYV");
    msg.op = 49U;
    msg.lat = 0.767458557328;
    msg.lon = 0.642936407534;
    msg.height = 0.136375124688;
    msg.x = 0.501545991524;
    msg.y = 0.703072563614;
    msg.z = 0.598941681358;
    msg.phi = 0.798517408593;
    msg.theta = 0.228362677277;
    msg.psi = 0.643369860001;
    msg.vx = 0.873931230241;
    msg.vy = 0.232268562549;
    msg.vz = 0.83266743343;
    msg.p = 0.959485457826;
    msg.q = 0.540604239486;
    msg.r = 0.222648239219;
    msg.svx = 0.625359658679;
    msg.svy = 0.799372825697;
    msg.svz = 0.924228152617;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("LeaderState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ReportedState msg;
    msg.setTimeStamp(0.624023177072);
    msg.setSource(13762U);
    msg.setSourceEntity(59U);
    msg.setDestination(46890U);
    msg.setDestinationEntity(119U);
    msg.lat = 0.798505778634;
    msg.lon = 0.052855799655;
    msg.depth = 0.949267686665;
    msg.roll = 0.823792493132;
    msg.pitch = 0.128409412351;
    msg.yaw = 0.122136824955;
    msg.rcp_time = 0.303294803937;
    msg.sid.assign("AGJFBFSEGRX");
    msg.s_type = 169U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ReportedState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ReportedState msg;
    msg.setTimeStamp(0.228053885426);
    msg.setSource(28544U);
    msg.setSourceEntity(222U);
    msg.setDestination(39780U);
    msg.setDestinationEntity(118U);
    msg.lat = 0.614499843655;
    msg.lon = 0.93733600401;
    msg.depth = 0.827580858339;
    msg.roll = 0.329599286451;
    msg.pitch = 0.649636997606;
    msg.yaw = 0.712818747406;
    msg.rcp_time = 0.780316155464;
    msg.sid.assign("VOPKWFRZPSQKZFSQSGEXYBDDJIEBOHTGDTAHYNLNUFJQAGWUZFQZWYTVZDIOE");
    msg.s_type = 51U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ReportedState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ReportedState msg;
    msg.setTimeStamp(0.35074382505);
    msg.setSource(21396U);
    msg.setSourceEntity(115U);
    msg.setDestination(14702U);
    msg.setDestinationEntity(87U);
    msg.lat = 0.238423301545;
    msg.lon = 0.3012107809;
    msg.depth = 0.711361472928;
    msg.roll = 0.712482398173;
    msg.pitch = 0.690302246227;
    msg.yaw = 0.684586045363;
    msg.rcp_time = 0.325673640297;
    msg.sid.assign("YYIIOXGZEQNSDVTXMURPYIVIOIPZHBRUUOMQQKOZKCYVHZXGFNWLUQJLOLLNJHSRXCKYFVFDYWMVJVGDXCQXPCMPEOLEKXISRXNLWANWAQUTUZFWTKCEENOMRRCVEAVBVTZWZGBPARHXIKBOHQADHOPEDAGSHSPAXHJRJWDQRCUWJEBGTZD");
    msg.s_type = 24U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ReportedState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteSensorInfo msg;
    msg.setTimeStamp(0.554385623677);
    msg.setSource(32892U);
    msg.setSourceEntity(200U);
    msg.setDestination(54403U);
    msg.setDestinationEntity(57U);
    msg.id.assign("JFGWEYDCJHHQJMZGSNCVULULBCADTUCMNVUUTXDHQIVKKZXKLPGOCOXIIOMRRDOFETQQGPMYXLDFDOISJTZPWYYRJJRNHPQIROZKERKSLQZNTKEGYFRVWFUELGUIWWJUSDNQPLAXYPUVWZYIXNJGBKBWSSZCXCUREJAPHFFHHPAMHYPGVYKVZQNBATPZMSV");
    msg.sensor_class.assign("VLZAVJNCBEERAFZVEEPRVKSRNOFTTWLXHIWEQOTJMRIAZVOUHRKPISFNLSBKZNCBHGDKTYHJKBWQHBVUDOYQUOEEMZUAFPCILCJPTWMDBQAWOXQCFAYCQTRNQMHOGRSRHSZXDCGMNMDTICBMLGFIJUPDJEDYNKFCECFLRGUABBBFMXHAISVPUWGXVSFQNWJGPTPGQYKILUSPNNLGPXAWILZKXYXMTESJYXJDYAHVGDVYTKJ");
    msg.lat = 0.81636225733;
    msg.lon = 0.0764730945115;
    msg.alt = 0.618884277006;
    msg.heading = 0.573380388993;
    msg.data.assign("OAKBVAYNKNASCKMNDAESGFJA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteSensorInfo #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteSensorInfo msg;
    msg.setTimeStamp(0.43798924597);
    msg.setSource(33046U);
    msg.setSourceEntity(238U);
    msg.setDestination(52939U);
    msg.setDestinationEntity(137U);
    msg.id.assign("DGYCNLJONDWRNFDPVGGEWRONCEYQZISPPAXIFZOFJNUVEULFLIJYVZFYZTHHLPCHIEODZXAVXCDRCEXFAPAYGTDRYTWCGHBNSAKKSIBKNKVTEANSVYYKTJRNSMGZURTBRJMRIUQUKXBPELYMZUVLEVRTGLYMMBDSHHULWCBUOKTXQWQZHOJJKMPLOQIJBOXKOZDUXIGQPXWCZTQOAFGMVRSTQHMWBGHFIVJDPANEELQCBW");
    msg.sensor_class.assign("JJHGUFIZLZHOFSIQCMUJAAPSGEECMUCOAFGIOKA");
    msg.lat = 0.0788305102608;
    msg.lon = 0.200144674904;
    msg.alt = 0.777117232633;
    msg.heading = 0.558294572333;
    msg.data.assign("IRHTFAUWOWKFAEKTTBNCJCIZUEXSGGJQJLNTORRDEVEBVBSDIHWYOFULUJZDPDEUPVANPLVGIRPRTLXMHCNVUAWPBEGRELAYVZCNPZMFAKCSJVHZYORVNOIDHITCGEZCZXVSWMBKSXQOYNUNB");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteSensorInfo #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteSensorInfo msg;
    msg.setTimeStamp(0.490101548562);
    msg.setSource(11510U);
    msg.setSourceEntity(170U);
    msg.setDestination(3138U);
    msg.setDestinationEntity(121U);
    msg.id.assign("FWQVYDMCDSKTPDZIERDDOMYYHPGYGKBIGKCWGWGBQKWHGFUQXOAGMMZSYREOVAIUXFWMSLGFENXPNNQFBGYALUFIQACCDZSBCOENFSIFKWRULXIWPDNYHARMHHNJZTNVRTUELLJAODOTVKLLJCAPVBSXKRVZLHVPTDBCTCTQEMWHIJQREJJHVPKQXVPNDOOZJLS");
    msg.sensor_class.assign("LXBWCLABXCUZCQVLJDRNZWKWGJMZVEYEHNRILWAHXTROTSSFAXZRQNOUFBKPIRGVSSKNWSJHFYPFSJDZOFRRPHTWONDUXIUYOBQVGMKEFVJQBPSLBHWRKXYLXVNUFCSBKAUPASMGWMHJVZVEYLIHITWCMNGPBBVYDMFTOQJUMMPQQZGHGHNCBUEPTDCEEOIIQODFIWMEVPYYNJJLKGCQRPAFODSHTXGETIDXCATOKNZRE");
    msg.lat = 0.0449528022896;
    msg.lon = 0.588150375579;
    msg.alt = 0.3939092897;
    msg.heading = 0.731615889597;
    msg.data.assign("JNTXISEQZVLYXYRBZJUJZJDMLTCPPXMTTRUZPGCKYFKTMSBA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteSensorInfo #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Map msg;
    msg.setTimeStamp(0.335317007706);
    msg.setSource(21428U);
    msg.setSourceEntity(111U);
    msg.setDestination(19335U);
    msg.setDestinationEntity(246U);
    msg.id.assign("GGUMECOLEYFZLXFVASEPCFXVNPOSJZTUNYSRQWL");
    IMC::MapFeature tmp_msg_0;
    tmp_msg_0.id.assign("DKUHPGXKYAMKHYEJPTEQRVONGBDJYEVWOUYNWUYUUARDSBNXXJILZEFEHPJZPZPPIYGUCDTSHINMJXWXTMHSSLFTSHXLRRBIPQLZYCVRFPKSLZQOAYTXUPNGLCFSUXBJTZSQLGBLMMWRIKCFHWNBWELCGTAXAVITFHZHJVDEJDHNARIDKODNAOMKPFACEZQGFNQBIVVWYIAOOQVTMXOCBKOGUYBFFODCUKMBRITZZNWCV");
    tmp_msg_0.feature_type = 230U;
    tmp_msg_0.rgb_red = 227U;
    tmp_msg_0.rgb_green = 117U;
    tmp_msg_0.rgb_blue = 106U;
    msg.features.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Map #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Map msg;
    msg.setTimeStamp(0.328045263835);
    msg.setSource(20402U);
    msg.setSourceEntity(14U);
    msg.setDestination(43723U);
    msg.setDestinationEntity(188U);
    msg.id.assign("PTQSTUVCVHPEOAPFOJMCYVVYWPITIPGWXEHOOKOOAXN");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Map #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Map msg;
    msg.setTimeStamp(0.312152887921);
    msg.setSource(22410U);
    msg.setSourceEntity(206U);
    msg.setDestination(11789U);
    msg.setDestinationEntity(91U);
    msg.id.assign("WYJSZMOCQDTRCKFQXULZMUEOLDWEQACNGMV");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Map #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MapFeature msg;
    msg.setTimeStamp(0.999959462078);
    msg.setSource(43792U);
    msg.setSourceEntity(128U);
    msg.setDestination(9535U);
    msg.setDestinationEntity(228U);
    msg.id.assign("ZLJPYQUFMPSMXGUPJSVSRLKKLEJJNGVAFIHHSTQAZJETCUMUNQXTUNSTGFUOYIAHWSHWMIKGWQJLOIPKCYQNEBYVSHBEVZSCGVODKAS");
    msg.feature_type = 170U;
    msg.rgb_red = 7U;
    msg.rgb_green = 22U;
    msg.rgb_blue = 66U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MapFeature #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MapFeature msg;
    msg.setTimeStamp(0.702938099669);
    msg.setSource(29641U);
    msg.setSourceEntity(163U);
    msg.setDestination(30446U);
    msg.setDestinationEntity(19U);
    msg.id.assign("PJZKLPBWDXRJEQWLXAZPMEPFIA");
    msg.feature_type = 91U;
    msg.rgb_red = 106U;
    msg.rgb_green = 194U;
    msg.rgb_blue = 129U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MapFeature #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MapFeature msg;
    msg.setTimeStamp(0.460585738718);
    msg.setSource(38018U);
    msg.setSourceEntity(172U);
    msg.setDestination(8695U);
    msg.setDestinationEntity(250U);
    msg.id.assign("LISOGYZEQKIRUMHTFECXYZIHSHFFXVUQTOCJXNSPWAVGDVJXWSBGQUWANZOGMSIBHAFPRIERSXUBINEZZTPDZHAFWYVOQONFBNMAEDKXCXDIDATJSCWLUBKEPRICVLBJVYMVJLDGBTSARPAWPBZMXK");
    msg.feature_type = 110U;
    msg.rgb_red = 91U;
    msg.rgb_green = 216U;
    msg.rgb_blue = 234U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MapFeature #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MapPoint msg;
    msg.setTimeStamp(0.129574986901);
    msg.setSource(51043U);
    msg.setSourceEntity(229U);
    msg.setDestination(54436U);
    msg.setDestinationEntity(6U);
    msg.lat = 0.434052244543;
    msg.lon = 0.934416302907;
    msg.alt = 0.399655678017;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MapPoint #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MapPoint msg;
    msg.setTimeStamp(0.807641799547);
    msg.setSource(4621U);
    msg.setSourceEntity(252U);
    msg.setDestination(56614U);
    msg.setDestinationEntity(18U);
    msg.lat = 0.491763044703;
    msg.lon = 0.874205430471;
    msg.alt = 0.55003269141;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MapPoint #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::MapPoint msg;
    msg.setTimeStamp(0.806555992105);
    msg.setSource(47305U);
    msg.setSourceEntity(219U);
    msg.setDestination(54859U);
    msg.setDestinationEntity(84U);
    msg.lat = 0.352070585849;
    msg.lon = 0.374905640019;
    msg.alt = 0.268700199678;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("MapPoint #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CcuEvent msg;
    msg.setTimeStamp(0.709110486284);
    msg.setSource(58152U);
    msg.setSourceEntity(87U);
    msg.setDestination(12549U);
    msg.setDestinationEntity(150U);
    msg.type = 67U;
    msg.id.assign("YROOCIPSUXQVZBCVAJEYFFDAUBNEDDWKJYUFEKZVRVDKUMYPSHQMLMXDMOOHGJNMXXSWGXIHEPAWTSPXOSUVXORQDAWCZCQEJBUCKRYBBLNSVGLAQFIFSWHQLDCYKAGQVNHTMJQZRYGIJVLOKVUXSNZXIAVEL");
    IMC::LoggingControl tmp_msg_0;
    tmp_msg_0.op = 197U;
    tmp_msg_0.name.assign("GIEKIDMXVEUMTDCCNMNPQGWGUBAFHNCYLSLGIQCNLRMHQVZZATSFQEJTZLPBMVMQKYFGOLJGJDDOKAXJXYCOKHJUFYAKORPWVIUUTWTHACBWWQHBXJRDNWSYDSHFBLSWRZ");
    msg.arg.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CcuEvent #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CcuEvent msg;
    msg.setTimeStamp(0.464497067429);
    msg.setSource(52778U);
    msg.setSourceEntity(251U);
    msg.setDestination(45695U);
    msg.setDestinationEntity(22U);
    msg.type = 210U;
    msg.id.assign("TORFMIRWXKVDMT");
    IMC::MonitorEntityState tmp_msg_0;
    tmp_msg_0.command = 211U;
    tmp_msg_0.entities.assign("BWLUWXVUFFWVGJGMIUOXHJPDKRTHMMHYMWJSKYUMPALWUFTWVPPDYPQGOSAWBSIKBE");
    msg.arg.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CcuEvent #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CcuEvent msg;
    msg.setTimeStamp(0.322347454252);
    msg.setSource(43453U);
    msg.setSourceEntity(254U);
    msg.setDestination(47830U);
    msg.setDestinationEntity(158U);
    msg.type = 103U;
    msg.id.assign("PXSHFGTHFIMXCKYUVVIPVYLTEKHUAVQUJCBHZTOYIJWENNZWBWYFYQYONPUBTJTYLEDJDEWHZHZPDBKEJZYICGMAQITZXOQOXXNGDLMFLACDRYFLXSOLLSUDADJWVWMKIIKVBUBOPZGHRTGMVIRTDVNVUUBWZQMQEFTPSFXCSLQCSMMERAJJSGSAXURGBSNINYDLCWQOCNDUWPHNJEQFKHPIPR");
    IMC::DepthOffset tmp_msg_0;
    tmp_msg_0.value = 0.605903248784;
    msg.arg.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CcuEvent #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleLinks msg;
    msg.setTimeStamp(0.988227958041);
    msg.setSource(22151U);
    msg.setSourceEntity(166U);
    msg.setDestination(19032U);
    msg.setDestinationEntity(7U);
    msg.localname.assign("PYZAVHKVIXEMVJKGQYIUQEIBKXCMXMNCFIHJULYDMSLLSEUOGCDKUTRWANXKCFBTVLVLXCBNKCRMZJHDDOPZQSIVHBWPKZAIWYGFCAXPHEQTUTIKAOXQIJJYNXPJSEFRSGZVTUHYRVPWLIWONDTFGOHLHLZRDFWWSQACGYNKQCBUBQMJSNAMBJYGRBCZRZSFIT");
    IMC::Announce tmp_msg_0;
    tmp_msg_0.sys_name.assign("KVQZOJJQOJVXNIRXZLLGYNVKAYRCHCBFJMATDQFMYVRTQNQSUSZETKZTOVEFARZFSKHWNLKCOBQSIBIZAUAXUMVSXAMRKBGWPBNPUZPEPYSNPJHVOYHDKWGFBEZGXJEPQGFXIUJCDEEHVDHYNICTLIMVSMEBHOZDXWFWCTYVODUOPTXOALEWQULIJGZHKC");
    tmp_msg_0.sys_type = 175U;
    tmp_msg_0.owner = 17729U;
    tmp_msg_0.lat = 0.777215063466;
    tmp_msg_0.lon = 0.215650628624;
    tmp_msg_0.height = 0.167738779306;
    tmp_msg_0.services.assign("ZOPNYEWLMLNVOHNAZMGDPUBNWBLWEEVBDHYWTLDDRVMRPEURXJGPXRCGQOSRPXOKWFGIUGKJSAZIIEPCGNOAMOGKJTMOVBHKTWBHPJXDWYADXIDRYTXNVULVSQHOSJFIIAAZGHUQNAJFVDTCNNCXLZXUFJTEYPFZRQWYXCYKLHZBWJACKBINBZDEHCPOSLHEYTMEQMRUFSYSJFKFGLF");
    msg.links.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleLinks #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleLinks msg;
    msg.setTimeStamp(0.0654278028189);
    msg.setSource(22975U);
    msg.setSourceEntity(39U);
    msg.setDestination(10224U);
    msg.setDestinationEntity(162U);
    msg.localname.assign("TJEAATUZEQTQYQDEUSSCASDRSNDDMQCRKHYALWRPKYUYWHMVTEBPUZCOKIJHSDOGANJVFILUMFDKKDIOSWFMPBWQOFVHXCDRZPRBCLCLVOFUYHYEONVDYCUVB");
    IMC::Announce tmp_msg_0;
    tmp_msg_0.sys_name.assign("SMYATJEVSWGMKNDFAXGYDCNNTMIHRJGGMLWMEYHQCLTFFOJNIXAIGBBUSUXVEKHUSDRLWQIKDWCBVADRZVICNSEPUPEOJTWIGDMJGBKZEQZVZZYUJKRDRERQAJBIBNZVVUSLCREPAHJKFQKYHLNNCSOUXTDLBTHNMXZLIYO");
    tmp_msg_0.sys_type = 71U;
    tmp_msg_0.owner = 65027U;
    tmp_msg_0.lat = 0.247541452087;
    tmp_msg_0.lon = 0.0216456718373;
    tmp_msg_0.height = 0.975055645429;
    tmp_msg_0.services.assign("RCHXPVWDKEKARFEFTJQRBDWIVGSAHOVYFEGYDIVAMZRTDPFEWHXLSNXYCJSONHPPHBTUSUZSCULGAKKRJBPWVZDLGZOCFSJHYPAJMJKQXKLXDOLZNWTNBVTPFDMBCSKQOZXHRWBXYPV");
    msg.links.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleLinks #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VehicleLinks msg;
    msg.setTimeStamp(0.563092961181);
    msg.setSource(27372U);
    msg.setSourceEntity(206U);
    msg.setDestination(85U);
    msg.setDestinationEntity(211U);
    msg.localname.assign("ARQDUKDJLEVCZYISRAWYIBHPCWWEUDPWHPUVBTTOCFGKAKEXSALZXLDVQFGDTSBXRXTAVWGUJYXNIHAVWCJEJZLOQDYMBPSZPF");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VehicleLinks #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexObservation msg;
    msg.setTimeStamp(0.863227214153);
    msg.setSource(30002U);
    msg.setSourceEntity(224U);
    msg.setDestination(13909U);
    msg.setDestinationEntity(178U);
    msg.timeline.assign("SOHJHXRTCGHRGFJMEIZLVMTTEXZSAZNEUOV");
    msg.predicate.assign("GKXYSTNSFHLZRTEQTBCPKIZGJQBGBXMIGARCMYMODEQFIXQPWZFVLQTHYIFBAYKWTAZQUBDPWGFTWUHEBLHBKIKOAFMATRMFSHXRYSLOUXQWONUCXGNMYJRNISJZSJNVOAUFKPEAQPPGSFJXJIZSBOGIWWQPUWVLONARAHOAZGMV");
    msg.attributes.assign("WSDVAKJZFVOCZ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexObservation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexObservation msg;
    msg.setTimeStamp(0.601445074796);
    msg.setSource(10984U);
    msg.setSourceEntity(93U);
    msg.setDestination(20681U);
    msg.setDestinationEntity(34U);
    msg.timeline.assign("TJZQIFLUSIKDIRWQGAXYYELZSOSFJCRCECMJXDFWYLCDFUKULMTDMZAXNHPOXIQGGPJPABWUWDDBQRLUSSKMVEBISODWNIIHWAAMHFYAHJGVTPYNXZXR");
    msg.predicate.assign("MSBEJBYUOGEGLEIHCHDWDHEKYIKJPBNTNLNUPVGJHEXOGJQORSNWGRKPNDCAWONIQTZRNLRQHPLFOYXKXWRTDFEMJFRAJZWVFSDFQZIGSKWKBGHYVOOTYMATPTZSYVQQWCRJVVFRCDXLLTSBCUVQCAGVLBPVSMHYBFMMVYEPGUEEZUZSPQKLPIAMOZMDKXOYBXAFCWXLJAWSFDTZ");
    msg.attributes.assign("QHYWRCFMCLKEVJUBNKDZGDZODBYUOBVLRDOEUBDUQDCUBBSYWFPWFDSUEXZMKIOMMTLUNPDNJEHHRWZEWPAPCSGCUIYHMQAJTNCWAXHKBZVIXLMHOJSOLYWJDMHEFVFZBIQKQVFJBIOXSNGNNXNXWTFTQQJIJIQSLYA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexObservation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexObservation msg;
    msg.setTimeStamp(0.202455418283);
    msg.setSource(14600U);
    msg.setSourceEntity(31U);
    msg.setDestination(5045U);
    msg.setDestinationEntity(165U);
    msg.timeline.assign("UMJNJONRMNSREWPSSZGKHCRSZKOXLHYYHFQTSXZQEYKBDBRLTPKFQKEFRKMTYCYOWRTFOEIGXVLGJIHBUMASRNQGYBIYNWJPTOGJZKDAAOXTRRCAUIJVNWPJCWFFWRMLOEZPQYGXLHJLVHPNVATLKSMZACXCAODBLNLVJCVGGBHVHEJCIXTZFDZVMSZVUQXWADDWBPUN");
    msg.predicate.assign("UHKWJMFMOTOINKQPPSRSTMEJVIICOHQRIVFWAHCBCGPJBHBLLZPULIMEDNGAJYWEXPDGYBWXYXQABHMRXLZPQIMMDAUIZRVWLYKZJWFVUAQAGEXHVPXSNSDOLPYYPSQKMZBFVEUQJUYOVMSJRKFQBDEUNUFFJZO");
    msg.attributes.assign("RMVCORMWFLZXIJSJRHNWOJVBHVMHYAZYYIYYGNUCNTCIUJOGOVMHNGDHDBUYFHZZXIEPMXTWQUCAJORIXSGZCGDOXJBALGLGRLVNDAVYRYZKAEAHPEQOEQISKMYEALJSTCDXQDKCUDRCTTWSPENYIZLLQPFPGXOEUADBKLASSAHOWKRIEBG");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexObservation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexCommand msg;
    msg.setTimeStamp(0.413036155826);
    msg.setSource(24923U);
    msg.setSourceEntity(195U);
    msg.setDestination(63805U);
    msg.setDestinationEntity(26U);
    msg.command = 149U;
    msg.goal_id.assign("XIVWDUEVCGWCNDUYJOWFSRBDVTWHHXTABWTRGNQFWPDNGRXYQIOHDBDFBAEAOTKPVHCYZYSUOWQYUSCPVLJPMMLTNXMRSXZWCRKKCXGRIBXQCEMZQEASHKJNFJUASQKXXDFESTGLFLGKPNRGSNYJBCMIOVAULZSZLQRDIJBRWNFGVJJLPZFJHEFBHBMUTDAEVONYAYRO");
    msg.goal_xml.assign("NMNTHMSVRPBNGWGUPNAWSDAEJBFACUBZKEILRHTZYMGTWXKBUCGYEMWVXIEBEKZZVQPFJBALGVIMWQLGJFHQYAVIXLSSWYNCS");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexCommand #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexCommand msg;
    msg.setTimeStamp(0.151677392587);
    msg.setSource(15184U);
    msg.setSourceEntity(222U);
    msg.setDestination(60539U);
    msg.setDestinationEntity(57U);
    msg.command = 239U;
    msg.goal_id.assign("FGXLVSQKOEYNUDCCJTAZOOZABZDVZZEHYRHDFFEUAIFSLTKDIJY");
    msg.goal_xml.assign("UOHZJVSRQPZLKEKNBEKFVEEMENCSYAWXISBKPMWBHR");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexCommand #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexCommand msg;
    msg.setTimeStamp(0.137332965308);
    msg.setSource(56597U);
    msg.setSourceEntity(36U);
    msg.setDestination(16165U);
    msg.setDestinationEntity(199U);
    msg.command = 104U;
    msg.goal_id.assign("TDDNZLOBGVZSHSRZOIFQDXZBHFTMOHK");
    msg.goal_xml.assign("SVKSEJUKMUIPLFWZLZTQFDWAFFBEFYYPKZOSLOHBRNUISKSEKLHIJNMKMQEITKIYXBSCQETALXAPDCVYQLUKVVCTXTUWPBXMFRRMJHGWJHKZPLEPVUOYTWCCAMNXUPJDPCDNDGRTGEONSW");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexCommand #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexOperation msg;
    msg.setTimeStamp(0.638425501725);
    msg.setSource(18990U);
    msg.setSourceEntity(21U);
    msg.setDestination(30787U);
    msg.setDestinationEntity(117U);
    msg.op = 9U;
    msg.goal_id.assign("SIWLTUGLEGVQVNTTHDXZZWNVAD");
    IMC::TrexToken tmp_msg_0;
    tmp_msg_0.timeline.assign("VZQUJZKZARPUKPGVLLIGRXZSTQQTUBJDODNGTAMLPXNLWAUIJDDPCAGECKBSNVUPSFDWEQXEARYOGQQKLXREBBBOWMSCQYROOWIFVFSKVCRYZETYGNJTVQYJVRRNBUALDNCDITHSSGPJNALPYMIQDKFDNGXKXZIZHVNEJCHIXYTABEFHGFLAIKVRKEMCCMMTUHOWSUWZHYJFOWWUX");
    tmp_msg_0.predicate.assign("DRFQBYIEYOMNPZFQYWPWNWIQRCSQ");
    msg.token.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexOperation #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexOperation msg;
    msg.setTimeStamp(0.318907512082);
    msg.setSource(31360U);
    msg.setSourceEntity(27U);
    msg.setDestination(51381U);
    msg.setDestinationEntity(37U);
    msg.op = 137U;
    msg.goal_id.assign("OUKADXZEVYCNZHJGVLUK");
    IMC::TrexToken tmp_msg_0;
    tmp_msg_0.timeline.assign("UGWBFYQXHPPSPUHXXDTLDCRLIJTHYICMZDIYCBEQSSNOWUAAGAHXQNVQSFFWAEIVOUZIOGYXKIFQZYAEQGHMWPBKMKYBODVGJHCPLEASXMZWSFNTLAPHRPMFOOKHPZERGVMBXDQIEBONDUCNSMFLKFCKJZETWLCUVTORRWVRYZBBWYGXJTLUEEZTJJCYNDGXHOSJVMADLSNTRJHMNPQLKAKFPBADMJVRGV");
    tmp_msg_0.predicate.assign("CBZVIGJJOQPNQFXDKBRXBENHQYOZKOFTISVXRRVZJTWZEVSLOKGSTISOERYLTILYRXUKNJRZGVWCPMMMBOFHDECIWFZBKMKNISCLROPLFFVQNRGSPZDVNLKNGSHHUBFALPWNYYAQGTXMRCMRUBHUSLGXKNJJQTAZDJ");
    IMC::TrexAttribute tmp_tmp_msg_0_0;
    tmp_tmp_msg_0_0.name.assign("AJLJBEOBKGFVMMSELURYXKKRTVHSITRYDRFQGKLNVHRFJNZZSMBDSNQITPQTINZDQXSWYQOVUOJPATXFYSOFCSOYH");
    tmp_tmp_msg_0_0.attr_type = 202U;
    tmp_tmp_msg_0_0.min.assign("WQVVZBXSYVQFGUWBBFTZGOAAKFBXOMLLXDBMUOTVRJLRKJEOWONXNNRGHTXPOPAYPWUQEUEXDEQUWLHLKHZZCIBIWKRMYMGFLLGRSRJYDYVONVAUMKXYQDNMTWEPCZTTBSZJHNQFAHYCSRHXJPSNKXPSFPQEJNMUYGCLMIVWBGHECIHVIXFTBWKCOAQVSUUJGSNLDPBIJZGOADNASDYSFKTWDCEDTPRIIZFPVKRRKJMU");
    tmp_tmp_msg_0_0.max.assign("BGDBMRPINRHRUCFBEDFZZGMVSHIAQOTMNVIDDMMMXACOUCCQFYVHGQLPLVYLSWGKWLVPTRZPUTULQVUSJDZLZEBLJRAHDPSBUJYHVEZYRHXIBFIASDWKTUNEMLTNPUAYJOKIDCJKKEVXIUQSGGAXPTQEGHPZKHCXKBLNLNYOTTWBVIFXO");
    tmp_msg_0.attributes.push_back(tmp_tmp_msg_0_0);
    msg.token.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexOperation #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexOperation msg;
    msg.setTimeStamp(0.0182305221173);
    msg.setSource(52507U);
    msg.setSourceEntity(151U);
    msg.setDestination(1284U);
    msg.setDestinationEntity(126U);
    msg.op = 207U;
    msg.goal_id.assign("JSKCXXMEZMGZFKQPZBBKFVBLATDZRPGILGOQYRBXCOXMTEFIZOQLFLTNDNQBCBYNSOULJIEWEZSWSDKMZLUWHPFVWXUJWIGZFXYRYJPJVDSGYMNHGKLJJANRAUHFFDREWRQQVNTLKYIYXVJHHSNSCPE");
    IMC::TrexToken tmp_msg_0;
    tmp_msg_0.timeline.assign("KLFNRAGZMYWNCUXAFGZRVVINYSWEQWMUQFSZTDEPLEEQCARUSWFBTZJFEJCPGHPYAMXGCMGQXPTRRSILOJDPLLDDCOKLTF");
    tmp_msg_0.predicate.assign("WLCBRERSKBJSRDCOKAPWVRPGLWQSVNMRYGPIHNGUTYUKDMZPORFUFNODHHIZLLNCDKJCBFQGJQCNXWQBUEHEWBYJJOWKFDKTIEBVAECKRMIQTVAXLHPHYGDFZJVEUZDSKMOSWFIGBJWYMHISLZWEQPGCTAEACORAADEYQXLXUPUIMFHPSKVFZNFMXOXZXQXXATHR");
    msg.token.set(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexOperation #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexAttribute msg;
    msg.setTimeStamp(0.0100312263931);
    msg.setSource(21682U);
    msg.setSourceEntity(16U);
    msg.setDestination(733U);
    msg.setDestinationEntity(189U);
    msg.name.assign("SUKRKLZAAATGRUSABKGWFBBKEXDSIXYZBWIOYLHJYGPLKZOJRCEETIKDLAVNPYDQWDQKUCNZTMTWPXJFQMZDFZBMMWYMHGJLVJSUKIVUZGUCYVXOTGPBYTGQEQUWEFV");
    msg.attr_type = 178U;
    msg.min.assign("CHKWKNFSDFFFHFJSXOHJCACTRGLAUTJIWMFGJQSBSDCEVDSXRUTUPXHKIJVYEGEBYNZBQWAITDMLMMIIEJJRISYTZYWNIZYFXGOCFYUWZBNKCXMQWKWJVDNOVHZAJNEMRIGELEQDLLTEKPWONVPBARTZGGTLXMQWFKJQGVTZHXLLGPDPHUPLRWRXYRDBEUBOMVNVQMCAGKYPNQUZRKBPCYZRZVDHSAQPFHASSOLSXUBXOY");
    msg.max.assign("IQJAEHXVOHTNJLBEMZGDQAKYJQHHWCVHDNBRHRUPQLDDFCUCMYGKPTKWXGCVAFAKZDMNWJBZXIHNYZCIMEFSZPZDPFBQEQOCLXYGBKNISTRWMAGCFLLGFUFHCAJEKWFJLSGXQVRXZDKDHTWPTCFONSVLSBMRSKJEOFSOXRIEDTGMMBLXOESZUYOHPAVLJXAUIDINO");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexAttribute #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexAttribute msg;
    msg.setTimeStamp(0.685896161836);
    msg.setSource(333U);
    msg.setSourceEntity(223U);
    msg.setDestination(41257U);
    msg.setDestinationEntity(217U);
    msg.name.assign("PJYPZSXUYFXRRDOCJVLSOCTMPEVNBLBZMKFFZDBCXNKKEYRJVHUMFCHUEOAYGGZOEJNVFIWWZYMSLNUWXHCOSRVAQQRAPZE");
    msg.attr_type = 14U;
    msg.min.assign("ILWOIRASNXMHKDNEPUZXGVULIYLWWJDKNAGXWOZVAMDZGCJTDCQPBKEWHPWOYVGQBXMGJYDIVZJGTBHJXKKCPDAHPZFWRFCSCTYPCIHTCUEKRKZUQBVLASMQBSGWJRESZHTXPKILZRAXQMLMRNMCEERZMNTOUZABBOQOWOTEAQGLJJHFS");
    msg.max.assign("CSANZUIMJZDPYWRILXKPYFLCWOEVXSHAMDGQAKLPORGVSFFGMEMZTSJAUTPTKIOWNZQDHXPVPCAFHHKZJOIMPCRQLUFTERGALBWEBONDKVYICNKVRORVIWCKLUSQEZZWDUHJRYDWBGAHGGFFEVALM");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexAttribute #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexAttribute msg;
    msg.setTimeStamp(0.402655794283);
    msg.setSource(35273U);
    msg.setSourceEntity(26U);
    msg.setDestination(20293U);
    msg.setDestinationEntity(125U);
    msg.name.assign("BBPTVZYCIEGLCJFACWGJVVDBESHZZTKZZTNSRBQGORWQOAPHJJANXAWKHDMHDRXKFVYWLQQUWISYHNGWITRMUUKEDNXXJHTJESSPDAQIXSHFOLTZZBCKVGNWISEDFEUFZKTRLQUOCMNAGAMRZWTBFIDJVLKVMFTYBXAXOPLJPNKHWLBYOJIIYAQQYMROJVNSDYMOCHYBSE");
    msg.attr_type = 9U;
    msg.min.assign("APQORCQHFTNFDBWNQSBHQIVEKGGVJMEKCSUKKEXNCJOWYLUCNGCQKIAZWAYDPXQDGRXSCJZMKMZDDVHOXYTJXNYTZGBMUWXOCMHAFAYVY");
    msg.max.assign("GDNBXHQIGYZRKPVYDXFWBFWQQELPKTPHJVVPRSPOIUZLSVQUTYTSOZDFCJYZPQOKTFAJAKOPGJHGQXVXNPVODSSSPUHSCWNUYMLULYSDX");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexAttribute #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexToken msg;
    msg.setTimeStamp(0.172384219571);
    msg.setSource(9320U);
    msg.setSourceEntity(234U);
    msg.setDestination(17663U);
    msg.setDestinationEntity(62U);
    msg.timeline.assign("PXMIXZVOINTVKRIZQYBESZAPNHDRCPUTXWDLPKBTJZXRZWZENLHPKSMRCKSNWDMRXZKNXNWSSLHZTUCIIKWUJCLIQRKSMJFZDOVUUEBACIQCGPMJNNOKXQWPPTVDLAAECLCFGGYNRAYOQQ");
    msg.predicate.assign("QANKUWMQSLLFSXKMIAGSLLHEYNGJVNBNFVOSUATPKOUFXDPLHLRIDZGCWQYGIQCWYVVIPJKRWYTJIJAUEQGNTNWVAIRHSUKFLHUMSXMTZCBOYFROSBMAKHPYTPKHOTVQMTKECLCEQEUEUTVHOORXDEVBDIMJJZDDWAERMNMFFWGJPPX");
    IMC::TrexAttribute tmp_msg_0;
    tmp_msg_0.name.assign("UEVRTHWQHBEAWUXOILCQYO");
    tmp_msg_0.attr_type = 32U;
    tmp_msg_0.min.assign("IWTLCWPNBMZVHAXHLLFJYLISWFCZMQEMHCGRNZYEESDCGKYFPFRBYNGMPENFCDHDSWQOQGRTDIQZVHJRPCXXIDWZRLOXHJAKMSTUEWVIATXLMOJSKXOCLPCUQVXTZHDGZLJASXTUQFWQW");
    tmp_msg_0.max.assign("XEUJDYOVWDBOMXLHSEAHQUAGWKNTNMAPMZNDIYVBNTURRYQCEIDFECRPINHHSPFQBPKFCEAMLIASJVHBLCCUEQFKUWWGYVCIJMFMJHWLZEMZZJPRPGGCVHTSQZFSIQOKOGDJHZUBOMNYQXRAVOVKXYUUVTTICLJMWLWVDM");
    msg.attributes.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexToken #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexToken msg;
    msg.setTimeStamp(0.412401927293);
    msg.setSource(3853U);
    msg.setSourceEntity(234U);
    msg.setDestination(34809U);
    msg.setDestinationEntity(100U);
    msg.timeline.assign("HTXYUPXLIBGAJXPIASTSYNOTTTQKRDMJSWYEUGYJ");
    msg.predicate.assign("AURDNKZVQYGCNIEHZBAIDPQIENCSGYKRZJOXEFOTEYWQIKBPLNZIFDDRMKHFSWQGCGCHEPIBAUHLYOTSASMCTYVVDDRUSPMHCPX");
    IMC::TrexAttribute tmp_msg_0;
    tmp_msg_0.name.assign("EXHVXZHQHDOZMDZSLBFZGPNUANZVIVMSJNCTJBBLJWYPHFGQFPBHTMAZWLAXTNRTUMDEKIALFOVPLJDTRCKWKVXSWTSKMBUKNHVROYBGGQOISUPVKZAUXEKCEMHOQRALMJGCDVNPLYKRTNJGIYFJCIEPWAQOXVDZNMPLWISDJSBU");
    tmp_msg_0.attr_type = 139U;
    tmp_msg_0.min.assign("RGKALUJDFXKPJBIRZZPYDUQWATXGEWNMCZTIOFXODAXOOGORNSQSKWPQIT");
    tmp_msg_0.max.assign("LFIXIITMGXUWKSZGFOHTVBOCRDAHAPRTDEWNDDTQQOGKCOACDNBDZJENPWNYCGWVUANKATEOJZBFRXEXUIWVVJJOEZWSXMOHAKBGPVHBQSDYSRGANPQWAEUHLEURYLJGCMJZMHWSCWERQTUXVIMNH");
    msg.attributes.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexToken #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexToken msg;
    msg.setTimeStamp(0.872951517389);
    msg.setSource(2092U);
    msg.setSourceEntity(145U);
    msg.setDestination(21614U);
    msg.setDestinationEntity(221U);
    msg.timeline.assign("EMMXAVIKLREEWJMQHDTHBZKNSXNZNQFMTVYBDAQJYAEVWPHKJMBSKQZBKWCNUNROFAMXBPCRSDAOSSBTPNVCZUDUHNFYHQOEVECHKBVYMEKFPPAWHUZASB");
    msg.predicate.assign("DYPMNEJVSYLXGSMKDKRRUBPFWQWMEMKBAZAVZKTIRBIFSYXPQOUYZNTHKUGHIFCNTATH");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexToken #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexPlan msg;
    msg.setTimeStamp(0.933137843373);
    msg.setSource(63417U);
    msg.setSourceEntity(233U);
    msg.setDestination(1308U);
    msg.setDestinationEntity(105U);
    msg.reactor.assign("ECTZDYTBRWHFZOIFJPAPBMFGUMHNRDTCRKTUCBVYZXUKHOAPQPWKAHXJUNPQQJFGPQQGWVOVXCRHBAUGAVZMAFJD");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexPlan #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexPlan msg;
    msg.setTimeStamp(0.0438745872293);
    msg.setSource(28333U);
    msg.setSourceEntity(114U);
    msg.setDestination(36762U);
    msg.setDestinationEntity(171U);
    msg.reactor.assign("ZLDAXQCRAMXIGQUAGCTWTAFRODGKURESUBNDWV");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexPlan #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::TrexPlan msg;
    msg.setTimeStamp(0.0532512980493);
    msg.setSource(7866U);
    msg.setSourceEntity(113U);
    msg.setDestination(31573U);
    msg.setDestinationEntity(54U);
    msg.reactor.assign("GGSDLKMODEUQQXUIIJKRVDDKJCMTYDYAFWIJBCHKTNTFSORAVLGLFOCIMTKJNTQHQWURHOZKBBQPHWA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("TrexPlan #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VideoData msg;
    msg.setTimeStamp(0.231776732997);
    msg.setSource(16489U);
    msg.setSourceEntity(103U);
    msg.setDestination(47943U);
    msg.setDestinationEntity(60U);
    msg.id = 159U;
    msg.width = 17371U;
    msg.height = 14472U;
    msg.widthstep = 37041U;
    msg.channels = 200U;
    msg.depth = 190U;
    msg.finaldata = 210U;
    const char tmp_msg_0[] = {57, -32, 33, -42, -69, -45, -111, 47, 86, -78, -32, -98, 23, -79, 36, -65, -14, 120, -21, -15, -60, 11, -73, -8, -103, -127, 123, -64, 121, 9, 38, 67, 42, 81, -90, -105, -99, -67, -16, 46, -44, 32, -51, 63, 121, -4, -98, -90, 21, -125, 107, -41, 113, -121, -115, 63, -39, 71, -41, 48, -108, -16, -56, -116, 54, -8, 34, 49, 10, -23, -69, 2, 44, 85, 11, -93, -25, -58, 14, -120, -70, 43, 66, -117, 97, -97, -69, -73, 50, 80, -116, 56, 98, 9, 74, -111, -117, 58, 7, 90, 106, 97, -15, 76, -89, 11, 68, 63, -127, 3, -107, -68, -81, 88, 16, -41, 34, -41, 4, 22, -123, -74, -120, 40, -57, 51, 31, -111, 15, 87, 25, -94, -32, -20, -17, -22, 50, -9, 106, 63, -128, 1, 53, -125, 21, -55, 75, 62, 55, 87, 2, 77, 6, -3, 94, 55, 57, 30, 9, -27, -109, 30, -44, -113, 8, 59, -127, 47, -71, 15, -98, 122, -76, 112, -96, -57, 58, 59, -68};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VideoData #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VideoData msg;
    msg.setTimeStamp(0.0138897050166);
    msg.setSource(31507U);
    msg.setSourceEntity(233U);
    msg.setDestination(4669U);
    msg.setDestinationEntity(171U);
    msg.id = 145U;
    msg.width = 5641U;
    msg.height = 4482U;
    msg.widthstep = 9862U;
    msg.channels = 103U;
    msg.depth = 71U;
    msg.finaldata = 56U;
    const char tmp_msg_0[] = {-4, 97, -114, 96, 125, -65, 72, 0, -38, 70, -89, 13, -65, 41, -107, -99, -113, 24, 97, -58, -22, -44, -51, 71, -111, 19, 13, 53, 9, -14, -71, 109, -54, -93, -78, 82, 57, 11, 40, -23, -56, 13, 80, -112, -56, -9, -74, -123, -55, 86, 102, 45, -4, -84, -9, 90, 93, 42, -49, 104, 39, -55, 44, 17, 10, -42, -64, -62, -127, -45, -67, 70, 80, -11, 35, 86, -79, 120, 24, -2, -11, 35, 51, 41, 21, -115, -125, -26, -59, -73, -96, -90, -74, 43, -104, -85, -79, 25, 72, 123, -115, -68, 93, 111, 57, -76, 91, 114, 61, 48, -83, 66, 12, 101, -70, 25, -94, 66, -124, -75, 31, -121, 43, -9, 93, 50, 90, 8, 76, -78, -112, 7, -53, 95, 70, 32, -117, -89, -84, -30, 89, -69, -82, -123, 8, -4, -104, -116, 106, 23, 86, 30, -88, 11, -111, -76, -35, -114, -22, 18, 59, -92, 55, -22, 96, -36, -81, 62, -80, 78, 126, 56, -55, -119, 53, -106, -76, 68, 123, -114, -46, 123, 42, -71, 123, -20, 55, -61, -73, -45, -114, 39, -47, -16, -96, -128, -92, -92, -113, -95, 78, -88, 32, -86, 78, -58, 119, -73, -115, 70, -64, -85, -52, -64, 113, -36, -76, -39, 44, 86, 83, 39, -85, 2, 51, 3, 68, 81, 118, 13, -48, 9, -126, 40, 92, 38, -53};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VideoData #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::VideoData msg;
    msg.setTimeStamp(0.964216611031);
    msg.setSource(7453U);
    msg.setSourceEntity(30U);
    msg.setDestination(18936U);
    msg.setDestinationEntity(153U);
    msg.id = 103U;
    msg.width = 23059U;
    msg.height = 22302U;
    msg.widthstep = 53313U;
    msg.channels = 134U;
    msg.depth = 25U;
    msg.finaldata = 25U;
    const char tmp_msg_0[] = {65, -31, 71, 90, -73, -92, 96, 56, 84, -95, 85, 63, -98, -56, 102, -117, -11, -19, 114, -118, -38, -26, 77, -14, -14, 121, 75, -123, -128, -70, 85, -13, -63, -52, 80, -110, 34, 20, 23, -72, 101, 94, -108, -29, 23, -71, -41, 60, 74, 70, -120, -60, 42, 4, -2, 5, -6, -60, -127, -8, -58, 63, 44, -16, -79, 42, 84, -123, -125, 43, -106, -1, -56, -62, -53, -61, -50, 44, 71, 107, 124, -124, -75, 30, 107, 96, -89, -90, 67, -16, -57, 107, -118, 121, 114, -12, -4, 36, -84, 4, 95, 27, 0, -9, -68, -125, -110, -9, 67, 84, -29};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("VideoData #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RawImage msg;
    msg.setTimeStamp(0.0540534486724);
    msg.setSource(39732U);
    msg.setSourceEntity(122U);
    msg.setDestination(62228U);
    msg.setDestinationEntity(105U);
    msg.width = 42982U;
    msg.height = 13289U;
    msg.channels = 52U;
    msg.depth = 219U;
    const char tmp_msg_0[] = {-98, -46, -15, 110, 116, 68, 25, 9, -73, 60, 69, 114, -49, 85, 2, -126, 95, -6, 99, 125, 38, -79, 25, 69, 21, -5, -62, -64, -91, -89, -102, -95, 49, 10, -99, -121, -103, -19, -65, 107, -18, 26, 83, -64, -64, 5, 75, -32, -2, -16, 105, 125, 115, -109, 63, -86, 89, -38, -58, -103, -68, -51, -88, 40, -15, -100, -4, 58, 85, 19, -95, -34, -110, 60, -123, 80, -61, -80, 8, -29, 75, -95, -106, -51, -54, -34, 114, 21, -114, 41, -102, 106, 2, -73, -54, 48, -23, -20, -115, 38, 35, -34, 57, -6, 42, 98, 86, -23, 68, -28, 99, 28, -22, 106, 85, -93, -107, -5, 100, 32, -45, -106, 83, 84, -106, 33, 75, -25, -114, -25, -55, -60, 72, 108, 122, -107, 123, -107, 116, 43, 44, 11, 69, -99, 105, 43, 55, 13, 58, -103, 92, 119, 81, 65, 93, -64, 35, 32, 27, 122, 56, -79, -59, -63, -95, 100, 78, -114, -124, 73, -23, 104, 7, -36, -87, -86, 12, -93, -121, 105, -96, 62, -80, -87, 18, -27, -117, 46, -38, 31, -4, 54, -18, -20, -118, 48, -44, -92, 96, -82, -110, 82, -65, -57, -128, -73, 108};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RawImage #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RawImage msg;
    msg.setTimeStamp(0.662885276224);
    msg.setSource(38845U);
    msg.setSourceEntity(244U);
    msg.setDestination(39449U);
    msg.setDestinationEntity(150U);
    msg.width = 50815U;
    msg.height = 10898U;
    msg.channels = 239U;
    msg.depth = 49U;
    const char tmp_msg_0[] = {90, -75, -33, 75, -63, -48, -109, 2, 21, 85, -34, -50, 10, 77, 13, -40, -29, -65, -73, 74, 25, -74, 57, -100, -43, -103, -58, -92, 43};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RawImage #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RawImage msg;
    msg.setTimeStamp(0.274855490831);
    msg.setSource(40530U);
    msg.setSourceEntity(30U);
    msg.setDestination(3444U);
    msg.setDestinationEntity(104U);
    msg.width = 555U;
    msg.height = 34511U;
    msg.channels = 198U;
    msg.depth = 250U;
    const char tmp_msg_0[] = {83, -101, -92, 106, 38, -28, 59, 113, -53, 25, 119, -39, 73, -28, -44, 74, 107, -127, -75, 67, 15, -67, 56, -128, 65, -56, -62, 2, 120, 68, 113, -19, 2, -29, 92, 122, -78, -67, -73, -80, -83, -93, -110, -29, -35, -7, 87, -81, 24, 7, 99, 29, -7, -75, -119, -69, -4, 81, 56, -10, -108, 85, -4, 29, -51, 93, -19, -9, 60, 97, 28, -118, -63, -34, 76, -13, -113, -41, 106, 13, 117, -71, 24, 23, 21, -12, -98, 5, 79, 15, 71, -111, -36, -40, -81, -76, 76, 96, -19, 114, 43, 33, -76, 4, -14, -26, 72, -26, 107, 7, 66, -103, 41, 77, 97, 28, 23, -88, -85, -21, 40, -24, -100, 39, 64, 36, -10, 60, -38, 56, -99, -53, -65, 60, 84, -43, 61, -52, -69, -62, -73, -52, -68, -4};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RawImage #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CompressedImage msg;
    msg.setTimeStamp(0.466576966213);
    msg.setSource(38936U);
    msg.setSourceEntity(32U);
    msg.setDestination(28107U);
    msg.setDestinationEntity(229U);
    msg.frameid = 143U;
    const char tmp_msg_0[] = {-42, -125, 120, -56, 43, -41, -59, 5, 123, 40, 99, -101, 111, -21, -124, 124, -58, -9, 3, -78, -83, -36, 67, -114, 117, -84, -83, 105, 111, -81, 55, -75, -52, -112, 120, -40, -30, 32, 62, 120, 100, 40, 114, -83, 1, 17, -11, 32, -34, -102, -59, 36, 42, 94, 38, 54, -118, 54, -54, 105, 39, -114, -4, -39, 91, -24, 40, -109, -14, 42, 125, -79, -6, -24, 44, 66, -44, -3, -41, 15, -117, 31, 114, 102, 67, 18, -117, -127, -106, 75, 24, -19, -20, -63, -16, -127, -64, -125, -26, -123, 74, 23, -90, -80, -52, -16, -69, 109, 11, 39, -100, 52, 110, -52, 14, -90, 108, -25, -70, -72, 98, -86, -104, -103, -44, 16, -57, -108, -51, -15, 9, 90, -82, 75, -47, -114, -48, 49, 13, -44, -57, 87, -42, -10, 8, 114, -95, -36, -23, 104, -46, 26, 82, 27, -5, -80, -113, 119, -88, -23, -38, 24, -22, 117, 116, -49, 48, -50, -59, -45, 97, -98, -66, 114, 97, -18, 21, 93, -70, 109, -128, 50, -39, -110, 98, 8, -45, -81, 58, -24, 56, 105, 99, 37, -18};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CompressedImage #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CompressedImage msg;
    msg.setTimeStamp(0.212935197631);
    msg.setSource(26159U);
    msg.setSourceEntity(138U);
    msg.setDestination(54382U);
    msg.setDestinationEntity(231U);
    msg.frameid = 150U;
    const char tmp_msg_0[] = {110, 115, 122, -102, 79, -72, 91, 49, -94, -119, -104, 87, 10, -108, 117, 2, -20, -59, 51, -29, -8, 119, 77, -98, 113, -77, -95, 4, 4, 76, 96, 56, 120, 60, -22, -15, -103, 96, -34, -75, 33, 117, 86, 7, -81, -53, 24, 81, -22, -14, 15, 3, -77, -9, -25, -115, 94, -74, 93, -85, -2, 19, -116, -67, 75, 13, 10, 13, -8, -51, -98, -80, 112, 54, 42, 120, 101, 109, 17, 15, 71, 57, -126, -27, -49, 37, -99, 41, 106, -73, -6, -59, 53, 28, -82, -128, 54, -80, -127, 125, -112, 38, 7, -61, -108, 52, -111, -101, 3, -43, 64, 124, -83, -9, -31, 12, -23, 98, -46, -22, 88, 13, 94, -92, -119, 93, -65, -77, 17, 96, -106, 121, 68, 55, 33};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CompressedImage #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CompressedImage msg;
    msg.setTimeStamp(0.226464692258);
    msg.setSource(15153U);
    msg.setSourceEntity(111U);
    msg.setDestination(54694U);
    msg.setDestinationEntity(224U);
    msg.frameid = 14U;
    const char tmp_msg_0[] = {-23, 33, 121, 111, -53, -77, 47, 42, 66, -101, -49, 9, -7, 97, 86, -22, -2, 47, 67, -65, 102, -80, -45, -128, 81, -117, -56, 45, -99, 103, -112, 9, -38, -9, -92, -7, 105, 119, 0, 26, -68, 18, 29, 125, -123, 55, -104, -35, -85, 46, 5, -107, 31, -48, -121, -4, -50, -31, 112, 116, 14, -76, -19, 22, 82, -23, 24, 110, 88, 21, -91, -22, -125, -65, -49, 64, -16, -85, 67, 63, -114, -32, 112, -64, 77, 30, -52, 100, -79, 52, -7, 39, 18, -102, -80, -105, -108, -13, -78, 103, 58, 65, 91, -100, -121, -93, 96, -112, 81, 32, 74, 38, 96, -59, 124, -39, -6, 90, -52, 111, 89, 119, -26, -20, -5, 25, -33, 80, -64, 114, -103, 111, -50, -25, -22, -111, 4, -9, 74, -17, -70, -122, 61, -77, 109, 104, 55, 7, 14, -53, -76, 37, 41, 112, 21, 97, -15, -79, 65, -92, -77, 43, -114, -66, -75, 42, 81, 10, -101, -104, 121, 41, 15, 74, 114, 85, 40, -64, -60, -112, -83, 77, 59, -28, 84, -66, 64, 6, -11, -48, -106, 23, -18, -83, -107, 47, -81, 71, -42, 96};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CompressedImage #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ImageTxSettings msg;
    msg.setTimeStamp(0.290705343771);
    msg.setSource(31183U);
    msg.setSourceEntity(237U);
    msg.setDestination(18082U);
    msg.setDestinationEntity(251U);
    msg.fps = 230U;
    msg.quality = 194U;
    msg.reps = 221U;
    msg.tsize = 43U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ImageTxSettings #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ImageTxSettings msg;
    msg.setTimeStamp(0.226202849909);
    msg.setSource(63794U);
    msg.setSourceEntity(71U);
    msg.setDestination(53807U);
    msg.setDestinationEntity(150U);
    msg.fps = 222U;
    msg.quality = 66U;
    msg.reps = 105U;
    msg.tsize = 24U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ImageTxSettings #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::ImageTxSettings msg;
    msg.setTimeStamp(0.319152866401);
    msg.setSource(14541U);
    msg.setSourceEntity(94U);
    msg.setDestination(38827U);
    msg.setDestinationEntity(211U);
    msg.fps = 99U;
    msg.quality = 192U;
    msg.reps = 148U;
    msg.tsize = 152U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("ImageTxSettings #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteState msg;
    msg.setTimeStamp(0.388219763181);
    msg.setSource(7808U);
    msg.setSourceEntity(135U);
    msg.setDestination(18287U);
    msg.setDestinationEntity(164U);
    msg.lat = 0.546114802417;
    msg.lon = 0.261779395061;
    msg.depth = 200U;
    msg.speed = 0.936689710982;
    msg.psi = 0.518637627003;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteState #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteState msg;
    msg.setTimeStamp(0.452764278139);
    msg.setSource(38432U);
    msg.setSourceEntity(92U);
    msg.setDestination(24071U);
    msg.setDestinationEntity(185U);
    msg.lat = 0.952637402214;
    msg.lon = 0.150901998549;
    msg.depth = 10U;
    msg.speed = 0.111486162902;
    msg.psi = 0.732568519829;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteState #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::RemoteState msg;
    msg.setTimeStamp(0.405659218337);
    msg.setSource(18353U);
    msg.setSourceEntity(112U);
    msg.setDestination(62256U);
    msg.setDestinationEntity(185U);
    msg.lat = 0.224505957891;
    msg.lon = 0.196059364051;
    msg.depth = 208U;
    msg.speed = 0.43583406958;
    msg.psi = 0.225598394508;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("RemoteState #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Target msg;
    msg.setTimeStamp(0.296489357072);
    msg.setSource(25148U);
    msg.setSourceEntity(158U);
    msg.setDestination(15212U);
    msg.setDestinationEntity(87U);
    msg.label.assign("HJKNQKTEYYEYMUFE");
    msg.lat = 0.517851756722;
    msg.lon = 0.266713926493;
    msg.z = 0.191457296406;
    msg.z_units = 245U;
    msg.cog = 0.246537158812;
    msg.sog = 0.414458929488;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Target #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Target msg;
    msg.setTimeStamp(0.540934307168);
    msg.setSource(62216U);
    msg.setSourceEntity(132U);
    msg.setDestination(28770U);
    msg.setDestinationEntity(101U);
    msg.label.assign("MXQGYBRELTNDGCKOKAOSLWPUVXISKAIUKFJNOGHYRNLEZJHCPZNSYZAGOOEKKGDYWTXVTTFACVQPXNAGSKFAAELYTUOHFULTVYZBWHSNVCDKUEFLXCXDORNTHOSCMBVHWPZCLFUJRPWZOMFVLQMWQIQHBTVKQYQPGQOXBUMMWFJMJUFGGXEBNPUJREZQEBETDCCPRNWDHJKXDHISWBZAQVMRSNIAJFMDSJHYYRLIAPDMVLZEJWPIIR");
    msg.lat = 0.666008374586;
    msg.lon = 0.867866496475;
    msg.z = 0.755390437012;
    msg.z_units = 89U;
    msg.cog = 0.451866084756;
    msg.sog = 0.552285928228;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Target #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::Target msg;
    msg.setTimeStamp(0.957188571221);
    msg.setSource(62670U);
    msg.setSourceEntity(217U);
    msg.setDestination(21059U);
    msg.setDestinationEntity(92U);
    msg.label.assign("IMXYADEEYNQLUCGIYTOTZPWTMZBOPURGX");
    msg.lat = 0.111713863702;
    msg.lon = 0.287087960123;
    msg.z = 0.2980357586;
    msg.z_units = 157U;
    msg.cog = 0.0412772425267;
    msg.sog = 0.937707605433;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("Target #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityParameter msg;
    msg.setTimeStamp(0.634360535424);
    msg.setSource(31323U);
    msg.setSourceEntity(254U);
    msg.setDestination(23098U);
    msg.setDestinationEntity(67U);
    msg.name.assign("YMVLAQOAJWYADPAMNDXGIOLXPGJCXVR");
    msg.value.assign("VJTHMDJSVGIDYKWMSXERFGIULKPCXRPBBOPEFCXZBFQOPBLTDHAHKAWILXBNKGRXIVEYVXJNSSZTOPCOVIECJSYNNAQYAPZBKEKWPMWVGGNVLC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityParameter #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityParameter msg;
    msg.setTimeStamp(0.324782044198);
    msg.setSource(30137U);
    msg.setSourceEntity(171U);
    msg.setDestination(21863U);
    msg.setDestinationEntity(58U);
    msg.name.assign("MBBMGFNEGKCZIWVNTQGMAMNHLFOZLGHIAEOUUBAWRJRHLQUKKPSHSGJJBQWSNS");
    msg.value.assign("YKMPWFTXHPDDREHYMYLVTPNXEQAVFVZDRHYEPGRLBZPTAHHXXHCESJJEBCXZOKIPYWLIBKNUMSJBSRQONSGOQWINVCQAUDGLNDJVMZTDZVJUCKUIFFZTIIEUBAIITDCMQHKMYOQTAWHKQBOWTJEHONMLOFWBYXXGAIFWUL");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityParameter #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityParameter msg;
    msg.setTimeStamp(0.00787404032386);
    msg.setSource(45831U);
    msg.setSourceEntity(22U);
    msg.setDestination(55710U);
    msg.setDestinationEntity(10U);
    msg.name.assign("ERIWOMTUUROTBWEMLYGSNMGXBARVLXZESDOBAPFCHKPFCNXYVEJGDNMGTOSKRHHVCNXMCONYFATIMEZHZOFGVZRFZTTFCLZICXHMQJJQXAPTAUPYZXZNQUYSWORUAKJVPEZPTFWIBBIAXHDSPKQDKQGDVVBAQQWCNDHLYAMWFBDILIFHDLOUYOEWJRJSWBPGCLEECVYJIVPMDQRUSEPKBIQKL");
    msg.value.assign("MBLKDUHRRUGQSGPJSKKWINOQXZLMVKHXMXFHESZCASDNXUVOAGGSERJOIYXULGLPBQQUMUIUEVHJYLWSJTXSYYNUIZKLWCOTCGHYPTVZNZXXXPBUCZQASGELHRJCAMAYBWBBRBIRTQDQRVFZDZLFEAAMPEMJEJINMLOMVYBGRWWWJASTPTWTKOUJGQNIZIONCPAVHENOFACIDQYFOHSKIBFDDGEDBCKTKVLOWFMRFP");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityParameter #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityParameters msg;
    msg.setTimeStamp(0.39374411944);
    msg.setSource(1690U);
    msg.setSourceEntity(167U);
    msg.setDestination(34285U);
    msg.setDestinationEntity(73U);
    msg.name.assign("BWFXFLEMXBMPYDGTKIKUOOBAPVAEIFDOMQPFQZQVLUZKLB");
    IMC::EntityParameter tmp_msg_0;
    tmp_msg_0.name.assign("SYAJYSYLMENWHVLCISOUNQVKDWHYCTNZACRRBTRHBRMDMJUWPMSLKWXCJPVHXXVQPCZOFTEKBKAOOEQEOSLYRGROEZUNMIPPAQNYIGTLKOSJDZDDKHCQE");
    tmp_msg_0.value.assign("FCMORSSENYBPEACXTVMYTZPLIVMVQMMJFLRXCXTPYSATUKZIJSDLQZGVCKRERIQEFTGVHQFNFUOAXLZGMABHQHECYGACPNXPIDPGSZALUDAXWPNMMWBHVGLDVYLEJTUKKAUWDZJLQFCKIMNAJZWRDLGNILIFOWIQRJFWPEYDHNORP");
    msg.params.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityParameters #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityParameters msg;
    msg.setTimeStamp(0.890806363989);
    msg.setSource(22382U);
    msg.setSourceEntity(196U);
    msg.setDestination(21733U);
    msg.setDestinationEntity(68U);
    msg.name.assign("JYRVIMQWZIVGWJGFLTFMTGSCUMZFEYCSBDOHUMHDCNQSUZYAQSUZLKSUNCKWSRLWWWVCQGRAWINJUFVXIKHPNADJMYXVXMQNAOPOTEJYJSLHFPPWXTRXCFKEALEZIKCLNZCOPBJRLXRYDOBVJQKBMRTSTFHZUNCSIFP");
    IMC::EntityParameter tmp_msg_0;
    tmp_msg_0.name.assign("ARYSLCGDOHSXEMJDSRCRXEJMNCNIWPYWLHKYUQIOEWPFIXCDOZFUBKAPQVHDDXZKHLCHRJBCRWGEXSCJZMTQVOPJBRWMGFOFNAEZYFOBXMGEDTUGKLCHNQRXWQIOMJGGUQBRZXFYKNKBIMNYKFNBLIUWVUFXQGOEJILHLZAZWHTLUSMDVCZIDSBC");
    tmp_msg_0.value.assign("ZZQJGUWCXRWVLJTVVSBLUKJXDIMYUUYMVMOTDJZAZPWGXJAUCOWZFNBRFOJKHDIHAR");
    msg.params.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityParameters #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::EntityParameters msg;
    msg.setTimeStamp(0.898439966683);
    msg.setSource(49260U);
    msg.setSourceEntity(184U);
    msg.setDestination(44347U);
    msg.setDestinationEntity(235U);
    msg.name.assign("SFIANEUGHRVGYAWJPNKJGYDQKRCXKBWNHZOY");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("EntityParameters #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityParameters msg;
    msg.setTimeStamp(0.941896024417);
    msg.setSource(45579U);
    msg.setSourceEntity(102U);
    msg.setDestination(29111U);
    msg.setDestinationEntity(205U);
    msg.name.assign("EIWVIZSUKZUKWJOUNEEWZOJFLCMQEVWXWSVZBQPWOYAZAZWAWODPHHSIIUDIFKYYUFEUHGFCNLDJPADJRKXIXJHRCPHRBKVMGOGJAITTFJQCUCBMPVEZGDUQCGJTCCSXCLXTPMBSKBWGEYUASRVFQWRETVNFROGY");
    msg.visibility.assign("BJCDWMEUAHXBHRLXAIYDMDIZBOAWZXQROSJFPOLCFPJYNPYJCIQRIVJTBDI");
    msg.scope.assign("FAXSLFVUASJKOEXMBGGUCLHFJCZLNZJCQLKMYMFPIUHN");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityParameters #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityParameters msg;
    msg.setTimeStamp(0.629618425241);
    msg.setSource(15252U);
    msg.setSourceEntity(105U);
    msg.setDestination(19330U);
    msg.setDestinationEntity(194U);
    msg.name.assign("UQMKWNYMXDYCIQUARZDESFMOFXANNEMVOXMGBY");
    msg.visibility.assign("LLQUQGVJMURHZZDRQHSJISRYQETSGKAYFSKCKYOGEPLXFFFTTUR");
    msg.scope.assign("PKVRWWXZMFMOECOKBGKXSODFVOBMCKPBARYKXYAFTQMJNLLIUYCQPDZVHKBNXFZGVRSQZUXWAXZJRLKYHWUQEZEAAQHHC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityParameters #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::QueryEntityParameters msg;
    msg.setTimeStamp(0.846367284829);
    msg.setSource(15161U);
    msg.setSourceEntity(141U);
    msg.setDestination(24580U);
    msg.setDestinationEntity(150U);
    msg.name.assign("BYOLZLFMVKWFPLCRCGEUOQPIXPKYNDAJJBDYQZVVTDFIWFTJIPCHHNNBITZCMDAATYOOIKUARNBAPJAEMMLRUSCEZKQDP");
    msg.visibility.assign("BRUPURVKBBLOCLYYHFGHANFPWWXKVCHIAWDEBGCQJBZQIHQJRQWDWNQXLEAEPSLZWQVYGXYGTCTMKLFNPYNJSRGRVOUAXUBUZTCDSOOMPJBJFKTUEWHAJPMNNZDEUOETVOAQZMUXSISYZGCQWZAMDCXNEZYEGCODTTVMWRTBFMTJXKIDLFAZVQINOYIUVTDSNSHBFJILPSOESXEALLHHHCDKSAXHVRZFKPJXRRIYWKYFUIIKJNLBOQPGGMFVG");
    msg.scope.assign("RRTVBAYZXGITVJHSPAWPQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("QueryEntityParameters #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetEntityParameters msg;
    msg.setTimeStamp(0.146324765736);
    msg.setSource(50628U);
    msg.setSourceEntity(153U);
    msg.setDestination(13101U);
    msg.setDestinationEntity(51U);
    msg.name.assign("TCJLYGZMQIANWJBLAYMQJTCDEEZLSXTKGRNRNSSNFYKEOTJOMVZQOUGSFMVCPYCGPZGQZKGKIKZJPABDHRHLHMXWVXUUVFHWSBSEGLUWKXOZLL");
    IMC::EntityParameter tmp_msg_0;
    tmp_msg_0.name.assign("UMBFATMNBRVGIJSRQORKHXXGORJMDCBAPVOHGTMEZIIAJORCXQEMLKFTWSIGAYFJYQOKQFYHUPUDZSPCDEMJANMPDWJVKQAXDLUEGWGIQOMZVIWGXJNBHORYVLVGQZSHNJFKMZWWUQWHRZUXPSSTTFIQZLCNBHVXZKCCRT");
    tmp_msg_0.value.assign("SUZKPHLGOPCOSORAMAWXISPVWQQJTJCVFQLEBKVIILNHYESJMVDZHPQXYWVHIJBTJGYEEHUVKZNAFFIXRCQEXBWUKAFKUZAPY");
    msg.params.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetEntityParameters #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetEntityParameters msg;
    msg.setTimeStamp(0.438161838468);
    msg.setSource(56915U);
    msg.setSourceEntity(19U);
    msg.setDestination(5372U);
    msg.setDestinationEntity(33U);
    msg.name.assign("MHADUAFWETQQNYZBFLXNHOIKPXTRCMYJAMSFVPSSTHIUHIMFIKDKNPEDVGQWAKJBOVBNMLKIYFZGSUMFDGZXDIELMLTNUX");
    IMC::EntityParameter tmp_msg_0;
    tmp_msg_0.name.assign("NPDTGBPJNSCOHWMKEEZMXLRCYUWWIZDJIG");
    tmp_msg_0.value.assign("CQUJASDTALKQWPYKFRFKGKBNEAZVKIQHYNMHTFCDDNCGMXRMMOTTQRCWXJTNRVBUGRHDZLXNFALYPXVXZFDDVSFQSUZZSVSOILPSBCAMTUPTWQSGVWONOXIINNWVWUYIDOPLPEGIBBCWBFBBYHOQIGZTSHMYDHAAXHHAUIXRKNKACPUGGJXFELGSLPYFOWWJSOGPRHCNEWEUEDQEMPLJBYXEZKFJOCVQKIBJJTHMMLRYDRZLZQVJU");
    msg.params.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetEntityParameters #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SetEntityParameters msg;
    msg.setTimeStamp(0.841834904115);
    msg.setSource(30340U);
    msg.setSourceEntity(232U);
    msg.setDestination(28910U);
    msg.setDestinationEntity(41U);
    msg.name.assign("DJYLMDHGDMCSOJIYLAKMHKBCTXGKPWGXUIYMRJWGEDXJTIZWNOZYPZUFCBPQOVCSYNOSYBBRFHUSSIONNKDRJTSTWRVAOBHZRHXREMBNAVTYAFDVBAJQIKWIJTVTMELCZMAUILNUXHVVKZLHU");
    IMC::EntityParameter tmp_msg_0;
    tmp_msg_0.name.assign("MGOGJECLSPTFLHFEYEQFDKTUUBYOAGTAYKSWZMOCOPUUFWTQVCZISE");
    tmp_msg_0.value.assign("BWHPBXBRECXRXTFIXRVGYEZPCGNWFDRONBPRVRGXGHGJMYTKAZYLASBVDHQJADWOUEQTFAOMIJQTZYXQJERLLHGTMTNLPSKXWDCMGZWUVCFUDZFMIBSEVCNKUDSFKVAQJIXYKTPFJOLCIPNZZYUDJOMIFWGWCLHSUPQMMVHCFF");
    msg.params.push_back(tmp_msg_0);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SetEntityParameters #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SaveEntityParameters msg;
    msg.setTimeStamp(0.112851446381);
    msg.setSource(60095U);
    msg.setSourceEntity(98U);
    msg.setDestination(38994U);
    msg.setDestinationEntity(185U);
    msg.name.assign("ALZPAPZDELEOQIVFRVJSKANDTESWKGIPGCZNUVWEFEYYVCAUXKGUQPVJHRKZBSPAGRXMOLQQZSEZBCDIIRAWOPMHTZWDDOOUMCFXUHITNQUAFTJVCHBIJUCLUXHFXLCVDASTGTLRXYYEJBF");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SaveEntityParameters #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SaveEntityParameters msg;
    msg.setTimeStamp(0.264041981414);
    msg.setSource(35196U);
    msg.setSourceEntity(93U);
    msg.setDestination(34189U);
    msg.setDestinationEntity(168U);
    msg.name.assign("XFFBUNUVGRZTTCWAKLEEGHWHXGVFSLLOJLSTLKTVFENVNTQVRJHUOCJEPSQMXIMEIRHYAYGOQRRQMCHKFWABCMJHSUXKYZTUAROHYZMBQDDMWRNXTPFFOOPAMDWHEKIDVJNVZLUJWSCSTZBRNYODWJGXGSMTEQIFIDPPPLLYXGKJCPZSBNZFJWYQEMDXZQCBUYKISTURBJINBHABWANCEZQPZN");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SaveEntityParameters #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SaveEntityParameters msg;
    msg.setTimeStamp(0.682055726525);
    msg.setSource(5618U);
    msg.setSourceEntity(57U);
    msg.setDestination(62286U);
    msg.setDestinationEntity(252U);
    msg.name.assign("SJIYBRCLOGFFAUXJIFWHNLQPTLUMBLEQYTHMNVZAWZBGEQVQIXJPKFGPWCBLZIFXGSZLBQHWUYFDOKEBMCBAGXRTMEZXDTPIOVVINEJPCGAWPWSSZQZDJWTIAKKODEKVAOYTMPPGVSNUKLRAIADZVDJYUGRKTRJANUNZSKMQDLRPFHLVYGQQOOWCTUHBDDCTKIAHJ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SaveEntityParameters #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CreateSession msg;
    msg.setTimeStamp(0.190403957578);
    msg.setSource(17729U);
    msg.setSourceEntity(183U);
    msg.setDestination(9742U);
    msg.setDestinationEntity(106U);
    msg.timeout = 2127123186U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CreateSession #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CreateSession msg;
    msg.setTimeStamp(0.407495421349);
    msg.setSource(3795U);
    msg.setSourceEntity(30U);
    msg.setDestination(41337U);
    msg.setDestinationEntity(3U);
    msg.timeout = 3769519958U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CreateSession #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CreateSession msg;
    msg.setTimeStamp(0.974796372955);
    msg.setSource(29794U);
    msg.setSourceEntity(38U);
    msg.setDestination(53227U);
    msg.setDestinationEntity(183U);
    msg.timeout = 2356368650U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CreateSession #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CloseSession msg;
    msg.setTimeStamp(0.997726773237);
    msg.setSource(17009U);
    msg.setSourceEntity(48U);
    msg.setDestination(30532U);
    msg.setDestinationEntity(172U);
    msg.sessid = 934808155U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CloseSession #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CloseSession msg;
    msg.setTimeStamp(0.867179065272);
    msg.setSource(7336U);
    msg.setSourceEntity(135U);
    msg.setDestination(16824U);
    msg.setDestinationEntity(185U);
    msg.sessid = 960844860U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CloseSession #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::CloseSession msg;
    msg.setTimeStamp(0.315525353195);
    msg.setSource(32265U);
    msg.setSourceEntity(97U);
    msg.setDestination(11194U);
    msg.setDestinationEntity(192U);
    msg.sessid = 3686758941U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("CloseSession #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SessionSubscription msg;
    msg.setTimeStamp(0.057299525398);
    msg.setSource(55U);
    msg.setSourceEntity(129U);
    msg.setDestination(59910U);
    msg.setDestinationEntity(136U);
    msg.sessid = 4103209635U;
    msg.messages.assign("TQFXYIHFVJKGTBHUXHNOJEBMVDDWNXRXZNNVAHPOPSPYTBXFTRMQWCUPQLXUGCSGFWRQLNOZNDBFGXOTPGIMLXMJELNVHDKGBTYPWPOEZJABABCIYEDIMKFZRQWCTAVHDBSKJJOYIWSFWNMYPQGTCOLMESMISGQRUILLPTEZFRAZUADIFCDMJIYRJJJBHNUVKC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SessionSubscription #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SessionSubscription msg;
    msg.setTimeStamp(0.495722215346);
    msg.setSource(58026U);
    msg.setSourceEntity(0U);
    msg.setDestination(30129U);
    msg.setDestinationEntity(116U);
    msg.sessid = 382495710U;
    msg.messages.assign("QSOPWMLOXWJUEZESAIRKMECKZYDHRETVANXWDIDXQWGZMGCXNQKSRPLKVUMNIJZSSAHQCPCLTNJXQ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SessionSubscription #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SessionSubscription msg;
    msg.setTimeStamp(0.71543600347);
    msg.setSource(50685U);
    msg.setSourceEntity(208U);
    msg.setDestination(43977U);
    msg.setDestinationEntity(96U);
    msg.sessid = 1127501588U;
    msg.messages.assign("TPCQJUFCEHQRDMIOAQABHESALKWWZFPRMNNUUFYSVLVMNMMGWWYVGCDHBOIZKTHJVXAEIZCDQECYBJGYPNGPWQGGSVGCLJGYRPSXLPCOOLTZADNBROMUDKVOMZUVIPVCFSQMDLSBMDWUYTJLLFASKJOFOPIKKEHLTQAEIUXZSSDNROJTRNIFQEXSXBWXITJUA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SessionSubscription #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SessionKeepAlive msg;
    msg.setTimeStamp(0.216531481218);
    msg.setSource(401U);
    msg.setSourceEntity(69U);
    msg.setDestination(29180U);
    msg.setDestinationEntity(93U);
    msg.sessid = 709434185U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SessionKeepAlive #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SessionKeepAlive msg;
    msg.setTimeStamp(0.0580499461566);
    msg.setSource(17371U);
    msg.setSourceEntity(63U);
    msg.setDestination(37378U);
    msg.setDestinationEntity(147U);
    msg.sessid = 1217487291U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SessionKeepAlive #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SessionKeepAlive msg;
    msg.setTimeStamp(0.194345366557);
    msg.setSource(31760U);
    msg.setSourceEntity(234U);
    msg.setDestination(18864U);
    msg.setDestinationEntity(179U);
    msg.sessid = 3475703582U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SessionKeepAlive #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SessionStatus msg;
    msg.setTimeStamp(0.255331384532);
    msg.setSource(49375U);
    msg.setSourceEntity(166U);
    msg.setDestination(4213U);
    msg.setDestinationEntity(125U);
    msg.sessid = 338592383U;
    msg.status = 249U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SessionStatus #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SessionStatus msg;
    msg.setTimeStamp(0.748744842157);
    msg.setSource(66U);
    msg.setSourceEntity(3U);
    msg.setDestination(19405U);
    msg.setDestinationEntity(154U);
    msg.sessid = 3687911741U;
    msg.status = 93U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SessionStatus #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::SessionStatus msg;
    msg.setTimeStamp(0.744120172044);
    msg.setSource(34495U);
    msg.setSourceEntity(245U);
    msg.setDestination(59972U);
    msg.setDestinationEntity(78U);
    msg.sessid = 1619691880U;
    msg.status = 188U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("SessionStatus #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PushEntityParameters msg;
    msg.setTimeStamp(0.338769121669);
    msg.setSource(22867U);
    msg.setSourceEntity(75U);
    msg.setDestination(53351U);
    msg.setDestinationEntity(232U);
    msg.name.assign("GGOPCLTXBBOOCJHVAAD");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PushEntityParameters #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PushEntityParameters msg;
    msg.setTimeStamp(0.312956461106);
    msg.setSource(44429U);
    msg.setSourceEntity(6U);
    msg.setDestination(35784U);
    msg.setDestinationEntity(178U);
    msg.name.assign("LLXSFLFHMWPOMCIHDPGZGSPJIAWAZVTIFFXTGXIUGBQKREEWYGZMHTWIUYKNYXEMLIKCJZIQRBGQRMBENRAXSTFPKZRRWYOORCWAGAZJRUPLUNKXQLKBZOQVAWKAPSDBOYSTRNJJBIMKEFFPLNONSY");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PushEntityParameters #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PushEntityParameters msg;
    msg.setTimeStamp(0.173061974707);
    msg.setSource(65133U);
    msg.setSourceEntity(126U);
    msg.setDestination(14052U);
    msg.setDestinationEntity(167U);
    msg.name.assign("BGQMGHLFBASKQBXZWYCGOEISDNZDTVJEVVWIGFUNCJSJTFJUHEMKGOUISWRWZIUJTYLKRSDVQPSWEQTVDNBWNKXWEAJWFCFP");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PushEntityParameters #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PopEntityParameters msg;
    msg.setTimeStamp(0.984463046847);
    msg.setSource(14039U);
    msg.setSourceEntity(63U);
    msg.setDestination(44019U);
    msg.setDestinationEntity(70U);
    msg.name.assign("OVNRFMOVCNKYCDBDVPXWSQWETTUPGCKDUKMSOHUYEQSVZCFIQJOWZGXLJLAZHRFVFFUQSWIZBOHVVRZLSEDZSBFHWILHNPCXTFEUSKWUXENYBUSLKQBIKRGBDDTQGSTJKJIRBZTGCKJGMHLAXHCMJITAYO");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PopEntityParameters #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PopEntityParameters msg;
    msg.setTimeStamp(0.179508694266);
    msg.setSource(38902U);
    msg.setSourceEntity(78U);
    msg.setDestination(12682U);
    msg.setDestinationEntity(224U);
    msg.name.assign("MXLSTSCKWKBFKMBDSGQZPBWRTOJQNPWWCDCEEFQXRQNZAEKONMERIJQMKTEVDDXBIOFYJFKBXZCQJCHJRHFYYWUNNOQEOSGSUFKNCMNZITKUPDHALLVSVCLEGMAXYYUTSBGDORHOUUHPFVEXEBHLZYUTIIYCKXYPLLDGSJHVIIWSEDRHUPKBOPYVJHGAJTRZDAGNBUVTWORWVAAGAXZMWMPRAIQIMWPFNGQSFVLADXOGFQRUM");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PopEntityParameters #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::PopEntityParameters msg;
    msg.setTimeStamp(0.821163488743);
    msg.setSource(64983U);
    msg.setSourceEntity(0U);
    msg.setDestination(60580U);
    msg.setDestinationEntity(3U);
    msg.name.assign("YINCQMDDRFJPAYFTVSGMCJXEPLXVUFABYKWXLGJFOSVEKLASMLQLUNXWCBXSBGRQRGUARRORQTAUYWKQMEMWSLUETLBMHKZJECNFORCISQETSXAXCFYDHNOBDBZVAKUHETBBITJBTIIIJMJEHMGNNPGIOPWDOVUZIOFOSHMWOYCMLGCBJRDFZNVLSZPXQYJPKTLRFEWJTZRFUUSKAHHVTGNWDOVZADICIYKYQPEZAXVXWUQZHDY");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("PopEntityParameters #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IoEvent msg;
    msg.setTimeStamp(0.683235634158);
    msg.setSource(28788U);
    msg.setSourceEntity(10U);
    msg.setDestination(18772U);
    msg.setDestinationEntity(59U);
    msg.type = 179U;
    msg.error.assign("LKBRPNDUZXSKHC");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IoEvent #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IoEvent msg;
    msg.setTimeStamp(0.940802572631);
    msg.setSource(4082U);
    msg.setSourceEntity(250U);
    msg.setDestination(49703U);
    msg.setDestinationEntity(106U);
    msg.type = 90U;
    msg.error.assign("TZWJOQWWVATVLPUXABCEKPLDGEMKNMIRESUXHKCRKETOUQWGMPQQQASVECWAIWBLCGYVHFNKDNCGDXBOPFCREYQARURJGEZITDDFXLSAIFJHJZBKLZFQVSTTIONWNTDHYINBMIGHPRXLXVBIBCSYLLZXKRDWJRYNHPATCWDMA");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IoEvent #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::IoEvent msg;
    msg.setTimeStamp(0.788951386715);
    msg.setSource(60042U);
    msg.setSourceEntity(133U);
    msg.setDestination(62121U);
    msg.setDestinationEntity(78U);
    msg.type = 52U;
    msg.error.assign("YZCIADHQJBUFXNVVKEXECJTGPCOOWZHBSGQSLVFZTXEZRPWASLAUJQWSWTELREXISJLGYHBTCKGCEDLUMQIXBHTWWYSVPOHDOFPYI");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("IoEvent #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamTxFrame msg;
    msg.setTimeStamp(0.724613359356);
    msg.setSource(12603U);
    msg.setSourceEntity(220U);
    msg.setDestination(519U);
    msg.setDestinationEntity(119U);
    msg.seq = 52319U;
    msg.sys_dst.assign("SYIMTYESVFIR");
    msg.flags = 191U;
    const char tmp_msg_0[] = {-110, -84, 115, -49, 57, 12, 0, -113, 5, 35, -18, -27, -21, -54, 107, -34, 115, 106, 15, 26, 78, -76, -112, 95, -38, 13, -40, -74, -116, 8, 2, -57, -93, -76, -64, -36, -87, -73, 20, -91, -114, 88, -106, -46, 110, -62, 32, 111, -118, 47, 18, -116, 19, -85, -85, 98, -127, 92, -38, -84, -47, -41, 13, -44, -38, 85, -113, -31, -4, 39, 46, -15, -58, 28, -63, -36, -8, -91, -122, 123, 96, 31, -35, -88, -28, -46, -117};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamTxFrame #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamTxFrame msg;
    msg.setTimeStamp(0.870047255651);
    msg.setSource(52900U);
    msg.setSourceEntity(25U);
    msg.setDestination(12196U);
    msg.setDestinationEntity(80U);
    msg.seq = 39640U;
    msg.sys_dst.assign("BTPRLRHCNGCYEQQIZYWDVNZANTUQFIKZSXSCDSRELNJBGLIIQJWPXBDMGJVZUAJOMVKXMJZNSYIWXRFPIWXKBLSWCBWRJCKABXPTUPYDFUBIGCLDWKWSVGZJOMEEFDMSYFEKOSXEEQXBZOAOMYVLNXNHHHGCLLSMJTNUAZRGFADTMAHQHYUBKWCKOTFUQACYUDYFR");
    msg.flags = 120U;
    const char tmp_msg_0[] = {-20, -17, 35, -76, 86, -8, 26, -51, -10, -76, -57, 50, -81, 107, 6, -53, 38, 59, 50, 64, -66, -45, 45, -125, -93, -31, -5, -10, 31, 22, 92, 57, 111, -62, 123, -20, -1, -12, 57, 60, 92, -92, 76, -128, 60, 94, -97, 99, 116, -97, -31, 30, -79, 2, -50, -69, -66, -61, -98, 37, -19, 64, -81, -43, -54, 102, -123, -103, -124, 30, -111, -66, -8, 118, 36, 61, 75, 98, 19, 26, 110, -49, -68, -122, -3, 77, -71, -57, 54, 85, -38, 105, -2, 106, -24, 36, -108, -61, 86, -74, 112, 84, 124, -62, 24, -99, 29, -37, 25, -96, -116, -34, -107, 80, 3, 39, -58, 48, -41, 75, -84, -53, 51, -110, 92, -65, 53, -107, -98, 76, -18, -61, 36, -49, 7, -98, -6, 87, -24, 26, 110, 14, -57, 51, -99, -34, 32, -58, 10, 112, 17, 95, 41, -52, -60, -117, 73};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamTxFrame #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamTxFrame msg;
    msg.setTimeStamp(0.37249404111);
    msg.setSource(61903U);
    msg.setSourceEntity(131U);
    msg.setDestination(55725U);
    msg.setDestinationEntity(23U);
    msg.seq = 27188U;
    msg.sys_dst.assign("XUUWTJWTFRHLIGGZRGYZKMZUDBFSOMKNCZTAKTGGXBTCMDRPSQQJVXLVERBWMFLYONCIKAIIXUCJOGXLJMCDHYANANNQHBYBEBMJRBFPWQWVTYSOQTIVOEFMCAOKPWKSIHLLOFGKSSAHPEEYSTOUSELREXNVQJDUURSIHNVNAZDXWXVFZLJLGUAKQBKRMTYVRURJBSLDCOKBPEOZDMTGJHAPHVCIXWQYEZPPDPEMNANFCX");
    msg.flags = 217U;
    const char tmp_msg_0[] = {22, 15, 112, 80, -37, 114, 99, -11, 33, -82, -97, -26, -41, 101, 100, -93, 120, -65, -31, -104, 116, 57, -27, 68, -68, -5, -108, 45, 61, -36, 7, -56, 92, -106, 68, -2, -6, -83, -120, 46, -58, -47, 23, 121, 81, -119, 0, 0, 84, -54, 91, 22, -69, 75, -16, 4, 124, 24, -85, 58, -6, -51, -72, -108, -94, -104, 15, -79, -52, 38, 41, 58, 102, 4, 80, 44, 51, 34, -42, 24, 47, 103, 43, 37, -11, -33, 42, -13, -78, 72, -28, -61, 126, 71, 66, 102, 4, -106, -18, -105, -34, 125, 6, -69, -18, -107, -110, 5, -103, -72, 48, -98, -99, 40, 80, 126, -62, 92, -109, -80, 7, -109, -100, -28, -89, -59, 80, 30, -126, 115, 104, -48, -4, -22, -72, 4, -2, 49, 29, 110, -70, 6, -112, -109, 25, -70, 88, 54, 100, 119, -54, -46, 78, -13};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamTxFrame #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamRxFrame msg;
    msg.setTimeStamp(0.212387961133);
    msg.setSource(1900U);
    msg.setSourceEntity(241U);
    msg.setDestination(10215U);
    msg.setDestinationEntity(249U);
    msg.sys_src.assign("RFEPLFPFTBJORAUTHZAKLUPKBGYKHGDNOOMMPQJBNNVQVQQBOXZXOEVBWCQLETYRSMVGIOLPHSLWLHFWIAMKPUIFFMJQWCWHEWIZDSAEXUIRSKQZACYYKDQZNGGBBUVFIKUXPILNNIJCXQYFBJAT");
    msg.sys_dst.assign("RQGFPEAOYTNKPSMTDWOILCVXJYWTBBKYIYVFTOPUDVBZRWZEKQDHUGJWOGJFUDZHZCGSPNEMUDAXXSOXKHIIEQRQFAUIQUDYALVEFXGKZOCNHJLYDBOSGMGZETBAKNCHTIWPJWEHGYSQRXLNEDYVWBMWMMVJBHWFELFTVUOYLQJLCCVPICBYMVBILIRPQRKZZMFIJRSAKZSLAMPHMEWXTNPNUFDJRAHNNPDNLJ");
    msg.flags = 253U;
    const char tmp_msg_0[] = {56, 62, 52, -116, 126, -103, 88, 120, -36, -112, -32, 72, 96, 42, -48, 46, -98, 113, -124, 8, -26, -14, -124, 34, 94, 102, 102, -29, -124, -77, -16, -53, -64, 87, -114, -58, 59, 94, 70, 111, 51, -124, -97, -35, 76, -95, -18, 75, 30, 115, -113, 17, 40, 64, -106, -41, 10, -47, 44, 39, 61, -123, 34, -31, -66, -63, 54, -114, 69, -118, -40, 49, -16, 119, -10, -15, -93, 93, -10, 39, 78, 72, -44, 96, -9, 1, -125, -69, 14, 3, -80, -119, -95, -20, 22, 117, -10, -102, -94, 32, 1, -107, -71, -50, -63, 73, -103, -121, -63, 125, -83, 86, 76, -84, -94, 28, 120, 50, -67, -102, -9, -1, 69, 70, 30, -122, -77, -31, 114, 108, -9, 104, -83, 68, 78, -84, 46, 10, -65, 97, 49, -32, 75, 81, 118, -117, -24, 117, -54, 6, -93, 79, 108, 58, 84, 90, -41, -119, 14, -99, -94, 117, -56, 110, -11, 88};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamRxFrame #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamRxFrame msg;
    msg.setTimeStamp(0.736534128579);
    msg.setSource(38002U);
    msg.setSourceEntity(180U);
    msg.setDestination(28039U);
    msg.setDestinationEntity(218U);
    msg.sys_src.assign("PMMFLDCZWEFLSSLEUCPCTRDHOIGXERVDWPAVAZUGUVEWLXISKBYRU");
    msg.sys_dst.assign("PDIIVSDWDROCCSNJWALXEHGAUPEVKZUBOYGVSHEUJISLLYHLMBLJ");
    msg.flags = 63U;
    const char tmp_msg_0[] = {111, -122, 124, -48, -85, 97, 22, 19, 89, 8, 39, 28, -81, -43, -17, 126, 31, 91, 123, 58, -94, 25, -59, 95, 4, -67, 25, -113, 4, -122, -102, 101, -88, -72, -116, 13};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamRxFrame #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamRxFrame msg;
    msg.setTimeStamp(0.26139086331);
    msg.setSource(17042U);
    msg.setSourceEntity(42U);
    msg.setDestination(276U);
    msg.setDestinationEntity(161U);
    msg.sys_src.assign("FPWCLVLZMCMPHTUVQROYIQJXHOPWRADIQRHRCEGUUBNYTBVNMXMYGUKFNAWNGYHZCFCJFISSXWXUWVVAKQFVQIZCXDYSDLPCYTHYXRLKTWLFGJOONJDMYLQICAKBBMDJOZDIGORGHLDLRUXRHPQJHBZEANQPZXKSFCEBYPTJOZFPEKSCDRISU");
    msg.sys_dst.assign("LTZQRWDDJCXEVUIWORVZKEFLGXRIVPZLTIAFGFDVUDCJHXNSGUJLIEIJVDBCSCIMAITEBTESHSYMMYPWDODLZNBKBHUNRRAMVNGEOPQWEYJMWZGQPYWQOXRQQRPITGFHHMRQNGPXYMFDOKRWDGNZZDITHNOHIVUAQCQKCATBTPSAEJWNFBLHPGFMPOFKXOXSUYEJARUXSMUZFFBSKCY");
    msg.flags = 82U;
    const char tmp_msg_0[] = {52, 90, -57, -41, 112, 40, -45, -66, -73, 105, 37, -16, 46, 94, 123, 36, -97, 3, 14, 17, 124, -66, -77, 101, -107, -61, -91, 113, 70, -71, 35, 119, -119, -128, -59, 96, 111, 23, 88, 23, -13, -41, -1, -58, -94, 104, -101, -126, -63, 27, 13, 79, -23, -5, -56, -22, -54, -76, -60, -35, -85, 101, -6, -86, 69, 91, 37, -8, -27, 3, -75, 24, -72, -45, -125, 47, -12, 54, -112, -74, 68, -127, 65, -127, -58, 116, 122, 56, 57, 7, -76, 78, 115, -8, -3, -98, -127, 80, -93, -42, 73, -71, 110, 34, 115, -74, -33, 102, -32, 103, 1, 76, 105, -81, 105, -104, 37, 55, 102, -81, 27, 20, -87, 74, -71, -108, 11, 87, 35, 109, 28, -7, -77, -32, -85, 84, -64, 28, -114, 4, 102, 58, 37, 103, -61, 106, -120, 104, -82, 36, 56, -66, -55, -56, -127};
    msg.data.assign(tmp_msg_0, tmp_msg_0 + sizeof(tmp_msg_0));

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamRxFrame #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamTxStatus msg;
    msg.setTimeStamp(0.719590598735);
    msg.setSource(11447U);
    msg.setSourceEntity(101U);
    msg.setDestination(55734U);
    msg.setDestinationEntity(215U);
    msg.seq = 40709U;
    msg.value = 87U;
    msg.error.assign("UYREBJFQBKWLGKETZKRPPYXYNIHUNZWXGGDZ");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamTxStatus #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamTxStatus msg;
    msg.setTimeStamp(0.829280680869);
    msg.setSource(53623U);
    msg.setSourceEntity(74U);
    msg.setDestination(58673U);
    msg.setDestinationEntity(211U);
    msg.seq = 35789U;
    msg.value = 235U;
    msg.error.assign("BWHMGTANCXDNPBIRJCHYFYQIUJOBJSDUJDZCAFRNIGLUWIJKKWTWFZKYLLHWDHFNYDLAEVTGNUPQIHPFSMBSMUIUQEVJISN");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamTxStatus #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamTxStatus msg;
    msg.setTimeStamp(0.824937777883);
    msg.setSource(14720U);
    msg.setSourceEntity(215U);
    msg.setDestination(54318U);
    msg.setDestinationEntity(248U);
    msg.seq = 27469U;
    msg.value = 135U;
    msg.error.assign("DSUITCOVYHIXPKUJACIGXBVYGNWHHUJFBHKGCREDGPTDNUWVKHEDOPSEXYYDBXQYMNTTCKRTKLIZXBXFHPOTYRGCBAIJJEBKVBDHGZMCNXJQVNDKNRYEWLJURZPCILIWVAMVGLISQOTHPHSEDLNGCQWOILJYWZSPALRLPFWODMMRMWBSFGVXVKKEBHZFPAFSTFDEJQRJSZZMQZUTRUGVUMWFYAIWESZFXJASOUPQRZFXMBANANLCMTOL");

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamTxStatus #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamRxRange msg;
    msg.setTimeStamp(0.798319482915);
    msg.setSource(62180U);
    msg.setSourceEntity(128U);
    msg.setDestination(51770U);
    msg.setDestinationEntity(37U);
    msg.seq = 32552U;
    msg.sys.assign("ASZNCROETCHWE");
    msg.value = 0.750937562778;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamRxRange #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamRxRange msg;
    msg.setTimeStamp(0.750338660068);
    msg.setSource(28578U);
    msg.setSourceEntity(122U);
    msg.setDestination(40407U);
    msg.setDestinationEntity(15U);
    msg.seq = 15763U;
    msg.sys.assign("DLXZYCECBCVBCIRKEQTJKWXAAAOVPUFHFDNLHKYPVFKOFOMYBATLDGRLJIGMSUAYVWHPPLFIWGSDXQQUHVGLYMHICPSTFA");
    msg.value = 0.48970938347;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamRxRange #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::UamRxRange msg;
    msg.setTimeStamp(0.800910533584);
    msg.setSource(14404U);
    msg.setSourceEntity(242U);
    msg.setDestination(54425U);
    msg.setDestinationEntity(192U);
    msg.seq = 22096U;
    msg.sys.assign("ZSAXLRQWRNKWMQQJYZVIYAVCFYNNYMWSJMWHDTBPOFJLREOGORIUKTQQETKBANUWOHUXMPJBOBDIWAUBECWVNKVJRYYPGXGJXVYDLCEFRPVKPYENDVIGLNCGAHTPKGECNZKSAVWHQMCADPZTSHATCJMUBFCLKENIAXEIYIIQOSNH");
    msg.value = 0.378686912551;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("UamRxRange #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AbortAcked msg;
    msg.setTimeStamp(0.504970539857);
    msg.setSource(31901U);
    msg.setSourceEntity(191U);
    msg.setDestination(11218U);
    msg.setDestinationEntity(114U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AbortAcked #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AbortAcked msg;
    msg.setTimeStamp(0.468127720923);
    msg.setSource(53204U);
    msg.setSourceEntity(124U);
    msg.setDestination(33105U);
    msg.setDestinationEntity(224U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AbortAcked #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::AbortAcked msg;
    msg.setTimeStamp(0.983239684744);
    msg.setSource(23983U);
    msg.setSourceEntity(65U);
    msg.setDestination(35670U);
    msg.setDestinationEntity(44U);

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("AbortAcked #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormCtrlParam msg;
    msg.setTimeStamp(0.264773891896);
    msg.setSource(17812U);
    msg.setSourceEntity(170U);
    msg.setDestination(1735U);
    msg.setDestinationEntity(89U);
    msg.action = 17U;
    msg.longain = 1192636294U;
    msg.latgain = 980091881U;
    msg.bondthick = 45244471U;
    msg.leadgain = 4232257195U;
    msg.deconflgain = 2874305223U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormCtrlParam #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormCtrlParam msg;
    msg.setTimeStamp(0.702903644171);
    msg.setSource(35671U);
    msg.setSourceEntity(236U);
    msg.setDestination(49055U);
    msg.setDestinationEntity(85U);
    msg.action = 53U;
    msg.longain = 22371056U;
    msg.latgain = 1075276645U;
    msg.bondthick = 1573237844U;
    msg.leadgain = 2424885542U;
    msg.deconflgain = 427927086U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormCtrlParam #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::FormCtrlParam msg;
    msg.setTimeStamp(0.107074146094);
    msg.setSource(14229U);
    msg.setSourceEntity(230U);
    msg.setDestination(15807U);
    msg.setDestinationEntity(141U);
    msg.action = 104U;
    msg.longain = 1125317877U;
    msg.latgain = 2892376729U;
    msg.bondthick = 1697662211U;
    msg.leadgain = 4047844946U;
    msg.deconflgain = 1105835015U;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("FormCtrlParam #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DetectedObject msg;
    msg.setTimeStamp(0.942850652254);
    msg.setSource(48477U);
    msg.setSourceEntity(151U);
    msg.setDestination(12388U);
    msg.setDestinationEntity(250U);
    msg.id = 3262147713U;
    msg.state = 137U;
    msg.lat = 0.309987555128;
    msg.lon = 0.903176172444;
    msg.time = 0.431748612433;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DetectedObject #0", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #0", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DetectedObject msg;
    msg.setTimeStamp(0.349908587475);
    msg.setSource(14007U);
    msg.setSourceEntity(28U);
    msg.setDestination(37486U);
    msg.setDestinationEntity(63U);
    msg.id = 3191965521U;
    msg.state = 155U;
    msg.lat = 0.6646584376;
    msg.lon = 0.377763121027;
    msg.time = 0.333452215957;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DetectedObject #1", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #1", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  {
    IMC::DetectedObject msg;
    msg.setTimeStamp(0.42370221155);
    msg.setSource(34731U);
    msg.setSourceEntity(191U);
    msg.setDestination(40315U);
    msg.setDestinationEntity(42U);
    msg.id = 1297494530U;
    msg.state = 11U;
    msg.lat = 0.836522608445;
    msg.lon = 0.928557542905;
    msg.time = 0.674419341458;

    try
    {
      Utils::ByteBuffer bfr;
      IMC::Packet::serialize(&msg, bfr);
      IMC::Message* msg_d = IMC::Packet::deserialize(bfr.getBuffer(), bfr.getSize());
      test.boolean("DetectedObject #2", msg == *msg_d);
      delete msg_d;
    }
    catch (IMC::InvalidMessageSize& e)
    {
      (void)e;
      test.boolean("msg #2", msg.getSerializationSize() > DUNE_IMC_CONST_MAX_SIZE);
    }
  }

  return test.getReturnValue();
}
